#ifndef _DAQNAVI_SMART_PTR_H
#define _DAQNAVI_SMART_PTR_H

//  
//smart_ptr.h : our simple smart pointer  
//  
template <typename T>  
class smart_ptr
{  
public:  
   smart_ptr() : m_ptr(0), m_ref(0)
   {  
      m_ref = new int32(1);  // create a ref_cnt here though we don't have any object to point to 
   }  

   smart_ptr(T* p) : m_ptr(p) 
   { 
      m_ref = new int32(1);  // create a reference counter in heap  
   }  

   ~smart_ptr() 
   { 
      if(AtomicDecrement(m_ref) == 0) {  // delete only if our ref count is 0  
         delete m_ref;  
         delete m_ptr;  
      }  
   }  

   T& operator*() 
   {  
      return *m_ptr;  
   }  

   T& operator*() const
   {  
      return *m_ptr;  
   }  

   T* operator->()
   {  
      return m_ptr;  
   }  

   T* operator->() const
   {  
      return m_ptr;  
   }  

   //add assignment operator and copy constructor  
   smart_ptr(smart_ptr<T> const &sp) : m_ptr(sp.m_ptr),m_ref(sp.m_ref)
   {  
      AtomicIncrement(m_ref);
   }    
   smart_ptr<T>& operator = (smart_ptr<T> const &sp)
   {  
      if(&sp != this) {  
         // handle the ref_cnt our smart_ptr previously pointed to 
         if(AtomicDecrement(m_ref) == 0) {  
            delete m_ref;  
            delete m_ptr;  
         }  
         //copy the ptr and ref_cnt and increment the ref_cnt  
         m_ptr = sp.m_ptr;  
         m_ref = sp.m_ref;  
         AtomicIncrement(m_ref);  
      }  
      return *this;  
   }    

private:  
   T*     m_ptr;  
   int32* m_ref;  
};  

#endif
