#ifndef _DAQNAVI_AI_CTRL_BASE_IMPL_H
#define _DAQNAVI_AI_CTRL_BASE_IMPL_H

class BDAQ_NO_VTABLE AiCtrlBaseImpl : public DaqCtrlBaseImpl
{
   typedef ArrayImpl<AiChannelImpl> AiChannelArrayImpl;

public:
   AiFeaturesImpl* getFeatures() { return &m_features; }

   int32 getChannelCount()
   {
      int32 count = 0;
      if (getInitialized()) { ModProp().Get(CFG_ChannelCount, count); }
      return count;
   }

   ArrayImpl<AiChannelImpl> * getChannels()
   {
      return (ArrayImpl<AiChannelImpl> *)&m_channels;
   }

   ErrorCode runSensorCalibration(int32 ch, CalibrationData const *data)
   {
      return ModuleRef().RunSensorCalibration(ch, data);
   }
   
   // -----------------------------------------------------------------------
   // helpers
   //------------------------------------------------------------------------
   // Get the other channel number of a channel pair.
   int GetCoupledChan(int ch)
   {
      int32 phyMap[MAX_AI_CH_COUNT] = {0};
      if (ModProp().Get(CFG_OrderOfChannels, MAX_AI_CH_COUNT, phyMap) != Success) {
         return ch; 
      }

      return GetCoupledChan(phyMap, ch);
   }

   int GetCoupledChan(int32 const chMap[MAX_AI_CH_COUNT], int ch)
   {
      for (int i = 0; i < m_features.getChannelCountMax(); ++i) {
         if (i != ch && chMap[i] == chMap[ch]) {  
            return i; 
         }
      }
      return ch;
   }

public: // ctor & dtor
   AiCtrlBaseImpl() : DaqCtrlBaseImpl(DaqAi), m_channels(false) 
   { 
      m_features.setOwner(this); 
   }

   virtual ~AiCtrlBaseImpl() {  }

   DaqAiModule & ModuleRef() { return *(DaqAiModule*)getModule(); }

protected: 

   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual ModuleType WantedType() { return DaqAi;  }

   virtual bool IsSuitable(DaqModule & module)
   {
      return module.getType() == DaqAi;
   }

   // initialize/de-initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = DaqCtrlBaseImpl::InitializeControl(x);
      if (ret == Success) {
         ret = m_features.getChannelCountMax() > 0 ? Success : ErrorFuncNotSpted;;
      }

      // Bug fix: 19068: cache the channel value range 
      // to avoid reading back the incorrect setting when set channel value range
      if (ret == Success) {
         LoadProperties();
      }

      // Force creating AI channel array
      {
         assert(m_channels.getLength() == 0);

         int count = m_features.getChannelCountMax();
         m_channels.setCapacity(count);

         for (int i = 0; i < count; ++i) {
            m_channels.Add(AiChannelImpl(this, i));
         }
      }

      return ret;
   }

   virtual void DeinitializeControl()
   {
      // reset all configuration to null
      m_channels.Clear();

      DaqCtrlBaseImpl::DeinitializeControl();
   }

   //
   // Reload the properties' value from device
   virtual void ReloadProperties()
   {
      DaqCtrlBaseImpl::ReloadProperties();
      LoadProperties();
   }

   void LoadProperties()
   {
   }

private:
   AiFeaturesImpl     m_features;
   AiChannelArrayImpl m_channels;   
};

#endif
