#ifndef _DAQNAVI_AI_FEATURE_IMPL_H
#define _DAQNAVI_AI_FEATURE_IMPL_H

class AiCtrlBaseImpl;

class AiFeaturesImpl
{
public:
   // ADC features
   int32 getResolution();
   int32 getDataSize();
   int32 getDataMask();

   // channel features
   int32                      getChannelCountMax();
   AiChannelType              getChannelType();
   bool                       getOverallValueRange();
   ArrayImpl<ValueRange>*     getValueRanges();
   ArrayImpl<BurnoutRetType>* getBurnoutReturnTypes();

   // filter
   ArrayImpl<FilterType>*     getFilterTypes();
   void                       getFilterCutoffFreqRange(MathInterval &x);
   void                       getFilterCutoffFreq1Range(MathInterval &x);
                             
   // CJC features
   bool                       getThermoSupported();
   ArrayImpl<int32>*          getCjcChannels();
                              
   // buffered ai->basic features
   bool                       getBufferedAiSupported();
   SamplingMethod             getSamplingMethod();
   int32                      getChannelStartBase();
   int32                      getChannelCountBase();
                              
   // buffered ai->conversion clock features
   ArrayImpl<SignalDrop>*     getConvertClockSources();
   void                       getConvertClockRange(MathInterval &x);
                              
   // buffered ai->burst scan
   bool                       getBurstScanSupported();
   ArrayImpl<SignalDrop>*     getScanClockSources();
   void                       getScanClockRange(MathInterval &x);
   int32                      getScanCountMax();

   // buffered ai->trigger features, multi-trigger supported
   int32                      getTriggerCount();
   bool                       getRetriggerable();
   ArrayImpl<FilterType>*     getTriggerFilterTypes(uint32 /*reserved*/);
   void                       getTriggerFilterCutoffFreqRange(uint32 /*reserved*/, MathInterval &x);

   ArrayImpl<TriggerAction>*  getTriggerActions(int32 trigger);
   void                       getTriggerDelayRange(int32 trigger, MathInterval &x);
   ArrayImpl<SignalDrop>*     getTriggerSources(int32 trigger);
   ValueRange                 getTriggerSourceVrg(int32 trigger);
   double                     getTriggerHysteresisIndexMax(int32 trigger);
   int32                      getTriggerHysteresisIndexStep(int32 trigger);

   // --- DN4
   ArrayImpl<AiSignalType>*   getConnectionTypes();
   bool                       getOverallConnection();
   // ---

   // new: coupling & IEPE & Impedance
   ArrayImpl<CouplingType> *  getCouplingTypes();
   ArrayImpl<IepeType>     *  getIepeTypes();
   ArrayImpl<ImpedanceType>*  getImpedanceTypes();

   // new: timestamp resolution
   double                     getTimestampResolution();
   
   // new: measure type
   ArrayImpl<MeasureType>*    getMeasureTypes();
   ArrayImpl<int32>*          getBridgeResistances();
   void                       getExcitingVoltageRange(MathInterval &x);

public:
   AiFeaturesImpl(AiCtrlBaseImpl * owner = NULL) 
   { 
      m_owner = owner; 
   }
   void setOwner(AiCtrlBaseImpl * owner) 
   {  
      assert(owner);  
      m_owner = owner;
   }

private:
   AiCtrlBaseImpl * m_owner;
};

#endif
