//
// AI function implementation
//
#include <std_decl.h>

#include "ai_features.h"
#include "cjc_setting.h"
#include "ai_channel.h"
#include "ai_ctrl_base.h"

//------------------------------------------------------------------
// Help MACRO used to get/set property and check error
//------------------------------------------------------------------
#undef  MOD_PROP
#define MOD_PROP (m_owner->ModProp())

// -----------------------------------------------------------------------
// functions of AiFeaturesImpl
//------------------------------------------------------------------------

// ADC features
int32 AiFeaturesImpl::getResolution()
{
   int32 res = 0;
   MOD_PROP.Get(CFG_FeatureResolutionInBit, res);
   return res;
}
int32 AiFeaturesImpl::getDataSize()
{
   int32 size = 0;
   MOD_PROP.Get(CFG_FeatureDataSize, size);
   return size;
}
int32 AiFeaturesImpl::getDataMask()
{
   int32 mask = 0;
   MOD_PROP.Get(CFG_FeatureDataMask, mask);
   return mask;
}

// channel features
int32 AiFeaturesImpl::getChannelCountMax()
{
   int32 count = 0;
   if (MOD_PROP.Get(CFG_FeatureChannelNumberMax, count) == Success) {
      count += 1; // convert from channel number to channel count
   }
   return count;
}
AiChannelType AiFeaturesImpl::getChannelType()
{
   ArrayImpl<AiSignalType> *types = getConnectionTypes();
   if (!types) { return AllSingleEnded; }

   AiChannelType t;
   if (types->getLength() == 1) {
      t = types->Item(0) == SingleEnded ? AllSingleEnded : AllDifferential;
   } else {
      bool se_spt = false;
      for (int i = 0; !se_spt && i < types->getLength(); ++i) {
         se_spt = types->Item(0) == SingleEnded;
      }

      if (se_spt) {
         t = getOverallConnection() ? AllSeDiffAdj : MixedSeDiffAdj;
      } else { // Differential, PseudoDifferential
         t = AllDifferential;
      }      
   }
   types->Dispose();
   return t;
}
bool AiFeaturesImpl::getOverallValueRange()
{
   int32 ret = 0;
   MOD_PROP.Get(CFG_FeatureOverallVrgType, ret);
   return ret != 0;
}
ArrayImpl<ValueRange>* AiFeaturesImpl::getValueRanges()
{
   return MOD_PROP.GetValueArrayI32<ValueRange>(CFG_FeatureVrgTypes);
}
ArrayImpl<BurnoutRetType>* AiFeaturesImpl::getBurnoutReturnTypes()
{
   return MOD_PROP.GetValueArrayI32<BurnoutRetType>(CFG_FeatureBurnDetectedReturnTypes);
}

ArrayImpl<AiSignalType>* AiFeaturesImpl::getConnectionTypes()
{
   return MOD_PROP.GetValueArrayI32<AiSignalType>(CFG_FeatureConnectionTypes);
}
bool AiFeaturesImpl::getOverallConnection()
{
   int32 ret = 0;
   MOD_PROP.Get(CFG_FeatureOverallConnection, ret);
   return ret != 0;
}

// filter
ArrayImpl<FilterType>* AiFeaturesImpl::getFilterTypes()
{
   return MOD_PROP.GetValueArrayI32<FilterType>(CFG_FeatureFilterTypes);
}
void AiFeaturesImpl::getFilterCutoffFreqRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureFilterCutoffFreqRange, x) != Success) {
      ResetMathInterval(x);
   }
}
void AiFeaturesImpl::getFilterCutoffFreq1Range(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureFilterCutoffFreq1Range, x) != Success) {
      ResetMathInterval(x);
   }
}

// CJC features
bool AiFeaturesImpl::getThermoSupported()
{
   bool   spt = false;
   uint32 xbuf[128] = {0};
   uint32 size = sizeof(xbuf);

   MOD_PROP.Read(CFG_FeatureVrgTypes, &size, xbuf);
   if (size) {
      for (unsigned i = 0; i < size / sizeof(int32); ++i) {
         if (xbuf[i] >= Jtype_0To760C && xbuf[i] <= Ttype_Neg20To135C) {
            spt = true;
            break;
         }
      }
   }

   return spt;
}
ArrayImpl<int32>* AiFeaturesImpl::getCjcChannels()
{
   return MOD_PROP.GetValueArrayI32<int32>(CFG_FeatureCjcChannels);
}

// buffered ai->basic features
bool AiFeaturesImpl::getBufferedAiSupported()
{
   return MOD_PROP.GetLength(CFG_FeatureConvertClockSources) != 0;
}
SamplingMethod AiFeaturesImpl::getSamplingMethod()
{
   int32 ret = (int32)EqualTimeSwitch;
   MOD_PROP.Get(CFG_FeatureChannelScanMethod, ret);
   return (SamplingMethod)ret;
}
int32 AiFeaturesImpl::getChannelStartBase()
{
   int32 startBase = 1;
   MOD_PROP.Get(CFG_FeatureScanChannelStartBase, startBase);
   return startBase;
}
int32 AiFeaturesImpl::getChannelCountBase()
{
   int32 countBase = 1;
   MOD_PROP.Get(CFG_FeatureScanChannelCountBase, countBase);
   return countBase;
}

// buffered ai->conversion clock features
ArrayImpl<SignalDrop> * AiFeaturesImpl::getConvertClockSources()
{
   return MOD_PROP.GetValueArrayI32<SignalDrop>(CFG_FeatureConvertClockSources);
}
void AiFeaturesImpl::getConvertClockRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureConvertClockRateRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// buffered ai->burst scan
bool AiFeaturesImpl::getBurstScanSupported()
{
   return MOD_PROP.GetLength(CFG_FeatureScanClockSources) != 0;
}
ArrayImpl<SignalDrop> * AiFeaturesImpl::getScanClockSources()
{
   return MOD_PROP.GetValueArrayI32<SignalDrop>(CFG_FeatureScanClockSources);
}
void AiFeaturesImpl::getScanClockRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureScanClockRateRange, x) != Success) {
      ResetMathInterval(x);
   }
}
int32 AiFeaturesImpl::getScanCountMax()
{
   int32 maxCount = 0;
   MOD_PROP.Get(CFG_FeatureScanCountMax, maxCount);
   return maxCount;
}

// buffered ai->trigger features
bool AiFeaturesImpl::getRetriggerable()
{
   int32 retriggerable = 0;
   MOD_PROP.Get(CFG_FeatureRetriggerable, retriggerable);
   return retriggerable != 0;
}
int32 AiFeaturesImpl::getTriggerCount()
{
   int32 count = 0;
   MOD_PROP.Get(CFG_FeatureTriggersCount, count);
   return count;
}

ArrayImpl<TriggerAction> * AiFeaturesImpl::getTriggerActions(int32 trigger)
{
   PropertyId id = trigger < MAX_TRIG_COUNT ? feat_id_trigAction[trigger] : PropertyId(-1);
   return MOD_PROP.GetValueArrayI32<TriggerAction>(id);
}
void AiFeaturesImpl::getTriggerDelayRange(int32 trigger, MathInterval &x)
{
   PropertyId id = trigger < MAX_TRIG_COUNT ? feat_id_trigDlyRng[trigger] : PropertyId(-1);
   if (MOD_PROP.Get(id, x) != Success) { ResetMathInterval(x); }
}
ArrayImpl<SignalDrop> * AiFeaturesImpl::getTriggerSources(int32 trigger)
{
   PropertyId id = trigger < MAX_TRIG_COUNT ? feat_id_trigSrc[trigger] : PropertyId(-1);
   return MOD_PROP.GetValueArrayI32<SignalDrop>(id);
}
ValueRange AiFeaturesImpl::getTriggerSourceVrg(int32 trigger)
{
   PropertyId id = trigger < MAX_TRIG_COUNT ? feat_id_trigSrcVrg[trigger] : PropertyId(-1);

   int32 vrg = 0;
   MOD_PROP.Get(id, vrg);
   return ValueRange(vrg);
}
ArrayImpl<FilterType>* AiFeaturesImpl::getTriggerFilterTypes(uint32 /*reserved*/)
{
   return MOD_PROP.GetValueArrayI32<FilterType>(CFG_FeatureTriggerFilterTypes);
}
void AiFeaturesImpl::getTriggerFilterCutoffFreqRange(uint32 /*reserved*/, MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureTriggerFilterCutoffFreqRange, x) != Success) {
      ResetMathInterval(x);
   }
}
double AiFeaturesImpl::getTriggerHysteresisIndexMax(int32 trigger)
{
   PropertyId id = trigger < MAX_TRIG_COUNT ? feat_id_trigHystIdxMax[trigger] : PropertyId(-1);

   double indexMax = 0;
   MOD_PROP.Get(id, indexMax);
   return indexMax;
}
int32 AiFeaturesImpl::getTriggerHysteresisIndexStep(int32 trigger)
{
   PropertyId id = trigger < MAX_TRIG_COUNT ? feat_id_trigHystStep[trigger] : PropertyId(-1);

   int32 step = 0;
   MOD_PROP.Get(id, step);
   return step;
}
// new: coupling & IEPE & Impedance
ArrayImpl<CouplingType> * AiFeaturesImpl::getCouplingTypes()
{
   return MOD_PROP.GetValueArrayI32<CouplingType>(CFG_FeatureCouplingTypes);
}
ArrayImpl<IepeType> * AiFeaturesImpl::getIepeTypes()
{
   return MOD_PROP.GetValueArrayI32<IepeType>(CFG_FeatureIepeTypes);
}
ArrayImpl<ImpedanceType> * AiFeaturesImpl::getImpedanceTypes()
{
   return MOD_PROP.GetValueArrayI32<ImpedanceType>(CFG_FeatureImpedanceTypes);
}

// new: timestamp resolution
double AiFeaturesImpl::getTimestampResolution()
{
   double res = 0;
   return MOD_PROP.Get(CFG_FeatureTimestampResolution, res);
   return res;
}

// new: measure type
ArrayImpl<MeasureType>* AiFeaturesImpl::getMeasureTypes()
{
   return MOD_PROP.GetValueArrayI32<MeasureType>(CFG_FeatureMeasureTypes);
}

ArrayImpl<int32>* AiFeaturesImpl::getBridgeResistances()
{
   return MOD_PROP.GetValueArrayI32<int32>(CFG_FeatureBridgeResistances);
}

void AiFeaturesImpl::getExcitingVoltageRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureExcitingVoltageRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// -----------------------------------------------------------------------
// functions of AiChannelImpl
//------------------------------------------------------------------------
int32 AiChannelImpl::getChannel()
{
   return (int32)m_tag;
}
ValueRange AiChannelImpl::getValueRange()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_VrgTypeOfChannels, m_tag, value);
   return ValueRange(value);
}
ErrorCode AiChannelImpl::setValueRange(ValueRange value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   int32 xbuf[MAX_AI_CH_COUNT] = {0};
   MOD_PROP.Get(CFG_VrgTypeOfChannels, MAX_AI_CH_COUNT, xbuf);
   if ((unsigned)xbuf[m_tag] == value) { return Success; }

   if (m_owner->getFeatures()->getOverallValueRange()) {
      for (int i = 0; i < m_owner->getFeatures()->getChannelCountMax(); ++i) {
         xbuf[i] = value;
      }
   } else {
      if (getSignalType() == SingleEnded) {  xbuf[m_tag] = value; }
      else { xbuf[m_tag] = xbuf[m_owner->GetCoupledChan(m_tag)] = value; }
   }

   return MOD_PROP.Set(CFG_VrgTypeOfChannels, MAX_AI_CH_COUNT, xbuf);
}
AiSignalType AiChannelImpl::getSignalType()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   int32 value = 0; 
   MOD_PROP.GetValueNth(CFG_ConnectionTypeOfChannels, m_tag, value);
   return AiSignalType(value);
}
ErrorCode AiChannelImpl::setSignalType(AiSignalType value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   int32 xbuf[MAX_AI_CH_COUNT] = {0};
   MOD_PROP.Get(CFG_ConnectionTypeOfChannels, MAX_AI_CH_COUNT, xbuf);
   if (uint32(xbuf[m_tag]) == value) { return Success; }

   if (m_owner->getFeatures()->getOverallConnection()) {
      for (int i = 0; i < m_owner->getFeatures()->getChannelCountMax(); ++i) {
         xbuf[i] = value;
      }
   } else {
      // Check supported connection types
      ArrayImpl<AiSignalType>* cnnt_types = m_owner->getFeatures()->getConnectionTypes();
      int xtype = 0;
      for (int i = 0; i < cnnt_types->getLength(); ++i) {
         xtype |= 0x1 << cnnt_types->Item(i);
      }
      cnnt_types->Dispose();

      // Just differential and pseudo-differential are supported, 
      // changes only current channel's signal type
      if (xtype == ((0x1 << Differential) | (0x1 << PseudoDifferential))) {
         xbuf[m_tag] = value;
      } else {
         int startCH = m_tag & (~0x1);
         xbuf[startCH] = xbuf[startCH + 1] = value;
      }
   }

   return MOD_PROP.Set(CFG_ConnectionTypeOfChannels, MAX_AI_CH_COUNT, xbuf);
}
BurnoutRetType AiChannelImpl::getBurnoutRetType()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_BurnDetectedReturnTypeOfChannels, m_tag, value);
   return BurnoutRetType(value);
}
ErrorCode AiChannelImpl::setBurnoutRetType(BurnoutRetType value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_BurnDetectedReturnTypeOfChannels, m_tag, (int32)value);
}
double AiChannelImpl::getBurnoutRetValue()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   double value = 0;
   MOD_PROP.GetValueNth(CFG_BurnoutReturnValueOfChannels, m_tag, value);
   return value;
}
ErrorCode AiChannelImpl::setBurnShortRetValue(double value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_BurnShortReturnValueOfChannels, m_tag, value);
}
double AiChannelImpl::getBurnShortRetValue()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   double value = 0;
   MOD_PROP.GetValueNth(CFG_BurnShortReturnValueOfChannels, m_tag, value);
   return value;
}
ErrorCode AiChannelImpl::setBurnoutRetValue(double value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_BurnoutReturnValueOfChannels, m_tag, value);
}
FilterType AiChannelImpl::getFilterType()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_FilterTypeOfChannels, m_tag, value);
   return FilterType(value);
}
ErrorCode AiChannelImpl::setFilterType(FilterType value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_FilterTypeOfChannels, m_tag, (int32)value);
}
double AiChannelImpl::getFilterCutoffFreq()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   double value = 0;
   MOD_PROP.GetValueNth(CFG_FilterCutoffFreqOfChannels, m_tag, value);
   return value;
}
ErrorCode AiChannelImpl::setFilterCutoffFreq(double value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_FilterCutoffFreqOfChannels, m_tag, value);
}
double AiChannelImpl::getFilterCutoffFreq1()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   double value = 0;
   MOD_PROP.GetValueNth(CFG_FilterCutoffFreq1OfChannels, m_tag, value);
   return value;
}
ErrorCode AiChannelImpl::setFilterCutoffFreq1(double value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_FilterCutoffFreq1OfChannels, m_tag, value);
}
int32 AiChannelImpl::getLogicalNumber()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   int32 value = -1;
   MOD_PROP.GetValueNth(CFG_OrderOfChannels, m_tag, value); 
   return value;
}

//new : Couping & IEPE & Impedance
CouplingType AiChannelImpl::getCouplingType()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_CouplingTypeOfChannels, m_tag, value);
   return CouplingType(value);
}
ErrorCode  AiChannelImpl::setCouplingType(CouplingType value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_CouplingTypeOfChannels, m_tag, (int32)value);
}

IepeType AiChannelImpl::getIepeType()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_IepeTypeOfChannels, m_tag, value);
   return IepeType(value);
}
ErrorCode  AiChannelImpl::setIepeType(IepeType value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_IepeTypeOfChannels, m_tag, (int32)value);
}

ImpedanceType AiChannelImpl::getImpedanceType()
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_ImpedanceTypeOfChannels, m_tag, value);
   return ImpedanceType(value);
}
ErrorCode  AiChannelImpl::setImpedanceType(ImpedanceType value)
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return MOD_PROP.SetValueNth(CFG_ImpedanceTypeOfChannels, m_tag, (int32)value);
}

//new : Sensor Scaling
ErrorCode AiChannelImpl::getSensorDescription(int32 &length, wchar_t desc[])
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   return MOD_PROP.GetValueNth(CFG_SensorDescriptionOfChannels, m_tag, length, desc);
}
ErrorCode AiChannelImpl::setSensorDescription(int32 length, wchar_t desc[])
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }
   if (!desc) { return ErrorBufferIsNull; }

   return MOD_PROP.SetValueNth(CFG_SensorDescriptionOfChannels, m_tag, length, desc);
}

// new: scale table
ErrorCode AiChannelImpl::getScaleTable(int32 &size, MapFuncPiece table[])
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   return m_owner->ModuleRef().GetScaleTable(m_tag, (uint32&)size, table);
}
ErrorCode AiChannelImpl::setScaleTable(int32  size, MapFuncPiece table[])
{
   assert(unsigned(m_tag) < MAX_AI_CH_COUNT);

   if (!m_owner->CanEditProperty()) { return ErrorPrivilegeNotHeld; }
   if (m_owner->IsRunning()) { return ErrorFuncBusy; }

   return m_owner->ModuleRef().SetScaleTable(m_tag, size, table);
}

// -----------------------------------------------------------------------
// functions of CjcSettingImpl
//------------------------------------------------------------------------
int32 CjcSettingImpl::getChannel()
{
   int32 ch = -1;
   MOD_PROP.Get(CFG_CjcChannel, ch);
   return ch;
}
ErrorCode CjcSettingImpl::setChannel(int32 ch)
{
   if (!m_owner->CanEditProperty()) { 
      return ErrorPrivilegeNotHeld; 
   }
   if (m_owner->IsRunning()) { 
      return ErrorFuncBusy; 
   }

   return MOD_PROP.Set(CFG_CjcChannel, ch);
}
double CjcSettingImpl::getValue()
{
   double value = 0;
   MOD_PROP.Get(CFG_CjcValue, value);
   return value;
}
ErrorCode CjcSettingImpl::setValue(double value)
{
   if (!m_owner->CanEditProperty()) {
      return ErrorPrivilegeNotHeld;
   }
   return MOD_PROP.Set(CFG_CjcValue, value);
}
double CjcSettingImpl::getUpdateFreq()
{
   double value = 0;
   MOD_PROP.Get(CFG_CjcUpdateFrequency, value);
   return value;
}
ErrorCode CjcSettingImpl::setUpdateFreq(double value)
{
   if (!m_owner->CanEditProperty()) {
      return ErrorPrivilegeNotHeld;
   }
   return MOD_PROP.Set(CFG_CjcUpdateFrequency, value);
}

