#ifndef _DAQNAVI_AO_CTRL_BASE_IMPL_H
#define _DAQNAVI_AO_CTRL_BASE_IMPL_H

class BDAQ_NO_VTABLE AoCtrlBaseImpl : public DaqCtrlBaseImpl
{
   typedef ArrayImpl<AoChannelImpl> AoChannelArrayImpl;

public:
   AoFeaturesImpl* getFeatures() { return &m_features; }

   int32 getChannelCount()
   {
      return m_features.getChannelCountMax();
   }

   ArrayImpl<AoChannelImpl> * getChannels()
   {
      AutoLock<MutexLock> lock(PropLock());

      if (getInitialized()) {
         if (m_channels.getLength() == 0) {
            int count = m_features.getChannelCountMax();
            m_channels.setCapacity(count);
            
            for (int i = 0; i < count; ++i) {
               m_channels.Add(AoChannelImpl(this, i));
            }
         }
      }

      return (ArrayImpl<AoChannelImpl> *)&m_channels;
   }

   // obsolete : using channel's 'ExtRefUnipolar' instead.
   double getExtRefValueForUnipolar()
   {
      double value = 0;
      ModProp().Get(CFG_ExtRefValueForUnipolar, value);
      return value;
   }
   ErrorCode setExtRefValueForUnipolar(double value)
   {
      if (!CanEditProperty()) { return ErrorPrivilegeNotHeld; }
      if (IsRunning()) { return ErrorFuncBusy; }

      MathInterval range;
      getFeatures()->getExternalRefRange(range);
      value = x_max(x_min(value, range.Max), range.Min);

      double xbuf[128];
      std::fill_n(xbuf, 128, value);

      return ModProp().Set(CFG_ExtRefValueForUnipolar, 128, xbuf);
   }

   // obsolete : using channel's 'ExtRefBipolar' instead.
   double getExtRefValueForBipolar()
   {
      double value = 0;
      ModProp().Get(CFG_ExtRefValueForBipolar, value);
      return value;
   }
   ErrorCode setExtRefValueForBipolar(double value)
   {
      if (!CanEditProperty()) { return ErrorPrivilegeNotHeld; }
      if (IsRunning()) { return ErrorFuncBusy; }

      MathInterval range;
      getFeatures()->getExternalRefRange(range);
      value = x_max(x_min(value, range.Max), range.Min);

      double xbuf[128];
      std::fill_n(xbuf, 128, value);

      return ModProp().Set(CFG_ExtRefValueForBipolar, 128, xbuf);
   }

public: // ctor & dtor
   AoCtrlBaseImpl() : DaqCtrlBaseImpl(DaqAo), m_channels(false) { m_features.setOwner(this);  }

   virtual ~AoCtrlBaseImpl() { }

   DaqAoModule & ModuleRef() { return *(DaqAoModule*)getModule(); }

protected: 

   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual ModuleType WantedType() { return DaqAo; }

   virtual std::vector<AccessMode> WantedModes()
   {
      AccessMode spted[] = { ModeWrite, ModeWriteWithReset };
      return std::vector<AccessMode>(&spted[0], &spted[2]);
   }

   virtual bool IsSuitable(DaqModule & module)
   {
      return module.getType() == DaqAo;
   }

   // initialize/de-initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = DaqCtrlBaseImpl::InitializeControl(x);
      if (ret == Success) {
         ret = m_features.getChannelCountMax() > 0 ? Success : ErrorFuncNotSpted;;
      }
      return ret;
   }

   virtual void DeinitializeControl()
   {
      // reset all configuration to null
      m_channels.Clear();

      DaqCtrlBaseImpl::DeinitializeControl();
   }

private:
   AoFeaturesImpl     m_features;
   AoChannelArrayImpl m_channels;
};

#endif
