#ifndef _DAQNAVI_COLLECTION_H
#define _DAQNAVI_COLLECTION_H

//---------------------------------------------------------------
// Collection implementation
//---------------------------------------------------------------
//*****
// CAUTION: SHARED WITH DN3, DONOT CHANGE THE CLASS LAYOUT AND METHODS(NAME,
// PARAMETERS, etc.). BUT YOU CAN APPEND NEW METHODS TO THE CLASS, IF NECESSARY.
//*****
class XArray
{
public:
   virtual void   BDAQCALL Dispose()  = 0;   // destroy the instance
   virtual int32  BDAQCALL getLength() = 0;
   virtual void * BDAQCALL getItem(int32 idx) = 0;
};

template<class T>
class ArrayImpl : public XArray
{
   typedef std::vector<T>  value_array;

public:
   // ctor / dtor
   ArrayImpl(bool disposable = true) { m_disposable = disposable; }
   virtual ~ArrayImpl()              {  }

   // -------------------------------------------
   // methods of Array
   // -------------------------------------------
   virtual void  BDAQCALL Dispose()         { Dispose(false); }
   virtual int32 BDAQCALL getLength()       { return (int32)m_data.size(); }
   virtual void* BDAQCALL getItem(int32 idx){ return &Item(idx); }

   // -------------------------------------------
   // methods for internal 
   // -------------------------------------------
   T &  Item(int32 index)                  { return m_data[index]; }
   void Dispose(bool force)                { if (force || m_disposable) delete this; }

   void Assign(T * first, size_t count)    { m_data.assign(first, first + count); }
   void Assign(value_array const & values) { m_data = values; }
   void Add( const T & value)              { m_data.push_back(value); }
   void setCapacity(size_t capacity)       { m_data.reserve(capacity); }
   void Clear()                            { m_data.clear(); }

private:
   bool         m_disposable;
   value_array  m_data;
};

#endif
