#ifndef _DAQNAVI_BFD_CNTR_TRIGGER_IMPL_H
#define _DAQNAVI_BFD_CNTR_TRIGGER_IMPL_H

// -----------------------------------------------------------------
// implementation of Trigger, multi-trigger supported
// -----------------------------------------------------------------

class CntrCtrlBaseImpl;

//*****
// CAUTION: SHARED WITH DN3, DONOT CHANGE THE CLASS LAYOUT AND METHODS(NAME,
// PARAMETERS, etc.). BUT YOU CAN APPEND NEW METHODS TO THE CLASS, IF NECESSARY.
//*****
class BfdCntrTriggerImpl : public TriggerImpl
{
public:
   virtual SignalDrop    BDAQCALL getSource();
   virtual ErrorCode     BDAQCALL setSource(SignalDrop value);
   virtual ActiveSignal  BDAQCALL getEdge();
   virtual ErrorCode     BDAQCALL setEdge(ActiveSignal value);
   virtual double        BDAQCALL getLevel();
   virtual ErrorCode     BDAQCALL setLevel(double value);
   virtual TriggerAction BDAQCALL getAction();
   virtual ErrorCode     BDAQCALL setAction(TriggerAction value);
   virtual int32         BDAQCALL getDelayCount();
   virtual ErrorCode     BDAQCALL setDelayCount(int32 value);
                         
   virtual double        BDAQCALL getHysteresisIndex();
   virtual ErrorCode     BDAQCALL setHysteresisIndex(double value);

   virtual FilterType    BDAQCALL getFilterType();
   virtual ErrorCode     BDAQCALL setFilterType(FilterType value);

   virtual double        BDAQCALL getFilterCutoffFreq();
   virtual ErrorCode     BDAQCALL setFilterCutoffFreq(double value);

public:
   BfdCntrTriggerImpl(CntrCtrlBaseImpl *owner = NULL, int32 tag = 0)
   {
      m_owner = owner;
      m_tag   = tag;
      m_trigDelayPerChan = 0;
      m_edge   = RisingEdge;
      m_src    = SignalNone;
      m_level  = 0;
      m_action = ActionNone;
   }
   void setOwner(CntrCtrlBaseImpl *owner, int32 tag)
   {
      m_owner = owner;
      m_tag   = tag;
   }
   void adjustDelayCount(int32 value)
   {
      m_trigDelayPerChan = value;
   }

protected:
   CntrCtrlBaseImpl *m_owner;
   int32           m_tag;
   int             m_trigDelayPerChan; // cached value which can not be mapped directly to a driver define property.
   SignalDrop      m_src;
   ActiveSignal    m_edge;
   double          m_level;
   TriggerAction   m_action;
   int32           m_delayCount;
   double          m_hysteresis;
   FilterType      m_filterType;
   double          m_cutoffFreq;
};

#endif
