#ifndef _DAQNAVI_DAQ_CTRL_BASE_IMPL_H
#define _DAQNAVI_DAQ_CTRL_BASE_IMPL_H

//------------------------------------------------------
// Implementation of interface 'DaqCtrlBase'
//------------------------------------------------------
typedef void (BDAQCALL * DaqEventProc)(void * sender, void * args, void *userParam);

class BDAQ_NO_VTABLE DaqCtrlBaseImpl
{
   class DummyModules
   {
      class DummyAiModule : public DaqAiModule {
      public: 
         virtual AccessMode BDAQCALL getMode()                                             { return ModeRead;          }
         virtual ErrorCode  BDAQCALL PropAccess(PropertyId, uint32 & size, void *, uint32) { size = 0; return ErrorFuncNotInited;}
         virtual ErrorCode  BDAQCALL ReadSamples(uint32, uint32, void *, double *)         { return ErrorFuncNotInited;}
      };
      class DummyAoModule : public DaqAoModule {
      public: 
         virtual AccessMode BDAQCALL getMode()                                             { return ModeRead;          }
         virtual ErrorCode  BDAQCALL PropAccess(PropertyId, uint32 & size, void *, uint32) { size = 0; return ErrorFuncNotInited;}
         virtual ErrorCode  BDAQCALL WriteSamples(uint32, uint32, void *, double *)        { return ErrorFuncNotInited;}
      };
      class DummyDioModule : public DaqDioModule {
      public: 
         virtual AccessMode BDAQCALL getMode()                                             { return ModeRead;          }
         virtual ErrorCode  BDAQCALL PropAccess(PropertyId, uint32 & size, void *, uint32) { size = 0; return ErrorFuncNotInited;}
      };
      class DummyCntrModule : public DaqCtModule {
      public: 
         virtual AccessMode BDAQCALL getMode()                                             { return ModeRead;          }
         virtual ErrorCode  BDAQCALL PropAccess(PropertyId, uint32 & size, void *, uint32) { size = 0; return ErrorFuncNotInited;}
         virtual ErrorCode  BDAQCALL CtReset(uint32, uint32)                               { return ErrorFuncNotInited;}
      };
      class DummyCaliModule : public DaqCaliModule {
         virtual AccessMode BDAQCALL getMode()                                             { return ModeRead;           }
         virtual uint32     BDAQCALL getSolutionList(uint32 size, ObjectId buffer[])       { return 0;                  }         
      };
      class DummyFwModule : public DaqFwModule {
         virtual AccessMode BDAQCALL getMode()                                             { return ModeRead;           }
      };

   public:
      static DaqModule* getInstance(ModuleType modType)
      {
         static DummyAiModule   dummyAi;
         static DummyAoModule   dummyAo;
         static DummyDioModule  dummyDio;
         static DummyCntrModule dummyCntr;
         static DummyCaliModule dummyCali;
         static DummyFwModule   dummyFw;
         switch(modType)
         {
         case DaqAi:      return &dummyAi;
         case DaqAo:      return &dummyAo;
         case DaqDio:     return &dummyDio;
         case DaqCounter: return &dummyCntr;
         case DaqCali:    return &dummyCali;
         case DaqFw:      return &dummyFw;
         default:         return NULL;
         }
      }
   };
    
public: // interface 'DaqCtrlBase'
   // Event
   virtual void addEventHandler(EventId id, DaqEventProc proc, void * param)
   {
      //
      // Add device event handler, for compatible with daqnavi3
      if (IsDeviceCtrlEvent(id)) {
         UserEventManager::Add(getOuter(), id, (UserEventProc)proc, param);
         m_deviceCtrl.addEventHandler(id, DeviceEventProxyProc, this);
         return;
      }

      //
      // Add 'X' module event
      if (!IsValidEvent(id)) { return; }

      UserEventManager::Add(getOuter(), id, (UserEventProc)proc, param);
      RegisterKernEvent(id);
   }

   virtual void removeEventHandler(EventId id, DaqEventProc proc, void * param)
   {
      //
      // Remove device event handler, for compatible with daqnavi3
      if (IsDeviceCtrlEvent(id)) {
         UserEventManager::Remove(getOuter(), id, (UserEventProc)proc, param);
         m_deviceCtrl.removeEventHandler(id, DeviceEventProxyProc, this);
         return;
      }

      //
      // Remove 'X' module event
      if (!IsValidEvent(id)) { return; }

      UserEventManager::Remove(getOuter(), id, (UserEventProc)proc, param);
      if (!UserEventManager::IsExist(getOuter(), id)) {
         UnregisterKernEvent(id);
      }
   }

   // Method
   void Cleanup() 
   { 
      DeinitializeControl(); 
   }

   ErrorCode LoadProfile(wchar_t const *fileName)
   {
      if (!getInitialized()) {
         return ErrorFuncNotInited;
      }
      if (m_deviceCtrl.getAccessMode() == ModeRead) {
         return ErrorPrivilegeNotHeld;
      }
      if (IsRunning()) {
         return ErrorFuncBusy;
      }

      char *path     = NULL;
      bool fileValid = fileName != NULL;
      if (fileValid) {
         size_t len = wcslen(fileName) + 1;
         path = new char[len];
         assert(path);

#if defined(_WIN32) || defined(WIN32)
         wcstombs_s(NULL, path, len, fileName, len);
         fileValid = _access(path, 0) == 0;
#else
         wcstombs(path, fileName, len);
         fileValid = access(path, F_OK) == 0;
#endif
      }
      
      // Attempt to initialize the hardware with the profile
      ErrorCode ret = WarningProfileNotValid;
      if (fileValid) { 
         ret = TryLoadProfile(path); 
         if (ret == Success || ret == WarningPropPartialFailed) {
            ReloadProperties();
         }
      }

      if (path) {delete[] path;}

      return ret;
   }

   // property
   void getSelectedDevice(DeviceInformation &x)
   { 
      x.DeviceNumber = m_deviceCtrl.getDeviceNumber(); 
      x.DeviceMode   = m_deviceCtrl.getAccessMode();
      x.ModuleIndex  = m_module->getNumber();

      m_deviceCtrl.getDescription(ARRAY_SIZE(x.Description), x.Description);
   }

   ErrorCode setSelectedDevice(DeviceInformation const &x)
   {
      // reset the control before attach to new device
      DeinitializeControl();

      // attach to new device information and attach to it
      ErrorCode ret = InitializeControl(x);
      if (ret != Success) { DeinitializeControl(); }

      return ret;
   }

   bool getInitialized()   
   { 
      return m_ctrlState != Uninited; 
   }
   
   DeviceCtrlImpl* getDevice()
   { 
      return &m_deviceCtrl; 
   }
   
   DaqModule* getModule()
   { 
      return m_module; 
   }

   ArrayImpl<DeviceTreeNode>* getSupportedDevices() 
   { 
      return EnumerateDeviceHelper(WantedType(), this);
   }

   ArrayImpl<AccessMode> * getSupportedModes()
   {
      ArrayImpl<AccessMode> * modes = new ArrayImpl<AccessMode>();
      modes->Assign(WantedModes());
      return modes;
   }

   ControlState getState() 
   { 
      RefreshState(m_ctrlState);
      return m_ctrlState;
   }

public: 
   
   DaqProperty  ModProp()              { return DaqProperty(m_module);}
   DaqEvent     ModEvent()             { return DaqEvent(m_module);   }
   void *       getOuter()             { return m_outer;  }
   void         setOuter(void * outer) { m_outer = outer; }

   bool         IsReady()              { return getState() == Ready || getState() == Stopped;}
   bool         IsRunning()            { return getState() == Running; }
   bool         IsIdle()               { return getState() <= Idle;    }      
   bool         CanEditProperty()      { return getInitialized() && m_deviceCtrl.getAccessMode() != ModeRead; }

   ModuleType   ModType()                 { return WantedType(); }
   bool         ModSuitable(DaqModule &m) { return IsSuitable(m);}

   MutexLock   &PropLock()             { return m_propLock;  }
   MutexLock   &BuffLock()             { return m_buffLock;  }

   //
   // Notification method when some of the control properties were changed by user.
   virtual void OnPropChanged(PropertyId id) { setState(Idle); }

   //
   // Notification method when the device state was changed (Removed & Re-connected)
   virtual void OnDeviceStateChanged(int32 newState) 
   {
      if (newState == DevState::Removed) {
         setState(Idle);
      }
   }

public:
   DaqCtrlBaseImpl(ModuleType type)
   {
      m_module = DummyModules::getInstance(type);
      m_outer  = this;
      m_deviceCtrl.setOwner(this);
      setState(Uninited);
   }

   virtual ~DaqCtrlBaseImpl()                 
   { 
      // The object must had been de-inited by using method 'Cleanup' 
      // before destruct the object due to we cannot call virtual method 'DeinitializeControl' here.
      assert(getState() == Uninited);
   }

protected:
   // Update the control running state
   void setState(ControlState ns)
   {
      m_ctrlState = ns;
   }

   // register/unregister device event into event manager
   ErrorCode RegisterKernEvent(EventId id, KernEventProc proc = ProcessKernEvent)
   {
      if (!getInitialized()) { return ErrorFuncNotInited; }

      HANDLE handle = GetEventHandle(id);
      if (handle == NULL) { return ErrorEventNotSpted; }

      KernEventMonitor::Register(m_deviceCtrl.getDeviceNumber(), handle, id, proc, this);
      return Success;
   }

   void UnregisterKernEvent(EventId id, KernEventProc proc = ProcessKernEvent)
   {
      if (!getInitialized()) { return; }

      KernEventMonitor::Unregister(m_deviceCtrl.getDeviceNumber(), id, proc, this);
   }

   HANDLE GetEventHandle(EventId id)
   {
      HANDLE handle = NULL;
      ModEvent().GetHandle(id, handle);
      return handle;
   }

   static void BDAQCALL ProcessKernEvent(EventId id, void *param)
   {
      ((DaqCtrlBaseImpl*)param)->OnKernEvent(id);
   }

   // For compatible with DN3:
   // This proxy is used to convert the sender from 'device ctrl' to DaqCtrlBase derived class.
   static void BDAQCALL DeviceEventProxyProc(void *sender, DeviceEventArgs *args, void *param)
   {
      UserEventManager::Fire(((DaqCtrlBaseImpl*)param)->getOuter(), args->Id, args);
   }

protected: // internal virtual methods

   // these function should be override in the derived class
   // Get the module type the derived class wanted
   virtual ModuleType WantedType() { return DaqAny; }

   // Get the access mode the derived class wanted
   virtual std::vector<AccessMode> WantedModes()
   {
      AccessMode spted[] = { ModeRead, ModeWrite, ModeWriteWithReset };
      return std::vector<AccessMode>(&spted[0], &spted[3]);
   }

   // Determine whether a module is suitable for the derived class 
   virtual bool IsSuitable(DaqModule &module) { return true; }

   // refresh the control running state according the current state
   virtual void RefreshState(ControlState &curr)
   {
      // default control state is just the same as current state.
   }

   // Initialize/Deinitialize the control.
   // these two functions should be override in the derived class
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // initialize the underlying device
      // must release the device had been opened at first
      assert(!getInitialized());

      if (x.ModuleIndex < 0) { return ErrorFuncNotSpted; }

      // Open device by description if it is available
      ErrorCode ret = m_deviceCtrl.Initialize(x.DeviceNumber, x.Description, x.DeviceMode);
      if (ret != Success) { return ret; }

      // get the module needed
      DaqModule *module = m_deviceCtrl.GetModule(WantedType(), (int)x.ModuleIndex);
      if (module == NULL || !IsSuitable(*module)) {
         return ErrorFuncNotSpted;
      }
      m_module = module;

      // step the control state
      setState(Idle);
      return Success;
   }

   virtual void DeinitializeControl()
   {
      m_deviceCtrl.Deinitialize();

      m_module = DummyModules::getInstance(WantedType());

      // step the control state
      setState(Uninited);

      // Fix bug#21444: close config DB when the control is de-initialized.
      CloseDaqNaviConfigDB();      
   }

   //
   // Reload the properties' value from device
   virtual void ReloadProperties()
   {
   }   
   
   //
   // Check whether the event id is supported by a counter control or not. 
   // This function must be overridden in the derived class which support event.
   virtual bool IsValidEvent(EventId id) 
   {  
      return false; 
   }
   
   //
   // Process the kernel event
   // This function must be overridden in the derived class which support event.
   // This function will be called under the protect of event lock.
   virtual void OnKernEvent(EventId id) 
   {
   }

   //
   // try to initialize the HW with the specified profile
   ErrorCode TryLoadProfile(char const *path)
   {
      TiXmlDocument doc(path);
      doc.LoadFile();

      TiXmlElement *daqnavi = doc.RootElement();
      if (!daqnavi) { return WarningProfileNotValid; }

      // function module
      char const *name = NULL;
      switch(m_module->getType())
      {
      case DaqAi:  name = "DaqAi";  break;
      case DaqAo:  name = "DaqAo";  break;
      case DaqDio: name = "DaqDio"; break;
      case DaqCounter: name = "DaqCounter"; break;
      default: 
         return ErrorFuncNotSpted;
      }

      TiXmlElement *modProps = daqnavi->FirstChildElement(name);
      if (!modProps) { return WarningProfileNotValid; }

      // check the module index
      bool found = false;
      do {
         int moduleIndex = 0;
         modProps->QueryIntAttribute("ModuleIndex", &moduleIndex);      
         found = moduleIndex == getModule()->getNumber();
      } while (!found && (modProps = modProps->NextSiblingElement(name)));

      if (!found) {
         return WarningProfileNotValid; 
      }

      return InitializeFromXml(modProps->FirstChildElement());
   }

   //
   // initialize HW from profile file
   ErrorCode InitializeFromXml(TiXmlElement *prop)
   {
      assert(prop);
      DaqProperty daqProp(m_module);

      uint32 write_count = 0;
      uint32 fail_count = 0;

      uint32 size = 4096;
      uint8  buffer[4096];

      ErrorCode ret;
      for (; prop; prop = (TiXmlElement *)prop->NextSibling()) {

         char const         *idTxt = prop->Attribute("ID");
         TiXmlElement const  *attr = prop->FirstChildElement("Attribute");
         TiXmlElement const *value = prop->FirstChildElement("Value");
         assert(idTxt && attr && value);

         int id = atoi(idTxt);

         if (atoi(attr->GetText()) & Writable) {
            ++write_count;

            size = 4096;
            ret = PropStringToValueA(id, value->GetText(), &size, buffer);
            if (ret == Success) {
               ret = daqProp.Write(PropertyId(id), size, buffer);
               if (BioFailed(ret) 
                  && ret != ErrorPropNotSpted) { 
                     ++fail_count; 
               }
            }
         }
      }

      //
      // No property need write or all properties write into module successfully
      if (!write_count || !fail_count) {  return Success; }

      //
      // some properties failed to write into module
      if (write_count != fail_count) { return WarningPropPartialFailed; }

      //
      // all failed
      return ErrorPropAllFailed;
   }

   // For compatible with DAQNAVI3
   bool IsDeviceCtrlEvent(EventId id)
   {
      return EvtDeviceRemoved <=id && id <= EvtPropertyChanged;
   }
   
protected:
   DeviceCtrlImpl    m_deviceCtrl;
   DaqModule         *m_module;
   void              *m_outer;

   // Internal property, derived class can access it directly
   ControlState      m_ctrlState; 
   
   // Control lock, for thread-safe. 
   // ANY operation which will change the control's data(state)/Event should be protected by the lock.
   // Usage: AutoLock<MutexLock> lock;
   MutexLock         m_propLock;  
   MutexLock         m_buffLock;  
};

#endif
