#ifndef _DAQNAVI_DEVICE_CTRL_IMPL_H
#define _DAQNAVI_DEVICE_CTRL_IMPL_H

ArrayImpl<DeviceTreeNode>* EnumerateDeviceHelper(ModuleType typeWanted, void *daqCtrl);

class DaqCtrlBaseImpl;

class DeviceCtrlImpl
{
   class DummyDeviceObject : public DaqDevModule
   {
   public:
      virtual int32      BDAQCALL getNumber()                                      { return -1;                }
      virtual AccessMode BDAQCALL getMode()                                        { return ModeRead;          }
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId, uint32 &, void *, uint32) { return ErrorFuncNotInited;}
      virtual void       BDAQCALL Close()                                          { /*Do nothing*/            } 
      virtual ErrorCode  BDAQCALL Refresh()                                        { return ErrorFuncNotInited;}
      virtual ErrorCode  BDAQCALL Reset(uint32 state)                              { return ErrorFuncNotInited;}
      virtual ErrorCode  BDAQCALL GetModule(ModuleType, uint32, DaqModule *&)      { return ErrorFuncNotInited;}
   public:
      static DummyDeviceObject * getInstance() 
      {
         static DummyDeviceObject dummyDevice;
         return &dummyDevice;
      }
   };

public:   
   // -----------------------------------------------------------------------
   // event, 2015-4-03, add argument 'sender' for legacy support
   // -----------------------------------------------------------------------
   void addEventHandler(EventId id, DeviceEventProc proc, void * param)
   {
      if (!IsValidEvent(id)) { return; }

      UserEventManager::Add(this, id, (UserEventProc)proc, param);
      RegisterKernEvent(id);
   }

   void removeEventHandler(EventId id, DeviceEventProc proc, void * param)
   {
      if (!IsValidEvent(id)) { return; }

      UserEventManager::Remove(this, id, (UserEventProc)proc, param);
      if (!UserEventManager::IsExist(this, id)) {
         UnregisterKernEvent(id);
      }
   }

   // -----------------------------------------------------------------------
   // method
   // -----------------------------------------------------------------------
   ErrorCode Refresh()
   {
      m_device->Refresh();
      return Success;
   }

   ErrorCode ReadRegister(int32 space, int32 offset, int32 length, void *data)
   {
      return m_device->ReadRegister(space, offset, length, data);
   }
   ErrorCode WriteRegister(int32 space, int32 offset, int32 length, void *data)
   {
      return m_device->WriteRegister(space, offset, length, data);
   }

   ErrorCode ReadPrivateRegion(int32 signature, int32 length, uint8 *data)
   {
      return m_device->ReadPrivRgn(signature, length, data);
   }
   ErrorCode WritePrivateRegion(int32 signature, int32 length, uint8 *data)
   {
      return m_device->WritePrivRgn(signature, length, data);
   }

   //-------------------------------------------------------------------
   // timestamp methods
   ErrorCode SynchronizeTimebase()
   {
      return m_device->SynchronizeTimebase();
   }
   double CalculateAbsoluteTime(double const &relativeTime)
   {
      return m_device->CalculateAbsoluteTime(relativeTime);
   }

   // -----------------------------------------------------------------------
   // property
   // -----------------------------------------------------------------------
   int32 getDeviceNumber()
   {
      return m_device->getNumber();
   }
   ErrorCode getDescription(int32 length, wchar_t *desc)
   {
      if (!getInitialized()) {
         #define NODEV_DESC  L"No Device"
         length *= sizeof(wchar_t);
         return DefCopyPtr(NODEV_DESC, sizeof(NODEV_DESC), desc, (uint32&)length);
      } else {
         return DaqProperty(m_device).Get(CFG_Description, length, desc);
      }
   }
   ErrorCode setDescription(int32 length, wchar_t *desc)
   {
      if (!desc || wcslen(desc) == 0) { return ErrorBufferIsNull; }
      if (length < 1 || length > 64) { return ErrorDataLenExceedLimit; }
      
      wchar_t xbuf[64 + 1];

#if defined(_WIN32) || defined(WIN32)
      wcsncpy_s(xbuf, desc, length);
#else
      wcsncpy(xbuf, desc, length);
#endif

      xbuf[length] = 0;

      // Update sys DB 
      UpdateDeviceFieldDB(m_device->getNumber(), DFN_DESC, xbuf);

      // To specify the 'SysDB' as the target, here we have to use 'Write' method. 
      //return DaqProperty(m_device).Set(CFG_Description, length, desc);
      return DaqProperty(m_device).Write(CFG_Description, length * sizeof(wchar_t), xbuf, PropFlag::SysDB);
   }
   AccessMode getAccessMode()
   {
      return m_device->getMode();
   }

   ProductId getProductId()
   {
      int32 id;
      DaqProperty(m_device).Get(CFG_DeviceProductID, id);
      return (ProductId)id;
   }
   int32 getBoardId()
   {
      int32 id;
      DaqProperty(m_device).Get(CFG_DeviceBoardID, id);
      return id;
   }
   ErrorCode setBoardId(int32 id)
   {
      ErrorCode ret = DaqProperty(m_device).Set(CFG_DeviceBoardID, id);
      if (ret != Success) { return ret; }

      // Update sys DB after we had changed the hardware setting
      ret = UpdateDeviceFieldI32DB(m_device->getNumber(), DFN_BOARDID, id);

#if defined(_WIN32) || defined(WIN32)
      ret = DaqProperty(m_device).Write(CFG_DeviceBoardID, sizeof(id), &id, PropFlag::SysDB);
#endif
      return ret;
   }
   ErrorCode getBoardVersion(int32 length, wchar_t *ver)
   {
      return DaqProperty(m_device).Get(CFG_DeviceBoardVersion, length, ver);
   }
   ErrorCode getDriverVersion(int32 length, wchar_t *ver)
   {
      return DaqProperty(m_device).Get(CFG_DeviceDriverVersion, length, ver);
   }
   ErrorCode getDllVersion(int32 length, wchar_t *ver)
   {
      return DaqProperty(m_device).Get(CFG_DeviceDllVersion, length, ver);
   }
   ErrorCode getLocation(int32 length, wchar_t *loc)
   {
      return DaqProperty(m_device).Get(CFG_DeviceLocation, length, loc);
   }
   int32 getPrivateRegionLength()
   {
      int32 len = 0;
      DaqProperty(m_device).Get(CFG_DevicePrivateRegionLength, len);
      return len;
   }
   int32 getHotResetPreventable()
   {
      int32 hrp = 0;
      DaqProperty(m_device).Get(CFG_DeviceHotResetPreventable, hrp);
      return hrp;
   }
   int32 getBusSlot()
   {
      int32 bus_slot = -1;
      QueryDeviceFieldI32DB(m_device->getNumber(), DFN_BUSSLOT, &bus_slot);
      return bus_slot;
   }
   ArrayImpl<int64>* getBaseAddresses()
   {
      uint64 xbuf[6];
      uint32 size = sizeof(xbuf);
      DaqProperty(m_device).Read(CFG_DeviceBaseAddresses, &size, xbuf);
      if (size == 0) { return NULL; }

      ArrayImpl<int64> *coll = new ArrayImpl<int64>();

      // Note: 
      // The driver returned the device's addresses in two formats:
      //    1. 'int64' for memory-mapped IO address, or
      //    2. 'int32' for IO-mapped IO address and return only one address.
      // Here we had to determine which format the driver used.
      if (size > sizeof(uint32)) {  
         int64 *ptr = (int64 *)xbuf;
         for (unsigned i = 0; i < size / sizeof(int64); ++i) {
            coll->Add(ptr[i]);
         }
      } else { 
         uint32 *ptr = (uint32 *)xbuf;
         for (unsigned i = 0; i < size / sizeof(int32); ++i) {
            coll->Add(ptr[i]);
         }
      }
      return coll;
   }
   ArrayImpl<int32>* getInterrupts()
   {
      int32 count = DaqProperty(m_device).GetLength(CFG_DeviceInterrupts) / sizeof(int32);
      if (count > 0) {
         int32 irqs[6];
         DaqProperty(m_device).Get(CFG_DeviceInterrupts, 6, irqs);
         ArrayImpl<int32> *coll = new ArrayImpl<int32>();
         for (int i = 0; i < count; ++i) {
            coll->Add(irqs[i]);
         }
         return coll;
      }
      return NULL;
   }
   ArrayImpl<TerminalBoard>* getSupportedTerminalBoard()
   {
      int32 count = DaqProperty(m_device).GetLength(CFG_DeviceSupportedTerminalBoardTypes) / sizeof(int32);
      if (count > 0) {
         int32 types[10];
         DaqProperty(m_device).Get(CFG_DeviceSupportedTerminalBoardTypes, 10, types);
         ArrayImpl<TerminalBoard> *coll = new ArrayImpl<TerminalBoard>();
         for (int i = 0; i < count; ++i) {
            coll->Add((TerminalBoard)types[i]);
         }
         return coll;
      }
      return NULL;
   }
   ArrayImpl<EventId>* getSupportedEvents()
   {
      int32 count = DaqProperty(m_device).GetLength(CFG_DeviceSupportedEvents) / sizeof(int32);
      if (count > 0) {
         int32 events[10];
         DaqProperty(m_device).Get(CFG_DeviceSupportedEvents, 10, events);
         ArrayImpl<EventId> *coll = new ArrayImpl<EventId>();
         for (int i = 0; i < count; ++i) {
            coll->Add((EventId)events[i]);
         }
         return coll;
      }
      return NULL;
   }
 
   int32 getSupportedScenarios(); // implemented in cpp   

   TerminalBoard getTerminalBoard()
   {
      int32 board = WiringBoard;
      DaqProperty(m_device).Get(CFG_DeviceTerminalBoardType, board);
      return (TerminalBoard)board;
   }
   ErrorCode setTerminalBoard(TerminalBoard board)
   {
      ErrorCode ret = DaqProperty(m_device).Set(CFG_DeviceTerminalBoardType, (int32)board);
      if (ret != Success) { return ret; }

      // Update sys DB after we had changed the hardware setting
      ret = UpdateDeviceFieldI32DB(m_device->getNumber(), DFN_TERMBD, board);
#if defined(_WIN32) || defined(WIN32)
      ret = DaqProperty(m_device).Write(CFG_DeviceTerminalBoardType, sizeof(board), &board, PropFlag::SysDB);
#endif
      return ret;
   }
   ErrorCode setLocateEnabled(int32 value)
   {
      return DaqProperty(m_device).Set(CFG_DeviceLocateEnabled, value);
   }

   //-------------------------------------------------------------------
   // static property
   static ArrayImpl<DeviceTreeNode>* getInstalledDevices() 
   { 
      return EnumerateDeviceHelper(DaqDevice, NULL);
   }

   //-------------------------------------------------------------------
   // device specific properties
   static PropertyId idofProp(wchar_t const *name)
   {
      static struct {
         wchar_t const *name;
         PropertyId id;
      } x_map [] = {
         {HWSPEC_WDT_DO_ENABLED, CFG_DeviceWdtDoEnabled},
         {HWSPEC_WDT_DO_PATTERN, CFG_DeviceWdtDoPattern}, 
         {HWSPEC_WDT_TIME,       CFG_DeviceWdtTime},      
         {HWSPEC_WDT_FLAG,       CFG_DeviceWdtFlag}, 
      };
      for (unsigned i = 0; i < ARRAY_SIZE(x_map); ++i) {
         if (wcscmp(x_map[i].name, name) == 0) {
            return x_map[i].id;
         }
      }
      return PropertyId(-1);
   }

   ErrorCode getHwSpecific(wchar_t const *name, int32 *size, void *buffer)
   {
      PropertyId id = idofProp(name);
      if (id == PropertyId(-1)) {
         return ErrorPropNotSpted;
      }

      return DaqProperty(m_device).Read(id, (uint32*)size, buffer);
   }
   ErrorCode setHwSpecific(wchar_t const *name, int32 size, void *buffer)
   {
      PropertyId id = idofProp(name);
      if (id == PropertyId(-1)) {
         return ErrorPropNotSpted;
      }

      return DaqProperty(m_device).Write(id, (uint32)size, buffer);
   }

   // Property for fusion device
   ArrayImpl<ProductId>* getSupportedFusionTypes()
   {
      int32  types[16];
      uint32 size = sizeof(types);
      ErrorCode ret = DaqProperty(m_device).Read(CFG_DeviceSupportedFusionTypes, &size, types);

      ArrayImpl<ProductId> *spted = NULL;
      if (ret == Success && size) {
         spted = new ArrayImpl<ProductId>();
         for (unsigned i = 0; i < size / sizeof(int32); ++i) {
            spted->Add(ProductId(types[i]));
         }
      }

      return spted;
   }

   // Features & properties for PFP pins
   ArrayImpl<SignalDrop>* getProgrammableSignals()
   {
      int32  sigs[64];
      uint32 size = sizeof(sigs);
      ErrorCode ret = DaqProperty(m_device).Read(CFG_FeatureDeviceRtProgrammableSignals, &size, sigs);

      ArrayImpl<SignalDrop> *spted = NULL;
      if (ret == Success && size) {
         spted = new ArrayImpl<SignalDrop>();
         for (unsigned i = 0; i < size / sizeof(int32); ++i) {
            spted->Add(SignalDrop(sigs[i]));
         }
      }

      return spted;
   }
   ErrorCode getProgrammableSignalsDirection(int32 length, SignalDirection dir[])
   {
      return DaqProperty(m_device).Get(CFG_DeviceRtProgrammableSignalDirections, length, (int32 *)dir);
   }
   ErrorCode setProgrammableSignalsDirection(int32 length, SignalDirection dir[])
   {
      return DaqProperty(m_device).Set(CFG_DeviceRtProgrammableSignalDirections, length, (int32 *)dir);
   }

   // Features & properties for signal routing
   ArrayImpl<SignalDrop>* getRtSignalEndpoints()
   {
      int32  types[64];
      uint32 size = sizeof(types);
      ErrorCode ret = DaqProperty(m_device).Read(CFG_FeatureDeviceRtSignalEndpoints, &size, types);

      ArrayImpl<SignalDrop> *spted = NULL;
      if (ret == Success && size) {
         spted = new ArrayImpl<SignalDrop>();
         for (unsigned i = 0; i < size / sizeof(int32); ++i) {
            spted->Add(SignalDrop(types[i]));
         }
      }

      return spted;
   }
   ArrayImpl<SignalDrop>* getRtEndpointXSupportedSources(int32 epIndex)
   {
      ArrayImpl<SignalDrop> *spted = NULL;
      if (epIndex < 0 || 15 < epIndex) {
         return spted;
      }

      int32  types[64];
      uint32 size = sizeof(types);
      ErrorCode ret = DaqProperty(m_device).Read(PropertyId(CFG_FeatureDeviceRtEndpoint0Sources + epIndex), &size, types);

      if (ret == Success && size) {
         spted = new ArrayImpl<SignalDrop>();
         for (unsigned i = 0; i < size / sizeof(int32); ++i) {
            spted->Add(SignalDrop(types[i]));
         }
      }

      return spted;
   }
   SignalDrop getRtEndpointXSource(int32 epIndex)
   {
      if (epIndex < 0 || 15 < epIndex) {
         return SignalNone;
      }

      int32 source = SignalNone;
      DaqProperty(m_device).Get(PropertyId(CFG_DeviceRtEndpoint0Source + epIndex), source);

      return SignalDrop(source);
   }
   ErrorCode setRtEndpointXSource(int32 epIndex, SignalDrop source)
   {
      if (epIndex < 0 || 15 < epIndex) {
         return ErrorParamOutOfRange;
      }

      return DaqProperty(m_device).Set(PropertyId(CFG_DeviceRtEndpoint0Source + epIndex), int32(source));
   }

public:
   DeviceCtrlImpl(bool disposable = false)
   {
      m_owner       = NULL;
      m_disposable  = disposable;
      m_device      = DummyDeviceObject::getInstance();
      m_devPnpState = DevState::Arrived;
   }

   virtual ~DeviceCtrlImpl()
   {
      Deinitialize();

      // clear all user event delegate
      for (unsigned i = 0; i < ARRAY_SIZE(dev_ctl_events); ++i) {
         UserEventManager::Remove(this, dev_ctl_events[i]);
      }
   }

   void setOwner(DaqCtrlBaseImpl *owner)
   {
      m_owner = owner;
   }

   DaqDevModule * getDevice()
   {
      return m_device;
   }

   bool getInitialized()
   {
      return m_device != DummyDeviceObject::getInstance();
   }

   bool getDisposable()
   {
      return m_disposable;
   }

   int32 getDeviceState()
   {
      return m_devPnpState;
   }

   // Initialize/Deinitialize the control.
   ErrorCode Initialize(int32 deviceNumber, wchar_t const *description, AccessMode mode)
   {
      // Must de-initialize before we open a new device object
      assert(!getInitialized());

      // open the specified device
      DaqDevModule *newDevice;
      ErrorCode    ret;

      // Open device by description if it is available
      if (description && description[0] != L'\0') {
         ret = XDevManager::OpenDevice(description, mode, newDevice);
      } else if (deviceNumber >= 0) {
         // Open device by device number
         ret = XDevManager::OpenDevice(deviceNumber, mode, newDevice);
      } else {
         ret = ErrorDeviceNotExist;
      }

      if (ret != Success) {
         return ret;
      }
      
      m_device = newDevice;

      // activate the device events
      for (unsigned i = 0; i < ARRAY_SIZE(dev_ctl_events); ++i) {
         if (UserEventManager::IsExist(this, dev_ctl_events[i])) {
            RegisterKernEvent(dev_ctl_events[i]);
         }
      }

      // Fix bug#20425: Register device 'Removed' & 'Reconnected' events only if the user required
      // Always register device 'Removed' & 'Reconnected' event for internal using, if available.
      //{ 
      //   RegisterKernEvent(EvtDeviceRemoved);
      //   RegisterKernEvent(EvtDeviceReconnected);
      //}

      // initialize device state: Arrived 
      m_devPnpState = DevState::Arrived;

      return Success;
   }

   void Deinitialize()
   {
      if (!getInitialized()) {
         return;
      }

      // deactivate device events
      for (unsigned i = 0; i < ARRAY_SIZE(dev_ctl_events); ++i) {
         UnregisterKernEvent(dev_ctl_events[i]);
      }
     
      // close device
      XDevManager::CloseDevice(m_device);
      m_device = DummyDeviceObject::getInstance();
   }

   DaqModule * GetModule(ModuleType type, int32 index)
   {
      DaqModule *m = NULL;
      m_device->GetModule(type, index, m);
      return m;
   }

   static int32 CounterCapsToScenarios(ArrayImpl<CounterCapability> &caps)
   {
      int32 sc = 0;
      for (int i = 0; i < caps.getLength(); ++i){
         switch(caps.Item(i)) 
         {
         case InstantEventCount:   sc |= SceEventCounter;         break;
         case OneShot:             sc |= SceOneShot;              break;
         case TimerPulse:          sc |= SceTimerPulse;           break;
         case InstantFreqMeter:    sc |= SceFreqMeter;            break;
         case InstantPwmIn:        sc |= ScePwMeter;              break;   
         case InstantPwmOut:       sc |= ScePwModulator;          break;
         case UpDownCount:         sc |= SceUdCounter;            break;
         case BufferedEventCount:  sc |= SceBufferedEventCounter; break;
         case BufferedPwmIn:       sc |= SceBufferedPwMeter;      break;
         case BufferedPwmOut:      sc |= SceBufferedPwModulator;  break;
         case BufferedUpDownCount: sc |= SceBufferedUdCounter;    break;
         default: break;
         }
      }
      return sc;
   }

protected: // internal methods

   // register/unregister device event into event manager
   ErrorCode RegisterKernEvent(EventId id)
   {
      if (!getInitialized()) { return ErrorFuncNotInited; }

      HANDLE handle = GetEventHandle(id);
      if (handle == NULL) { return ErrorEventNotSpted; }

      KernEventMonitor::Register(getDeviceNumber(), handle, id, (KernEventProc)OnKernEvent, this);
      return Success;
   }
   
   void UnregisterKernEvent(EventId id)
   {
      if (!getInitialized()) { return; }

      KernEventMonitor::Unregister(getDeviceNumber(), id, (KernEventProc)OnKernEvent, this);
   }
   
   HANDLE GetEventHandle(EventId id)
   {
      assert(EvtDeviceRemoved <= id && id <= EvtPropertyChanged);
      HANDLE handle = NULL;
      m_device->EventGetHandle(id, handle);
      return handle;
   }

   bool IsValidEvent(EventId id)
   {
      return ARRAY_INDEX_OF(dev_ctl_events, id) != -1;
   }

   // -----------------------------------------------------------------
   // driver event receiver for device event
   // Move the implement into .cpp due to we want to use the method of
   // class DaqCtrlBaseImpl
   // -----------------------------------------------------------------
   static void OnKernEvent(EventId id, DeviceCtrlImpl *ctrl);
  
private:   
   DaqCtrlBaseImpl  *m_owner;
   bool              m_disposable;
   DaqDevModule     *m_device;
   int32             m_devPnpState;
};

#endif
