#ifndef _DAQNAVI_TRIGGER_IMPL_H
#define _DAQNAVI_TRIGGER_IMPL_H

// -----------------------------------------------------------------
// implementation of Trigger, multi-trigger supported
// -----------------------------------------------------------------

class DaqCtrlBase;

//*****
// CAUTION: SHARED WITH DN3, DONOT CHANGE THE CLASS LAYOUT AND METHODS(NAME,
// PARAMETERS, etc.). BUT YOU CAN APPEND NEW METHODS TO THE CLASS, IF NECESSARY.
//*****
class TriggerImpl
{
public:
   virtual SignalDrop    BDAQCALL getSource();
   virtual ErrorCode     BDAQCALL setSource(SignalDrop value);
   virtual ActiveSignal  BDAQCALL getEdge();
   virtual ErrorCode     BDAQCALL setEdge(ActiveSignal value);
   virtual double        BDAQCALL getLevel();
   virtual ErrorCode     BDAQCALL setLevel(double value);
   virtual TriggerAction BDAQCALL getAction();
   virtual ErrorCode     BDAQCALL setAction(TriggerAction value);
   virtual int32         BDAQCALL getDelayCount();
   virtual ErrorCode     BDAQCALL setDelayCount(int32 value);
                         
   virtual double        BDAQCALL getHysteresisIndex();
   virtual ErrorCode     BDAQCALL setHysteresisIndex(double value);

   virtual FilterType    BDAQCALL getFilterType();
   virtual ErrorCode     BDAQCALL setFilterType(FilterType value);

   virtual double        BDAQCALL getFilterCutoffFreq();
   virtual ErrorCode     BDAQCALL setFilterCutoffFreq(double value);

public:
   TriggerImpl(DaqCtrlBaseImpl *owner = NULL, int32 tag = 0, bool isDio = false)
   {
      m_owner = owner;
      m_tag   = tag;
      m_trigDelayPerChan = 0;
      m_isDio = isDio;
   }
   void setOwner(DaqCtrlBaseImpl *owner, int32 tag = 0, bool isDio = false)
   {
      m_owner = owner;
      m_tag   = tag;
      m_isDio = isDio;
   }
   void adjustDelayCount(int32 value)
   {
      m_trigDelayPerChan = value;
   }

protected:
   DaqCtrlBaseImpl *m_owner;
   int32           m_tag;
   int             m_trigDelayPerChan; // cached value which can not be mapped directly to a driver define property.
   bool            m_isDio;
};

#endif

