#ifndef _DAQNAVI_CALIBRATION_CTRL_IMPL_H
#define _DAQNAVI_CALIBRATION_CTRL_IMPL_H

class CalibrationCtrlImpl;
class CaliSolutionImpl;
class CaliSectionImpl;

class CaliSubjectImpl
{
public:
   //********************************************************************************
   // property
   //--------------------------------------------------------------------------------
   wchar_t const * getDescription(void);
   wchar_t const * getInstruction(void);
   wchar_t const * getTargetRange(void);

   //********************************************************************************
   // method
   //--------------------------------------------------------------------------------
   CaliState       AdjCodeGetCurrentState(double *value, uint32 *count, uint32 code[]);
   void            AdjCodeGetRange(uint32 lower[], uint32 upper[]);

   ErrorCode       ManualAdjStart(void);
   ErrorCode       ManualAdjStop(void);
   ErrorCode       ManualAdjSetCode(uint32 count, uint32 code[]);

public:
   CaliSubjectImpl(CalibrationCtrlImpl *owner = NULL, uint32 id = -1)
   {
      m_owner      = owner;
      m_subId.xval = id;
   }

   virtual ~CaliSubjectImpl()
   {
   }

private:
   CalibrationCtrlImpl    *m_owner;
   DaqCaliModule::ObjectId m_subId;

   std::wstring            m_description;
   std::wstring            m_instruction;
   std::wstring            m_targetRange;
};

class CaliSectionImpl
{
public:
   //********************************************************************************
   // property
   //--------------------------------------------------------------------------------
   wchar_t const * getTypeName(void);
   wchar_t const * getDescription(void);
   wchar_t const * getInstruction(void);

   ArrayImpl<CaliSubjectImpl> * getSubjects(void);

   //********************************************************************************
   // method
   //--------------------------------------------------------------------------------
   ErrorCode       Prepare(void);
   ErrorCode       BatchStart(void);
   void            BatchTerminate(void);

public:
   CaliSectionImpl(CalibrationCtrlImpl *owner = NULL, uint32 id = -1)
   {
      m_owner       = owner;
      m_sectId.xval = id;
   }

   virtual ~CaliSectionImpl()
   {
      m_subjects.Clear();
   }

private:
   CalibrationCtrlImpl       *m_owner;
   DaqCaliModule::ObjectId    m_sectId;
   ArrayImpl<CaliSubjectImpl> m_subjects;

   std::wstring               m_typeName;
   std::wstring               m_description;
   std::wstring               m_instruction;
};

class CaliSolutionImpl 
{
public:
   //********************************************************************************
   // property
   //--------------------------------------------------------------------------------
   wchar_t const * getTypeName(void);
   wchar_t const * getDescription(void);
   wchar_t const * getInstruction(void);
   uint8   const * getAuxiliaryImage(uint32 *size);

   ArrayImpl<CaliSectionImpl> * getSections(void);

   //********************************************************************************
   // method
   //--------------------------------------------------------------------------------
   ErrorCode       AdjCodesSave(void);
   ErrorCode       AdjCodesLoad(void);                          
   ErrorCode       AdjCodesSaveToGoldBackup(void);              
   ErrorCode       AdjCodesLoadFromGoldBackup(int32 action);

public:
   CaliSolutionImpl(CalibrationCtrlImpl *owner = NULL, uint32 id = -1)
   {
      m_owner      = owner;
      m_slnId.xval = id;
   }

   virtual ~CaliSolutionImpl()
   {
      m_sections.Clear();
   }

private:
   CalibrationCtrlImpl       *m_owner;
   DaqCaliModule::ObjectId    m_slnId;
   ArrayImpl<CaliSectionImpl> m_sections;

   std::wstring               m_typeName;
   std::wstring               m_description;
   std::wstring               m_instruction;
};

class CalibrationCtrlImpl : public DaqCtrlBaseImpl
{
public:
   //********************************************************************************
   // property
   //--------------------------------------------------------------------------------
   ArrayImpl<CaliSolutionImpl> * getSolutions(void)
   {
      if (getInitialized() && m_solutions.getLength() == 0) {
         DaqCaliModule::ObjectId ids[64];
         uint32 count = ModuleRef().getSolutionList(64, ids);

         for (unsigned i = 0; i < count; ++i) {
            m_solutions.Add(CaliSolutionImpl(this, ids[i].xval));
         }
      }

      return &m_solutions;
   }

public: // ctor & dtor
   CalibrationCtrlImpl() : DaqCtrlBaseImpl(DaqCali), m_solutions(false)
   {
   }
   virtual ~CalibrationCtrlImpl()
   {
   }

   DaqCaliModule & ModuleRef() { return *(DaqCaliModule*)getModule(); }

protected: 
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual ModuleType WantedType() { return DaqCali;  }

   virtual bool IsSuitable(DaqModule & module)
   {
      return module.getType() == DaqCali;
   }

   // initialize/de-initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = DaqCtrlBaseImpl::InitializeControl(x);

      return ret;
   }

   virtual void DeinitializeControl()
   {
      // reset all configuration to null
      m_solutions.Clear();

      DaqCtrlBaseImpl::DeinitializeControl();
   }

private:
   ArrayImpl<CaliSolutionImpl> m_solutions; 
};

#endif
