//
// Calibration function implementation
//
#include <std_decl.h>
#include "cali_ctrl.h"

//********************************************************************************
// Helper MACRO for get string information
//--------------------------------------------------------------------------------
#define CALI_GET_STRING(var, id, method)                     \
   do {                                                      \
      wchar_t  xbuf[1024];                                   \
      wchar_t *ptr  = xbuf;                                  \
      uint32   size = method(id, 1024, ptr);                 \
      if (size > 1024) {                                     \
         ptr = (wchar_t *)_malloca(size * sizeof(wchar_t));  \
         size = method(id, size, ptr);                       \
       }                                                     \
       if (size)        { var = ptr;   }                     \
       if (ptr != xbuf) { _freea(ptr); }                     \
   } while (0)                                                     
                                    
#define CALI_SUBJ_GET_STRING(var, id, fn)\
   CALI_GET_STRING(var, id, m_owner->ModuleRef().subject_##fn)

#define CALI_SECT_GET_STRING(var, id, fn)\
   CALI_GET_STRING(var, id, m_owner->ModuleRef().section_##fn)

#define CALI_SLN_GET_STRING(var, id, fn)\
   CALI_GET_STRING(var, id, m_owner->ModuleRef().solution_##fn)

//********************************************************************************
// class CaliSubjectImpl
//--------------------------------------------------------------------------------
wchar_t const * CaliSubjectImpl::getDescription(void)
{
   if (!m_description.size()) {
      CALI_SUBJ_GET_STRING(m_description, m_subId, getDescription);
   }
   
   return m_description.c_str();
}
wchar_t const * CaliSubjectImpl::getInstruction(void)
{
   if (!m_instruction.size()) {
      CALI_SUBJ_GET_STRING(m_instruction, m_subId, getInstruction);
   }

   return m_instruction.c_str();
}
wchar_t const * CaliSubjectImpl::getTargetRange(void)
{
   if (!m_targetRange.size()) {
      CALI_SUBJ_GET_STRING(m_targetRange, m_subId, getTargetRange);
   }

   return m_targetRange.c_str();
}

CaliState CaliSubjectImpl::AdjCodeGetCurrentState(double *value, uint32 *count, uint32 code[])
{
   return m_owner->ModuleRef().subject_AdjCodeGetCurrentState(m_subId, value, count, code);
}
void CaliSubjectImpl::AdjCodeGetRange(uint32 lower[], uint32 upper[])
{
   m_owner->ModuleRef().subject_AdjCodeGetRange(m_subId, lower, upper);
}

ErrorCode CaliSubjectImpl::ManualAdjStart(void)
{
   return m_owner->ModuleRef().subject_ManualAdjStart(m_subId);
}
ErrorCode CaliSubjectImpl::ManualAdjStop(void)
{
   return m_owner->ModuleRef().subject_ManualAdjStop(m_subId);
}
ErrorCode CaliSubjectImpl::ManualAdjSetCode(uint32 count, uint32 code[])
{
   return m_owner->ModuleRef().subject_ManualAdjSetCode(m_subId, count, code);
}

//********************************************************************************
// class CaliSectionImpl
//--------------------------------------------------------------------------------
wchar_t const * CaliSectionImpl::getTypeName(void)
{
   if (!m_typeName.size()) {
      CALI_SECT_GET_STRING(m_typeName, m_sectId, getTypeName);
   }

   return m_typeName.c_str();
}
wchar_t const * CaliSectionImpl::getDescription(void)
{
   if (!m_description.size()) {
      CALI_SECT_GET_STRING(m_description, m_sectId, getDescription);
   }

   return m_description.c_str();
}
wchar_t const * CaliSectionImpl::getInstruction(void)
{
   if (!m_instruction.size()) {
      CALI_SECT_GET_STRING(m_instruction, m_sectId, getInstruction);
   }

   return m_instruction.c_str();
}
ArrayImpl<CaliSubjectImpl> * CaliSectionImpl::getSubjects(void)
{
   if (m_owner->getInitialized() && m_subjects.getLength() == 0) {
      DaqCaliModule::ObjectId xbuf[64];
      DaqCaliModule::ObjectId *data = xbuf;

      uint32 count = m_owner->ModuleRef().section_getSubjectList(m_sectId, 64, data);

      // Buffer too small, re-allocate one and try again
      if (count > 64) { 
         data = (DaqCaliModule::ObjectId *)_malloca(count * sizeof(DaqCaliModule::ObjectId));
         count = m_owner->ModuleRef().section_getSubjectList(m_sectId, count, data);
      }

      for (unsigned i = 0; i < count; ++i) {
         m_subjects.Add(CaliSubjectImpl(m_owner, data[i].xval));
      }

      if (data != xbuf) { _freea(data); }
   }

   return &m_subjects;
}

ErrorCode CaliSectionImpl::Prepare(void)
{
    return m_owner->ModuleRef().section_Prepare(m_sectId);
}

ErrorCode CaliSectionImpl::BatchStart(void)
{
   return m_owner->ModuleRef().section_BatchStart(m_sectId);
}
void CaliSectionImpl::BatchTerminate(void)
{
   m_owner->ModuleRef().section_BatchTerminate(m_sectId);
}

//********************************************************************************
// class CaliSolutionImpl 
//--------------------------------------------------------------------------------
wchar_t const * CaliSolutionImpl::getTypeName(void)
{
   if (!m_typeName.size()) {
      CALI_SLN_GET_STRING(m_typeName, m_slnId, getTypeName);
   }

   return m_typeName.c_str();
}
wchar_t const * CaliSolutionImpl::getDescription(void)
{
   if (!m_description.size()) {
      CALI_SLN_GET_STRING(m_description, m_slnId, getDescription);
   }

   return m_description.c_str();
}
wchar_t const * CaliSolutionImpl::getInstruction(void)
{
   if (!m_instruction.size()) {
      CALI_SLN_GET_STRING(m_instruction, m_slnId, getInstruction);
   }

   return m_instruction.c_str();
}
uint8 const * CaliSolutionImpl::getAuxiliaryImage(uint32 *size)
{
   return m_owner->ModuleRef().solution_getAuxImage(m_slnId, size);
}
ArrayImpl<CaliSectionImpl> * CaliSolutionImpl::getSections(void)
{
   if (m_owner->getInitialized() && m_sections.getLength() == 0) {
      DaqCaliModule::ObjectId xbuf[64];
      DaqCaliModule::ObjectId *data = xbuf;

      uint32 count = m_owner->ModuleRef().solution_getSectionList(m_slnId, 64, data);

      // Buffer too small, re-allocate one and try again
      if (count > 64) { 
         data = (DaqCaliModule::ObjectId *)_malloca(count * sizeof(DaqCaliModule::ObjectId));
         count = m_owner->ModuleRef().solution_getSectionList(m_slnId, count, data);
      }

      for (unsigned i = 0; i < count; ++i) {
         m_sections.Add(CaliSectionImpl(m_owner, data[i].xval));
      }

      if (data != xbuf) { _freea(data); }
   }

   return &m_sections;
}

ErrorCode CaliSolutionImpl::AdjCodesSave(void)
{
   return m_owner->ModuleRef().solution_AdjCodesSave(m_slnId);
}
ErrorCode CaliSolutionImpl::AdjCodesLoad(void)
{
   return m_owner->ModuleRef().solution_AdjCodesLoad(m_slnId);
}
ErrorCode CaliSolutionImpl::AdjCodesSaveToGoldBackup(void)
{
   return m_owner->ModuleRef().solution_AdjCodesSaveToGoldBackup(m_slnId);
}
ErrorCode CaliSolutionImpl::AdjCodesLoadFromGoldBackup(int32 action)
{
   return m_owner->ModuleRef().solution_AdjCodesLoadFromGoldBackup(m_slnId, action);
}