#ifndef _DAQNAVI_BFD_PULSE_WIDTH_METER_CTRL_IMPL_H
#define _DAQNAVI_BFD_PULSE_WIDTH_METER_CTRL_IMPL_H

class BfdPwMeterCtrlImpl : public CntrCtrlBaseImpl
{

public:
   // Method
   ErrorCode GetData(int32 cntr, int32 count, void *buffer, int32 timeout, int32 *returned, double *startTime)
   {
      ErrorCode   ret;
      EventStatus es  = {sizeof(es)};
      EventId     eid = EventId(EvtCntrStopped0 + cntr);
      uint32      dtLen;
      uint8      *dtPos = (uint8*)buffer;

      AutoLock<MutexLock> lock(BuffLock());

      do {
         // Get running state before we read data
         ModEvent().GetStatus(eid, es);

         // Get data
         dtLen = count;
         ret = ModuleRef().BfdCtGetData(cntr, (uint32 &)dtLen, dtPos, NULL, NULL);

         if(BioFailed(ret)) {break;} // Error occurred!

         count -= dtLen;
         dtPos += dtLen * sizeof(PulseWidth);

         // Update the 'Read Position'
         m_dataReadyCleared[cntr] = true;
         ModEvent().ClearFlag(EventId(EvtCiDataReady0 + cntr), dtLen, 0);

         // Wait data continuously if it is necessary
         if(!count || ret != Success || es.General.lParam == DAQ_FN_STOPPED) {break;}

         // Wait data ready and update the timeout value
         if(timeout) { timeout = WaitEventTimeout(m_dataArrived[cntr], timeout); }
      } while (timeout);

      if(returned) { *returned = uint32(dtPos - (uint8*)buffer) / sizeof(PulseWidth); }

      if(count && ret == Success) {
         ret = es.General.lParam ? WarningFuncStopped : WarningFuncTimeout;
      }

      return ret;
   }

   int32 getBufferCapacity()
   {
      return m_dataBufLen; 
   }

   TriggerImpl* getTrigger(int32 trigger)
   {
      TriggerImpl *x = NULL;

      // Bug#14478
      //if ((uint32)trigger < (uint32)getFeatures()->getTriggerCount()) {
      if ((uint32)trigger < ARRAY_SIZE(m_triggers)) {
         x = &m_triggers[trigger];
      }

      return x;
   }

   RecordImpl * getRecord()
   {
      return &m_record;
   }

   bool getBufferStatus(int32 ch, int32 & count, int32 & offset)
   {
      EventStatus es = {sizeof(es)};
      ModEvent().GetStatus(EventId(EvtCiDataReady0 + ch), es);
      count  = es.General.lParam;
      offset = es.General.rParam;

      return false;
   }

   ErrorCode setSampleClkSrc(SignalDrop value)
   {
      if (IsRunning()) {
         return ErrorFuncBusy;
      }

      ArrayImpl<SignalDrop>* clkSrcs = ModProp().GetValueArrayI32<SignalDrop>(PropertyId(CFG_FeaturePiSampleClkSourceOfCounter0 + m_chStart));
      for(int32 i = 0; i < clkSrcs->getLength(); i++) {

         if(value == clkSrcs->Item(i)) {
            m_sampleClkSrc = value;

            return ModProp().SetValueNth<int32>(CFG_SamplingClockSources, m_chStart, m_sampleClkSrc);
         }
      }

      return ErrorPropValueNotSpted;
   }

   SignalDrop getSampleClkSrc()
   {
      ModProp().GetValueNth<SignalDrop>(CFG_SamplingClockSources, m_chStart, m_sampleClkSrc);

      return m_sampleClkSrc;
   }

   void getChannelMap(int32 count, int8 *data)
   {
      ModProp().Read(CFG_ScanEnabledChannels, count, data);
   }

   ErrorCode setChannelMap(int32 count, int8 *data)
   {
      if (!CanEditProperty()) { return ErrorPrivilegeNotHeld; }
      if (IsRunning())        { return ErrorFuncBusy;         }

      CHK_USER_BUF_LEN(data, count >= getFeatures()->getChannelCountMax());

      ErrorCode ret = ModProp().Set(CFG_ScanEnabledChannels, count, (uint8 *)data);

      int ch_start = 0; // update channelStart and channelCount
      int ch_count = 0; 
      
      m_chEnabled= 0;
      for (int i = 0; i < getFeatures()->getChannelCountMax(); ++i) {
         if (((uint8*)data)[i] == 0) { continue; }

         m_chEnabled |= 1 << i;

         ++ch_count;
         if (ch_count == 1) { ch_start = i; }
      }

      if (m_chEnabled) {
         m_chStart = ch_start;
         m_chCount = ch_count;
      }

      return ret;
   }

   virtual ErrorCode setChannelStart(int32 ch)
   {
      if (!getInitialized()) { return ErrorFuncNotInited; }
      if(IsRunning())        { return ErrorFuncBusy; }

      ErrorCode warning = Success;
      warning = ModProp().Set(CFG_ScanChannelStart, ch);
      ModProp().Get(CFG_ScanChannelStart, m_chStart);

      m_chEnabled = CalculateChEnabled();

      return warning;
   }

   virtual ErrorCode setChannelCount(int32 count)
   {
      if (!getInitialized()) { return ErrorFuncNotInited; }
      if(IsRunning())        { return ErrorFuncBusy; }

      ErrorCode warning = Success;
      warning = ModProp().Set(CFG_ScanChannelCount, count);
      ModProp().Get(CFG_ScanChannelCount, m_chCount);

      m_chEnabled = CalculateChEnabled();

      return warning;
   }

   ErrorCode Prepare()
   {
      AutoLock<MutexLock> lock(PropLock());

      ErrorCode ret;
      if(!getInitialized()) {
         ret = ErrorFuncNotInited;
      } else if(IsRunning()) {
         ret = ErrorFuncBusy;
      } else if(getState() == Ready) {
         ret = Success;
      } else {
         ret = UpdateChEnabled();
         CHK_EXPR(ret == Success, ret);
         // Configure each enabled counter channel
         uint32 chEnabled = m_chEnabled;
         for (int cntr = 0; cntr < getFeatures()->getChannelCountMax(); ++cntr, chEnabled >>= 1) {
            if(chEnabled & 0x1) { // enable counter
               ret = ModProp().SetValueNth<uint32>(CFG_BufferedModeOfCounters, cntr, WantedCapability());
               CHK_RESULT(ret);

               ret = ModProp().SetValueNth<int32>(CFG_RecordSectionLengthes,  cntr, m_record.getSectionLength());
               CHK_RESULT(ret);

               ret = ModProp().SetValueNth<int32>(CFG_RecordSectionCounts,    cntr, m_record.getSectionCount());
               CHK_RESULT(ret);

               ret = ModProp().SetValueNth<int32>(CFG_SamplingClockSources,   cntr, m_sampleClkSrc);
               CHK_RESULT(ret);

               int32 count = getFeatures()->getTriggerCount();
               for (int32 i = 0; i < count; i++) {
                  ret = ModProp().SetValueNth<int32>(CFG_TriggerSources, cntr, m_triggers[i].getSource());
                  CHK_RESULT(ret);

                  ret = ModProp().SetValueNth<double>(CFG_TriggerSourcesLevel, cntr, m_triggers[i].getLevel());
                  CHK_RESULT(ret);

                  ret = ModProp().SetValueNth<int32>(CFG_TriggerSourcesEdge, cntr, m_triggers[i].getEdge());
                  CHK_RESULT(ret);

                  ret = ModProp().SetValueNth<double>(CFG_TriggerSourcesHysteresisIndex, cntr, m_triggers[i].getHysteresisIndex());
                  CHK_RESULT(ret);
               }
            }
         }

         m_dataBufLen = 0;
         ret = ModuleRef().BfdCtPrepare((uint32 &)m_dataBufLen);
         if(ret == Success) { // adjust the section length.
            int32 sectLength = 0;
            ret = ModProp().GetValueNth<int32>(CFG_RecordSectionLengthes, m_chStart, sectLength);
            CHK_RESULT(ret);
            m_record.setSectionLength(sectLength);
         }

         setState(ret == Success ? Ready : Idle);
      }

      return ret;
   }

   void Release()
   {
      // Fix bug#19706: Call driver 'Release' function anyway to release resource, if any.
      // It is harmless operation even if the device is in 'FAI IDLE' state.
      // if (IsIdle()) { return; }
      // Call driver 'Release' only if we had called the method 'Prepare'
      if (m_dataBufLen) {
         ModuleRef().BfdCtRelease();
      }

      setState(Idle);
      m_dataBufLen = 0;
   }

   // Property
   ArrayImpl<PiChannelImpl>* getChannels()
   {
      if (getInitialized()) {
         if (m_channels.getLength() == 0) {
            int count = getFeatures()->getChannelCountMax();
            m_channels.setCapacity(count);

            for (int i = 0; i < count; ++i) {
               m_channels.Add(PiChannelImpl(this, i));
            }
         }
      }

      return &m_channels;
   }

   // event
   virtual void addEventHandler(EventId id, DaqEventProc proc, void * param)
   {
      if(!IsValidEvent(id)) {
         CntrCtrlBaseImpl::addEventHandler(id, proc, param);
         return;
      }

      UserEventManager::Add(getOuter(), id, (UserEventProc)proc, param);
   }

   virtual void removeEventHandler(EventId id, DaqEventProc proc, void * param)
   {
      if(!IsValidEvent(id)) {
         CntrCtrlBaseImpl::removeEventHandler(id, proc, param);
         return;
      }

      UserEventManager::Remove(getOuter(), id, (UserEventProc)proc, param);
   }

public: // ctor & dtor
   BfdPwMeterCtrlImpl() : m_channels(false), m_dn4EventRegistered(false), m_canFire(false), m_dataBufLen(0)
   { 
      memset(m_dataReadyCleared, 0, sizeof(m_dataReadyCleared));

      for(int8 ch = 0; ch < MAX_CNTR_CH_COUNT; ch++) {
         m_dataArrived[ch] = CreateEvent(NULL, false, false, NULL);
      }

      for (int i = 0; i < MAX_TRIG_COUNT; ++i) {
         m_triggers[i].setOwner(this, i);
      }

      m_record.setOwner(this);
      m_sampleClkSrc = SignalNone;
   }

   virtual ~BfdPwMeterCtrlImpl() 
   { 
      for(int8 ch = 0; ch < MAX_CNTR_CH_COUNT; ch++) {
         if(m_dataArrived[ch]) {
            CloseHandle(m_dataArrived[ch]);
         }
      }

      // clear all user event delegate
      for (unsigned i = 0; i < ARRAY_SIZE(bfd_ci_events); ++i) {
         UserEventManager::Remove(getOuter(), bfd_ci_events[i]);
      }
   }

protected:
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual CounterCapability WantedCapability() { return BufferedPwmIn; }

   // update the control state
   virtual void RefreshState(ControlState &curr)
   {
      if (curr == Running) {
         EventStatus es = {sizeof(es)};
         uint32 chEnabled = m_chEnabled;
         for (int cntr = 0; cntr < getFeatures()->getChannelCountMax(); ++cntr, chEnabled >>= 1) {
            if(chEnabled & 0x1) {

               ModEvent().GetStatus(EventId(EvtCntrStopped0 + cntr), es);
               if (!es.CntrStoped.Stopped) {
                  return;
               }
            }
         }

         setState(Stopped);
         m_enabled = false;
      }
   }

   // initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = CntrCtrlBaseImpl::InitializeControl(x);
      if (ret != Success) { return ret; }

      m_channels.Clear();

      LoadProperties();
      return Success;
   }

   virtual void DeinitializeControl()
   {
      m_canFire = false;
      Release();

      // deactivate the events     
      const int32 chMax = getFeatures()->getChannelCountMax();
      for (unsigned evt_idx = 0; evt_idx < ARRAY_SIZE(bfd_ci_events); ++evt_idx) {
         for (int cntr = 0; cntr < chMax; ++cntr) {
            UnregisterKernEvent(EventId(bfd_ci_events[evt_idx] + cntr));
         }
      }

      for(int8 ch = 0; ch < getChannels()->getLength(); ch++) {
         UnregisterKernEvent(EventId(EvtCiDataReady0 + ch), DN4_ProcessKernEvent);
         UnregisterKernEvent(EventId(EvtCntrStopped0 + ch), DN4_ProcessKernEvent);
      }

      m_dn4EventRegistered = false;
      memset(m_dataReadyCleared, 0, sizeof(m_dataReadyCleared));

      CntrCtrlBaseImpl::DeinitializeControl();
   }

   virtual ErrorCode StartCounter(bool start) // for start and stop operation
   {
      if(start) {
         m_canFire = false;
         ErrorCode ret = Prepare();
         CHK_RESULT(ret);

         uint32 chEnabled = m_chEnabled;
         const int32 chMax = getFeatures()->getChannelCountMax();

         //
         // Rocky changed the following loop to:
         // assign the same kind events to different thread evenly
         for (unsigned evt_idx = 0; evt_idx < ARRAY_SIZE(bfd_ci_events); ++evt_idx) {
            for (int cntr = 0; cntr < chMax; ++cntr) {
               // enable counter
               if(chEnabled & (0x1 << cntr)) { 
                  if(UserEventManager::IsExist(getOuter(), bfd_ci_events[evt_idx])) { 
                     EnableKernEvent(EventId(bfd_ci_events[evt_idx] + cntr), cntr);       
                  }
               } else { // disable counter
                  UnregisterKernEvent(EventId(bfd_ci_events[evt_idx] + cntr));
               }
            }
         }

         // for blocked 'GetData'
         if(!m_dn4EventRegistered) {
            m_dn4EventRegistered = true;
            for (int cntr = 0; cntr < chMax; ++cntr) {
               if(chEnabled & (0x1 << cntr)) { 
                  RegisterKernEvent(EventId(EvtCiDataReady + cntr), DN4_ProcessKernEvent);
                  RegisterKernEvent(EventId(EvtCntrStopped + cntr), DN4_ProcessKernEvent);
               }
            }
         }

         // start buffered CNTR
         m_canFire = true;
         ret = ModuleRef().BfdCtStart();
         if(ret == Success) {setState(Running);}

         return ret;
      }

      if(IsIdle()) { return Success; }

      setState(Stopped);
      return ModuleRef().BfdCtStop();
   }

   // -----------------------------------------------------------------
   // Check whether the event id is supported by a counter control or not
   // -----------------------------------------------------------------
   virtual bool IsValidEvent(EventId id)
   {
      return ARRAY_INDEX_OF(bfd_ci_events, id) != -1;
   }

   // -----------------------------------------------------------------
   // driver event receiver for buffered Counter IN
   // -----------------------------------------------------------------
   virtual void OnKernEvent(EventId id)
   {
      // Discard the un-expected events
      if (IsIdle()) { return; }

      EventStatus      es   = {sizeof(es)};
      BfdCntrEventArgs args = { id };

      switch(id)
      {
      default:
         break;
      case EvtCiDataReady0:  /* data ready */
      case EvtCiDataReady1:
      case EvtCiDataReady2:
      case EvtCiDataReady3:
      case EvtCiDataReady4:
      case EvtCiDataReady5:
      case EvtCiDataReady6:
      case EvtCiDataReady7:
         m_dataReadyCleared[id - EvtCiDataReady0] = false;

         ModEvent().GetStatus(id, es); 
         args.Count  = es.General.lParam;
         if (args.Count && m_canFire) {
            args.Channel = id - EvtCiDataReady;
            args.Offset  = es.General.rParam;

            UserEventManager::Fire(getOuter(), EvtCiDataReady, &args);
         }
         // clear the event flag if the used hadn't clear it by calling 'GetData'.
         // using a count '0' to clear only the 'had sent' flag, but not effect the 'read pos'.
         if (!m_dataReadyCleared[id - EvtCiDataReady0]) { ModEvent().ClearFlag(id); }
         break;
      case EvtCiOverrun0:  /* overrun */
      case EvtCiOverrun1:
      case EvtCiOverrun2:
      case EvtCiOverrun3:
      case EvtCiOverrun4:
      case EvtCiOverrun5:
      case EvtCiOverrun6:
      case EvtCiOverrun7:
         if(m_canFire) {
            ModEvent().GetStatus(id, es);
            args.Channel = id - EvtCiOverrun;
            args.Count   = es.General.lParam;
            args.Offset  = es.General.rParam;

            UserEventManager::Fire(getOuter(), EvtCiOverrun, &args);
            ModEvent().ClearFlag(id);            // clear the event flag after the user had processed it.
         }
         break;
      case EvtCntrStopped0:    /* stopped */ 
      case EvtCntrStopped1:
      case EvtCntrStopped2:
      case EvtCntrStopped3:
      case EvtCntrStopped4:
      case EvtCntrStopped5:
      case EvtCntrStopped6:
      case EvtCntrStopped7:
         if(m_canFire) {
            ModEvent().GetStatus(id, es);
            args.Channel = id - EvtCntrStopped;
            args.Count   = es.CntrStoped.DataCount;
            args.Offset  = es.CntrStoped.DataOffset; 

            UserEventManager::Fire(getOuter(), EvtCntrStopped, &args);
         }
         // need not clear the event flag
         break;
      case EvtCiCacheOverflow0:  /* cache overflow */
      case EvtCiCacheOverflow1:
      case EvtCiCacheOverflow2:
      case EvtCiCacheOverflow3:
      case EvtCiCacheOverflow4:
      case EvtCiCacheOverflow5:
      case EvtCiCacheOverflow6:
      case EvtCiCacheOverflow7:
         if(m_canFire) {
            ModEvent().GetStatus(id, es);
            args.Channel = id - EvtCiCacheOverflow;

            UserEventManager::Fire(getOuter(), EvtCiCacheOverflow, &args);
            ModEvent().ClearFlag(id);            // clear the event flag after the user had processed it.
         }
         break;
      }
   }

   ErrorCode EnableKernEvent(int eventId, int eventSrc)
   {
      EventId id = (EventId)eventId;

      // try to register the event into driver before we start the snapping
      ErrorCode ret = RegisterKernEvent(id);
      if (ret != Success) { return ret; }

      // clear the flag of un-expected event signaled during we are starting DI snap.
      ModEvent().ClearFlag(id);
      return ret;
   }

   virtual void ReloadProperties()
   {
      CntrCtrlBaseImpl::ReloadProperties();
      LoadProperties();
   }

   void LoadProperties()
   {
      ModProp().Get(CFG_ScanChannelStart, m_chStart);
      ModProp().Get(CFG_ScanChannelCount, m_chCount);
      ModProp().Get(CFG_ScanEnabledChannels, m_chEnabled);

      ModProp().GetValueNth(CFG_SamplingClockSources, m_chStart, m_sampleClkSrc);

      int32 sectLength = 0;
      int32 sectCount  = 0;
      ModProp().GetValueNth(CFG_RecordSectionLengthes, m_chStart, sectLength);
      ModProp().GetValueNth(CFG_RecordSectionCounts, m_chStart, sectCount);

      m_record.setSectionLength(sectLength);
      m_record.setSectionCount(sectCount);

      int32 index = 0;
      ArrayImpl<SignalDrop>* clkSrcs = ModProp().GetValueArrayI32<SignalDrop>(PropertyId(CFG_FeaturePiSampleClkSourceOfCounter0 + m_chStart));
      for(index = 0; index < clkSrcs->getLength(); index++) {
         if(m_sampleClkSrc == clkSrcs->Item(index)) {
            break;
         }
      }

      if(index == clkSrcs->getLength()) {
         m_sampleClkSrc = clkSrcs->Item(0);
      }
   }

   //------------------------------------------------------------------------
   // internal methods
   //------------------------------------------------------------------------
   // DN4 event receiver for blocked 'GetData'
   static void BDAQCALL DN4_ProcessKernEvent(EventId id, void *param)
   {
      BfdPwMeterCtrlImpl *_this = (BfdPwMeterCtrlImpl *)param;

      if(EvtCiDataReady0 <= id && id <= EvtCiDataReady7) {
         SetEvent(_this->m_dataArrived[id - EvtCiDataReady0]);
      } else if(EvtCntrStopped0 <= id && id <= EvtCntrStopped7) {
         SetEvent(_this->m_dataArrived[id - EvtCntrStopped0]);
      }
   }

   static uint32 WaitEventTimeout(HANDLE ev, uint32 timeout)
   {
      uint32 tick = GetTickCount();
      WaitForMultipleObjects(1, &ev, false, timeout);
      uint32 elapsed = GetTickCount() - tick;
      if (timeout != (uint32)INFINITE) { 
         timeout = timeout > elapsed ? timeout - elapsed : 0;
      }
      return timeout;
   }

private:
   uint32 CalculateChEnabled()
   {
      uint32 ch_start, ch_count, ch_enabled = 0;

      ch_start = m_chStart;
      ch_count = m_chCount;

      while (ch_count--) {
         ch_enabled |= 1 << ch_start;
         ch_start = (ch_start + 1) % getFeatures()->getChannelCountMax();
      }

      return ch_enabled;
   }

   ErrorCode UpdateChEnabled()
   {
      int32 count = MAX_CNTR_CH_COUNT;
      uint8 data[MAX_CNTR_CH_COUNT] = {0};

      CHK_USER_BUF_LEN(data, count >= getFeatures()->getChannelCountMax());
      ErrorCode ret = ModProp().Read(CFG_ScanEnabledChannels, count, data);
      CHK_EXPR(Success == ret, ret);

      m_chEnabled= 0;
      for (int i = 0; i < getFeatures()->getChannelCountMax(); ++i) {
         if (((uint8*)data)[i] == 0) { continue; }

         m_chEnabled |= 1 << i;
      }

      return Success;
   }

private:
   ArrayImpl<PiChannelImpl> m_channels;
   BfdCntrRecordImpl        m_record;
   BfdCntrTriggerImpl       m_triggers[MAX_TRIG_COUNT];
   SignalDrop               m_sampleClkSrc;

   bool          m_dn4EventRegistered;
   bool          m_dataReadyCleared[MAX_CNTR_CH_COUNT];
   HANDLE        m_dataArrived[MAX_CNTR_CH_COUNT];

   bool          m_canFire;
   int32         m_chEnabled;
   int32         m_dataBufLen;
};

#endif
