#ifndef _DAQNAVI_BFD_PULSE_WIDTH_MODULATOR_CTRL_IMPL_H
#define _DAQNAVI_BFD_PULSE_WIDTH_MODULATOR_CTRL_IMPL_H

class BfdPwModulatorCtrlImpl : public CntrCtrlBaseImpl
{

public:
   // Method
   ErrorCode SetData(int32 cntr, uint32 count, void *buffer)
   {
      ErrorCode ret;
      if (getState() == Idle || getState() == Stopped) { 
         ret = Prepare();
         if (ret != Success) { return ret; }
      }

      AutoLock<MutexLock> lock(BuffLock());

      ret = ModuleRef().BfdCtSetData(cntr, buffer, count);
      if(ret != Success) { return ret; }
      // Update the 'Write Position'
      m_dataTransedCleared[cntr] = true;
      ModEvent().ClearFlag(EventId(EvtCoDataTransmitted0 + cntr), count);

      return Success;
   }

   TriggerImpl* getTrigger(int32 trigger)
   {
      TriggerImpl *x = NULL;

      // Bug#14478
      //if ((uint32)trigger < (uint32)getFeatures()->getTriggerCount()) {
      if ((uint32)trigger < ARRAY_SIZE(m_triggers)) {
         x = &m_triggers[trigger];
      }

      return x;
   }

   int32 getSamples()
   {
      return m_samplesCount;
   }

   ErrorCode setSamples(int32 value)
   {
      if(IsRunning()) { return ErrorFuncBusy; }
      m_samplesCount = value;

      return Success;
   }

   int32 getBufferCapacity()
   {
      return m_dataBufLen; 
   }

   int32 getIntervalCount()
   {
      ModProp().GetValueNth<int32>(CFG_RecordSectionLengthes, m_chStart, m_intervalCount);

      return m_intervalCount;
   }

   ErrorCode setIntervalCount(int32 value)
   {
      if (IsRunning()) { return ErrorFuncBusy; }

      ErrorCode ret = ModProp().SetValueNth<int32>(CFG_RecordSectionLengthes, m_chStart, value);
      CHK_RESULT(ret);

      m_intervalCount = value;
      return Success;
   }

   bool getStreaming() 
   { 
      return m_sectionCount ? false : true; 
   }

   bool getBufferStatus(int32 ch, int32 & count, int32 & offset)
   {
      EventStatus es = {sizeof(es)};
      ModEvent().GetStatus(EventId(EvtCoDataTransmitted0 + ch), es);
      count  = es.General.lParam;
      offset = es.General.rParam;

      return false;
   }

   ErrorCode setStreaming(bool value)
   {
      if(IsRunning()) { return ErrorFuncBusy; }

      m_sectionCount = (value ? 0 : 1);
      return Success;
   }

   ErrorCode setSampleClkSrc(SignalDrop value)
   {
      if (IsRunning()) {
         return ErrorFuncBusy;
      }

      ArrayImpl<SignalDrop>* clkSrcs = ModProp().GetValueArrayI32<SignalDrop>(PropertyId(CFG_FeaturePoSampleClkSourceOfCounter0 + m_chStart));
      for(int32 i = 0; i < clkSrcs->getLength(); i++) {

         if(value == clkSrcs->Item(i)) {
            m_sampleClkSrc = value;
            
            return ModProp().SetValueNth<int32>(CFG_SamplingClockSources, m_chStart, m_sampleClkSrc);
         }
      }

      return ErrorPropValueNotSpted;
   }

   SignalDrop getSampleClkSrc()
   {
      ModProp().GetValueNth<SignalDrop>(CFG_SamplingClockSources, m_chStart, m_sampleClkSrc);

      return m_sampleClkSrc;
   }

   void getChannelMap(int32 count, int8 *data)
   {
      ModProp().Read(CFG_ScanEnabledChannels, count, data);
   }

   ErrorCode setChannelMap(int32 count, int8 *data)
   {
      if (!CanEditProperty()) { return ErrorPrivilegeNotHeld; }
      if (IsRunning())        { return ErrorFuncBusy;         }

      CHK_USER_BUF_LEN(data, count >= getFeatures()->getChannelCountMax());

      ErrorCode ret = ModProp().Set(CFG_ScanEnabledChannels, count, (uint8 *)data);

      int ch_start = 0; // update channelStart and channelCount
      int ch_count = 0; 
      
      m_chEnabled= 0;
      for (int i = 0; i < getFeatures()->getChannelCountMax(); ++i) {
         if (((uint8*)data)[i] == 0) { continue; }

         m_chEnabled |= 1 << i;

         ++ch_count;
         if (ch_count == 1) { ch_start = i; }
      }

      if (m_chEnabled) {
         m_chStart = ch_start;
         m_chCount = ch_count;
      }

      return ret;
   }

   virtual ErrorCode setChannelStart(int32 ch)
   {
      if (!getInitialized()) { return ErrorFuncNotInited; }
      if(IsRunning())        { return ErrorFuncBusy; }

      ErrorCode warning = Success;
      warning = ModProp().Set(CFG_ScanChannelStart, ch);
      ModProp().Get(CFG_ScanChannelStart, m_chStart);

      m_chEnabled = CalculateChEnabled();

      return warning;
   }

   virtual ErrorCode setChannelCount(int32 count)
   {
      if (!getInitialized()) { return ErrorFuncNotInited; }
      if(IsRunning())        { return ErrorFuncBusy; }

      ErrorCode warning = Success;
      warning = ModProp().Set(CFG_ScanChannelCount, count);
      ModProp().Get(CFG_ScanChannelCount, m_chCount);

      m_chEnabled = CalculateChEnabled();

      return warning;
   }

   ErrorCode Prepare()
   {
      AutoLock<MutexLock> lock(PropLock());

      ErrorCode ret;
      if(!getInitialized()) {
         ret = ErrorFuncNotInited;
      } else if(IsRunning()) {
         ret = ErrorFuncBusy;
      } else if(getState() == Ready) {
         ret = Success;
      } else {
         ret = UpdateChEnabled();
         CHK_EXPR(ret == Success, ret);

         uint32 chEnabled = m_chEnabled;
         for (int cntr = 0; cntr < getFeatures()->getChannelCountMax(); ++cntr, chEnabled >>= 1) {
            if(chEnabled & 0x1) { // enable counter
               ret = ModProp().SetValueNth<uint32>(CFG_BufferedModeOfCounters, cntr, WantedCapability());
               CHK_RESULT(ret);

               ret = ModProp().SetValueNth<int32>(CFG_RecordSectionLengthes,  cntr, m_intervalCount);
               CHK_RESULT(ret);

               ret = ModProp().SetValueNth<int32>(CFG_RecordSectionCounts,    cntr, m_sectionCount);
               CHK_RESULT(ret);

               ret = ModProp().SetValueNth<int32>(CFG_SamplingClockSources,   cntr, m_sampleClkSrc);
               CHK_RESULT(ret);

               int32 count = getFeatures()->getTriggerCount();
               for (int32 i = 0; i < count; i++) {
                  ret = ModProp().SetValueNth<int32>(CFG_TriggerSources, cntr, m_triggers[i].getSource());
                  CHK_RESULT(ret);

                  ret = ModProp().SetValueNth<double>(CFG_TriggerSourcesLevel, cntr, m_triggers[i].getLevel());
                  CHK_RESULT(ret);

                  ret = ModProp().SetValueNth<int32>(CFG_TriggerSourcesEdge, cntr, m_triggers[i].getEdge());
                  CHK_RESULT(ret);

                  ret = ModProp().SetValueNth<double>(CFG_TriggerSourcesHysteresisIndex, cntr, m_triggers[i].getHysteresisIndex());
                  CHK_RESULT(ret);
               }
            }
         }

         m_dataBufLen = m_samplesCount;
         ret = ModuleRef().BfdCtPrepare((uint32 &)m_dataBufLen);
         if(ret == Success) { // adjust the section length.
            ret = ModProp().GetValueNth<int32>(CFG_RecordSectionLengthes, m_chStart, m_intervalCount);
            CHK_RESULT(ret);
         }

         setState(ret == Success ? Ready : Idle);
      }

      return ret;
   }

   void Release()
   {
      // Fix bug#19706: Call driver 'Release' function anyway to release resource, if any.
      // It is harmless operation even if the device is in 'FAI IDLE' state.
      // if (IsIdle()) { return; }
      // Call driver 'Release' only if we had called the method 'Prepare'
      if (m_dataBufLen) {
         ModuleRef().BfdCtRelease();
      }

      setState(Idle);
      m_dataBufLen = 0;
   }

   // Property
   ArrayImpl<PoChannelImpl>* getChannels()
   {
      if (getInitialized()) {
         if (m_channels.getLength() == 0) {
            int count = getFeatures()->getChannelCountMax();
            m_channels.setCapacity(count);

            for (int i = 0; i < count; ++i) {
               m_channels.Add(PoChannelImpl(this, i));
            }
         }
      }

      return &m_channels;
   }

   // event
   virtual void addEventHandler(EventId id, DaqEventProc proc, void * param)
   {
      if(!IsValidEvent(id)) {
         CntrCtrlBaseImpl::addEventHandler(id, proc, param);
         return;
      }

      UserEventManager::Add(getOuter(), id, (UserEventProc)proc, param);
   }

   virtual void removeEventHandler(EventId id, DaqEventProc proc, void * param)
   {
      if(!IsValidEvent(id)) {
         CntrCtrlBaseImpl::addEventHandler(id, proc, param);
         return;
      }

      UserEventManager::Add(getOuter(), id, (UserEventProc)proc, param);
   }

public: // ctor & dtor
   BfdPwModulatorCtrlImpl() : m_channels(false), m_sampleClkSrc(SignalNone), m_canFire(false), m_dataBufLen(0)
   { 
      memset(m_dataTransedCleared, 0, sizeof(m_dataTransedCleared));
      for (int i = 0; i < MAX_TRIG_COUNT; ++i) {
         m_triggers[i].setOwner(this, i);
      }
   }

   virtual ~BfdPwModulatorCtrlImpl() 
   {
      // clear all user event delegate
      for (unsigned i = 0; i < ARRAY_SIZE(bfd_co_events); ++i) {
         UserEventManager::Remove(getOuter(), bfd_co_events[i]);
      }
   }

protected:
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual CounterCapability WantedCapability() { return BufferedPwmOut; }

   // update the control state
   virtual void RefreshState(ControlState &curr)
   {
      if (curr == Running) {
         EventStatus es = {sizeof(es)};
         uint32 chEnabled = m_chEnabled;
         for (int cntr = 0; cntr < getFeatures()->getChannelCountMax(); ++cntr, chEnabled >>= 1) {
            if(chEnabled & 0x1) {

               ModEvent().GetStatus(EventId(EvtCntrStopped0 + cntr), es);
               if (!es.General.lParam) {
                  return;
               }
            }
         }

         setState(Stopped);
         m_enabled = false;
      }
   }

   // initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = CntrCtrlBaseImpl::InitializeControl(x);
      if (ret != Success) { return ret; }

      m_channels.Clear();

      LoadProperties();
      return Success;
   }

   virtual void DeinitializeControl()
   {
      m_canFire = false;
      Release();

      // deactivate the events     
      const int32 chMax = getFeatures()->getChannelCountMax();
      for (unsigned evt_idx = 0; evt_idx < ARRAY_SIZE(bfd_co_events); ++evt_idx) {
         for (int cntr = 0; cntr < chMax; ++cntr) {
            UnregisterKernEvent(EventId(bfd_co_events[evt_idx] + cntr));
         }
      }

      CntrCtrlBaseImpl::DeinitializeControl();
   }

   virtual ErrorCode StartCounter(bool start) // for start and stop operation
   {
      if(start) {
         ErrorCode ret = Prepare();
         CHK_RESULT(ret);

         m_canFire         = false;
         uint32 chEnabled  = m_chEnabled;
         const int32 chMax = getFeatures()->getChannelCountMax();

         //
         // Rocky changed the following loop to:
         // assign the same kind events to different thread evenly
         for (unsigned evt_idx = 0; evt_idx < ARRAY_SIZE(bfd_co_events); ++evt_idx) {
            for (int cntr = 0; cntr < chMax; ++cntr) {
               // enable counter
               if(chEnabled & (0x1 << cntr)) { 
                  if(UserEventManager::IsExist(getOuter(), bfd_co_events[evt_idx])) { 
                     EnableKernEvent(EventId(bfd_co_events[evt_idx] + cntr), cntr);       
                  }
               } else { // disable counter
                  UnregisterKernEvent(EventId(bfd_co_events[evt_idx] + cntr));
               }
            }
         }

         // start buffered CNTR
         ret = ModuleRef().BfdCtStart();
         if(ret == Success) {setState(Running);}

         m_canFire = true;
         return ret;
      }

      if(IsIdle()) { return Success; }
      setState(Stopped);

      return ModuleRef().BfdCtStop();
   }

   // -----------------------------------------------------------------
   // Check whether the event id is supported by a counter control or not
   // -----------------------------------------------------------------
   virtual bool IsValidEvent(EventId id)
   {
      return ARRAY_INDEX_OF(bfd_co_events, id) != -1;
   }

   // -----------------------------------------------------------------
   // driver event receiver for buffered Counter IN
   // -----------------------------------------------------------------
   virtual void OnKernEvent(EventId id)
   {
      // Discard the un-expected events
      if (IsIdle()) { return; }

      EventStatus      es   = {sizeof(es)};
      BfdCntrEventArgs args = { id };

      switch(id)
      {
      default:
         break;
      case EvtCoDataTransmitted0:  /* data transmitted */
      case EvtCoDataTransmitted1:
      case EvtCoDataTransmitted2:
      case EvtCoDataTransmitted3:
      case EvtCoDataTransmitted4:
      case EvtCoDataTransmitted5:
      case EvtCoDataTransmitted6:
      case EvtCoDataTransmitted7:
         m_dataTransedCleared[id - EvtCoDataTransmitted0] = false;

         ModEvent().GetStatus(id, es); 
         args.Count  = es.General.lParam;
         if (args.Count && m_canFire) {
            args.Channel = id - EvtCoDataTransmitted;
            args.Offset  = es.General.rParam;

            UserEventManager::Fire(getOuter(), EvtCoDataTransmitted, &args); 
         }
         // clear the event flag if the used hadn't clear it by calling 'GetData'.
         // using a count '0' to clear only the 'had sent' flag, but not effect the 'read pos'.
         if (!m_dataTransedCleared[id - EvtCoDataTransmitted0]) { ModEvent().ClearFlag(id); }
         break;
      case EvtCoUnderrun0:  /* under-run */
      case EvtCoUnderrun1:
      case EvtCoUnderrun2:
      case EvtCoUnderrun3:
      case EvtCoUnderrun4:
      case EvtCoUnderrun5:
      case EvtCoUnderrun6:
      case EvtCoUnderrun7:
         if(m_canFire) {
            ModEvent().GetStatus(id, es);
            args.Channel = id - EvtCoUnderrun;
            args.Count   = es.General.lParam;
            args.Offset  = es.General.rParam;

            UserEventManager::Fire(getOuter(), EvtCoUnderrun, &args); 
            ModEvent().ClearFlag(id);            // clear the event flag after the user had processed it.
         }
         break;
      case EvtCoCacheEmptied0:    /* stopped */ 
      case EvtCoCacheEmptied1:
      case EvtCoCacheEmptied2:
      case EvtCoCacheEmptied3:
      case EvtCoCacheEmptied4:
      case EvtCoCacheEmptied5:
      case EvtCoCacheEmptied6:
      case EvtCoCacheEmptied7:
         if(m_canFire) {
            ModEvent().GetStatus(id, es);
            args.Channel = id - EvtCoCacheEmptied;
            args.Count   = es.General.lParam;
            args.Offset  = es.General.rParam; 

            UserEventManager::Fire(getOuter(), EvtCoCacheEmptied, &args); 
            ModEvent().ClearFlag(id);
         }
         break;
      case EvtCoTransStopped0:  /* cache overflow */
      case EvtCoTransStopped1:
      case EvtCoTransStopped2:
      case EvtCoTransStopped3:
      case EvtCoTransStopped4:
      case EvtCoTransStopped5:
      case EvtCoTransStopped6:
      case EvtCoTransStopped7:
         if(m_canFire) {
            uint32 ch = id - EvtCoTransStopped0;
            ModEvent().GetStatus(EventId(EvtCoDataTransmitted0 + ch), es);

            args.Channel = ch;
            args.Count   = es.General.lParam;
            args.Offset  = es.General.rParam; 

            UserEventManager::Fire(getOuter(), EvtCoTransStopped, &args); 
            // need not clear the event flag
         }
         break;
      case EvtCntrStopped0:
      case EvtCntrStopped1:
      case EvtCntrStopped2:
      case EvtCntrStopped3:
      case EvtCntrStopped4:
      case EvtCntrStopped5:
      case EvtCntrStopped6:
      case EvtCntrStopped7:
         if(m_canFire) {
            uint32 ch = id - EvtCntrStopped0;
            ModEvent().GetStatus(EventId(EvtCoDataTransmitted0 + ch), es);

            args.Channel = ch;
            args.Count   = es.General.lParam;
            args.Offset  = es.General.rParam; 

            UserEventManager::Fire(getOuter(), EvtCntrStopped, &args); 
            // need not clear the event flag
         }
         break;
      }
   }

   ErrorCode EnableKernEvent(int eventId, int eventSrc)
   {
      EventId id = (EventId)eventId;

      // try to register the event into driver before we start the snapping
      ErrorCode ret = RegisterKernEvent(id);
      if (ret != Success) { return ret; }

      // clear the flag of un-expected event signaled during we are starting DI snap.
      ModEvent().ClearFlag(id);
      return ret;
   }

   virtual void ReloadProperties()
   {
      CntrCtrlBaseImpl::ReloadProperties();
      LoadProperties();
   }

   void LoadProperties()
   {
      ModProp().Get(CFG_ScanChannelStart, m_chStart);
      ModProp().Get(CFG_ScanChannelCount, m_chCount);
      ModProp().Get(CFG_ScanEnabledChannels, m_chEnabled);
      ModProp().GetValueNth<int32>(CFG_RecordSectionLengthes,  m_chStart, m_intervalCount);

      ModProp().GetValueNth(CFG_SamplingClockSources, m_chStart, m_sampleClkSrc);

      int32 index = 0;
      ArrayImpl<SignalDrop>* clkSrcs = ModProp().GetValueArrayI32<SignalDrop>(PropertyId(CFG_FeaturePoSampleClkSourceOfCounter0 + m_chStart));
      for(index = 0; index < clkSrcs->getLength(); index++) {
         if(m_sampleClkSrc == clkSrcs->Item(index)) {
            break;
         }
      }

      if(index == clkSrcs->getLength()) {
         m_sampleClkSrc = clkSrcs->Item(0);
      }

      m_sectionCount  = 1;
      m_samplesCount  = m_intervalCount * 2;
   }

private:
   uint32 CalculateChEnabled()
   {
      uint32 ch_start, ch_count, ch_enabled = 0;

      ch_start = m_chStart;
      ch_count = m_chCount;

      while (ch_count--) {
         ch_enabled |= 1 << ch_start;
         ch_start = (ch_start + 1) % getFeatures()->getChannelCountMax();
      }

      return ch_enabled;
   }

   ErrorCode UpdateChEnabled()
   {
      int32 count = MAX_CNTR_CH_COUNT;
      uint8 data[MAX_CNTR_CH_COUNT] = {0};

      CHK_USER_BUF_LEN(data, count >= getFeatures()->getChannelCountMax());
      ErrorCode ret = ModProp().Read(CFG_ScanEnabledChannels, count, data);
      CHK_EXPR(Success == ret, ret);

      m_chEnabled= 0;
      for (int i = 0; i < getFeatures()->getChannelCountMax(); ++i) {
         if (((uint8*)data)[i] == 0) { continue; }

         m_chEnabled |= 1 << i;
      }

      return Success;
   }

private:
   ArrayImpl<PoChannelImpl> m_channels;
   BfdCntrTriggerImpl       m_triggers[MAX_TRIG_COUNT];
   SignalDrop               m_sampleClkSrc;

   bool          m_dataTransedCleared[MAX_CNTR_CH_COUNT];
   bool          m_canFire;

   int32         m_intervalCount;
   int32         m_samplesCount;
   int32         m_sectionCount;

   int32         m_chEnabled;
   int32         m_dataBufLen;
};

#endif
