#ifndef _DAQNAVI_CNTR_CHANNEL_IMPL_H
#define _DAQNAVI_CNTR_CHANNEL_IMPL_H

class CntrCtrlBaseImpl;

class EcChannelImpl
{
public:
   // Common Property
   int32     getChannel();
   bool      getNoiseFiltered();
   ErrorCode setNoiseFiltered(bool value);

   // Event counting Property
   SignalDrop     getClockSource();
   ErrorCode      setClockSource(SignalDrop value);
   SignalPolarity getClockPolarity();
   ErrorCode      setClockPolarity(SignalPolarity value);
   SignalPolarity getGatePolarity();
   ErrorCode      setGatePolarity(SignalPolarity value);
   bool           getGated();
   ErrorCode      setGated(bool value);

public:
   EcChannelImpl(CntrCtrlBaseImpl * owner = NULL, int tag = -1)
   {
      m_owner = owner;
      m_tag   = tag;
   }

   void setOwner(CntrCtrlBaseImpl * owner, int tag) 
   {
      m_owner = owner;
      m_tag   = tag;
   }

protected:
   CntrCtrlBaseImpl *m_owner;
   int32            m_tag;
};

class FmChannelImpl
{
public:
   // Common Property
   int32     getChannel();
   bool      getNoiseFiltered();
   ErrorCode setNoiseFiltered(bool value);

   // Frequency measurement Property
   FreqMeasureMethod getFmMethod();
   ErrorCode         setFmMethod(FreqMeasureMethod value);
   double            getCollectionPeriod();
   ErrorCode         setCollectionPeriod(double value);

public:
   FmChannelImpl(CntrCtrlBaseImpl * owner = NULL, int tag = -1)
   {
      m_owner = owner;
      m_tag   = tag;
   }

   void setOwner(CntrCtrlBaseImpl * owner, int tag) 
   {
      m_owner = owner;
      m_tag   = tag;
   }

protected:
   CntrCtrlBaseImpl *m_owner;
   int32            m_tag;
};

class OsChannelImpl
{
public:
   // Common
   int32     getChannel();
   bool      getNoiseFiltered();
   ErrorCode setNoiseFiltered(bool value);

   // One-shot Property
   int32          getDelayCount();
   ErrorCode      setDelayCount(int32 value);
   SignalDrop     getClockSource();
   ErrorCode      setClockSource(SignalDrop value);
   SignalPolarity getClockPolarity();
   ErrorCode      setClockPolarity(SignalPolarity value);
   SignalDrop     getGateSource();
   ErrorCode      setGateSource(SignalDrop value);
   SignalPolarity getGatePolarity();
   ErrorCode      setGatePolarity(SignalPolarity value);
   OutSignalType  getOutSignal();
   ErrorCode      setOutSignal(OutSignalType value);

public:
   OsChannelImpl(CntrCtrlBaseImpl * owner = NULL, int tag = -1)
   {
      m_owner = owner;
      m_tag   = tag;
   }

   void setOwner(CntrCtrlBaseImpl * owner, int tag) 
   {
      m_owner = owner;
      m_tag   = tag;
   }

protected:
   CntrCtrlBaseImpl *m_owner;
   int32            m_tag;
};

class TmrChannelImpl
{
public:
   // Common
   int32     getChannel();
   bool      getNoiseFiltered();
   ErrorCode setNoiseFiltered(bool value);

   // Timer/Pulse Property
   double         getFrequency();
   ErrorCode      setFrequency(double value);
   bool           getGated();
   ErrorCode      setGated(bool value);
   SignalPolarity getGatePolarity();
   ErrorCode      setGatePolarity(SignalPolarity value);
   OutSignalType  getOutSignal();
   ErrorCode      setOutSignal(OutSignalType value);

public:
   TmrChannelImpl(CntrCtrlBaseImpl * owner = NULL, int tag = -1)
   {
      m_owner = owner;
      m_tag   = tag;
   }

   void setOwner(CntrCtrlBaseImpl * owner, int tag) 
   {
      m_owner = owner;
      m_tag   = tag;
   }

protected:
   CntrCtrlBaseImpl *m_owner;
   int32            m_tag;
};

class PiChannelImpl
{
public:
   // Common
   int32     getChannel();
   bool      getNoiseFiltered();
   ErrorCode setNoiseFiltered(bool value);

public:
   PiChannelImpl(CntrCtrlBaseImpl * owner = NULL, int tag = -1)
   {
      m_owner = owner;
      m_tag   = tag;
   }

   void setOwner(CntrCtrlBaseImpl * owner, int tag) 
   {
      m_owner = owner;
      m_tag   = tag;
   }

protected:
   CntrCtrlBaseImpl *m_owner;
   int32            m_tag;
};

class PoChannelImpl
{
public:
   // Common
   int32          getChannel();
   bool           getNoiseFiltered();
   ErrorCode      setNoiseFiltered(bool value);

   // Pulse Width Modulation Property
   void           getPulseWidth(PulseWidth &width);
   ErrorCode      setPulseWidth(PulseWidth const &width);
   bool           getGated();
   ErrorCode      setGated(bool value);
   SignalPolarity getGatePolarity();
   ErrorCode      setGatePolarity(SignalPolarity value);
   OutSignalType  getOutSignal();
   ErrorCode      setOutSignal(OutSignalType value);
   int32          getOutCount();
   ErrorCode      setOutCount(int32 value);

   // new: trigger settings
   SignalDrop     getTriggerSource();
   ErrorCode      setTriggerSource(SignalDrop value);
   ActiveSignal   getTriggerEdge();
   ErrorCode      setTriggerEdge(ActiveSignal value);
   TriggerAction  getTriggerAction();
   ErrorCode      setTriggerAction(TriggerAction value);
   double         getTriggerDelayCount();
   ErrorCode      setTriggerDelayCount(double value);

public:
   PoChannelImpl(CntrCtrlBaseImpl * owner = NULL, int tag = -1)
   {
      m_owner = owner;
      m_tag   = tag;
   }

   void setOwner(CntrCtrlBaseImpl * owner, int tag) 
   {
      m_owner = owner;
      m_tag   = tag;
   }

protected:
   CntrCtrlBaseImpl *m_owner;
   int32            m_tag;
};

class UdChannelImpl
{
public:
   // Common
   int32     getChannel();
   bool      getNoiseFiltered();
   ErrorCode setNoiseFiltered(bool value);

   // Up-down Counter Property
   CountingType getCountingType();
   ErrorCode    setCountingType(CountingType value);
   int32        getInitialValue();
   ErrorCode    setInitialValue(int32 value);
   int32        getResetTimesByIndex();
   ErrorCode    setResetTimesByIndex(int32 value);

   // new properties for Counter continue comparing: outputting pulse settings
   void           getPulseWidth(PulseWidth &width);
   ErrorCode      setPulseWidth(PulseWidth const &width);
   bool           getGated();
   ErrorCode      setGated(bool value);
   SignalPolarity getGatePolarity();
   ErrorCode      setGatePolarity(SignalPolarity value);
   OutSignalType  getOutSignal();
   ErrorCode      setOutSignal(OutSignalType value);
   int32          getOutCount();
   ErrorCode      setOutCount(int32 value);

public:
   UdChannelImpl(CntrCtrlBaseImpl * owner = NULL, int tag = -1)
   {
      m_owner = owner;
      m_tag   = tag;
   }

   void setOwner(CntrCtrlBaseImpl * owner, int tag) 
   {
      m_owner = owner;
      m_tag   = tag;
   }

protected:
   CntrCtrlBaseImpl *m_owner;
   int32            m_tag;
};

#endif
