//
// Counter function implementation
//
#include <std_decl.h>

#include "cntr_features.h"
#include "cntr_channel.h"
#include "cntr_ctrl_base.h"

//------------------------------------------------------------------
// Help MACRO used to get/set property and check error
//------------------------------------------------------------------
#undef  MOD_PROP
#define MOD_PROP (m_owner->ModProp())

#define SCE_EVCTR_MASK  (SceEventCounter | SceBufferedEventCounter)
#define SCE_PWMTR_MASK  (ScePwMeter      | SceBufferedPwMeter     )
#define SCE_PWMDL_MASK  (ScePwModulator  | SceBufferedPwModulator )
#define SCE_UDCTR_MASK  (SceUdCounter    | SceBufferedUdCounter   )

// -----------------------------------------------------------------------
// functions of CntrFeaturesImpl
//------------------------------------------------------------------------
// channel features
int32 CntrFeaturesImpl::getChannelCountMax()
{
   int32 count = 0;
   if (MOD_PROP.Get(CFG_FeatureChannelNumberMax, count) == Success) {
      count += 1; // convert from channel number to channel count
   }
   return count;
}
int32 CntrFeaturesImpl::getResolution()
{
   int32 res = 0;
   MOD_PROP.Get(CFG_FeatureResolutionInBit, res);
   return res;
}
int32 CntrFeaturesImpl::getDataSize()
{
   int32 size = 0;
   MOD_PROP.Get(CFG_FeatureDataSize, size);
   return size;
}
CounterCapabilityIndexerImpl * CntrFeaturesImpl::getCapabilities()
{
   return &m_capsIndexer;
}

// noise filter features
bool CntrFeaturesImpl::getNoiseFilterSupported()
{
   return MOD_PROP.GetLength(CFG_FeatureNoiseFilterOfChannels) != 0;
}
ArrayImpl<uint8>* CntrFeaturesImpl::getNoiseFilterOfChannels()
{
   return MOD_PROP.GetValueArrayI8(CFG_FeatureNoiseFilterOfChannels);
}
void CntrFeaturesImpl::getNoiseFilterBlockTimeRange(MathInterval &x) 
{
   if (MOD_PROP.Get(CFG_FeatureNoiseFilterBlockTimeRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// new: measurement timeout range
void CntrFeaturesImpl::getMeasurementTimeoutRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureMeasurementTimeoutRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// event counting features
CounterClockSourceIndexerImpl* CntrFeaturesImpl::getEcClockSources()
{
   if (!(m_scenarios & SCE_EVCTR_MASK)) return NULL;
   return &m_ecClkSrcIndexer;
}
ArrayImpl<SignalPolarity>* CntrFeaturesImpl::getEcClockPolarities()
{
   if (!(m_scenarios & SCE_EVCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<SignalPolarity>(CFG_FeatureEcClkPolarities);
}
ArrayImpl<int32>* CntrFeaturesImpl::getEcGateControlOfChannels()
{
   if (!(m_scenarios & SCE_EVCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<int32>(CFG_FeatureEcGateControlOfCounters);
}
ArrayImpl<SignalPolarity>* CntrFeaturesImpl::getEcGatePolarities()
{
   if (!(m_scenarios & SCE_EVCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<SignalPolarity>(CFG_FeatureEcGatePolarities);
}

// frequency measurement features
ArrayImpl<FreqMeasureMethod>* CntrFeaturesImpl::getFmMethods()
{
   if (!(m_scenarios & SceFreqMeter)) return NULL;
   return MOD_PROP.GetValueArrayI32<FreqMeasureMethod>(CFG_FeatureFmMethods);
}

// one-shot features
CounterClockSourceIndexerImpl* CntrFeaturesImpl::getOsClockSources()
{
   if (!(m_scenarios & SceOneShot)) return NULL;
   return &m_osClkSrcIndexer;
}
ArrayImpl<SignalPolarity>* CntrFeaturesImpl::getOsClockPolarities()
{
   if (!(m_scenarios & SceOneShot)) return NULL;
   return MOD_PROP.GetValueArrayI32<SignalPolarity>(CFG_FeatureOsClkPolarities);
}
CounterGateSourceIndexerImpl* CntrFeaturesImpl::getOsGateSources()
{
   if (!(m_scenarios & SceOneShot)) return NULL;
   return &m_osGateSrcIndexer;
}
ArrayImpl<SignalPolarity>* CntrFeaturesImpl::getOsGatePolarities()
{
   if (!(m_scenarios & SceOneShot)) return NULL;
   return MOD_PROP.GetValueArrayI32<SignalPolarity>(CFG_FeatureOsGatePolarities);
}
ArrayImpl<OutSignalType>* CntrFeaturesImpl::getOsOutSignals()
{
   if (!(m_scenarios & SceOneShot)) return NULL;
   return MOD_PROP.GetValueArrayI32<OutSignalType>(CFG_FeatureOsOutSignals);
}
void CntrFeaturesImpl::getOsDelayCountRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureOsDelayCountRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// Timer/pulse features
ArrayImpl<int32>* CntrFeaturesImpl::getTmrGateControlOfChannels()
{
   if (!(m_scenarios & SceTimerPulse)) return NULL;
   return MOD_PROP.GetValueArrayI32<int32>(CFG_FeatureTmrGateControlOfCounters);
}
ArrayImpl<SignalPolarity>* CntrFeaturesImpl::getTmrGatePolarities()
{
   if (!(m_scenarios & SceTimerPulse)) return NULL;
   return MOD_PROP.GetValueArrayI32<SignalPolarity>(CFG_FeatureTmrGatePolarities);
}
ArrayImpl<OutSignalType>* CntrFeaturesImpl::getTmrOutSignals()
{
   if (!(m_scenarios & SceTimerPulse)) return NULL;
   return MOD_PROP.GetValueArrayI32<OutSignalType>(CFG_FeatureTmrOutSignals);
}
void CntrFeaturesImpl::getTmrFrequencyRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureTmrFrequencyRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// pulse width measurement features
ArrayImpl<CounterCascadeGroup>* CntrFeaturesImpl::getPiCascadeGroup()
{
   if (!(m_scenarios & SCE_PWMTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<CounterCascadeGroup>(CFG_FeaturePiCascadeGroups);
}

// pulse width modulation features
ArrayImpl<int32>* CntrFeaturesImpl::getPoGateControlOfChannels()
{
   if (!(m_scenarios & SCE_PWMDL_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<int32>(CFG_FeaturePoGateControlOfCounters);
}
ArrayImpl<SignalPolarity>* CntrFeaturesImpl::getPoGatePolarities()
{
   if (!(m_scenarios & SCE_PWMDL_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<SignalPolarity>(CFG_FeaturePoGatePolarities);
}
ArrayImpl<OutSignalType>*  CntrFeaturesImpl::getPoOutSignals()
{
   if (!(m_scenarios & SCE_PWMDL_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<OutSignalType>(CFG_FeaturePoOutSignals);
}
void CntrFeaturesImpl::getPoHiPeriodRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeaturePoHiPeriodRange, x) != Success) {
      ResetMathInterval(x);
   }
}
void CntrFeaturesImpl::getPoLoPeriodRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeaturePoLoPeriodRange, x) != Success) {
      ResetMathInterval(x);
   }
}
void CntrFeaturesImpl::getPoOutCountRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeaturePoOutCountRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// new: trigger features for pulse width modulation 
int32 CntrFeaturesImpl::getPoTriggerCount()
{
   int32 count = 0;
   MOD_PROP.Get(CFG_FeaturePoTriggersCount, count);
   return count;
}
ArrayImpl<SignalDrop>* CntrFeaturesImpl::getPoTriggerSources()
{
   if (!(m_scenarios & SCE_PWMDL_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<SignalDrop>(CFG_FeaturePoTriggerSources);
}
ArrayImpl<TriggerAction>* CntrFeaturesImpl::getPoTriggerActions()
{
   if (!(m_scenarios & SCE_PWMDL_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<TriggerAction>(CFG_FeaturePoTriggerActions);
}
void CntrFeaturesImpl::getPoTriggerDelayRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeaturePoTriggerDelayCountRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// Up-down counter features
ArrayImpl<CountingType>* CntrFeaturesImpl::getUdCountingTypes()
{
   if (!(m_scenarios & SCE_UDCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<CountingType>(CFG_FeatureUdCountingTypes);
}

ArrayImpl<int32>* CntrFeaturesImpl::getUdInitialValues()
{
   if (!(m_scenarios & SCE_UDCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<int32>(CFG_FeatureUdInitialValues);
}

ArrayImpl<int32>* CntrFeaturesImpl::getUdSnapEventSources()
{
   if (!(m_scenarios & SCE_UDCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<int32>(CFG_FeatureSnapEventSources);
}

void CntrFeaturesImpl::getUdValueResetTimes(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureUdCountValueResetTimes, x) != Success) {
      ResetMathInterval(x);
   }
}

// new features for Counter continue comparing: outputting pulse settings
ArrayImpl<int32>* CntrFeaturesImpl::getCcpGateControlOfChannels()
{
   if (!(m_scenarios & SCE_UDCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<int32>(CFG_FeatureCcpGateControlOfCounters);
}
ArrayImpl<SignalPolarity>* CntrFeaturesImpl::getCcpGatePolarities()
{
   if (!(m_scenarios & SCE_UDCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<SignalPolarity>(CFG_FeatureCcpGatePolarities);
}
ArrayImpl<OutSignalType>*  CntrFeaturesImpl::getCcpOutSignals()
{
   if (!(m_scenarios & SCE_UDCTR_MASK)) return NULL;
   return MOD_PROP.GetValueArrayI32<OutSignalType>(CFG_FeatureCcpOutSignals);
}
void CntrFeaturesImpl::getCcpHiPeriodRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureCcpHiPeriodRange, x) != Success) {
      ResetMathInterval(x);
   }
}
void CntrFeaturesImpl::getCcpLoPeriodRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureCcpLoPeriodRange, x) != Success) {
      ResetMathInterval(x);
   }
}
void CntrFeaturesImpl::getCcpOutCountRange(MathInterval &x)
{
   if (MOD_PROP.Get(CFG_FeatureCcpOutCountRange, x) != Success) {
      ResetMathInterval(x);
   }
}

// new: buffered counter features
int32 CntrFeaturesImpl::getTriggerCount()
{
   int32 count = 0;
   MOD_PROP.Get(CFG_FeatureTriggersCount, count);
   return count;
}

ArrayImpl<SignalDrop>* CntrFeaturesImpl::getTriggerSources(int32 trigger)
{
   PropertyId id = trigger < MAX_TRIG_COUNT ? feat_id_trigSrc[trigger] : PropertyId(-1);
   return MOD_PROP.GetValueArrayI32<SignalDrop>(id);
}

ArrayImpl<TriggerAction>* CntrFeaturesImpl::getTriggerActions(int32 trigger)
{
   PropertyId id = trigger < MAX_TRIG_COUNT ? feat_id_trigAction[trigger] : PropertyId(-1);
   return MOD_PROP.GetValueArrayI32<TriggerAction>(id);
}

CounterClockSourceIndexerImpl* CntrFeaturesImpl::getEcSampleClkSrcs()
{
   return &m_ecSampleSrcIndexer;
}

CounterClockSourceIndexerImpl* CntrFeaturesImpl::getPiSampleClkSrcs()
{
   return &m_piSampleSrcIndexer;
}

CounterClockSourceIndexerImpl* CntrFeaturesImpl::getPoSampleClkSrcs()
{
   return &m_poSampleSrcIndexer;
}

CounterClockSourceIndexerImpl* CntrFeaturesImpl::getUdSampleClkSrcs()
{
   return &m_udSampleSrcIndexer;
}

// -----------------------------------------------------------------------
// functions of EcChannelImpl
//------------------------------------------------------------------------
int32 EcChannelImpl::getChannel()
{
   return (int32)m_tag;
}
bool EcChannelImpl::getNoiseFiltered()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   uint8 value = 0;
   MOD_PROP.GetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, value);
   return value != 0;
}
ErrorCode EcChannelImpl::setNoiseFiltered(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, (uint8)value);
}

SignalDrop EcChannelImpl::getClockSource()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_EcClkSourceOfCounters, m_tag, value);
   return SignalDrop(value);
}
ErrorCode  EcChannelImpl::setClockSource(SignalDrop value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_EcClkSourceOfCounters, m_tag, (int32)value);
}
SignalPolarity EcChannelImpl::getClockPolarity()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_EcClkPolarityOfCounters, m_tag, value);
   return SignalPolarity(value);
}
ErrorCode EcChannelImpl::setClockPolarity(SignalPolarity value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_EcClkPolarityOfCounters, m_tag, (int32)value);
}
SignalPolarity EcChannelImpl::getGatePolarity()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_EcGatePolarityOfCounters, m_tag, value);
   return SignalPolarity(value);
}
ErrorCode EcChannelImpl::setGatePolarity(SignalPolarity value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_EcGatePolarityOfCounters, m_tag, (int32)value);
}
bool EcChannelImpl::getGated()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_EcGateEnabledOfCounters, m_tag, value);
   return value != 0;
}
ErrorCode EcChannelImpl::setGated(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_EcGateEnabledOfCounters, m_tag, (int32)value);
}
// -----------------------------------------------------------------------
// functions of FmChannelImpl
//------------------------------------------------------------------------
int32 FmChannelImpl::getChannel()
{
   return (int32)m_tag;
}
bool FmChannelImpl::getNoiseFiltered()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   uint8 value = 0;
   MOD_PROP.GetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, value);
   return value != 0;
}
ErrorCode FmChannelImpl::setNoiseFiltered(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, (uint8)value);
}
FreqMeasureMethod FmChannelImpl::getFmMethod()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_FmMethodOfCounters, m_tag, value);
   return FreqMeasureMethod(value);
}
ErrorCode FmChannelImpl:: setFmMethod(FreqMeasureMethod value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_FmMethodOfCounters, m_tag, (int32)value);
}
double FmChannelImpl::getCollectionPeriod()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   double value = 0;
   MOD_PROP.GetValueNth(CFG_FmCollectionPeriodOfCounters, m_tag, value);
   return value;
}
ErrorCode FmChannelImpl::setCollectionPeriod(double value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_FmCollectionPeriodOfCounters, m_tag, (double)value);
}

// -----------------------------------------------------------------------
// functions of OsChannelImpl
//------------------------------------------------------------------------
int32 OsChannelImpl::getChannel()
{
   return (int32)m_tag;
}
bool OsChannelImpl::getNoiseFiltered()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   uint8 value = 0;
   MOD_PROP.GetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, value);
   return value != 0;
}
ErrorCode OsChannelImpl::setNoiseFiltered(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, (uint8)value);
}
int32 OsChannelImpl::getDelayCount()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_OsDelayCountOfCounters, m_tag, value);
   return value;
}
ErrorCode  OsChannelImpl::setDelayCount(int32 value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_OsDelayCountOfCounters, m_tag, (int32)value);
}
SignalDrop OsChannelImpl::getClockSource()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_OsClkSourceOfCounters, m_tag, value);
   return SignalDrop(value);
}
ErrorCode  OsChannelImpl::setClockSource(SignalDrop value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_OsClkSourceOfCounters, m_tag, (int32)value);
}
SignalPolarity OsChannelImpl::getClockPolarity()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_OsClkPolarityOfCounters, m_tag, value);
   return SignalPolarity(value);
}
ErrorCode OsChannelImpl::setClockPolarity(SignalPolarity value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_OsClkPolarityOfCounters, m_tag, (int32)value);
}
SignalDrop OsChannelImpl::getGateSource()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_OsGateSourceOfCounters, m_tag, value);
   return SignalDrop(value);
}
ErrorCode OsChannelImpl::setGateSource(SignalDrop value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_OsGateSourceOfCounters, m_tag, (int32)value);
}
SignalPolarity OsChannelImpl::getGatePolarity()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_OsGatePolarityOfCounters, m_tag, value);
   return SignalPolarity(value);
}
ErrorCode OsChannelImpl::setGatePolarity(SignalPolarity value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_OsGatePolarityOfCounters, m_tag, (int32)value);
}
OutSignalType OsChannelImpl::getOutSignal()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_OsOutSignalOfCounters, m_tag, value);
   return OutSignalType(value);
}
ErrorCode OsChannelImpl::setOutSignal(OutSignalType value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_OsOutSignalOfCounters, m_tag, (int32)value);
}

// -----------------------------------------------------------------------
// functions of TmrChannelImpl
//------------------------------------------------------------------------
int32 TmrChannelImpl::getChannel()
{
   return (int32)m_tag;
}
bool TmrChannelImpl::getNoiseFiltered()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   uint8 value = 0;
   MOD_PROP.GetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, value);
   return value != 0;
}
ErrorCode TmrChannelImpl::setNoiseFiltered(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, (uint8)value);
}
double TmrChannelImpl::getFrequency()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   double value = 0;
   MOD_PROP.GetValueNth(CFG_TmrFrequencyOfCounters, m_tag, value);
   return value;
}
ErrorCode TmrChannelImpl::setFrequency(double value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_TmrFrequencyOfCounters, m_tag, (double)value);
}
bool TmrChannelImpl::getGated()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_TmrGateEnabledOfCounters, m_tag, value);
   return value != 0;
}
ErrorCode TmrChannelImpl::setGated(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_TmrGateEnabledOfCounters, m_tag, (int32)value);
}
SignalPolarity TmrChannelImpl::getGatePolarity()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_TmrGatePolarityOfCounters, m_tag, value);
   return SignalPolarity(value);
}
ErrorCode TmrChannelImpl::setGatePolarity(SignalPolarity value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_TmrGatePolarityOfCounters, m_tag, (int32)value);
}
OutSignalType TmrChannelImpl::getOutSignal()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_TmrOutSignalOfCounters, m_tag, value);
   return OutSignalType(value);
}
ErrorCode TmrChannelImpl::setOutSignal(OutSignalType value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_TmrOutSignalOfCounters, m_tag, (int32)value);
}

// -----------------------------------------------------------------------
// functions of PiChannelImpl
//------------------------------------------------------------------------
int32 PiChannelImpl::getChannel()
{
   return (int32)m_tag;
}
bool PiChannelImpl::getNoiseFiltered()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   uint8 value = 0;
   MOD_PROP.GetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, value);
   return value != 0;
}
ErrorCode PiChannelImpl::setNoiseFiltered(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, (uint8)value);
}

// -----------------------------------------------------------------------
// functions of PoChannelImpl
//------------------------------------------------------------------------
int32 PoChannelImpl::getChannel()
{
   return (int32)m_tag;
}
bool PoChannelImpl::getNoiseFiltered()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   uint8 value = 0;
   MOD_PROP.GetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, value);
   return value != 0;
}
ErrorCode PoChannelImpl::setNoiseFiltered(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, (uint8)value);
}
void PoChannelImpl::getPulseWidth(PulseWidth &width)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   width.HiPeriod = width.LoPeriod = 0;
   MOD_PROP.GetValueNth(CFG_PoHiPeriodOfCounters, m_tag, width.HiPeriod);
   MOD_PROP.GetValueNth(CFG_PoLoPeriodOfCounters, m_tag, width.LoPeriod);
}
ErrorCode PoChannelImpl::setPulseWidth(PulseWidth const &width)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   ErrorCode retHo = MOD_PROP.SetValueNth(CFG_PoHiPeriodOfCounters, m_tag, width.HiPeriod);
   ErrorCode retLo = MOD_PROP.SetValueNth(CFG_PoLoPeriodOfCounters, m_tag, width.LoPeriod, true);
   return retHo != Success ? retHo : retLo;
}
bool PoChannelImpl::getGated()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_PoGateEnabledOfCounters, m_tag, value);
   return value != 0;
}
ErrorCode PoChannelImpl::setGated(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_PoGateEnabledOfCounters, m_tag, (int32)value);
}
SignalPolarity PoChannelImpl::getGatePolarity()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_PoGatePolarityOfCounters, m_tag, value);
   return SignalPolarity(value);
}
ErrorCode PoChannelImpl::setGatePolarity(SignalPolarity value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_PoGatePolarityOfCounters, m_tag, (int32)value);
}
OutSignalType PoChannelImpl::getOutSignal()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_PoOutSignalOfCounters, m_tag, value);
   return OutSignalType(value);
}
ErrorCode PoChannelImpl::setOutSignal(OutSignalType value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_PoOutSignalOfCounters, m_tag, (int32)value);
}
int32 PoChannelImpl::getOutCount()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_PoOutCountOfCounters, m_tag, value);
   return value;
}
ErrorCode PoChannelImpl::setOutCount(int32 value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_PoOutCountOfCounters, m_tag, (int32)value);
}

//
// new: trigger settings
SignalDrop PoChannelImpl::getTriggerSource()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;   
   MOD_PROP.GetValueNth(CFG_PoTriggerSourceOfCounters, m_tag, value);

   return SignalDrop(value);
}
ErrorCode PoChannelImpl::setTriggerSource(SignalDrop value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_PoTriggerSourceOfCounters, m_tag, (int32)value);
}
ActiveSignal PoChannelImpl::getTriggerEdge()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;   
   MOD_PROP.GetValueNth(CFG_PoTriggerEdgeOfCounters, m_tag, value);

   return ActiveSignal(value);
}
ErrorCode PoChannelImpl::setTriggerEdge(ActiveSignal value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_PoTriggerEdgeOfCounters, m_tag, (int32)value);
}
TriggerAction PoChannelImpl::getTriggerAction()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;   
   MOD_PROP.GetValueNth(CFG_PoTriggerActionOfCounters, m_tag, value);

   return TriggerAction(value);
}
ErrorCode PoChannelImpl::setTriggerAction(TriggerAction value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_PoTriggerActionOfCounters, m_tag, (int32)value);
}
double PoChannelImpl::getTriggerDelayCount()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   double value = 0;   
   MOD_PROP.GetValueNth(CFG_PoTriggerDelayCountOfCounters, m_tag, value);

   return value;
}
ErrorCode PoChannelImpl::setTriggerDelayCount(double value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_PoTriggerDelayCountOfCounters, m_tag, value);
}

// -----------------------------------------------------------------------
// functions of UdChannelImpl
//------------------------------------------------------------------------
int32 UdChannelImpl::getChannel()
{
   return (int32)m_tag;
}
bool UdChannelImpl::getNoiseFiltered()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   uint8 value = 0;
   MOD_PROP.GetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, value);
   return value != 0;
}
ErrorCode UdChannelImpl::setNoiseFiltered(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_NoiseFilterEnabledChannels, m_tag, (uint8)value);
}
CountingType UdChannelImpl::getCountingType()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_UdCountingTypeOfCounters, m_tag, value);
   return CountingType(value);
}
ErrorCode UdChannelImpl::setCountingType(CountingType value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_UdCountingTypeOfCounters, m_tag, (int32)value);
}
int32 UdChannelImpl::getInitialValue()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_UdInitialValueOfCounters, m_tag, value);
   return value;
}
ErrorCode UdChannelImpl::setInitialValue(int32 value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_UdInitialValueOfCounters, m_tag, (int32)value);
}
int32 UdChannelImpl::getResetTimesByIndex()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_UdCountValueResetTimesByIndexs, m_tag, value);
   return value;
}
ErrorCode UdChannelImpl::setResetTimesByIndex(int32 value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   if (value < -1) {
      return ErrorPropValueOutOfRange;
   }
   return MOD_PROP.SetValueNth(CFG_UdCountValueResetTimesByIndexs, m_tag, (int32)value);
}
void UdChannelImpl::getPulseWidth(PulseWidth &width)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   width.HiPeriod = width.LoPeriod = 0;
   MOD_PROP.GetValueNth(CFG_CcpHiPeriodOfCounters, m_tag, width.HiPeriod);
   MOD_PROP.GetValueNth(CFG_CcpLoPeriodOfCounters, m_tag, width.LoPeriod);
}
ErrorCode UdChannelImpl::setPulseWidth(PulseWidth const &width)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   ErrorCode retHo = MOD_PROP.SetValueNth(CFG_CcpHiPeriodOfCounters, m_tag, width.HiPeriod);
   ErrorCode retLo = MOD_PROP.SetValueNth(CFG_CcpLoPeriodOfCounters, m_tag, width.LoPeriod, true);
   return retHo != Success ? retHo : retLo;
}
bool UdChannelImpl::getGated()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_CcpGateEnabledOfCounters, m_tag, value);
   return value != 0;
}
ErrorCode UdChannelImpl::setGated(bool value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_CcpGateEnabledOfCounters, m_tag, (int32)value);
}
SignalPolarity UdChannelImpl::getGatePolarity()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_CcpGatePolarityOfCounters, m_tag, value);
   return SignalPolarity(value);
}
ErrorCode UdChannelImpl::setGatePolarity(SignalPolarity value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_CcpGatePolarityOfCounters, m_tag, (int32)value);
}
OutSignalType UdChannelImpl::getOutSignal()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_CcpOutSignalOfCounters, m_tag, value);
   return OutSignalType(value);
}
ErrorCode UdChannelImpl::setOutSignal(OutSignalType value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_CcpOutSignalOfCounters, m_tag, (int32)value);
}
int32 UdChannelImpl::getOutCount()
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   int32 value = 0;
   MOD_PROP.GetValueNth(CFG_CcpOutCountOfCounters, m_tag, value);
   return value;
}
ErrorCode UdChannelImpl::setOutCount(int32 value)
{
   assert((unsigned)m_tag < MAX_CNTR_CH_COUNT);
   return MOD_PROP.SetValueNth(CFG_CcpOutCountOfCounters, m_tag, (int32)value);
}
