#ifndef _DAQNAVI_EVENT_COUNTER_CTRL_IMPL_H
#define _DAQNAVI_EVENT_COUNTER_CTRL_IMPL_H

class EventCounterCtrlImpl : public CntrCtrlBaseImpl
{
public:
   // Method
   ErrorCode Read(int32 count, int32 *data)
   {
      if (count > getChannelCount()) {
         count = getChannelCount();
      }

      ErrorCode ret = ModuleRef().EcRead(getChannelStart(), count, (uint32*)data);

      // Cache the data
      if (ret == Success) {
         for (int i = 0; i < count; ++i) {
            m_values[(getChannelStart() + i) % MAX_CNTR_CH_COUNT] = data[i];
         }
      } else { // Return the last good value
         for (int i = 0; i < count; ++i) {
            data[i] = m_values[(getChannelStart() + i) % MAX_CNTR_CH_COUNT];
         }
      }
      return ret;
   }

   // Property
   ArrayImpl<EcChannelImpl>* getChannels()
   {
      if (getInitialized()) {
         if (m_channels.getLength() == 0) {
            int count = getFeatures()->getChannelCountMax();
            m_channels.setCapacity(count);

            for (int i = 0; i < count; ++i) {
               m_channels.Add(EcChannelImpl(this, i));
            }
         }
      }

      return &m_channels;
   }

public: // ctor & dtor
   EventCounterCtrlImpl() : m_channels(false)
   { 
      memset(m_values, 0, sizeof(m_values));
   }
   virtual ~EventCounterCtrlImpl() 
   { 
   }

protected:
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual CounterCapability WantedCapability() { return InstantEventCount; }

   // initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = CntrCtrlBaseImpl::InitializeControl(x);
      if (ret != Success) { return ret; }

      m_channels.Clear();
      return Success;
   }

   // initialize the counter channel. 
   virtual ErrorCode InitializeCounter(bool init)
   {
      ErrorCode ret = CntrCtrlBaseImpl::InitializeCounter(init);
      if (ret != Success) { return ret; }

      memset(m_values, 0, sizeof(m_values));
      return ret;
   }

   // start/restart counter after something had been changed
   virtual ErrorCode StartCounter(bool start)
   {
      if (!IsValidChannel()) {
         return ErrorParamOutOfRange;
      }

      // stop counter at first
      if (IsRunning()) {
         ModuleRef().CtReset(getChannelStart(), getChannelCount());
         setState(Stopped);
      }

      // start action
      ErrorCode err = Success;
      if (start && getEnabled()) {
         err = ModuleRef().EcStart(getChannelStart(), getChannelCount());
         if (err == Success) {
            setState(Running);
         }
      }
      return err;
   }

private:
   ArrayImpl<EcChannelImpl> m_channels;
   int32                    m_values[MAX_CNTR_CH_COUNT]; // cached counter value for device broken down unexpected.
};

#endif
