#ifndef _DAQNAVI_FREQ_METER_CTRL_IMPL_H
#define _DAQNAVI_FREQ_METER_CTRL_IMPL_H

class FreqMeterCtrlImpl : public CntrCtrlBaseImpl
{
public:
   // Method
   ErrorCode Read(int32 count, double *data)
   {
      if (count > getChannelCount()) {
         count = getChannelCount();
      }

      ErrorCode ret = ModuleRef().FmRead(getChannelStart(), count, data);

      // Cache the data
      if (ret == Success) {
         for (int i = 0; i < count; ++i) {
            m_values[(getChannelStart() + i) % MAX_CNTR_CH_COUNT] = data[i];
         }
      } else { // Return the last good value
         for (int i = 0; i < count; ++i) {
            data[i] = m_values[(getChannelStart() + i) % MAX_CNTR_CH_COUNT];
         }
      }
      return ret;
   }

   // Property
   ArrayImpl<FmChannelImpl>* getChannels()
   {
      if (getInitialized()) {
         if (m_channels.getLength() == 0) {
            int count = getFeatures()->getChannelCountMax();
            m_channels.setCapacity(count);

            for (int i = 0; i < count; ++i) {
               m_channels.Add(FmChannelImpl(this, i));
            }
         }
      }

      return &m_channels;
   }

public: // ctor & dtor
   FreqMeterCtrlImpl() : m_channels(false)
   {
      memset(m_values, 0, sizeof(m_values));
   }
   virtual ~FreqMeterCtrlImpl() 
   { 
   }

protected:
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual CounterCapability WantedCapability() { return InstantFreqMeter; }
   
   // initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = CntrCtrlBaseImpl::InitializeControl(x);
      if (ret != Success) { return ret; }

      m_channels.Clear();
      return Success;
   }

   // initialize the counter channel. 
   virtual ErrorCode InitializeCounter(bool init)
   {
      ErrorCode ret = CntrCtrlBaseImpl::InitializeCounter(init);
      if (ret != Success) { return ret; }

      memset(m_values, 0, sizeof(m_values));
      return ret;
   }

   // start/restart counter after something had been changed
   virtual ErrorCode StartCounter(bool start)
   {
      if (!IsValidChannel()) {
         return ErrorParamOutOfRange;
      }

      // stop counter at first
      if (IsRunning()) {
         ModuleRef().CtReset(getChannelStart(), getChannelCount());
         setState(Stopped);
      }

      // start action
      ErrorCode err = Success;
      if (start && getEnabled()) {
         err = ModuleRef().FmStart(getChannelStart(), getChannelCount());
         if (err == Success) {
            setState(Running);
         }
      }
      return err;
   }

private:
   ArrayImpl<FmChannelImpl> m_channels;
   double                   m_values[MAX_CNTR_CH_COUNT];  // cached counter value for device broken down unexpected.
};

#endif
