#ifndef _DAQNAVI_ONE_SHOT_CTRL_IMPL_H
#define _DAQNAVI_ONE_SHOT_CTRL_IMPL_H


class OneShotCtrlImpl : public CntrCtrlBaseImpl
{
public:
   // Property
   ArrayImpl<OsChannelImpl>* getChannels()
   {
      if (getInitialized()) {
         if (m_channels.getLength() == 0) {
            int count = getFeatures()->getChannelCountMax();
            m_channels.setCapacity(count);

            for (int i = 0; i < count; ++i) {
               m_channels.Add(OsChannelImpl(this, i));
            }
         }
      }

      return &m_channels;
   }

public: // ctor & dtor
   OneShotCtrlImpl() : m_channels(false)        
   {
   }
   virtual ~OneShotCtrlImpl() 
   { 
      UserEventManager::Remove(getOuter(), this->GetCntrEventId());
   }

protected:
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual CounterCapability WantedCapability() { return OneShot; }

   // initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = CntrCtrlBaseImpl::InitializeControl(x);
      if (ret != Success) { return ret; }

      m_channels.Clear();
      return Success;
   }
   
   // start/restart counter after something had been changed
   virtual ErrorCode StartCounter(bool start)
   {
      if (!IsValidChannel()) {
         return ErrorParamOutOfRange;
      }

      // stop counter at first
      if (IsRunning()) {
         ModuleRef().CtReset(getChannelStart(), getChannelCount());
         setState(Stopped);
      }

      // start action
      ErrorCode err = Success;
      if (start && getEnabled()) {
         err = ModuleRef().OsStart(getChannelStart(), getChannelCount());
         if (err == Success) {
            setState(Running);
         }
      }
      return err;
   }

   // -----------------------------------------------------------------
   // Supported Counter Event ID, uses 'EvtCntOneShot0' as a general event id for one-shot
   // -----------------------------------------------------------------
   virtual EventId GetCntrEventId() { return EvtCntOneShot0; }

   // -----------------------------------------------------------------
   // driver event receiver for one-shot
   // -----------------------------------------------------------------
   virtual void OnKernEvent(EventId id)
   {
      CntrEventArgs args;
      args.Id = id;
      args.Channel = id - EvtCntOneShot0;
      UserEventManager::Fire(getOuter(), EvtCntOneShot0, &args);

      // automatically clear the event flag after the user had processed it.
      ModEvent().ClearFlag(id);
   };

private:
   ArrayImpl<OsChannelImpl> m_channels;
};

#endif
