#ifndef _DAQNAVI_PULSE_WIDTH_METER_CTRL_IMPL_H
#define _DAQNAVI_PULSE_WIDTH_METER_CTRL_IMPL_H

class PwMeterCtrlImpl : public CntrCtrlBaseImpl
{
public:
   // Method
   ErrorCode Read(int32 count, PulseWidth *data)
   {
      if (count > getChannelCount()) {
         count = getChannelCount();
      }

      double hi[MAX_CNTR_CH_COUNT] = {0}, lo[MAX_CNTR_CH_COUNT] = {0}; 
      ErrorCode ret = ModuleRef().PiRead(getChannelStart(), count, hi, lo);

      // Cache the data
      if (ret == Success) {
         for (int i = 0; i < count; ++i) {
            m_values[(getChannelStart() + i) % MAX_CNTR_CH_COUNT].HiPeriod = data[i].HiPeriod = hi[i];
            m_values[(getChannelStart() + i) % MAX_CNTR_CH_COUNT].LoPeriod = data[i].LoPeriod = lo[i];
         }
      } else { // Return the last good value
         for (int i = 0; i < count; ++i) {
            data[i].HiPeriod = m_values[(getChannelStart() + i) % MAX_CNTR_CH_COUNT].HiPeriod;
            data[i].LoPeriod = m_values[(getChannelStart() + i) % MAX_CNTR_CH_COUNT].LoPeriod;
         }
      }
      return ret;
   }

   // Property
   ArrayImpl<PiChannelImpl>* getChannels()
   {
      if (getInitialized()) {
         if (m_channels.getLength() == 0) {
            int count = getFeatures()->getChannelCountMax();
            m_channels.setCapacity(count);

            for (int i = 0; i < count; ++i) {
               m_channels.Add(PiChannelImpl(this, i));
            }
         }
      }

      return &m_channels;
   }

public: // ctor & dtor
   PwMeterCtrlImpl() : m_channels(false)
   {  
      memset(m_values, 0, sizeof(m_values));
   }
   virtual ~PwMeterCtrlImpl() 
   { 
      UserEventManager::Remove(getOuter(), this->GetCntrEventId());
   }

protected:
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual CounterCapability WantedCapability() { return InstantPwmIn; }

   // initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = CntrCtrlBaseImpl::InitializeControl(x);
      if (ret != Success) { return ret; }

      m_channels.Clear();
      return Success;
   }

   // initialize the counter channel. 
   virtual ErrorCode InitializeCounter(bool init)
   {
      ErrorCode ret = CntrCtrlBaseImpl::InitializeCounter(init);
      if (ret != Success) { return ret; }

      memset(m_values, 0, sizeof(m_values));
      return ret;
   }

   // start/restart counter after something had been changed
   virtual ErrorCode StartCounter(bool start)
   {
      if (!IsValidChannel()) {
         return ErrorParamOutOfRange;
      }

      // stop counter at first
      if (IsRunning()) {
         ModuleRef().CtReset(getChannelStart(), getChannelCount());
         setState(Stopped);
      }

      // start action
      ErrorCode err = Success;
      if (start && getEnabled()) {
         err = ModuleRef().PiStart(getChannelStart(), getChannelCount());
         if (err == Success) {
            setState(Running);
         }
      }
      return err;
   }

   // -----------------------------------------------------------------
   // Supported Counter Event ID, uses 'EvtCntPwmInOverflow0' as a general event id for pwmin
   // -----------------------------------------------------------------
   virtual EventId GetCntrEventId() { return EvtCntPwmInOverflow0; }

   // -----------------------------------------------------------------
   // driver event receiver for pulse-width-measurement
   // -----------------------------------------------------------------
   virtual void OnKernEvent(EventId id)
   {
      CntrEventArgs args;
      args.Id = id;
      args.Channel = id - EvtCntPwmInOverflow0;
      UserEventManager::Fire(getOuter(), EvtCntPwmInOverflow0, &args);

      // automatically clear the event flag after the user had processed it.
      ModEvent().ClearFlag(id);
   };

private:
   ArrayImpl<PiChannelImpl> m_channels;
   PulseWidth               m_values[MAX_CNTR_CH_COUNT];    // cached counter value for device broken down unexpected.
};

#endif
