#ifndef _DAQNAVI_TIMER_PLUSE_CTRL_IMPL_H
#define _DAQNAVI_TIMER_PLUSE_CTRL_IMPL_H


class TimerPulseCtrlImpl : public CntrCtrlBaseImpl
{
public:
   // Property
   ArrayImpl<TmrChannelImpl>* getChannels()
   {
      if (getInitialized()) {
         if (m_channels.getLength() == 0) {
            int count = getFeatures()->getChannelCountMax();
            m_channels.setCapacity(count);

            for (int i = 0; i < count; ++i) {
               m_channels.Add(TmrChannelImpl(this, i));
            }
         }
      }

      return &m_channels;
   }

public: // ctor & dtor
   TimerPulseCtrlImpl() : m_channels(false)
   { 
   }
   virtual ~TimerPulseCtrlImpl() 
   {  
      UserEventManager::Remove(getOuter(), this->GetCntrEventId());
   }

protected:
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual CounterCapability WantedCapability() { return TimerPulse; }

   // initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = CntrCtrlBaseImpl::InitializeControl(x);
      if (ret != Success) { return ret; }

      m_channels.Clear();
      return Success;
   }
   
   // start/restart counter after something had been changed
   virtual ErrorCode StartCounter(bool start)
   {
      if (!IsValidChannel()) {
         return ErrorParamOutOfRange;
      }

      // stop counter at first
      if (IsRunning()) {
         ModuleRef().CtReset(getChannelStart(), getChannelCount());
         setState(Stopped);
      }

      // start action
      ErrorCode err = Success;
      if (start && getEnabled()) {
         err = ModuleRef().TmrStart(getChannelStart(), getChannelCount());
         if (err == Success) {
            setState(Running);
         }
      }
      return err;
   }

   // -----------------------------------------------------------------
   // Supported Counter Event ID, uses 'EvtCntTimer0' as a general event id for timer/pluse
   // -----------------------------------------------------------------
   virtual EventId GetCntrEventId() { return EvtCntTimer0; }

   // -----------------------------------------------------------------
   // driver event receiver for timer/pulse
   // -----------------------------------------------------------------
   virtual void OnKernEvent(EventId id)
   {
      CntrEventArgs args;
      args.Id = id;
      args.Channel = id - EvtCntTimer0;
      UserEventManager::Fire(getOuter(), EvtCntTimer0, &args);

      // automatically clear the event flag after the user had processed it.
      ModEvent().ClearFlag(id);
   };

private:
   ArrayImpl<TmrChannelImpl> m_channels;
};

#endif
