#ifndef _DAQNAVI_DIO_CTRL_BASE_IMPL_H
#define _DAQNAVI_DIO_CTRL_BASE_IMPL_H

class BDAQ_NO_VTABLE DioCtrlBaseImpl : public DaqCtrlBaseImpl
{
public:
   DioFeaturesImpl * getFeatures() 
   { 
      return &m_features; 
   }

   int32 getPortCount()        
   { 
      return getFeatures()->getPortCount(); 
   }

   ArrayImpl<DioPortImpl> * getPorts()
   {
      if (!getInitialized()) { return NULL; }

      AutoLock<MutexLock> lock(PropLock());
      if (m_ports.getLength() == 0) {
         m_ports.setCapacity(getPortCount());

         for (int i = 0; i < getPortCount(); ++i) {
            m_ports.Add(DioPortImpl(this, i, WantedPort()));
         }
      }
      return &m_ports;
   }

public: // ctor & dtor
   DioCtrlBaseImpl() : DaqCtrlBaseImpl(DaqDio), m_ports(false) 
   { 
      m_features.setOwner(this);    
   }

   virtual ~DioCtrlBaseImpl() { }

   DaqDioModule & ModuleRef() { return *(DaqDioModule*)getModule(); }

protected: 
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual DioPortDir WantedPort() = 0;

   virtual ModuleType WantedType() { return DaqDio; }

   virtual bool IsSuitable(DaqModule & module)
   {
      return module.getType() == DaqDio;
   }

   // initialize/de-initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = DaqCtrlBaseImpl::InitializeControl(x);
      if (ret == Success) {
         ret = getPortCount() > 0 ? Success : ErrorFuncNotSpted;;
      }
      return ret;
   }

   virtual void DeinitializeControl()
   {
      // reset all configuration to null
      m_ports.Clear();

      DaqCtrlBaseImpl::DeinitializeControl();
   }

   friend class DioFeaturesImpl;
   friend class ScanPortImpl;
   friend class ConvertClockImpl;
   friend class TriggerImpl;
private:
   DioFeaturesImpl        m_features;
   ArrayImpl<DioPortImpl> m_ports;
};

#endif
