#ifndef _DAQNAVI_DIO_FEATURE_IMPL_H
#define _DAQNAVI_DIO_FEATURE_IMPL_H

class DioCtrlBaseImpl;

class DioFeaturesImpl
{
public:
   // DIO common
   bool                 getPortProgrammable();
   int32                getChannelCountMax();
   int32                getPortCount();
   ArrayImpl<uint8>*    getPortsType();
   bool                 getDiSupported();
   bool                 getDoSupported();   
   
   //Buffered DI / Buffered DO 
   bool                 getBufferedDiSupported();
   bool                 getBufferedDoSupported();
   int32                getDataSize();
   int32                getResolution();

   // DI features
   ArrayImpl<uint8>*    getDiDataMask();   

   // DI noise filter features
   bool                 getDiNoiseFilterSupported();
   ArrayImpl<uint8>*    getDiNoiseFilterOfChannels();
   void                 getDiNoiseFilterBlockTimeRange(MathInterval &x);

   // DI interrupt features
   bool                     getDiintSupported();
   bool                     getDiintGateSupported();
   bool                     getDiCosintSupported();
   bool                     getDiPmintSupported();
   ArrayImpl<ActiveSignal>* getDiintTriggerEdges();
   ArrayImpl<uint8>*        getDiintOfChannels();
   ArrayImpl<uint8>*        getDiintGateOfChannels();
   ArrayImpl<uint8>*        getDiCosintOfPorts();
   ArrayImpl<uint8>*        getDiPmintOfPorts();
   ArrayImpl<int32>*        getDiSnapEventSources();

   // DO features
   ArrayImpl<uint8>*         getDoDataMask();
   ArrayImpl<SignalDrop>*    getDoFreezeSignalSources();         
   void                      getDoReflectWdtFeedIntervalRange(MathInterval &x);
   Depository                getDoPresetValueDepository();
   ArrayImpl<DoCircuitType>* getDoCircuitSelectableTypes();

   // For compatible
   ArrayImpl<uint8>*         getDataMask();

   //Buffered DI Feature
   SamplingMethod            getDiSamplingMethod();

   // Buffered DI->conversion clock features
   ArrayImpl<SignalDrop>*     getDiConvertClockSources();
   void                       getDiConvertClockRange(MathInterval &x);

   // Buffered DI->trigger features, multi-trigger supported
   int32                      getDiTriggerCount();
   bool                       getDiRetriggerable();

   ArrayImpl<TriggerAction>*  getDiTriggerActions(int32 trigger);
   ArrayImpl<SignalDrop>*     getDiTriggerSources(int32 trigger);                              
   void                       getDiTriggerDelayRange(int32 trigger, MathInterval &x);
   ValueRange                 getDiTriggerSourceVrg(int32 trigger);
   double                     getDiTriggerHysteresisIndexMax(int32 trigger);
   int32                      getDiTriggerHysteresisIndexStep(int32 trigger);

   // Buffered DO Feature
   SamplingMethod             getDoSamplingMethod();

   // Buffered DO->conversion clock features
   ArrayImpl<SignalDrop>*     getDoConvertClockSources();
   void                       getDoConvertClockRange(MathInterval &x);

   // Buffered DO->trigger features, multi-trigger supported
   int32                      getDoTriggerCount();
   bool                       getDoRetriggerable();

   ArrayImpl<TriggerAction>*  getDoTriggerActions(int32 trigger);
   ArrayImpl<SignalDrop>*     getDoTriggerSources(int32 trigger);                              
   void                       getDoTriggerDelayRange(int32 trigger, MathInterval &x);
   ValueRange                 getDoTriggerSourceVrg(int32 trigger);
   double                     getDoTriggerHysteresisIndexMax(int32 trigger);
   int32                      getDoTriggerHysteresisIndexStep(int32 trigger);

public:
   DioFeaturesImpl(DioCtrlBaseImpl * owner = NULL)
   {
      m_owner = owner;
   }

   void setOwner(DioCtrlBaseImpl * owner)
   {
      assert(owner);
      m_owner = owner;
   }

private:
   DioCtrlBaseImpl * m_owner;
};


#endif
