#ifndef _DAQNAVI_INSTANT_DO_CTRL_IMPL_H
#define _DAQNAVI_INSTANT_DO_CTRL_IMPL_H

class InstantDoCtrlImpl : public DioCtrlBaseImpl
{
public:
   //********************************************************************************
   // method
   //--------------------------------------------------------------------------------
   ErrorCode WriteAny(int32 portStart, int32 portCount, uint8 data[])
   {
      return ModuleRef().DoWritePort(portStart, portCount, data);
   }

   ErrorCode ReadAny(int32 portStart, int32 portCount, uint8 data[])
   {
      return ModuleRef().DoReadPort(portStart, portCount, data);
   }

   ErrorCode WriteBit(int32 port, int32 bit, uint8 data)
   {
      if (uint32(bit) > 7) { return ErrorParamOutOfRange; }
      return ModuleRef().DoWriteBit(port, bit, data);
   }

   ErrorCode ReadBit(int32 port, int32 bit, uint8 *data)
   {
      ErrorCode warning = Success;
      if (uint32(bit) > 7) {
         bit &= 7; 
         warning = WarningParamOutOfRange;
      }

      ErrorCode ret = ModuleRef().DoReadBit(port, bit, *data);
      return ret != Success ? ret : warning;
   }

public: // ctor & dtor
   InstantDoCtrlImpl()
   {
   }
   virtual ~InstantDoCtrlImpl()
   {
   }

protected: 
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual DioPortDir WantedPort() { return Output; }

   virtual std::vector<AccessMode> WantedModes()
   {
      AccessMode spted[] = { ModeWrite, ModeWriteWithReset };
      return std::vector<AccessMode>(&spted[0], &spted[2]);
   }

   virtual bool IsSuitable(DaqModule & module)
   {      
      if (DioCtrlBaseImpl::IsSuitable(module)) {
         uint8 portType[MAX_DIO_PORT_COUNT];
         uint32 size = sizeof(portType);

         DaqProperty(module).Read(CFG_FeaturePortsType, &size, portType);
         for(unsigned i = 0; i < size / sizeof(uint8); ++i) {
            if (portType[i] != PortDi) {
               return true;
            }
         }
      }
      return false;
   }

   // initialize/de-initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = DioCtrlBaseImpl::InitializeControl(x);
      if (ret != Success) {
         return ret;
      }
      if (!getFeatures()->getDoSupported()) {
         return ErrorFuncNotSpted;
      }

      return ret;
   }

   virtual void DeinitializeControl()
   {
      // reset all configuration to null

      DioCtrlBaseImpl::DeinitializeControl();
   }
};


#endif
