#ifndef _DAQNAVI_FIRMWARE_CTRL_IMPL_H
#define _DAQNAVI_FIRMWARE_CTRL_IMPL_H

class FirmwareCtrlImpl;

class FirmwareImpl 
{
public:
   //********************************************************************************
   // property
   //--------------------------------------------------------------------------------
   wchar_t const * getDescription(void);
   wchar_t const * getNotice(void);
   wchar_t const * getActivationTitle();
   wchar_t const * getVersionOnDevice(void);
   wchar_t const * getVersionOnFile(void);
   int32           getOverwriteProgressing(void);

   //********************************************************************************
   // method
   //--------------------------------------------------------------------------------
   ErrorCode       LoadFwFile(wchar_t const *filePath);
   ErrorCode       Overwrite(FwAction action, wchar_t const *activationCode);

public:
   FirmwareImpl(FirmwareCtrlImpl *owner = NULL, uint32 index = -1)
   {
      m_owner = owner;
      m_index = index;
   }

   virtual ~FirmwareImpl()
   {
   }

private:
   FirmwareCtrlImpl *m_owner;
   int32              m_index;

   std::wstring       m_description;
   std::wstring       m_notice;
   std::wstring       m_activationTitle;
   std::wstring       m_versionOnDevice;
   std::wstring       m_versionOnFile;
};

class FirmwareCtrlImpl : public DaqCtrlBaseImpl
{
public:
   //********************************************************************************
   // property
   //--------------------------------------------------------------------------------
   ArrayImpl<FirmwareImpl> * getFirmwares(void)
   {
      if (getInitialized() && m_firmwares.getLength() == 0) {
         uint32 count = ModuleRef().getCount();

         for (unsigned i = 0; i < count; ++i) {
            m_firmwares.Add(FirmwareImpl(this, i));
         }
      }

      return &m_firmwares;
   }

public: // ctor & dtor
   FirmwareCtrlImpl() : DaqCtrlBaseImpl(DaqFw), m_firmwares(false)
   {
   }
   virtual ~FirmwareCtrlImpl()
   {
   }

   DaqFwModule & ModuleRef() { return *(DaqFwModule*)getModule(); }

protected: 
   //------------------------------------------------------------------------
   // override the method in base class
   //------------------------------------------------------------------------
   virtual ModuleType WantedType() { return DaqFw;  }

   virtual bool IsSuitable(DaqModule & module)
   {
      return module.getType() == DaqFw;
   }

   // initialize/de-initialize the control 
   virtual ErrorCode InitializeControl(DeviceInformation const &x)
   {
      // let's the base do initialization at first.
      ErrorCode ret = DaqCtrlBaseImpl::InitializeControl(x);

      return ret;
   }

   virtual void DeinitializeControl()
   {
      // reset all configuration to null
      m_firmwares.Clear();

      DaqCtrlBaseImpl::DeinitializeControl();
   }

private:
   ArrayImpl<FirmwareImpl> m_firmwares; 
};

#endif
