//
// Calibration function implementation
//
#include <std_decl.h>
#include "fw_ctrl.h"

//********************************************************************************
// Helper MACRO for get string information
//--------------------------------------------------------------------------------
#define FW_GET_STRING(var, id, method)                       \
   do {                                                      \
      wchar_t  xbuf[1024];                                   \
      wchar_t *ptr  = xbuf;                                  \
      uint32   size = method(id, 1024, ptr);                 \
      if (size > 1024) {                                     \
         ptr = (wchar_t *)_malloca(size * sizeof(wchar_t));  \
         size = method(id, size, ptr);                       \
       }                                                     \
       if (size)        { var = ptr;   }                     \
       if (ptr != xbuf) { _freea(ptr); }                     \
   } while (0)                                                     
                                    
//********************************************************************************
// class FirmwareImpl 
//--------------------------------------------------------------------------------
wchar_t const * FirmwareImpl::getDescription(void)
{
   if (!m_description.size()) {
      FW_GET_STRING(m_description, m_index, m_owner->ModuleRef().getDescription);
   }

   return m_description.c_str();
}
wchar_t const * FirmwareImpl::getNotice(void)
{
   if (!m_notice.size()) {
      FW_GET_STRING(m_notice, m_index, m_owner->ModuleRef().getNotice);
   }

   return m_notice.c_str();
}
wchar_t const * FirmwareImpl::getActivationTitle()
{
   if (!m_activationTitle.size()) {
      FW_GET_STRING(m_activationTitle, m_index, m_owner->ModuleRef().getActivationTitle);
   }

   return m_activationTitle.c_str();
}

wchar_t const * FirmwareImpl::getVersionOnDevice(void)
{
   if (!m_versionOnDevice.size()) {
      FW_GET_STRING(m_versionOnDevice, m_index, m_owner->ModuleRef().getVersionOnDevice);
   }

   return m_versionOnDevice.c_str();
}
wchar_t const * FirmwareImpl::getVersionOnFile(void)
{
   FW_GET_STRING(m_versionOnFile, m_index, m_owner->ModuleRef().getVersionOnFile);

   return m_versionOnFile.c_str();
}
int32 FirmwareImpl::getOverwriteProgressing(void)
{
   return m_owner->ModuleRef().getOverwriteProgressing(m_index);
}

ErrorCode FirmwareImpl::LoadFwFile(wchar_t const *filePath)
{
   return m_owner->ModuleRef().LoadFwFile(m_index, filePath);
}
ErrorCode FirmwareImpl::Overwrite(FwAction action, wchar_t const *activationCode)
{
   return m_owner->ModuleRef().Overwrite(m_index, action, activationCode);
}
