/*
 * DocAPI.h
 *
 *  Created on: 2011-8-23
 *      Author: rocky
 */

#ifndef _DAQNAVI3_API_H
#define _DAQNAVI3_API_H

#ifdef __cplusplus
extern "C" {
#endif

// ---------------------------------------------------------------
// Global Helper
// ---------------------------------------------------------------
ErrorCode BDAQCALL AdxGetValueRangeInformation(
   IN           uint32        valueRangeType,
   IN           uint32        descriptionSize,
   OUT OPTIONAL wchar_t       *description,
   OUT OPTIONAL PMathInterval range,
   OUT OPTIONAL uint32        *unit);

ErrorCode BDAQCALL AdxGetSignalConnectionInformation(
   IN           uint32  connectorType,
   IN           uint32  descriptionSize,
   OUT OPTIONAL wchar_t *description,
   OUT OPTIONAL uint32  *position);

double AdxTranslateTemperatureScale(
   IN uint32 degreeType,
   IN double degreeCelsius );

ErrorCode BDAQCALL AdxEnumToString(
   IN wchar_t const *enumTypeName,
   IN uint32        enumValue,
   IN uint32        enumStringLength,
   OUT wchar_t      *enumString);

ErrorCode BDAQCALL AdxStringToEnum(
   IN wchar_t const *enumTypeName,
   IN wchar_t const *enumString,
   OUT uint32       *enumValue);

// ---------------------------------------------------------------
// Event Function
// ---------------------------------------------------------------
ErrorCode BDAQCALL AdxEventGetHandle(
   IN  HANDLE handle,
   IN  uint32 eventType,
   OUT HANDLE *eventHandle);

ErrorCode BDAQCALL AdxEventGetLastStatus(
   IN           HANDLE handle,
   IN           uint32 eventType,
   OUT OPTIONAL uint32 *statusLParam,
   OUT OPTIONAL uint32 *statusRParam);

ErrorCode BDAQCALL AdxEventClearFlag(
   IN HANDLE handle,
   IN uint32 eventType,
   IN uint32 flagLParam,
   IN uint32 flagRParam);

// ---------------------------------------------------------------
// Property Access Function
// ---------------------------------------------------------------
ErrorCode BDAQCALL AdxPropertyRead(
   IN           HANDLE handle,
   IN           uint32 propertyId,
   IN           uint32 bufferSize,
   OUT OPTIONAL void   *buffer,
   OUT OPTIONAL uint32 *dataLength,
   OUT OPTIONAL uint32 *attribute);

ErrorCode BDAQCALL AdxPropertyWrite(
   IN HANDLE handle,
   IN uint32 propertyId,
   IN uint32 dataLength,
   IN void   *buffer,
   IN uint32 notifyNow);

ErrorCode BDAQCALL AdxPropertyReadFromRegistry(
   IN           HANDLE handle,
   IN           uint32 propertyId,
   IN           uint32 bufferSize,
   OUT OPTIONAL void   *buffer,
   OUT OPTIONAL uint32 *dataLength);

ErrorCode BDAQCALL AdxPropertyWriteToRegistry(
   IN HANDLE handle,
   IN uint32 propertyId,
   IN uint32 dataLength,
   IN void   *buffer);

// ---------------------------------------------------------------
// Device Function
// ---------------------------------------------------------------
ErrorCode BDAQCALL AdxDeviceGetLinkageInfo(
   IN           int32   deviceParent,
   IN           uint32  index,
   OUT          int32   *deviceNumber,
   OUT OPTIONAL wchar_t *deviceDescription,
   OUT OPTIONAL uint32  *subDeviceCount);

ErrorCode BDAQCALL AdxDeviceOpen(
   IN int32   deviceNumber,
   IN uint32  accessMode,
   OUT HANDLE *deviceHandle);

ErrorCode BDAQCALL AdxDeviceClose(
   IN HANDLE deviceHandle);

ErrorCode BDAQCALL AdxDeviceReset(
   IN HANDLE deviceHandle,
   IN uint32 state);

ErrorCode BDAQCALL AdxDeviceGetModuleHandle(
   IN HANDLE  deviceHandle,
   IN uint32  type,
   IN uint32  index,
   OUT HANDLE *moduleHandle );

ErrorCode BDAQCALL AdxDeviceRefreshProperties(
   IN HANDLE deviceHandle);

ErrorCode BDAQCALL AdxDeviceShowConfigDialogBox(
   IN HANDLE  deviceHandle,
   IN HANDLE  parentWindow,
   IN void    *region,
   IN uint32  dataSource,
   IN int32   modal,
   OUT HANDLE *dialogWindow);

ErrorCode BDAQCALL AdxDeviceReadPorts(
   IN  HANDLE deviceHandle,
   IN  uint32 startAddress,
   IN  uint32 length,
   OUT void   *data);

ErrorCode BDAQCALL AdxDeviceWritePorts(
   IN HANDLE deviceHandle,
   IN uint32 startAddress,
   IN uint32 length,
   IN void   *data);

ErrorCode BDAQCALL AdxDevicePrivateRegionRead(
   IN  HANDLE deviceHandle,
   IN  uint32 signature,
   IN  uint32 dataLen,
   OUT uint8  *dataBuf);

ErrorCode BDAQCALL AdxDevicePrivateRegionWrite(
   IN HANDLE deviceHandle,
   IN uint32 signature,
   IN uint32 dataLen,
   IN uint8  *dataBuf);

// ---------------------------------------------------------------
// AI Function
// ---------------------------------------------------------------
ErrorCode BDAQCALL AdxAiReadSamples(
   IN           HANDLE aiHandle,
   IN           uint32 channelStart,
   IN           uint32 channelCount,
   OUT OPTIONAL void   *rawData,
   OUT OPTIONAL double *scaledData);

ErrorCode BDAQCALL AdxBufferedAiPrepare(
   IN HANDLE aiHandle,
   IN uint32 dataCount,
   OUT void  **dataBuffer);

ErrorCode BDAQCALL AdxBufferedAiRunOnce(
   IN HANDLE aiHandle,
   IN int32  asynchronous);

ErrorCode BDAQCALL AdxBufferedAiRun(
   IN HANDLE aiHandle);

ErrorCode BDAQCALL AdxBufferedAiStop(
   IN HANDLE aiHandle);

ErrorCode BDAQCALL AdxBufferedAiRelease(
   IN HANDLE aiHandle);

ErrorCode BDAQCALL AdxBufferedAiScaleData(
   IN              HANDLE aiHandle,
   IN              void   *rawData,
   OUT             double *scaledData,
   IN              uint32 dataCount,
   IN OUT OPTIONAL uint32 *channelOffset);

ErrorCode BDAQCALL AdxAiGetScaleTable(
   IN           HANDLE           aiHandle,
   IN           uint32           valueRangeType,
   IN OUT       uint32           *size,
   OUT OPTIONAL MapFuncPiece * tables);

ErrorCode BDAQCALL AdxAiSetScaleTable(
   IN HANDLE       aiHandle,
   IN uint32       valueRangeType,
   IN uint32       size,
   IN MapFuncPiece *tables);

ErrorCode BDAQCALL AdxAiGenerateNewValueRange(
   IN          HANDLE       aiHandle,
   IN          uint32       baseType,
   IN          uint32       size,
   IN OPTIONAL MapFuncPiece *tables,
   OUT         uint32       *newType);

ErrorCode BDAQCALL AdxAiGetCjcScaleTable(
   IN           HANDLE       aiHandle,
   IN OUT       uint32       *size,
   OUT OPTIONAL MapFuncPiece *tables);

ErrorCode BDAQCALL AdxAiSetCjcScaleTable(
   IN HANDLE       aiHandle,
   IN uint32       size,
   IN MapFuncPiece *tables);

// ---------------------------------------------------------------
// AO Function
// ---------------------------------------------------------------
ErrorCode BDAQCALL AdxAoWriteSamples(
   IN          HANDLE aoHandle,
   IN          uint32 channelStart,
   IN          uint32 channelCount,
   IN OPTIONAL void   *rawData,
   IN OPTIONAL double *scaledData);

ErrorCode BDAQCALL AdxBufferedAoPrepare(
   IN HANDLE aoHandle,
   IN uint32 dataCount,
   OUT void  **dataBuffer);

ErrorCode BDAQCALL AdxBufferedAoRunOnce(
   IN HANDLE aoHandle,
   IN int32  asynchronous);

ErrorCode BDAQCALL AdxBufferedAoRun(
   IN HANDLE aoHandle);

ErrorCode BDAQCALL AdxBufferedAoStop(
   IN HANDLE aoHandle,
   IN uint32 action);

ErrorCode BDAQCALL AdxBufferedAoRelease(
   IN HANDLE aoHandle);

ErrorCode BDAQCALL AdxBufferedAoScaleData(
   IN              HANDLE aoHandle,
   IN              double *scaledData,
   OUT             void   *rawData,
   IN              uint32 dataCount,
   IN OUT OPTIONAL uint32 *channelOffset);

ErrorCode BDAQCALL AdxAoGetScaleTable(
   IN           HANDLE       aoHandle,
   IN           uint32       valueRangeType,
   IN OUT       uint32       *size,
   OUT OPTIONAL MapFuncPiece *tables);

ErrorCode BDAQCALL AdxAoSetScaleTable(
   IN HANDLE       aoHandle,
   IN uint32       valueRangeType,
   IN uint32       size,
   IN MapFuncPiece *tables);

ErrorCode BDAQCALL AdxAoGenerateNewValueRange(
   IN          HANDLE       aoHandle,
   IN          uint32       baseType,
   IN          uint32       size,
   IN OPTIONAL MapFuncPiece *tables,
   OUT         uint32       *newType);

// ---------------------------------------------------------------
// DI/O Function
// ---------------------------------------------------------------
ErrorCode BDAQCALL AdxDiReadPorts(
   IN  HANDLE dioHandle,
   IN  uint32 portStart,
   IN  uint32 portCount,
   OUT uint8  *dataBuffer);

ErrorCode BDAQCALL AdxDoWritePorts(
   IN HANDLE dioHandle,
   IN uint32 portStart,
   IN uint32 portCount,
   IN uint8  *dataBuffer);

ErrorCode BDAQCALL AdxDoReadBackPorts(
   IN HANDLE dioHandle,
   IN uint32 portStart,
   IN uint32 portCount,
   OUT uint8 *dataBuffer);

ErrorCode BDAQCALL AdxDiReadBit(
   IN  HANDLE dioHandle,
   IN  uint32 port,
   IN  uint32 bit,
   OUT uint8  *dataBuffer);

ErrorCode BDAQCALL AdxDoReadBackBit(
   IN  HANDLE dioHandle,
   IN  uint32 port,
   IN  uint32 bit,
   OUT uint8  *dataBuffer);

ErrorCode BDAQCALL AdxDoWriteBit(
   IN  HANDLE dioHandle,
   IN  uint32 port,
   IN  uint32 bit,
   IN  uint8  data);

ErrorCode BDAQCALL AdxBufferedDiPrepare(
   IN HANDLE dioHandle,
   IN uint32 dataCount,
   OUT void  **dataBuffer);

ErrorCode BDAQCALL AdxBufferedDiRunOnce(
   IN HANDLE dioHandle,
   IN int32  asynchronous);

ErrorCode BDAQCALL AdxBufferedDiRun(
   IN HANDLE dioHandle);

ErrorCode BDAQCALL AdxBufferedDiStop(
   IN HANDLE dioHandle);

ErrorCode BDAQCALL AdxBufferedDiRelease(
   IN HANDLE dioHandle);

ErrorCode BDAQCALL AdxBufferedDoPrepare(
   IN HANDLE dioHandle,
   IN uint32 dataCount,
   OUT void  **dataBuffer);

ErrorCode BDAQCALL AdxBufferedDoRunOnce(
   IN HANDLE dioHandle,
   IN int32  asynchronous);

ErrorCode BDAQCALL AdxBufferedDoRun(
   IN HANDLE dioHandle);

ErrorCode BDAQCALL AdxBufferedDoStop(
   IN HANDLE dioHandle,
   IN uint32 action);

ErrorCode BDAQCALL AdxBufferedDoRelease(
   IN HANDLE dioHandle);

ErrorCode BDAQCALL AdxDiSnapStart(
   IN  HANDLE dioHandle,
   IN  uint32 eventType,
   IN  uint32 portStart,
   IN  uint32 portCount,
   OUT uint8  **dataBuffer);

ErrorCode BDAQCALL AdxDiSnapStop(
   IN HANDLE dioHandle,
   IN uint32 eventType);

ErrorCode BDAQCALL AdxDoReflectWdtLockStart(
   IN HANDLE dioHandle);

ErrorCode BDAQCALL AdxDoReflectWdtFeed(
   IN HANDLE dioHandle);

ErrorCode BDAQCALL AdxDoReflectWdtLockStop(
   IN HANDLE dioHandle);

// ---------------------------------------------------------------
// Counter Function
// ---------------------------------------------------------------
ErrorCode BDAQCALL AdxCounterReset(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount );

ErrorCode BDAQCALL AdxCounterRead(
   IN  HANDLE cntrHandle,
   IN  uint32 counterStart,
   IN  uint32 counterCount,
   OUT uint32 *loadValue,
   OUT uint32 *holdValue);

ErrorCode BDAQCALL AdxEventCountStart(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount );

ErrorCode BDAQCALL AdxEventCountRead(
   IN  HANDLE cntrHandle,
   IN  uint32 counterStart,
   IN  uint32 counterCount,
   OUT uint32 *eventCount );

ErrorCode BDAQCALL AdxOneShotStart(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount);

ErrorCode BDAQCALL AdxTimerPulseStart(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount);

ErrorCode BDAQCALL AdxFrequencyMeasureStart(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount );

ErrorCode BDAQCALL AdxFrequencyMeasureRead(
   IN  HANDLE cntrHandle,
   IN  uint32 counterStart,
   IN  uint32 counterCount,
   OUT double *frequency );

ErrorCode BDAQCALL AdxPwmInStart(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 groupCount );

ErrorCode BDAQCALL AdxPwmInRead(
   IN  HANDLE cntrHandle,
   IN  uint32 counterStart,
   IN  uint32 groupCount,
   OUT double *hiPeriod,
   OUT double *lowPeriod);

ErrorCode BDAQCALL AdxPwmOutStart(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount );

ErrorCode BDAQCALL AdxUpDownCountStart(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount,
   IN uint32 resetCounter);

ErrorCode BDAQCALL AdxUpDownCountValueReset(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount);

ErrorCode BDAQCALL AdxUpDownCountRead(
   IN HANDLE cntrHandle,
   IN uint32 counterStart,
   IN uint32 counterCount,
   IN uint32 *values);

ErrorCode BDAQCALL AdxCounterSnapStart(
   IN HANDLE cntrHandle,
   IN uint32  eventType,
   IN uint32  counterStart,
   IN uint32  counterCount,
   OUT OPTIONAL uint32 **loadValues,
   OUT OPTIONAL uint32 **holdValues);

ErrorCode BDAQCALL AdxCounterSnapStop(
   IN HANDLE cntrHandle,
   IN uint32 eventType);

ErrorCode BDAQCALL AdxCounterCompareSetTable(
   IN HANDLE cntrHandle,
   IN uint32 counter,
   IN uint32 dataCount,
   IN uint32 *buffer);

ErrorCode BDAQCALL AdxCounterCompareSetInterval(
   IN HANDLE cntrHandle,
   IN uint32 counter,
   IN uint32 firstValue,
   IN int32  increment,
   IN uint32 count);

ErrorCode BDAQCALL AdxCounterCompareClear(
   IN HANDLE cntrHandle,
   IN uint32 counter);

ErrorCode BDAQCALL AdxCounterStart(
   IN HANDLE  cntrHandle,
   IN uint32  counterStart,
   IN uint32  counterCount);


#ifdef __cplusplus
}
#endif

#endif /* _DAQNAVI3_API_H */
