//
// DAQNAVI4 new device initialization routine
// Note:
//      The APIs is mainly used by DAQNAVI_DAEMON to do the startup initialization for 
// the devices enumerated by OS. 
//
#define  _CRT_SECURE_NO_WARNINGS
#include <std_decl.h>
#include <ioctl_helper.hpp>

ErrorCode FindFreeBoardId(uint32 productId, int32* boardId)
{
   *boardId     = -1;

   ErrorCode     ret;
   uint32        count    = 10;
   DEVICE_RECORD stack_buf[10]; 
   DEVICE_RECORD *record  = stack_buf;

   int const countofBoradId = 16;
   uint16    boardIdInUse   = 0;

#define MARK_DEVNUMBER(num)  boardIdInUse |= 1 << (num & 15)
#define TEST_DEVNUMBER(num)  (boardIdInUse & (1 << (num & 15)))

   do {
      ret = QueryDeviceInfoADB(ListAll, "", &count, record);
      if (ret == ErrorBufferTooSmall) {
         if (record != stack_buf) { 
            delete[] record; 
         }
         count  += 100;
         record = new DEVICE_RECORD[count];
      }
   } while (ret == ErrorBufferTooSmall);

   for(uint32 i = 0; i < count; i++) {
      if((record[i].ProductId == productId) && (0 == strcmp(record[i].DriverName, "biosimulator"))) {
         int32 tmpId = 0;
         if(Success == QueryDeviceFieldI32DB(record[i].DeviceNumber, DFN_BOARDID, &tmpId)) {
            MARK_DEVNUMBER(tmpId);
         }
      }
   }

   for (int i = 0; i < countofBoradId; ++i) {
      if (!TEST_DEVNUMBER(i)) {
         *boardId = i;
         break;
      }
   }

   if (record != stack_buf) { 
      delete[] record; 
   }

   return (*boardId == -1) ? ErrorDriverCountExceedLimit : Success;
}

//
// Helper method for the new version navigator
#if defined(_WIN32) || defined(WIN32) 
#  define BUILD_DEF_TREEPATH(info, bus, loc) _snwprintf(info.TreePath,    ARRAY_SIZE(info.TreePath),    L"/%S/%ls", bus, loc)
#  define BUILD_DEF_DEVDESC(info)            _snwprintf(info.Description, ARRAY_SIZE(info.Description), L"%s,BID#%d", info.DeviceName, info.BoardId)
#  define UPDATE_HWINFO_DESC(info, xdesc)    wcscpy_s(info.Description,  ARRAY_SIZE(info.Description), xdesc)
#else
#  define BUILD_DEF_TREEPATH(info, bus, loc) snprintf(info.TreePath,    ARRAY_SIZE(info.TreePath),    "/%s/%ls", bus, loc)
#  define BUILD_DEF_DEVDESC(info)            snprintf(info.Description, ARRAY_SIZE(info.Description), "%s,BID#%d", info.DeviceName, info.BoardId)
#  define UPDATE_HWINFO_DESC(info, xdesc)    wcstombs(info.Description, xdesc, ARRAY_SIZE(info.Description))
#endif

#define FIXUP_DEVDESC   (0x1 << 0)
#define FIXUP_TREEPATH  (0x1 << 1)

ErrorCode FixupDeviceTreePath(DeviceCtrlImpl &ctrl, DEVICE_HWINFO_EX &info)
{
   ErrorCode ret = Success;

   // Build the default device tree path
   if (info.TreePath[0] == 0) {
      //
      // Set locale information for string conversion
      std::setlocale(LC_CTYPE, "");  

      // Query device bus type
      char   bus[32];
      uint32 len = ARRAY_SIZE(bus);
      if (QueryDeviceBusTypeDB(ProductId(info.ProductId), &len, bus) != Success) {
         //
         // Extract the bus type from Device Name if we cannot get it from DB
         memset(bus, 0, sizeof(bus));
         for (int i = 0; i < ARRAY_SIZE(bus) - 1; ++i) {
            if ('-' == (char)info.DeviceName[i]) {
               break;
            }

            bus[i] = (char)info.DeviceName[i];
         }       
      } 

      // Query device location information
      wchar_t loc[64]; 
      loc[0] = 0;

      if (!ctrl.getInitialized()) {
         ret = ctrl.Initialize(info.DeviceNumber, NULL, ModeRead);
      }
      if (ctrl.getInitialized()) {
         ret = ctrl.getLocation(64, loc);
      }

      BUILD_DEF_TREEPATH(info, bus, loc);

   } else {
      // Just using the driver supplied tree path
   }

   return ret;
}

ErrorCode FixupDeviceDescription(DeviceCtrlImpl &ctrl, DEVICE_HWINFO_EX &info)
{
   ErrorCode ret = Success;

   //
   // Get current description
   wchar_t desc[MAX_DEVICE_DESC_LEN];
   uint32  len = ARRAY_SIZE(desc);
   desc[0] = 0;
   QueryDeviceFieldDB(info.DeviceNumber, DFN_DESC, &len, desc);

   // Build the default description if the current one is null
   if (desc[0] == 0) { 
      if (info.Description[0] == 0) {
         BUILD_DEF_DEVDESC(info);
      } else { 
         // Just using the kernel driver supplied description stored at info.Description
      }

   } else { // Update the device description according to the hardware information

      wchar_t xbuf[MAX_DEVICE_DESC_LEN];
      xbuf[0] = 0;

      // Call the device driver to update the description
      if (info.Description[0] != 0) {
         if (!ctrl.getInitialized()) {
            ret = ctrl.Initialize(info.DeviceNumber, NULL, ModeRead);
         }
         if (ctrl.getInitialized()) {
            ret = ctrl.getDevice()->UpdateDescription(info.BoardId, desc, xbuf);
         }

      } else {
          DeviceUpdateDescription(info.BoardId, desc, xbuf);
      }

      UPDATE_HWINFO_DESC(info, xbuf);
   }

   return ret;
}

ErrorCode FixupDeviceHwInfo(unsigned which, DEVICE_HWINFO_EX &info)
{
   ErrorCode ret = Success;
   DeviceCtrlImpl ctrl;
   
   if (which & FIXUP_TREEPATH) {
      ret = FixupDeviceTreePath(ctrl, info);
   }
   
   if (which & FIXUP_DEVDESC) {
      ret = FixupDeviceDescription(ctrl, info);
   }

   return ret;
}

///!!!+++ Yaojen+ Aug 2018
#if defined(_WIN32) || defined(WIN32) 
extern "C" ErrorCode BDAQCALL AdxDeviceStartupExtECAT(wchar_t const * devPath, DEVICE_HWINFO * phwinfo)
#else
extern "C" ErrorCode BDAQCALL AdxDeviceStartupExtECAT(char const * devPath, DEVICE_HWINFO * phwinfo)
#endif
{ 
   bool      newDevice = false;
   ErrorCode ret       = Success; 

   DEVICE_HWINFO_EX hwinfo;
   memcpy(&hwinfo, phwinfo, sizeof(*phwinfo));
   hwinfo.Description[0] = 0;
   hwinfo.TreePath[0]    = 0;

   //
   // Fix bug: 19773
   if (hwinfo.DriverSpec != DRVSPEC_DN4) { 
      return ErrorDeviceUnrecognized; 
   }

   //
   // Query device number of the device.
   ret = FindDeviceNumberDB(hwinfo, (int32*)&hwinfo.DeviceNumber);

   //
   // the fix number of device conflicts with other device's
   if (ret == ErrorFixNumberConflict) { return ret; }

   //
   // Allocate device number for the device if we had not done.
   if (ret == ErrorDeviceNotExist) { 
      newDevice = true;       
      ret = AllocDeviceNumberDB(hwinfo, devPath, (int32*)&hwinfo.DeviceNumber);
   }

   if (ret != Success) { return Success; }

   //
   // Update the DB with the HW info
   if (newDevice) { 
      // Set default terminal board
      uint32 size = sizeof(uint32);
      uint32 board = WiringBoard;
      PropReadValueDB(hwinfo.ProductId, catofModule(DaqDevice, 0), CFG_DeviceTerminalBoardType, &size, &board);
      UpdateDeviceFieldI32DB(hwinfo.DeviceNumber, DFN_TERMBD, board);

      // Set device tree path
      if (hwinfo.TreePath[0] == 0) { 
         FixupDeviceHwInfo(FIXUP_TREEPATH, hwinfo);   
      }

#if defined(_WIN32) || defined(WIN32) 
      UpdateDeviceFieldDB(hwinfo.DeviceNumber, DFN_TREEPATH, hwinfo.TreePath);
#else
      UpdateDeviceFieldADB(hwinfo.DeviceNumber, DFN_TREEPATH, hwinfo.TreePath);
#endif
   } else {
      // Note:
      // Update the device path before we fix up the HWINFo due to it maybe had changed,
      // and 'Fixup HWINFO' required it to open the device.
#if defined(_WIN32) || defined(WIN32) 
      UpdateDeviceFieldDB(hwinfo.DeviceNumber, DFN_DEVPATH, devPath);
#else
      UpdateDeviceFieldADB(hwinfo.DeviceNumber, DFN_DEVPATH, devPath);
#endif
      FixupDeviceHwInfo(FIXUP_DEVDESC | FIXUP_TREEPATH, hwinfo);

      //
      // Update the DB with the HW info which maybe had been changed.
      UpdateDeviceInfoDB(hwinfo, devPath);

#if !defined(_WIN32) && !defined(WIN32) 
      UpdateDeviceFieldADB(hwinfo.DeviceNumber, DFN_DESC,     hwinfo.Description);
      UpdateDeviceFieldADB(hwinfo.DeviceNumber, DFN_TREEPATH, hwinfo.TreePath);
#endif
   }
       
   //
   // DN4 device needn't to do this initialize, the user should initialize the device with 'LoadProfile' method
   // Initialize device using the value from config DB
   ModuleType mtypes[] = {DaqDevice, DaqAi, DaqAo, DaqDio, DaqCounter, DaqAny}; // Use 'DaqAny' as the end flag of array
   ret = XDevManager::SynchronizeDeviceWithDB(hwinfo.DeviceNumber, hwinfo.ProductId, mtypes);
       
   //
   // Release the driver dll we loaded.
   XDevFactory::UnloadIdleDriver();

   return ret;
}
///!!!---

#if defined(_WIN32) || defined(WIN32) 
extern "C" ErrorCode BDAQCALL AdxDeviceStartupInitialize(wchar_t const * devPath)
{
   DEVICE_HWINFO_EX hwinfo = {{(uint32)-1}}; 
   bool          newDevice = false;
   ErrorCode     ret       = Success;

   //
   // Get device hardware information via IOCTL
   HANDLE devHandle = CreateFile(devPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL); 
   if (devHandle == NULL) { return ErrorDeviceNotExist; }

   ret = IoctlDevGetHWInfo(devHandle, &hwinfo);
   CloseHandle(devHandle);

   if (ret != Success) { return ErrorDeviceUnrecognized; }
   
   //
   // Add null terminator for the description and tree path field to avoid unexpected gibberish.
   hwinfo.Description[63] = 0;
   hwinfo.TreePath[63]    = 0;
   
   //
   // Daqnavi3 driver, need do nothing and just return.
   if (hwinfo.DriverSpec < DRVSPEC_DN4) { 
      //
      // Note: Must update device info DB at first for DN3 device, 
      // otherwise we cannot get the correct tree path information 
      UpdateDeviceInfoDB(hwinfo, devPath);      

      if (hwinfo.TreePath[0] == 0) { 
         FixupDeviceHwInfo(FIXUP_TREEPATH, hwinfo);   
         UpdateDeviceFieldDB(hwinfo.DeviceNumber, DFN_TREEPATH, hwinfo.TreePath);
      }

      return Success; 
   }

   //
   // Fix bug: 19773
   if (hwinfo.DriverSpec != DRVSPEC_DN4) { 
      return ErrorDeviceUnrecognized; 
   }

   //
   // Query device number of the device.
   ret = FindDeviceNumberDB(hwinfo, (int32*)&hwinfo.DeviceNumber);

   // the fix number of device conflicts with other device's
   if (ret == ErrorFixNumberConflict) { return ret; }

   //
   // Allocate device number for the device if we had not done.
   if (ret == ErrorDeviceNotExist) { 
      newDevice = true;      
      ret = AllocDeviceNumberDB(hwinfo, devPath, (int32*)&hwinfo.DeviceNumber);
   }

   if (ret != Success) { return Success; }

   //
   // Update the DB with the HW info
   if (newDevice) { 
      // Set default terminal board
      uint32 size = sizeof(uint32);
      uint32 board = WiringBoard;
      PropReadValueDB(hwinfo.ProductId, catofModule(DaqDevice, 0), CFG_DeviceTerminalBoardType, &size, &board);
      UpdateDeviceFieldI32DB(hwinfo.DeviceNumber, DFN_TERMBD, board);

      // Set device tree path
      if (hwinfo.TreePath[0] == 0) { 
         FixupDeviceHwInfo(FIXUP_TREEPATH, hwinfo);   
      }

      UpdateDeviceFieldDB(hwinfo.DeviceNumber, DFN_TREEPATH, hwinfo.TreePath);
   } else {
      // Note:
      // Update the device path before we fix up the HWINFo due to it maybe had changed,
      // and 'Fixup HWINFO' required it to open the device.
      UpdateDeviceFieldDB(hwinfo.DeviceNumber, DFN_DEVPATH, devPath);
      FixupDeviceHwInfo(FIXUP_DEVDESC | FIXUP_TREEPATH, hwinfo);

      // Update the DB with the HW info which maybe had been changed.
      UpdateDeviceInfoDB(hwinfo, devPath);
   }

   //
   // DN4 device needn't to do this initialize, the user should initialize the device with 'LoadProfile' method
   // Initialize device using the value from config DB
   // ModuleType mtypes[] = {DaqDevice, DaqAi, DaqAo, DaqDio, DaqCounter, DaqAny}; // Use 'DaqAny' as the end flag of array
   // ret = XDevManager::SynchronizeDeviceWithDB(hwinfo.DeviceNumber, hwinfo.ProductId, mtypes);

   //
   // Notify the user new device is arrived
   HANDLE devEvt = DeviceCreatePnpEvent(EvtDeviceReconnected, hwinfo.ProductId, hwinfo.DeviceNumber);
   if (devEvt) {
      SetEvent(devEvt);
      CloseHandle(devEvt);
   }

   //
   // Release the driver dll we loaded.
   XDevFactory::UnloadIdleDriver();

   return ret;
}
#else
extern "C" ErrorCode BDAQCALL AdxDeviceStartupInitialize(char const * devPath)
{
   DEVICE_HWINFO_EX hwinfo = {(uint32)-1}; 
   ErrorCode        ret    = Success;

   //
   // Get device hardware information via IOCTL
   int devFd = open(devPath, O_RDONLY);
   if (devFd < 0) { return ErrorDeviceNotExist; }

   ret = IoctlDevGetHWInfo(devFd, &hwinfo);
   close(devFd);

   if (ret != Success) { return ErrorDeviceUnrecognized; }

   //
   // Add null terminator for the description and tree path field to avoid unexpected gibberish.
   hwinfo.Description[63] = 0;
   hwinfo.TreePath[63]    = 0;

   //
   // For linux, we haven't supported 'Fixed Device Number' at present,
   // so just update device info DB for DN3 & DN4 drivers all.
   if (hwinfo.DeviceNumber == uint32(-1)) {
      char const *daq = strstr(devPath, "daq");
      if (daq != NULL) {
         hwinfo.DeviceNumber = atoi(daq + 3);
      }
   }

   //
   // Note: Must update device info DB at first for DN3 device, 
   // otherwise we cannot get the correct tree path information 
   UpdateDeviceInfoDB(hwinfo, devPath);

   FixupDeviceHwInfo(FIXUP_DEVDESC | FIXUP_TREEPATH, hwinfo);
   UpdateDeviceFieldADB(hwinfo.DeviceNumber, DFN_DESC,     hwinfo.Description);
   UpdateDeviceFieldADB(hwinfo.DeviceNumber, DFN_TREEPATH, hwinfo.TreePath);

   //
   // Notify the user new device is arrived
   if (hwinfo.DriverSpec == DRVSPEC_DN4) {
      HANDLE devEvt = DaqDevModule::CreateDevicePnpEvent(EvtDeviceReconnected, hwinfo.ProductId, hwinfo.DeviceNumber);
      if (devEvt) {
         SetEvent(devEvt);
         CloseHandle(devEvt);
      }
   }

   //
   // Release the driver dll we loaded.
   XDevFactory::UnloadIdleDriver();

   return ret;
}
#endif

#if defined(_WIN32) || defined(WIN32) 
extern "C" void BDAQCALL AdxDeviceResetActiveFlag(wchar_t const * devPath)
#else
extern "C" void BDAQCALL AdxDeviceResetActiveFlag(char const * devPath)
#endif
{
   // reset all 'Active' flag if devPath is NULL, 
   // otherwise reset the specified device's 'Active' flag
   QueryType queryType = devPath ? ByDevPath : ListAll;

   uint32        count = 10;
   DEVICE_RECORD stack_buf[10]; 
   DEVICE_RECORD *record = stack_buf;
   ErrorCode ret; 

   do {
#if defined(_WIN32) || defined(WIN32)
      ret = QueryDeviceInfoDB(queryType, devPath ? devPath : L"", &count, record);
#else
      ret = QueryDeviceInfoADB(queryType, devPath ? devPath : "", &count, record);
#endif

      if (ret == ErrorBufferTooSmall) {
         if (record != stack_buf) { 
            delete[] record; 
         }
         count  += 100;
         record = new DEVICE_RECORD[count];
      }
   } while (ret == ErrorBufferTooSmall);

   //
   // Some devices should always keep active '1' due to they never send device PnP notification.
   uint32 const ignored[] = {BD_SUSIGPIO, BD_EAPIGPIO, BD_IDAQ1CHASSYNC};
   for (unsigned i = 0; i < count; ++i) {
      //
      // Keep the susiGPIO / eapiGPIO / iDAQ-Fused as active '1'
      if (ARRAY_INDEX_OF(ignored, record[i].ProductId) != -1) {
         daq_trace(("found Keeping Alive Device: %d, %d\n", record[i].DeviceNumber, record[i].OnSystem));
         continue;
      }  

      UpdateDeviceFieldI32DB(record[i].DeviceNumber, DFN_ACTIVE, 0);

      if (record[i].DriverSPEC == DRVSPEC_DN4) {
         HANDLE devEvt = DeviceCreatePnpEvent(EvtDeviceRemoved, record[i].ProductId, record[i].DeviceNumber);
         if (devEvt) {
            SetEvent(devEvt);
            CloseHandle(devEvt);
         }
      } 
   }

   if (record != stack_buf) { 
      delete[] record; 
   }
}

#if defined(_WIN32) || defined(WIN32) 
extern "C" ErrorCode BDAQCALL AdxAudioDeviceStartupInitialize(wchar_t const * devPath, wchar_t const* devDesc, const uint8 isActived)
#else
extern "C" ErrorCode BDAQCALL AdxAudioDeviceStartupInitialize(char const * devPath, char const * devDesc, const uint8 isActived)
#endif
{
   uint32        count = 10;
   DEVICE_RECORD stack_buf[10]; 
   DEVICE_RECORD *record = stack_buf;
   ErrorCode ret; 

   do {
#if defined(_WIN32) || defined(WIN32)
      ret = QueryDeviceInfoDB(ByDevPath, devPath, &count, record);
#else
      ret = QueryDeviceInfoADB(ByDevPath, devPath, &count, record);
#endif

      if (ret == ErrorBufferTooSmall) {
         if (record != stack_buf) { 
            delete[] record; 
         }
         count  += 100;
         record = new DEVICE_RECORD[count];
      }
   } while (ret == ErrorBufferTooSmall);

   for(unsigned i = 0; i < count; i++) { // this audio card is existed in config.db
      if((record[i].ProductId == BD_AudioCard) && (0 == strcmp(record[i].DriverName, "biosoundcard"))) {
         if(record != stack_buf) {
            delete[] record;
         }
 
         UpdateDeviceFieldI32DB(record[i].DeviceNumber, DFN_ACTIVE, isActived);
         return Success;
      }
   }

   if (record != stack_buf) { 
      delete[] record; 
   }

   if(!isActived) {
      return Success;
   }

   // create the new device name, and update db
#if defined(_WIN32) || defined(WIN32)
   uint32  len         = 32;
   wchar_t devName[32] = {0};
   ret = EnumToString(L"ProductId", BD_AudioCard, len, devName);
   CHK_EXPR(Success == ret, ret);

   DEVICE_HWINFO_EX hwInfo = {-1, BD_AudioCard, 0, 0, DRVSPEC_DN4, L"biosoundcard"};
   wcscpy_s(hwInfo.DeviceName, ARRAY_SIZE(hwInfo.DeviceName), devName);

   ret = AllocDeviceNumberDBForAudio(hwInfo, devPath, devDesc, (int32*)&hwInfo.DeviceNumber);
   if(ret == Success) {
      ret = UpdateDeviceFieldI32DB(hwInfo.DeviceNumber, DFN_ACTIVE, isActived);
   }
#endif

   return ret;

}

#if defined(_WIN32) || defined(WIN32) 
extern "C" void BDAQCALL AdxSetSimulatorActiveFlag(const uint8 isActived)
#else
extern "C" void BDAQCALL AdxSetSimulatorActiveFlag(const uint8 isActived)
#endif
{
   uint32        count = 10;
   DEVICE_RECORD stack_buf[10]; 
   DEVICE_RECORD *record = stack_buf;
   ErrorCode ret; 

   do {
#if defined(_WIN32) || defined(WIN32)
      ret = QueryDeviceInfoDB(ListAll, L"", &count, record);
#else
      ret = QueryDeviceInfoADB(ListAll, "", &count, record);
#endif

      if (ret == ErrorBufferTooSmall) {
         if (record != stack_buf) { 
            delete[] record; 
         }
         count  += 100;
         record = new DEVICE_RECORD[count];
      }
   } while (ret == ErrorBufferTooSmall);

   for (unsigned i = 0; i < count; ++i) {
      if(0 == strcmp(record[i].DriverName, "biosimulator")) {
         UpdateDeviceFieldI32DB(record[i].DeviceNumber, DFN_ACTIVE, isActived);
      }
   }

   if (record != stack_buf) { 
      delete[] record; 
   }
}

extern "C" void BDAQCALL AdxDeviceResyncDeviceInfo()
{
   ResyncDeviceInfoDB();
}

#if defined(_WIN32) || defined(WIN32) 
extern "C" ErrorCode BDAQCALL AdxAddSimulator(ProductId id, uint32 length, wchar_t* devPath)
#else
extern "C" ErrorCode BDAQCALL AdxAddSimulator(ProductId id, uint32 length, char * devPath)
#endif
{
   int32 boardId = -1;
   ErrorCode ret = FindFreeBoardId(id, &boardId);
   CHK_EXPR(Success == ret, ret);

   // create the new device name, and update db
#if defined(_WIN32) || defined(WIN32)
   uint32  len            = 32;
   wchar_t enumString[32] = {0};
   ret = EnumToString(L"ProductId", id, len, enumString);
   CHK_EXPR(Success == ret, ret);

   wchar_t devName[32] = {0};
   if(id == BD_DEMO) {
      swprintf_s(devName, ARRAY_SIZE(devName), L"%s", enumString);
   } else {
      swprintf_s(devName, ARRAY_SIZE(devName), L"%s%s", L"Sim", enumString);
   }

   DEVICE_HWINFO_EX hwInfo = {-1, id, boardId, 0, DRVSPEC_DN4, L"biosimulator"};
   wcscpy_s(hwInfo.DeviceName, ARRAY_SIZE(hwInfo.DeviceName), devName);

   swprintf_s(devPath, length, L"Simulator %s%s%d", devName, L"BID#", boardId);

   return AllocDeviceNumberDB(hwInfo, devPath, (int32*)&hwInfo.DeviceNumber);
#else
   return ErrorFuncNotSpted;
#endif
}
