
#include <std_decl.h>

//---------------------------------------------------------------------------------------
// DAQNAVI3 : Global method
//---------------------------------------------------------------------------------------
// Replaced by method: CalculateMaximumSectCount 
/*uint32 const MEM_THRESHOLD = 30 * 1024 * 1024;
static uint32 BfdAiCalcMaxSectCount(uint32 dataSize, double sampleRate, uint32 sectLength)
{
   uint32 sectCount = uint32(sampleRate * 1 / sectLength); // default: 1s

   if (sectCount < 2) {
      sectCount = 2;
   } 

   if (sectCount * sectLength > MEM_THRESHOLD) {
      sectCount = (MEM_THRESHOLD / dataSize)/ sectLength;
   }

   if (sectCount == 0) {
      sectCount = 1;
   } 

   return sectCount;
}
*/
uint32 DeviceSpaceToAddr(DaqDevModule *device, uint32 space, uint32 offset)
{
   uint64 xbuf[6];
   uint32 size = sizeof(xbuf);
   DaqProperty(device).Read(CFG_DeviceBaseAddresses, &size, xbuf);
   if (size == 0) { return -1; }

   // Note: 
   // The driver returned the device's addresses in two formats:
   //    1. 'int64' for memory-mapped IO address, or
   //    2. 'int32' for IO-mapped IO address and return only one address.
   // Here we had to determine which format the driver used.
   if (size > sizeof(uint32)) { 
      if (space < size / sizeof(int64)) {
         return uint32(xbuf[space]) + offset;
      }
   } else { 
      return uint32(xbuf[0]) + offset;
   }
   return -1;
}

//---------------------------------------------------------------------------------------
// DAQNAVI3 : API(Function table) based (win driver)
//---------------------------------------------------------------------------------------
// DaqNavi3Device
// Override : DaqModule
ErrorCode DN3Api::DN3Device::Initialize(int32 deviceNumber, AccessMode mode, DriverApis *apis)
{
   m_deviceNumber = deviceNumber;
   m_mode         = mode;
   m_apis         = apis;

   assert(m_apis);
   ErrorCode ret = m_apis->pfnDeviceOpen(deviceNumber, mode, &m_device);
   if (ret == Success && mode == ModeWriteWithReset) {
      m_apis->pfnDeviceReset(m_device, 1);
   }

   return ret;
}

int32 DN3Api::DN3Device::getNumber()    // device number 
{
   return m_deviceNumber;
}
AccessMode DN3Api::DN3Device::getMode()
{
   return m_mode;
}
ErrorCode DN3Api::DN3Device::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   ErrorCode ret;
   if (flag & PropFlag::OpGet) {
      if (flag & PropFlag::SysDB) { // read from system database
         ret = m_apis->pfnPropertyReadFromRegistry(m_device, id, size, buffer, &size);
      } else { // read from device object
         ret = m_apis->pfnPropertyRead(m_device, id, size, buffer, &size, NULL);
      }
   } else {
      if (flag & PropFlag::SysDB) { // write to system database
         ret = m_apis->pfnPropertyWriteToRegistry(m_device, id, size, buffer);
      } else {
         ret = m_apis->pfnPropertyWrite(m_device, id, size, buffer, flag);
      }
   }
   return ErrorCode(ret);
}
ErrorCode DN3Api::DN3Device::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_apis->pfnEventGetHandle(m_device, id, &handle);
}
ErrorCode DN3Api::DN3Device::EventGetStatus(EventId id, EventStatus &status)
{
   return m_apis->pfnEventGetLastStatus(m_device, id, &status.General.lParam, &status.General.rParam);
}
ErrorCode DN3Api::DN3Device::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_apis->pfnEventClearFlag(m_device, id, flags.General.lParam, flags.General.rParam);
}

// Override : DaqDevModule
void DN3Api::DN3Device::Close()
{
   // free all sub-modules
   ModuleArray::iterator itor = m_modules.begin();
   for (; itor != m_modules.end(); ++itor) {
      switch((*itor)->getType())
      {
      case DaqAi:      delete (DN3Ai   *)(*itor); break;
      case DaqAo:      delete (DN3Ao   *)(*itor); break;
      case DaqDio:     delete (DN3Dio  *)(*itor); break;
      case DaqCounter: delete (DN3Cntr *)(*itor); break;
      default: break;
      }
   }

   m_modules.clear();

   // close underlying device
   if (m_device) {
      m_apis->pfnDeviceClose(m_device);
   }

   m_device = NULL;

   delete this;
}

ErrorCode DN3Api::DN3Device::Refresh()
{
   return m_apis->pfnDeviceRefresh(m_device);
}
ErrorCode DN3Api::DN3Device::Reset(uint32 state)
{
   return m_apis->pfnDeviceReset(m_device, state);
}
ErrorCode DN3Api::DN3Device::GetModule(ModuleType type, uint32 index, DaqModule* &module)
{
   module = NULL;

   // Search the module in the cached module list
   ModuleArray::iterator itor = m_modules.begin();
   for (; itor != m_modules.end(); ++itor) {
      if ((*itor)->getType() == type && (*itor)->getNumber() == (int32)index) {
         module = *itor;
         return Success;
      }
   }

   // Create a new module
   HANDLE modHandle = NULL;
   ErrorCode ret = m_apis->pfnDeviceGetFuncHandle(m_device, type, index, &modHandle);
   if (ret != Success) {
      return ret;
   }

   switch(type)
   {
   case DaqAi:      module = new DN3Ai(this, index, modHandle, m_apis);   break;
   case DaqAo:      module = new DN3Ao(this, index, modHandle, m_apis);   break;
   case DaqDio:     module = new DN3Dio(this, index, modHandle, m_apis);  break;
   case DaqCounter: module = new DN3Cntr(this, index, modHandle, m_apis); break;
   case DaqFw:      module = new DN3Fw(this, index, modHandle, m_apis);   break;
   default: return ErrorFuncNotSpted;
   }

   m_modules.push_back(module);
   return Success;
}

// Undocumented functions
ErrorCode DN3Api::DN3Device::ReadRegister(uint32 space, uint32 offset, uint32 length, void *buffer)
{
   uint32 startAddr = DeviceSpaceToAddr(this, space, offset);
   return startAddr == uint32(-1) ? ErrorParamOutOfRange : m_apis->pfnDeviceReadPorts(m_device, startAddr, length, buffer);
}
ErrorCode DN3Api::DN3Device::WriteRegister(uint32 space, uint32 offset, uint32 length, void *buffer)
{
   uint32 startAddr = DeviceSpaceToAddr(this, space, offset);
   return startAddr == uint32(-1) ? ErrorParamOutOfRange : m_apis->pfnDeviceWritePorts(m_device, startAddr, length, buffer);
}
ErrorCode DN3Api::DN3Device::ReadPrivRgn(uint32 signature, uint32 dataLen, uint8 *dataBuf)
{
   return m_apis->pfnDevicePrivateRegionRead(m_device, signature, dataLen, dataBuf);
}
ErrorCode DN3Api::DN3Device::WritePrivRgn(uint32 signature, uint32 dataLen, uint8 *dataBuf)
{
   return m_apis->pfnDevicePrivateRegionWrite(m_device, signature, dataLen, dataBuf);
}

//
// DaqNavi3Ai
// Override : DaqModule
int32 DN3Api::DN3Ai::getNumber()
{
   return m_index;
}
AccessMode DN3Api::DN3Ai::getMode()
{
   return m_parent->getMode();
}
ErrorCode  DN3Api::DN3Ai::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   ErrorCode ret;
   if (flag & PropFlag::OpGet) {
      if (flag & PropFlag::SysDB) { // read from system database
         ret = m_apis->pfnPropertyReadFromRegistry(m_module, id, size, buffer, &size);
      } else { // read from device object
         switch(id)
         {
         case CFG_RecordSectionLength: ret = DefCopyT(m_sectLength, buffer, size); break;
         case CFG_RecordSectionCount:  ret = DefCopyT(m_sectCount, buffer, size); break;
         default: ret = m_apis->pfnPropertyRead(m_module, id, size, buffer, &size, NULL); break;
         }
      }
   } else {
      if (flag & PropFlag::SysDB) { // write to system database
         ret = m_apis->pfnPropertyWriteToRegistry(m_module, id, size, buffer);
      } else {
         switch(id)
         {
         case CFG_RecordSectionLength: 
            {
               CHK_USER_BUF_LEN(buffer, size >= sizeof(uint32));

               m_sectLength = *(uint32*)buffer;
               uint32 xval = m_sectLength * m_scanChanCount;
               ret = m_apis->pfnPropertyWrite(m_module, CFG_SectionDataCount, size, &xval, flag); 
            }
            break;
         case CFG_RecordSectionCount: 
            CHK_USER_BUF_LEN(buffer, size >= sizeof(uint32));

            m_sectCount = *(uint32*)buffer;
            ret = Success; 
            break;
         case CFG_ScanChannelCount:
            {
               CHK_USER_BUF_LEN(buffer, size >= sizeof(uint32));
               m_scanChanCount = *(uint32*)buffer;
               ret = m_apis->pfnPropertyWrite(m_module, id, size, buffer, flag); 

               // Fix bug: 9325 -- re-init 'CFG_SectionDataCount' after the 'CFG_ScanChannelCount' changed
               if (!m_useDN3SectDataCount) { // Fix bug: 12334
                  uint32 xval = m_sectLength * m_scanChanCount;
                  m_apis->pfnPropertyWrite(m_module, CFG_SectionDataCount, size, &xval, flag); 
               }
            }
            break;
         // Fix bug: 12334 -- wrong 'SectionDataCount' is used when DN3 control access DN3 driver via this DLL
         case CFG_SectionDataCount: 
            m_useDN3SectDataCount = true;
            ret = m_apis->pfnPropertyWrite(m_module, id, size, buffer, flag); 
            break;
         default:
            ret = m_apis->pfnPropertyWrite(m_module, id, size, buffer, flag); 
            break;
         }
      }
   }
   return ret;
}

ErrorCode DN3Api::DN3Ai::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_apis->pfnEventGetHandle(m_module, id, &handle);
}
ErrorCode DN3Api::DN3Ai::EventGetStatus(EventId id, EventStatus &status)
{
   if (id == EvtBufferedAiStopped) {
      return m_apis->pfnEventGetLastStatus(m_module, EvtBufferedAiStopped, &status.AiDataReady.Stopped, &status.AiDataReady.DataCount);
   }
   return m_apis->pfnEventGetLastStatus(m_module, id, &status.General.lParam, &status.General.rParam);
}
ErrorCode DN3Api::DN3Ai::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_apis->pfnEventClearFlag(m_module, id, flags.General.lParam, flags.General.rParam);
}

// Override : DaqAiModule
ErrorCode DN3Api::DN3Ai::ReadSamples(uint32 chStart, uint32 chCount, void *rawData, double *scaledData)
{
   return m_apis->pfnAiReadSamples(m_module, chStart, chCount, rawData, scaledData);
}
ErrorCode DN3Api::DN3Ai::BfdAiPrepare(uint32 &dataCount, void* &dataPtr)
{
   ErrorCode ret;

   if (dataCount == 0) {
      if (m_sectCount != 0) {
         dataCount = m_sectCount * m_sectLength * m_scanChanCount;
         ret = m_apis->pfnBufferedAiPrepare(m_module, dataCount, &dataPtr);
      } else {
         double sampleRate;
         uint32 size = sizeof(double);
         PropAccess(CFG_ConvertClockRatePerChannel, size, &sampleRate, PropFlag::OpGet);

         uint32 xlength = m_sectLength * m_scanChanCount;
         uint32 xcount  = CalculateMaximumSectCount(m_dataSize, sampleRate * m_scanChanCount, xlength);

         do {
            dataCount = xlength * xcount;
            ret = m_apis->pfnBufferedAiPrepare(m_module, dataCount, &dataPtr);
            if (ret == ErrorMemoryNotEnough) {
               // Buffer size too big, decrease it and try again
               if (xcount > 10) {
                  xcount /= 2;
               } else {
                  --xcount;
               }
            }
         } while (ret == ErrorMemoryNotEnough && xcount);
      }
   } else {
      ret = m_apis->pfnBufferedAiPrepare(m_module, dataCount, &dataPtr);
   }

   m_dataBufLen = dataCount;
   m_dataBuffer = (uint8*)dataPtr;
   return ret;
}

ErrorCode DN3Api::DN3Ai::BfdAiStart()
{
   if (m_sectCount) {
      return m_apis->pfnBufferedAiRunOnce(m_module, true);
   } else {
      return m_apis->pfnBufferedAiRun(m_module);
   }
}
ErrorCode DN3Api::DN3Ai::BfdAiStop()
{
   return m_apis->pfnBufferedAiStop(m_module);
}
ErrorCode DN3Api::DN3Ai::BfdAiRelease()
{
   return m_apis->pfnBufferedAiRelease(m_module);
}

ErrorCode DN3Api::DN3Ai::BfdAiGetData(uint32 &dataCount, void *rawBuf, double *dblBuf, double *startTime, uint32 *markCount, DataMark *)
{
   if (!m_dataBuffer) { return ErrorFuncNotInited; }

   if (startTime) { *startTime = 0; } // Unknown start time
   if (markCount) { *markCount = 0; } // No data marker

   // get current 'Read Position' and 'data count'
   uint32 count, offset;
   m_apis->pfnEventGetLastStatus(m_module, EvtBufferedAiDataReady, &count, &offset);
   count -= count % m_scanChanCount;
   
   if (dataCount > count) { dataCount = count; }
   if (!dataCount)        { return Success;    }

   // Raw Data
   if (rawBuf) {
      // copy the first segment, it's from 'offset' to 'buffer end', 'cpyLen' length
      uint32 cpyLen = x_min(dataCount, m_dataBufLen - offset);
      memcpy(rawBuf, m_dataBuffer + offset * m_dataSize, cpyLen * m_dataSize);

      // copy the rest from 'buffer start'
      if (dataCount > cpyLen) {
         memcpy((uint8*)rawBuf + cpyLen * m_dataSize, m_dataBuffer, (dataCount - cpyLen) * m_dataSize);
      }
   }

   // Scaled Data
   if (dblBuf) {
      uint32 chOffset = offset % m_scanChanCount;

      // copy the first segment, it's from 'offset' to 'buffer end', 'scaleLen' length
      uint32 scaleLen = x_min(dataCount, m_dataBufLen - offset);
      m_apis->pfnBufferedAiScaleData(m_module, m_dataBuffer + offset * m_dataSize, &dblBuf[0], scaleLen, &chOffset);

      // copy the rest from 'buffer start'
      if (dataCount > scaleLen) {
         m_apis->pfnBufferedAiScaleData(m_module, m_dataBuffer, &dblBuf[scaleLen], dataCount - scaleLen, &chOffset);
      }
   }

   return Success;
}

// Undocumented functions
ErrorCode DN3Api::DN3Ai::GetScaleTable(uint32 ch, uint32 &size, MapFuncPiece *tables)
{
   uint32 vrgs[MAX_AI_CH_COUNT];
   m_apis->pfnPropertyRead(m_module, CFG_VrgTypeOfChannels, sizeof(vrgs), vrgs, NULL, NULL);
   return m_apis->pfnAiGetScaleTable(m_module, vrgs[ch], &size, tables);
}
ErrorCode DN3Api::DN3Ai::SetScaleTable(uint32 ch, uint32 size, MapFuncPiece *tables)
{
   return ErrorFuncNotSpted;
}

// Legacy, For compatible ONLY!!!
ErrorCode DN3Api::DN3Ai::GetChanMapping(uint32 count, uint32 *buffer)
{
   return m_apis->pfnAiGetChanMapping(m_module, count, buffer);
}
//
// DaqNavi3Ao
// Override : DaqModule
int32 DN3Api::DN3Ao::getNumber()    // module index 
{
   return m_index;
}
AccessMode DN3Api::DN3Ao::getMode()
{
   return m_parent->getMode();
}
ErrorCode  DN3Api::DN3Ao::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   ErrorCode ret;
   if (flag & PropFlag::OpGet) {
      if (flag & PropFlag::SysDB) { // read from system database
         ret = m_apis->pfnPropertyReadFromRegistry(m_module, id, size, buffer, &size);
      } else { // read from device object
         ret = m_apis->pfnPropertyRead(m_module, id, size, buffer, &size, NULL);
      }
   } else {
      if (flag & PropFlag::SysDB) { // write to system database
         ret = m_apis->pfnPropertyWriteToRegistry(m_module, id, size, buffer);
      } else {
         ret = m_apis->pfnPropertyWrite(m_module, id, size, buffer, flag);
      }
   }
   return ret;
}
ErrorCode DN3Api::DN3Ao::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_apis->pfnEventGetHandle(m_module, id, &handle);
}
ErrorCode DN3Api::DN3Ao::EventGetStatus(EventId id, EventStatus &status)
{
   return m_apis->pfnEventGetLastStatus(m_module, id, &status.General.lParam, &status.General.rParam);
}
ErrorCode DN3Api::DN3Ao::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_apis->pfnEventClearFlag(m_module, id, flags.General.lParam, flags.General.rParam);
}
// Override : DaqAoModule
ErrorCode DN3Api::DN3Ao::WriteSamples(uint32 chStart, uint32 chCount, void *rawData, double *scaledData)
{
   return m_apis->pfnAoWriteSamples(m_module, chStart, chCount, rawData, scaledData);
}
ErrorCode DN3Api::DN3Ao::BfdAoPrepare(uint32 dataCount, void* &dataPtr)
{
   return m_apis->pfnBufferedAoPrepare(m_module, dataCount, &dataPtr);
}
ErrorCode DN3Api::DN3Ao::BfdAoRunOnce(bool asynchronous)
{
   return m_apis->pfnBufferedAoRunOnce(m_module, asynchronous);
}
ErrorCode DN3Api::DN3Ao::BfdAoRun()
{
   return m_apis->pfnBufferedAoRun(m_module);
}
ErrorCode DN3Api::DN3Ao::BfdAoStop(uint32 action)
{
   return m_apis->pfnBufferedAoStop(m_module, action);
}
ErrorCode DN3Api::DN3Ao::BfdAoRelease()
{
   return m_apis->pfnBufferedAoRelease(m_module);
}
ErrorCode DN3Api::DN3Ao::BfdAoScaleData(double *scaledData, void *rawData, uint32 count, uint32 *chanOffset)
{
   return m_apis->pfnBufferedAoScaleData(m_module, scaledData, rawData, count, chanOffset);
}

// Undocumented functions
ErrorCode DN3Api::DN3Ao::GetScaleTable(uint32 ch, uint32 &size, MapFuncPiece *tables)
{
   uint32 vrgs[MAX_AO_CH_COUNT];
   m_apis->pfnPropertyRead(m_module, CFG_VrgTypeOfChannels, sizeof(vrgs), vrgs, NULL, NULL);
   return m_apis->pfnAoGetScaleTable(m_module, vrgs[ch], &size, tables);
}
ErrorCode DN3Api::DN3Ao::SetScaleTable(uint32 ch, uint32 size, MapFuncPiece *tables)
{
   return ErrorFuncNotSpted;
}

//
// DaqNavi3Dio
// Override : DaqModule
int32 DN3Api::DN3Dio::getNumber()
{
   return m_index;
}
AccessMode DN3Api::DN3Dio::getMode()
{
   return m_parent->getMode();
}
ErrorCode DN3Api::DN3Dio::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   ErrorCode ret;
   if (flag & PropFlag::OpGet) {
      if (flag & PropFlag::SysDB) { // read from system database
         ret = m_apis->pfnPropertyReadFromRegistry(m_module, id, size, buffer, &size);
      } else { // read from device object
         ret = m_apis->pfnPropertyRead(m_module, id, size, buffer, &size, NULL);
      }
   } else {
      if (flag & PropFlag::SysDB) { // write to system database
         ret = m_apis->pfnPropertyWriteToRegistry(m_module, id, size, buffer);
      } else {
         ret = m_apis->pfnPropertyWrite(m_module, id, size, buffer, flag);
      }
   }
   return ret;
}
ErrorCode DN3Api::DN3Dio::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_apis->pfnEventGetHandle(m_module, id, &handle);
}
ErrorCode DN3Api::DN3Dio::EventGetStatus(EventId id, EventStatus &status)
{
   return m_apis->pfnEventGetLastStatus(m_module, id, &status.General.lParam, &status.General.rParam);
}
ErrorCode DN3Api::DN3Dio::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_apis->pfnEventClearFlag(m_module, id, flags.General.lParam, flags.General.rParam);
}

// Override : DaqDioModule
ErrorCode DN3Api::DN3Dio::DiReadPort(uint32 portStart, uint32 portCount, uint8 buffer[])
{
   return m_apis->pfnDiReadPorts(m_module, portStart, portCount, buffer);
}
ErrorCode DN3Api::DN3Dio::DiReadBit(uint32 port, uint32 bit, uint8 &data)
{
   return DaqDioModule::DiReadBit(port, bit, data);
}
ErrorCode DN3Api::DN3Dio::DoWritePort(uint32 portStart, uint32 portCount, uint8 buffer[])
{
   return m_apis->pfnDoWritePorts(m_module, portStart, portCount, buffer);
}
ErrorCode DN3Api::DN3Dio::DoWriteBit(uint32 port, uint32 bit, uint8 data)
{
   ErrorCode ret = m_apis->pfnDoWriteBit(m_module, port, bit, data);
   if (ret == ErrorFuncNotSpted) {
      ret = DaqDioModule::DoWriteBit(port, bit, data);
   }
   return ret;
}
ErrorCode DN3Api::DN3Dio::DoReadPort(uint32 portStart, uint32 portCount, uint8 buffer[])
{
   return m_apis->pfnDoReadBackPorts(m_module, portStart, portCount, buffer);
}
ErrorCode DN3Api::DN3Dio::DoReadBit(uint32 port, uint32 bit, uint8 &data)
{
   return DaqDioModule::DoReadBit(port, bit, data);
}
ErrorCode DN3Api::DN3Dio::DiSnapStart(EventId id, uint32 portStart, uint32 portCount, uint8 **dataPtr)
{
   return m_apis->pfnDiSnapStart(m_module, id, portStart, portCount, dataPtr);
}
ErrorCode DN3Api::DN3Dio::DiSnapStop(EventId id)
{
   return m_apis->pfnDiSnapStop(m_module, id);
}
// Undocumented functions
// 0 -- start, 1 -- stop, 2 -- feed
ErrorCode DN3Api::DN3Dio::WdtExec(int action)
{
   ErrorCode ret;
   switch(action)
   {
   case 0: ret = m_apis->pfnDoReflectWdtLockStart(m_module); break;
   case 1: ret = m_apis->pfnDoReflectWdtLockStop(m_module); break;
   case 2: ret = m_apis->pfnDoReflectWdtFeed(m_module); break;
   default: ret = ErrorFuncNotSpted;
   }
   return ret;
}

//
// DaqNavi3Cntr
// Override : DaqModule
int32 DN3Api::DN3Cntr::getNumber()
{
   return m_index;
}
AccessMode DN3Api::DN3Cntr::getMode()
{
   return m_parent->getMode();
}
ErrorCode DN3Api::DN3Cntr::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   ErrorCode ret;
   if (flag & PropFlag::OpGet) {
      if (flag & PropFlag::SysDB) { // read from system database
         ret = m_apis->pfnPropertyReadFromRegistry(m_module, id, size, buffer, &size);
      } else { // read from device object
         ret = m_apis->pfnPropertyRead(m_module, id, size, buffer, &size, NULL);
      }
   } else {
      if (flag & PropFlag::SysDB) { // write to system database
         ret = m_apis->pfnPropertyWriteToRegistry(m_module, id, size, buffer);
      } else {
         ret = m_apis->pfnPropertyWrite(m_module, id, size, buffer, flag);
      }
   }
   return ret;
}
ErrorCode DN3Api::DN3Cntr::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_apis->pfnEventGetHandle(m_module, id, &handle);
}
ErrorCode DN3Api::DN3Cntr::EventGetStatus(EventId id, EventStatus &status)
{
   return m_apis->pfnEventGetLastStatus(m_module, id, &status.General.lParam, &status.General.rParam);
}
ErrorCode DN3Api::DN3Cntr::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_apis->pfnEventClearFlag(m_module, id, flags.General.lParam, flags.General.rParam);
}

// Override : DaqCntrModule
ErrorCode DN3Api::DN3Cntr::CtReset(uint32 cntrStart, uint32 cntrCount)
{
   return m_apis->pfnCounterReset(m_module, cntrStart, cntrCount);
}
ErrorCode DN3Api::DN3Cntr::CtRead(uint32 cntrStart, uint32 cntrCount, uint32 loadVal[], uint32 holdVal[])
{
   return m_apis->pfnCounterRead(m_module, cntrStart, cntrCount, loadVal, holdVal);
}
ErrorCode DN3Api::DN3Cntr::CtStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_apis->pfnCounterStart(m_module, cntrStart, cntrCount);
}
ErrorCode DN3Api::DN3Cntr::EcStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_apis->pfnEventCountStart(m_module, cntrStart, cntrCount);
}
ErrorCode DN3Api::DN3Cntr::EcRead(uint32 cntrStart, uint32 cntrCount, uint32 cntrValue[])
{
   return m_apis->pfnEventCountRead(m_module, cntrStart, cntrCount, cntrValue);
}
ErrorCode DN3Api::DN3Cntr::OsStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_apis->pfnOneShotStart(m_module, cntrStart, cntrCount);
}
ErrorCode DN3Api::DN3Cntr::TmrStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_apis->pfnTimerPulseStart(m_module, cntrStart, cntrCount);
}
ErrorCode DN3Api::DN3Cntr::FmStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_apis->pfnFreqMeasureStart(m_module, cntrStart, cntrCount);
}
ErrorCode DN3Api::DN3Cntr::FmRead(uint32 cntrStart, uint32 cntrCount, double frequency[])
{
   return m_apis->pfnFreqMeasureRead(m_module, cntrStart, cntrCount, frequency);
}
ErrorCode DN3Api::DN3Cntr::PiStart(uint32 cntrStart, uint32 groupCount)
{
   return m_apis->pfnPwmInStart(m_module, cntrStart, groupCount);
}
ErrorCode DN3Api::DN3Cntr::PiRead(uint32 cntrStart, uint32 groupCount, double hiPeriod[], double lowPeriod[])
{
   return m_apis->pfnPwmInRead(m_module, cntrStart, groupCount, hiPeriod, lowPeriod);
}
ErrorCode DN3Api::DN3Cntr::PoStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_apis->pfnPwmOutStart(m_module, cntrStart, cntrCount);
}
ErrorCode DN3Api::DN3Cntr::UdStart(uint32 cntrStart, uint32 cntrCount, uint32 resetCounter)
{
   return m_apis->pfnUpDownCountStart(m_module, cntrStart, cntrCount, resetCounter);
}
ErrorCode DN3Api::DN3Cntr::UdResetValue(uint32 cntrStart, uint32 cntrCount)
{
   return m_apis->pfnUpDownCountValueReset(m_module, cntrStart, cntrCount);
}
ErrorCode DN3Api::DN3Cntr::UdRead(uint32 cntrStart, uint32 cntrCount, uint32 cntrValue[])
{
   return m_apis->pfnUpDownCountRead(m_module, cntrStart, cntrCount, cntrValue);
}
ErrorCode DN3Api::DN3Cntr::CtSnapStart(EventId id, uint32 cntrStart, uint32 cntrCount, uint32 **loadValues, uint32 **holdValues)
{
   return m_apis->pfnCounterSnapStart(m_module, id, cntrStart, cntrCount, loadValues, holdValues);
}
ErrorCode DN3Api::DN3Cntr::CtSnapStop(EventId id)
{
   return m_apis->pfnCounterSnapStop(m_module, id);
}
ErrorCode DN3Api::DN3Cntr::CCmpSetTable(uint32 counter, uint32 dataCount, uint32 buffer[])
{
   return m_apis->pfnCounterCompareSetTable(m_module, counter, dataCount, buffer);
}
ErrorCode DN3Api::DN3Cntr::CCmpSetInterval(uint32 counter, uint32 firstValue, uint32 increment, uint32 count)
{
   return m_apis->pfnCounterCompareSetInterval(m_module, counter, firstValue, increment, count);
}
ErrorCode DN3Api::DN3Cntr::CCmpClear(uint32 counter)
{
   return m_apis->pfnCounterCompareClear(m_module, counter);
}

//
// DaqNavi3FirmwareDownload
// Override : DaqModule
int32 DN3Api::DN3Fw::getNumber()
{
   return m_index;
}
AccessMode DN3Api::DN3Fw::getMode()
{
   return m_parent->getMode();
}

// Override : DaqFwModule
int32 DN3Api::DN3Fw::getCount()
{
   uint32 count = 0;
   m_apis->pfnFirmwareDownload(m_module, 0, FWDL_GET_COUNT, sizeof(count), &count);
   return count;
}
uint32 DN3Api::DN3Fw::getDescription(int32 index, uint32 size, wchar_t buffer[])
{
   buffer[0] = 0;
   m_apis->pfnFirmwareDownload(m_module, index, FWDL_GET_DESCRIPTION, size, buffer);
   return buffer[0] ? wcslen(buffer) + 1 : 0;
}
uint32 DN3Api::DN3Fw::getNotice(int32 index, uint32 size, wchar_t buffer[])
{
   buffer[0] = 0;
   m_apis->pfnFirmwareDownload(m_module, index, FWDL_GET_NOTICE, size, buffer);
   return buffer[0] ? wcslen(buffer) + 1 : 0;
}
uint32 DN3Api::DN3Fw::getVersionOnDevice(int32 index, uint32 size, wchar_t buffer[])
{
   buffer[0] = 0;
   m_apis->pfnFirmwareDownload(m_module, index, FWDL_GET_VERSION_ON_DEVICE, size, buffer);
   return buffer[0] ? wcslen(buffer) + 1 : 0;
}
uint32 DN3Api::DN3Fw::getVersionOnFile(int32 index, uint32 size, wchar_t buffer[])
{
   buffer[0] = 0;
   m_apis->pfnFirmwareDownload(m_module, index, FWDL_GET_VERSION_ON_FILE, size, buffer);
   return buffer[0] ? wcslen(buffer) + 1 : 0;
}
int32 DN3Api::DN3Fw::getOverwriteProgressing(int32 index)
{
   uint32 progress = 0;
   m_apis->pfnFirmwareDownload(m_module, 0, FWDL_GET_OVERWRITE_PROGRESSING, sizeof(progress), &progress);
   return progress;
}

ErrorCode DN3Api::DN3Fw::LoadFwFile(int32 index, wchar_t const *filePath)
{
   return m_apis->pfnFirmwareDownload(m_module, 0, FWDL_LOAD_FW_FILE, wcslen(filePath), (void *)filePath);
}
ErrorCode DN3Api::DN3Fw::Overwrite(int32 index, FwAction action, wchar_t const *activationCode)
{
   FWDLParam param;
   param.Action = action;
   if (activationCode) {
      wcsncpy(param.ActivationCode, activationCode, ARRAY_SIZE(param.ActivationCode));
   } else {
      param.ActivationCode[0] = 0;
   }
   return m_apis->pfnFirmwareDownload(m_module, 0, FWDL_OVERWRITE, sizeof(param), &param);
}


//---------------------------------------------------------------------------------------
// DAQNAVI3 : object based (linux driver)
//---------------------------------------------------------------------------------------
// DaqNavi3Device
// Override : DaqModule
ErrorCode DN3Obj::DN3Device::Initialize(int32 deviceNumber, AccessMode mode, FNOBJECTCREATE CreateObject)
{
   m_deviceNumber = deviceNumber;
   m_mode         = mode;

   assert(CreateObject);
   ErrorCode ret = CreateObject(deviceNumber, mode, &m_device);
   if (ret == Success && mode == ModeWriteWithReset) {
      m_device->Reset(1);
   }

   return ret;
}

int32 DN3Obj::DN3Device::getNumber()    // device number 
{
   return m_deviceNumber;
}
AccessMode DN3Obj::DN3Device::getMode()
{
   return m_mode;
}
ErrorCode DN3Obj::DN3Device::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   return m_device->PropAccess(id, size, buffer, NULL, flag );
}
ErrorCode DN3Obj::DN3Device::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_device->EventGetHandle(id, handle);
}
ErrorCode DN3Obj::DN3Device::EventGetStatus(EventId id, EventStatus &status)
{
   return m_device->EventGetStatus(id, &status.General.lParam, &status.General.rParam);
}
ErrorCode DN3Obj::DN3Device::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_device->EventClearFlag(id, flags.General.lParam, flags.General.rParam);
}

// Override : DaqDevModule
void DN3Obj::DN3Device::Close()
{
   // free all sub-modules
   ModuleArray::iterator itor = m_modules.begin();
   for (; itor != m_modules.end(); ++itor) {
      switch((*itor)->getType())
      {
      case DaqAi:      delete (DN3Ai   *)(*itor); break;
      case DaqAo:      delete (DN3Ao   *)(*itor); break;
      case DaqDio:     delete (DN3Dio  *)(*itor); break;
      case DaqCounter: delete (DN3Cntr *)(*itor); break;
      default: break;
      }
   }

   m_modules.clear();

   // close underlying device
   if (m_device) {
      m_device->Close();
   }
   m_device = NULL;

   delete this;
}

ErrorCode DN3Obj::DN3Device::Refresh()
{
   return m_device->Refresh();
}
ErrorCode DN3Obj::DN3Device::Reset(uint32 state)
{
   return m_device->Reset(state);
}
ErrorCode DN3Obj::DN3Device::GetModule(ModuleType type, uint32 index, DaqModule* &module)
{
   module = NULL;

   // Search the module in the cached module list
   ModuleArray::iterator itor = m_modules.begin();
   for (; itor != m_modules.end(); ++itor) {
      if ((*itor)->getType() == type && (*itor)->getNumber() == (int32)index) {
         module = (*itor);
         return Success;
      }
   }

   // Create a new module
   BDaqModule * xmodule = NULL;
   ErrorCode ret = m_device->GetModule(type, index, &xmodule);
   if (ret != Success) {
      return ret;
   }

   switch(type)
   {
   case DaqAi:      module = new DN3Ai(this, index,  (BDaqAi*)xmodule);   break;
   case DaqAo:      module = new DN3Ao(this, index,  (BDaqAo*)xmodule);   break;
   case DaqDio:     module = new DN3Dio(this, index, (BDaqDio*)xmodule);  break;
   case DaqCounter: module = new DN3Cntr(this, index,(BDaqCntr*)xmodule); break;
   case DaqFw:      module = new DN3Fw(this, index, (BDaqFw*)xmodule);    break;
   default: return ErrorFuncNotSpted;
   }

   m_modules.push_back(module);
   return Success;
}

// Undocumented functions
ErrorCode DN3Obj::DN3Device::ReadRegister(uint32 space, uint32 offset, uint32 length, void *buffer)
{
   uint32 startAddr = DeviceSpaceToAddr(this, space, offset);
   return startAddr == uint32(-1) ? ErrorParamOutOfRange : m_device->ReadPorts(startAddr, length, buffer);
}
ErrorCode DN3Obj::DN3Device::WriteRegister(uint32 space, uint32 offset, uint32 length, void *buffer)
{
   uint32 startAddr = DeviceSpaceToAddr(this, space, offset);
   return startAddr == uint32(-1) ? ErrorParamOutOfRange : m_device->WritePorts(startAddr, length, buffer);
}
ErrorCode DN3Obj::DN3Device::ReadPrivRgn(uint32 signature, uint32 dataLen, uint8 *dataBuf)
{
   return m_device->ReadPrivRgn(signature, dataLen, dataBuf);
}
ErrorCode DN3Obj::DN3Device::WritePrivRgn(uint32 signature, uint32 dataLen, uint8 *dataBuf)
{
   return m_device->WritePrivRgn(signature, dataLen, dataBuf);
}

//
// DaqNavi3Ai
// Override : DaqModule
int32 DN3Obj::DN3Ai::getNumber()
{
   return m_index;
}
AccessMode DN3Obj::DN3Ai::getMode()
{
   return m_module->getMode();
}
ErrorCode  DN3Obj::DN3Ai::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   ErrorCode ret;
   if (flag & PropFlag::OpGet) {
      switch(id)
      {
      case CFG_RecordSectionLength: ret = DefCopyT(m_sectLength, buffer, size); break;
      case CFG_RecordSectionCount:  ret = DefCopyT(m_sectCount, buffer, size); break;
      default: ret = m_module->PropAccess(id, size, buffer, NULL, flag | OPFLAG_READ); break;
      }
   } else {
      switch(id)
      {
      case CFG_RecordSectionLength: 
         {
            CHK_USER_BUF_LEN(buffer, size >= sizeof(uint32));

            m_sectLength = *(uint32*)buffer;
            uint32 xval = m_sectLength * m_scanChanCount;
            ret = m_module->PropAccess(CFG_SectionDataCount, size, &xval, NULL, flag); 
         }
         break;
      case CFG_RecordSectionCount: 
         CHK_USER_BUF_LEN(buffer, size >= sizeof(uint32));

         m_sectCount = *(uint32*)buffer;
         ret = Success; 
         break;
      case CFG_ScanChannelCount:
         {
            CHK_USER_BUF_LEN(buffer, size >= sizeof(uint32));
            m_scanChanCount = *(uint32*)buffer;
            ret = m_module->PropAccess(id, size, buffer, NULL, flag); 

            // Fix bug: 9325 -- re-init 'CFG_SectionDataCount' after the 'CFG_ScanChannelCount' changed
            if (!m_useDN3SectDataCount) { // Fix bug: 12334
               uint32 xval = m_sectLength * m_scanChanCount;
               m_module->PropAccess(CFG_SectionDataCount, size, &xval, NULL, flag); 
            }
         }
         break;
      // Fix bug: 12334 -- wrong 'SectionDataCount' is used when DN3 control access DN3 driver via this DLL
      case CFG_SectionDataCount: 
         m_useDN3SectDataCount = true;
         ret = m_module->PropAccess(id, size, buffer, NULL, flag); 
         break;
      default:
         ret = m_module->PropAccess(id, size, buffer, NULL, flag); 
         break;
      }
   }
   return ret;
}
ErrorCode  DN3Obj::DN3Ai::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_module->EventGetHandle(id, handle);
}
ErrorCode  DN3Obj::DN3Ai::EventGetStatus(EventId id, EventStatus &status)
{
   if (id == EvtBufferedAiStopped) {
      return m_module->EventGetStatus(EvtBufferedAiStopped, &status.AiDataReady.Stopped, &status.AiDataReady.DataCount);
   }
   return m_module->EventGetStatus(id, &status.General.lParam, &status.General.rParam);
}
ErrorCode  DN3Obj::DN3Ai::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_module->EventClearFlag(id, flags.General.lParam, flags.General.rParam);
}

// Override : DaqAiModule
ErrorCode DN3Obj::DN3Ai::ReadSamples(uint32 chStart, uint32 chCount, void *rawData, double *scaledData)
{
   return m_module->ReadSamples(chStart, chCount, rawData, scaledData);
}
ErrorCode DN3Obj::DN3Ai::BfdAiPrepare(uint32 &dataCount, void* &dataPtr)
{
   ErrorCode ret;

   if (dataCount == 0) {
      if (m_sectCount != 0) {
         dataCount = m_sectCount * m_sectLength * m_scanChanCount;
         ret = m_module->BfdAiPrepare(dataCount, &dataPtr);
      } else {
         double sampleRate;
         uint32 size = sizeof(double);
         PropAccess(CFG_ConvertClockRatePerChannel, size, &sampleRate, PropFlag::OpGet);

         uint32 xlength = m_sectLength * m_scanChanCount;
         uint32 xcount  = CalculateMaximumSectCount(m_dataSize, sampleRate * m_scanChanCount, xlength);

         do {
            dataCount = xlength * xcount;
            ret = m_module->BfdAiPrepare(dataCount, &dataPtr);
            if (ret == ErrorMemoryNotEnough) {
               // Buffer size too big, decrease it and try again
               if (xcount > 10) {
                  xcount /= 2;
               } else {
                  --xcount;
               }
            }
         } while (ret == ErrorMemoryNotEnough && xcount);
      }
   } else {
      ret = m_module->BfdAiPrepare(dataCount, &dataPtr);
   }

   m_dataBufLen = dataCount;
   m_dataBuffer = (uint8*)dataPtr;
   return ret;
}
ErrorCode DN3Obj::DN3Ai::BfdAiStart()
{
   if (m_sectCount) {
      return m_module->BfdAiRunOnce(true);
   } else {
      return m_module->BfdAiRun();
   }
}
ErrorCode DN3Obj::DN3Ai::BfdAiStop()
{
   return m_module->BfdAiStop();
}
ErrorCode DN3Obj::DN3Ai::BfdAiRelease()
{
   return m_module->BfdAiRelease();
}
ErrorCode DN3Obj::DN3Ai::BfdAiGetData(uint32 &dataCount, void *rawBuf, double *dblBuf, double *startTime, uint32 *markCount, DataMark *)
{
   if (!m_dataBuffer) { return ErrorFuncNotInited; }

   if (startTime) { *startTime = 0; } // Unknown start time
   if (markCount) { *markCount = 0; } // No data marker

   // get current 'Read Position' and 'data count'
   uint32 count, offset;
   m_module->EventGetStatus(EvtBufferedAiDataReady, &count, &offset);
   count -= count % m_scanChanCount;

   if (dataCount > count) { dataCount = count; }
   if (!dataCount)        { return Success;    }

   // Raw Data
   if (rawBuf) {
      // copy the first segment, it's from 'offset' to 'buffer end', 'cpyLen' length
      uint32 cpyLen = x_min(dataCount, m_dataBufLen - offset);
      memcpy(rawBuf, m_dataBuffer + offset * m_dataSize, cpyLen * m_dataSize);

      // copy the rest from 'buffer start'
      if (dataCount > cpyLen) {
         memcpy((uint8*)rawBuf + cpyLen * m_dataSize, m_dataBuffer, (dataCount - cpyLen) * m_dataSize);
      }
   }

   // Scaled Data
   if (dblBuf) {
      uint32 chOffset = offset % m_scanChanCount;

      // copy the first segment, it's from 'offset' to 'buffer end', 'scaleLen' length
      uint32 scaleLen = x_min(dataCount, m_dataBufLen - offset);
      m_module->BfdAiScaleData(m_dataBuffer + offset * m_dataSize, &dblBuf[0], scaleLen, &chOffset);

      // copy the rest from 'buffer start'
      if (dataCount > scaleLen) {
         m_module->BfdAiScaleData(m_dataBuffer, &dblBuf[scaleLen], dataCount - scaleLen, &chOffset);
      }
   }

   return Success;
}

// Undocumented functions
ErrorCode DN3Obj::DN3Ai::GetScaleTable(uint32 ch, uint32 &size, MapFuncPiece *tables)
{
   uint32 vrgs[MAX_AI_CH_COUNT];
   uint32 xlen = sizeof(vrgs);
   m_module->PropAccess(CFG_VrgTypeOfChannels, xlen, vrgs, NULL, PropFlag::OpGet);
   return m_module->GetScaleTable(vrgs[ch], &size, tables);
}
ErrorCode DN3Obj::DN3Ai::SetScaleTable(uint32 ch,uint32 size, MapFuncPiece *tables)
{
   return ErrorFuncNotSpted;
}


//
// DaqNavi3Ao
// Override : DaqModule
int32 DN3Obj::DN3Ao::getNumber()    // module index 
{
   return m_index;
}
AccessMode DN3Obj::DN3Ao::getMode()
{
   return m_module->getMode();
}
ErrorCode  DN3Obj::DN3Ao::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   return m_module->PropAccess(id, size, buffer, NULL, flag);
}
ErrorCode  DN3Obj::DN3Ao::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_module->EventGetHandle(id, handle);
}
ErrorCode  DN3Obj::DN3Ao::EventGetStatus(EventId id, EventStatus &status)
{
   return m_module->EventGetStatus(id, &status.General.lParam, &status.General.rParam);
}
ErrorCode  DN3Obj::DN3Ao::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_module->EventClearFlag(id, flags.General.lParam, flags.General.rParam);
}
// Override : DaqAoModule
ErrorCode DN3Obj::DN3Ao::WriteSamples(uint32 chStart, uint32 chCount, void *rawData, double *scaledData)
{
   return m_module->WriteSamples(chStart, chCount, rawData, scaledData);
}
ErrorCode DN3Obj::DN3Ao::BfdAoPrepare(uint32 dataCount, void* &dataPtr)
{
   return m_module->BfdAoPrepare(dataCount, &dataPtr);
}
ErrorCode DN3Obj::DN3Ao::BfdAoRunOnce(bool asynchronous)
{
   return m_module->BfdAoRunOnce(asynchronous);
}
ErrorCode DN3Obj::DN3Ao::BfdAoRun()
{
   return m_module->BfdAoRun();
}
ErrorCode DN3Obj::DN3Ao::BfdAoStop(uint32 action)
{
   return m_module->BfdAoStop(action);
}
ErrorCode DN3Obj::DN3Ao::BfdAoRelease()
{
   return m_module->BfdAoRelease();
}
ErrorCode DN3Obj::DN3Ao::BfdAoScaleData(double *scaledData, void *rawData, uint32 count, uint32 *chanOffset)
{
   return m_module->BfdAoScaleData(scaledData, rawData, count, chanOffset);
}

// Undocumented functions
ErrorCode DN3Obj::DN3Ao::GetScaleTable(uint32 ch, uint32 &size, MapFuncPiece *tables)
{
   uint32 vrgs[MAX_AI_CH_COUNT];
   uint32 xlen = sizeof(vrgs);
   m_module->PropAccess(CFG_VrgTypeOfChannels, xlen, vrgs, NULL, PropFlag::OpGet);
   return m_module->GetScaleTable(vrgs[ch], &size, tables);
}
ErrorCode DN3Obj::DN3Ao::SetScaleTable(uint32 ch, uint32 size, MapFuncPiece *tables)
{
   return ErrorFuncNotSpted;
}

//
// DaqNavi3Dio
// Override : DaqModule
int32 DN3Obj::DN3Dio::getNumber()
{
   return m_index;
}
AccessMode DN3Obj::DN3Dio::getMode()
{
   return m_module->getMode();
}
ErrorCode  DN3Obj::DN3Dio::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   return m_module->PropAccess(id, size, buffer, NULL, flag);
}
ErrorCode  DN3Obj::DN3Dio::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_module->EventGetHandle(id, handle);
}
ErrorCode  DN3Obj::DN3Dio::EventGetStatus(EventId id, EventStatus &status)
{
   return m_module->EventGetStatus(id, &status.General.lParam, &status.General.rParam);
}
ErrorCode  DN3Obj::DN3Dio::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_module->EventClearFlag(id, flags.General.lParam, flags.General.rParam);
}

// Override : DaqDioModule
ErrorCode DN3Obj::DN3Dio::DiReadPort(uint32 portStart, uint32 portCount, uint8 buffer[])
{
   return m_module->ReadDiPorts(portStart, portCount, buffer);
}
ErrorCode DN3Obj::DN3Dio::DiReadBit(uint32 port, uint32 bit, uint8 &data)
{
   return m_module->DiReadBit(port, bit, data);
}
ErrorCode DN3Obj::DN3Dio::DoWritePort(uint32 portStart, uint32 portCount, uint8 buffer[])
{
   return m_module->WriteDoPorts(portStart, portCount, buffer);
}
ErrorCode DN3Obj::DN3Dio::DoWriteBit(uint32 port, uint32 bit, uint8 data)
{
   return m_module->DoWriteBit(port, bit, data);
}
ErrorCode DN3Obj::DN3Dio::DoReadPort(uint32 portStart, uint32 portCount, uint8 buffer[])
{
   return m_module->ReadDoPorts(portStart, portCount, buffer);
}
ErrorCode DN3Obj::DN3Dio::DoReadBit(uint32 port, uint32 bit, uint8 &data)
{
   return m_module->DoReadBit(port, bit, data);
}
ErrorCode DN3Obj::DN3Dio::DiSnapStart(EventId id, uint32 portStart, uint32 portCount, uint8 **dataPtr)
{
   return m_module->StartDiSnap(id, portStart, portCount, dataPtr);
}
ErrorCode DN3Obj::DN3Dio::DiSnapStop(EventId id)
{
   return m_module->StopDiSnap(id);
}
// Undocumented functions
// 0 -- start, 1 -- stop, 2 -- feed
ErrorCode DN3Obj::DN3Dio::WdtExec(int action)
{
   return m_module->WdtExec(action);
}

//
// DaqNavi3Cntr
// Override : DaqModule
int32 DN3Obj::DN3Cntr::getNumber()
{
   return m_index;
}
AccessMode DN3Obj::DN3Cntr::getMode()
{
   return m_module->getMode();
}
ErrorCode  DN3Obj::DN3Cntr::PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag)
{
   return m_module->PropAccess(id, size, buffer, NULL, flag);
}
ErrorCode  DN3Obj::DN3Cntr::EventGetHandle(EventId id, HANDLE &handle)
{
   return m_module->EventGetHandle(id, handle);
}
ErrorCode  DN3Obj::DN3Cntr::EventGetStatus(EventId id, EventStatus &status)
{
   return m_module->EventGetStatus(id, &status.General.lParam, &status.General.rParam);
}
ErrorCode  DN3Obj::DN3Cntr::EventClearFlag(EventId id, EventStatus &flags)
{
   return m_module->EventClearFlag(id, flags.General.lParam, flags.General.rParam);
}

// Override : DaqCntrModule
ErrorCode DN3Obj::DN3Cntr::CtReset(uint32 cntrStart, uint32 cntrCount)
{
   return m_module->CounterReset(cntrStart, cntrCount);
}
ErrorCode DN3Obj::DN3Cntr::CtRead(uint32 cntrStart, uint32 cntrCount, uint32 loadVal[], uint32 holdVal[])
{
   return m_module->CounterRead(cntrStart, cntrCount, loadVal, holdVal);
}
ErrorCode DN3Obj::DN3Cntr::CtStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_module->CounterStart(cntrStart, cntrCount);
}
ErrorCode DN3Obj::DN3Cntr::EcStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_module->EventCountStart(cntrStart, cntrCount);
}
ErrorCode DN3Obj::DN3Cntr::EcRead(uint32 cntrStart, uint32 cntrCount, uint32 cntrValue[])
{
   return m_module->EventCountRead(cntrStart, cntrCount, cntrValue);
}
ErrorCode DN3Obj::DN3Cntr::OsStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_module->OneShotStart(cntrStart, cntrCount);
}
ErrorCode DN3Obj::DN3Cntr::TmrStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_module->TimerPulseStart(cntrStart, cntrCount);
}
ErrorCode DN3Obj::DN3Cntr::FmStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_module->FreqMeasureStart(cntrStart, cntrCount);
}
ErrorCode DN3Obj::DN3Cntr::FmRead(uint32 cntrStart, uint32 cntrCount, double frequency[])
{
   return m_module->FreqMeasureRead(cntrStart, cntrCount, frequency);
}
ErrorCode DN3Obj::DN3Cntr::PiStart(uint32 cntrStart, uint32 groupCount)
{
   return m_module->PwmInStart(cntrStart, groupCount);
}
ErrorCode DN3Obj::DN3Cntr::PiRead(uint32 cntrStart, uint32 groupCount, double hiPeriod[], double lowPeriod[])
{
   return m_module->PwmInRead(cntrStart, groupCount, hiPeriod, lowPeriod);
}
ErrorCode DN3Obj::DN3Cntr::PoStart(uint32 cntrStart, uint32 cntrCount)
{
   return m_module->PwmOutStart(cntrStart, cntrCount);
}
ErrorCode DN3Obj::DN3Cntr::UdStart(uint32 cntrStart, uint32 cntrCount, uint32 resetCounter)
{
   return m_module->UpDownCountStart(cntrStart, cntrCount, resetCounter);
}
ErrorCode DN3Obj::DN3Cntr::UdResetValue(uint32 cntrStart, uint32 cntrCount)
{
   return m_module->UpDownCountValueReset(cntrStart, cntrCount);
}
ErrorCode DN3Obj::DN3Cntr::UdRead(uint32 cntrStart, uint32 cntrCount, uint32 cntrValue[])
{
   return m_module->UpDownCountRead(cntrStart, cntrCount, cntrValue);
}
ErrorCode DN3Obj::DN3Cntr::CtSnapStart(EventId id, uint32 cntrStart, uint32 cntrCount, uint32 **loadValues, uint32 **holdValues)
{
   return m_module->CounterSnapStart(id, cntrStart, cntrCount, loadValues, holdValues);
}
ErrorCode DN3Obj::DN3Cntr::CtSnapStop(EventId id)
{
   return m_module->CounterSnapStop(id);
}
ErrorCode DN3Obj::DN3Cntr::CCmpSetTable(uint32 counter, uint32 dataCount, uint32 buffer[])
{
   return m_module->CounterCompareSetTable(counter, dataCount, buffer);
}
ErrorCode DN3Obj::DN3Cntr::CCmpSetInterval(uint32 counter, uint32 firstValue, uint32 increment, uint32 count)
{
   return m_module->CounterCompareSetInterval(counter, firstValue, increment, count);
}
ErrorCode DN3Obj::DN3Cntr::CCmpClear(uint32 counter)
{
   return m_module->CounterCompareClear(counter);
}

//
// DaqNavi3FirmwareDownload
// Override : DaqModule
int32 DN3Obj::DN3Fw::getNumber()
{
   return m_index;
}
AccessMode DN3Obj::DN3Fw::getMode()
{
   return m_parent->getMode();
}

// Override : DaqFwModule
int32 DN3Obj::DN3Fw::getCount()
{
   uint32 count = 0;
   return m_module->getCount();
}
uint32 DN3Obj::DN3Fw::getDescription(int32 index, uint32 size, wchar_t buffer[])
{
   return m_module->getDescription(index, size, buffer);
}
uint32 DN3Obj::DN3Fw::getNotice(int32 index, uint32 size, wchar_t buffer[])
{
   return m_module->getNotice(index, size, buffer);
}
uint32 DN3Obj::DN3Fw::getVersionOnDevice(int32 index, uint32 size, wchar_t buffer[])
{
   return m_module->getVersionOnDevice(index, size, buffer);
}
uint32 DN3Obj::DN3Fw::getVersionOnFile(int32 index, uint32 size, wchar_t buffer[])
{
   return m_module->getVersionOnFile(index, size, buffer);
}
int32 DN3Obj::DN3Fw::getOverwriteProgressing(int32 index)
{
   return m_module->getOverwriteProgressing(index);
}

ErrorCode DN3Obj::DN3Fw::LoadFwFile(int32 index, wchar_t const *filePath)
{
   return m_module->LoadFwFile(index, filePath);
}
ErrorCode DN3Obj::DN3Fw::Overwrite(int32 index, FwAction action, wchar_t const *activationCode)
{
   return m_module->Overwrite(index, action, activationCode);
}
