#ifndef _DAQNAVI_3_WRAPPER
#define _DAQNAVI_3_WRAPPER

#include <oal.h>
#include "functiontable.h" // DN3Api
#include "bdaqbase.h"      // DN3Obj*/

//---------------------------------------------------------------------------------------
// DAQNAVI3 : Global method
//---------------------------------------------------------------------------------------
extern ErrorCode PropReadHelper(DaqModule &, PropertyId, uint32 &, void *, uint32);

//---------------------------------------------------------------------------------------
// DAQNAVI3 : API(Function table) based (win driver)
//---------------------------------------------------------------------------------------
namespace DN3Api {

   typedef BioDriverFunctionTable DriverApis;
   typedef uint32 (BDAQCALL * FNDRIVERENTRY)(uint32, PBioDriverFunctionTable);

   inline char const * nameofFnDriverEntry()
   {
      return "DriverInitialize";
   }

   class DN3Device : public DaqDevModule
   {
      typedef std::vector<DaqModule *> ModuleArray;

   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_API; }

   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // device number 
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);
                         
      // Override : DaqDevModule
      virtual void       BDAQCALL Close();
      virtual ErrorCode  BDAQCALL Refresh();
      virtual ErrorCode  BDAQCALL Reset(uint32 state);
      virtual ErrorCode  BDAQCALL GetModule(ModuleType type, uint32 index, DaqModule* &module);

      // Undocumented functions
      virtual ErrorCode  BDAQCALL ReadRegister(uint32 space, uint32 offset, uint32 length, void *buffer);
      virtual ErrorCode  BDAQCALL WriteRegister(uint32 space, uint32 offset, uint32 length, void *buffer);
      virtual ErrorCode  BDAQCALL ReadPrivRgn(uint32 signature, uint32 dataLen, uint8 *dataBuf);
      virtual ErrorCode  BDAQCALL WritePrivRgn(uint32 signature, uint32 dataLen, uint8 *dataBuf);

   public: // Legacy support
#if defined(_WIN32) || defined(WIN32)
      ErrorCode ShowConfigDialogBox(HWND parentWnd, LPRECT region, uint32 dataSource, int32 modal, HWND *dialogWnd)
      {
         return m_apis->pfnDeviceShowCfgDlgBox(m_device, parentWnd, region, dataSource, modal, dialogWnd);
      }
#endif

   public:
      DN3Device() : m_deviceNumber(-1), m_mode(ModeRead), m_device(NULL), m_apis(NULL)
      {
      }
      virtual ~DN3Device()
      {
      }

      ErrorCode Initialize(int32 deviceNumber, AccessMode mode, DriverApis *apis);

   private:
      int32       m_deviceNumber;
      AccessMode  m_mode; 
      HANDLE      m_device;
      DriverApis *m_apis;
      ModuleArray m_modules;
   };

   class DN3Ai : public DaqAiModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_API; }

   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index 
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);
                   
      // Override : DaqAiModule
      virtual ErrorCode BDAQCALL ReadSamples(uint32 chStart, uint32 chCount, void *rawData, double *scaledData);
      virtual ErrorCode BDAQCALL BfdAiPrepare(uint32 &dataCount, void* &dataPtr);
      virtual ErrorCode BDAQCALL BfdAiStart();
      virtual ErrorCode BDAQCALL BfdAiStop();
      virtual ErrorCode BDAQCALL BfdAiRelease();
      virtual ErrorCode BDAQCALL BfdAiGetData(uint32 &dataCount, void *rawBuf, double *dblBuf, double *startTime, uint32 *markCount, DataMark *);

      // Undocumented functions
      virtual ErrorCode BDAQCALL GetScaleTable(uint32 ch, uint32 &size, MapFuncPiece *tables);
      virtual ErrorCode BDAQCALL SetScaleTable(uint32 ch, uint32  size, MapFuncPiece *tables);

   public:
      DN3Ai(DaqModule *parent, int32 index, HANDLE module, DriverApis *apis)
         : m_parent(parent), m_index(index), m_module(module), m_apis(apis)
      {
         m_dataBufLen = 0;
         m_dataBuffer = NULL;
         m_sectCount  = 0;
         m_useDN3SectDataCount = false;

         uint32 size = sizeof(uint32);
         PropReadHelper(*this, CFG_FeatureDataSize, size, &m_dataSize, PropFlag::OpGet);
         PropReadHelper(*this, CFG_ScanChannelCount, size, &m_scanChanCount, PropFlag::OpGet);
         PropReadHelper(*this, CFG_SectionDataCount, size, &m_sectLength, PropFlag::OpGet);

         if (m_scanChanCount == 0) { m_scanChanCount = 1; }

         m_sectLength /= m_scanChanCount;
         if (m_sectLength == 0) { m_sectLength = 1024; };
      }
      virtual ~DN3Ai()
      {
      }

      // Legacy, For compatible ONLY!!!
      ErrorCode BDAQCALL GetChanMapping(uint32 count, uint32 *buffer);    

      // Legacy
      ErrorCode BfdAiRunOnce(bool asynchronous)
      {
         return m_apis->pfnBufferedAiRunOnce(m_module, asynchronous);
      }

      // Legacy
      ErrorCode BfdAiScaleData(void *rawData, double *scaledData, uint32 dataCount, uint32 *channelOffset)
      {
         return m_apis->pfnBufferedAiScaleData(m_module, rawData, scaledData, dataCount, channelOffset);
      }

   private:
      DaqModule  *m_parent;
      int32       m_index;
      HANDLE      m_module;
      DriverApis *m_apis;

      // Cached FAI settings
      uint32      m_sectLength;
      uint32      m_sectCount;
      uint32      m_scanChanCount;
      bool        m_useDN3SectDataCount;

      // Data buffer info
      uint32      m_dataSize;
      uint32      m_dataBufLen;
      uint8      *m_dataBuffer;
   };

   class DN3Ao : public DaqAoModule
   {
   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);
                         
      // Override : DaqAoModule
      virtual ErrorCode BDAQCALL WriteSamples(uint32 chStart, uint32 chCount, void *rawData, double *scaledData);
      virtual ErrorCode BDAQCALL BfdAoPrepare(uint32 dataCount, void* &dataPtr);
      virtual ErrorCode BDAQCALL BfdAoRunOnce(bool asynchronous);
      virtual ErrorCode BDAQCALL BfdAoRun();
      virtual ErrorCode BDAQCALL BfdAoStop(uint32 action);
      virtual ErrorCode BDAQCALL BfdAoRelease();
      virtual ErrorCode BDAQCALL BfdAoScaleData(double *scaledData, void *rawData, uint32 count, uint32 *chanOffset);

      // Undocumented functions
      virtual ErrorCode BDAQCALL GetScaleTable(uint32 ch, uint32 &size, MapFuncPiece *tables);
      virtual ErrorCode BDAQCALL SetScaleTable(uint32 ch, uint32  size, MapFuncPiece *tables);

   public:
      DN3Ao(DaqModule *parent, int32 index, HANDLE module, DriverApis *apis)
         : m_parent(parent), m_index(index), m_module(module), m_apis(apis)
      {
      }
      virtual ~DN3Ao()
      {
      }

   private:
      DaqModule  *m_parent;
      int32       m_index;
      HANDLE      m_module;
      DriverApis *m_apis;
   };

   class DN3Dio : public DaqDioModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_API; }

   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index 
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);

      // Override : DaqDioModule
      virtual ErrorCode BDAQCALL DiReadPort(uint32 portStart, uint32 portCount, uint8 buffer[]);
      virtual ErrorCode BDAQCALL DiReadBit(uint32 port, uint32 bit, uint8 &data);
      virtual ErrorCode BDAQCALL DoWritePort(uint32 portStart, uint32 portCount, uint8 buffer[]);
      virtual ErrorCode BDAQCALL DoWriteBit(uint32 port, uint32 bit, uint8 data);
      virtual ErrorCode BDAQCALL DoReadPort(uint32 portStart, uint32 portCount, uint8 buffer[]);
      virtual ErrorCode BDAQCALL DoReadBit(uint32 port, uint32 bit, uint8 &data);
      virtual ErrorCode BDAQCALL DiSnapStart(EventId id, uint32 portStart, uint32 portCount, uint8 **dataPtr);
      virtual ErrorCode BDAQCALL DiSnapStop(EventId id);

      // Undocumented functions
      // 0 -- start, 1 -- stop, 2 -- feed
      virtual ErrorCode BDAQCALL WdtExec(int action);

   public:
      DN3Dio(DaqModule *parent, int32 index, HANDLE module, DriverApis *apis)
         : m_parent(parent), m_index(index), m_module(module), m_apis(apis)
      {
      }
      virtual ~DN3Dio()
      {
      }

   private:
      DaqModule  *m_parent;
      int32       m_index;
      HANDLE      m_module;
      DriverApis *m_apis;
   };

   class DN3Cntr : public DaqCtModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_API; }

   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index 
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);

      // Override : DaqCntrModule
      virtual ErrorCode BDAQCALL CtReset(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL CtRead(uint32 cntrStart, uint32 cntrCount, uint32 loadVal[], uint32 holdVal[]);
      virtual ErrorCode BDAQCALL CtStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL EcStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL EcRead(uint32 cntrStart, uint32 cntrCount, uint32 cntrValue[]);
      virtual ErrorCode BDAQCALL OsStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL TmrStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL FmStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL FmRead(uint32 cntrStart, uint32 cntrCount, double frequency[]);
      virtual ErrorCode BDAQCALL PiStart(uint32 cntrStart, uint32 groupCount);
      virtual ErrorCode BDAQCALL PiRead(uint32 cntrStart, uint32 groupCount, double hiPeriod[], double lowPeriod[]);
      virtual ErrorCode BDAQCALL PoStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL UdStart(uint32 cntrStart, uint32 cntrCount, uint32 resetCounter);
      virtual ErrorCode BDAQCALL UdResetValue(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL UdRead(uint32 cntrStart, uint32 cntrCount, uint32 cntrValue[]);
      virtual ErrorCode BDAQCALL CtSnapStart(EventId id, uint32 cntrStart, uint32 cntrCount, uint32 **loadValues, uint32 **holdValues);
      virtual ErrorCode BDAQCALL CtSnapStop(EventId id);
      virtual ErrorCode BDAQCALL CCmpSetTable(uint32 counter, uint32 dataCount, uint32 buffer[]);
      virtual ErrorCode BDAQCALL CCmpSetInterval(uint32 counter, uint32 firstValue, uint32 increment, uint32 count);
      virtual ErrorCode BDAQCALL CCmpClear(uint32 counter);
                       
   public:
      DN3Cntr(DaqModule *parent, int32 index, HANDLE module, DriverApis *apis)
         : m_parent(parent), m_index(index), m_module(module), m_apis(apis)
      {
      }
      virtual ~DN3Cntr()
      {
      }

   private:
      DaqModule  *m_parent;
      int32       m_index;
      HANDLE      m_module;
      DriverApis *m_apis;
   };

   class DN3Fw : public DaqFwModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_API; }

   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index 
      virtual AccessMode BDAQCALL getMode();

      // Override : firmware
      virtual int32      BDAQCALL getCount();
      virtual uint32     BDAQCALL getDescription(int32 index, uint32 size, wchar_t buffer[]);
      virtual uint32     BDAQCALL getNotice(int32 index, uint32 size, wchar_t buffer[]);
      virtual uint32     BDAQCALL getVersionOnDevice(int32 index, uint32 size, wchar_t buffer[]);
      virtual uint32     BDAQCALL getVersionOnFile(int32 index, uint32 size, wchar_t buffer[]);
      virtual int32      BDAQCALL getOverwriteProgressing(int32 index);

      virtual ErrorCode  BDAQCALL LoadFwFile(int32 index, wchar_t const *filePath);
      virtual ErrorCode  BDAQCALL Overwrite(int32 index, FwAction action, wchar_t const *activationCode);
   public:
      DN3Fw(DaqModule *parent, int32 index, HANDLE module, DriverApis *apis)
         : m_parent(parent), m_index(index), m_module(module), m_apis(apis)
      {
      }
      virtual ~DN3Fw()
      {
      }

   private:
      DaqModule  *m_parent;
      int32       m_index;
      HANDLE      m_module;
      DriverApis *m_apis;
   };

} // namespace DAQNAVI3API

//---------------------------------------------------------------------------------------
// DAQNAVI3 : object based (linux driver)
//---------------------------------------------------------------------------------------
namespace DN3Obj {

   typedef ErrorCode (BDAQCALL *FNOBJECTCREATE)(int32 deviceNumber, AccessMode mode, BDaqDevice **device);

   // Flag for PropAccess 
   // bit 0: Get/set. 1 -- get property, otherwise, set property
   // bit 1: data source. 1 -- from/to registry, otherwise, from/to device
   // bit 2: notify bit. 1 -- notify the user now.
   enum PropOpFlag {OPFLAG_READ = 0x1, OPFLAG_REG = 0x2, OPFLAG_NTFNOW = 0x4, OPFLAG_UPDHW = 0x100};

   inline char const * nameofFnObjectCreate() 
   { 
      return "BDaq3DeviceObjectCreate"; 
   }

   class DN3Device : public DaqDevModule
   {
      typedef std::vector<DaqModule *> ModuleArray;

   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_OBJ; }

   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // device number 
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);

      // Override : DaqDevModule
      virtual void       BDAQCALL Close();
      virtual ErrorCode  BDAQCALL Refresh();
      virtual ErrorCode  BDAQCALL Reset(uint32 state);
      virtual ErrorCode  BDAQCALL GetModule(ModuleType type, uint32 index, DaqModule* &module);

      // Undocumented functions
      virtual ErrorCode  BDAQCALL ReadRegister(uint32 space, uint32 offset, uint32 length, void *buffer);
      virtual ErrorCode  BDAQCALL WriteRegister(uint32 space, uint32 offset, uint32 length, void *buffer);
      virtual ErrorCode  BDAQCALL ReadPrivRgn(uint32 signature, uint32 dataLen, uint8 *dataBuf);
      virtual ErrorCode  BDAQCALL WritePrivRgn(uint32 signature, uint32 dataLen, uint8 *dataBuf);

   public:
      DN3Device() : m_deviceNumber(-1), m_mode(ModeRead), m_device(NULL)
      {
      }
      virtual ~DN3Device()
      {
      }
      ErrorCode Initialize(int32 deviceNumber, AccessMode mode, FNOBJECTCREATE CreateObject);

   private:
      int32       m_deviceNumber;
      AccessMode  m_mode; 
      BDaqDevice *m_device;
      ModuleArray m_modules;
   };


   class DN3Ai : public DaqAiModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_OBJ; }

   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index  
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);

      // Override : DaqAiModule
      virtual ErrorCode BDAQCALL ReadSamples(uint32 chStart, uint32 chCount, void *rawData, double *scaledData);
      virtual ErrorCode BDAQCALL BfdAiPrepare(uint32 &dataCount, void* &dataPtr);
      virtual ErrorCode BDAQCALL BfdAiStart();
      virtual ErrorCode BDAQCALL BfdAiStop();
      virtual ErrorCode BDAQCALL BfdAiRelease();
      virtual ErrorCode BDAQCALL BfdAiGetData(uint32 &dataCount, void *rawBuf, double *dblBuf, double *startTime, uint32 *markCount, DataMark *);

      // Undocumented functions
      virtual ErrorCode BDAQCALL GetScaleTable(uint32 ch, uint32 &size, MapFuncPiece *tables);
      virtual ErrorCode BDAQCALL SetScaleTable(uint32 ch, uint32  size, MapFuncPiece *tables);
                         
   public:               
      DN3Ai(DaqModule *parent, int32 index, BDaqAi *module)
         : m_parent(parent), m_index(index), m_module(module)
      {
         m_dataBufLen = 0;
         m_dataBuffer = NULL;
         m_sectCount  = 0;
         m_useDN3SectDataCount = false;

         uint32 size = sizeof(uint32);
         PropReadHelper(*this, CFG_FeatureDataSize, size, &m_dataSize, PropFlag::OpGet);
         PropReadHelper(*this, CFG_ScanChannelCount, size, &m_scanChanCount, PropFlag::OpGet);
         PropReadHelper(*this, CFG_SectionDataCount, size, &m_sectLength, PropFlag::OpGet);

         if (m_scanChanCount == 0) { m_scanChanCount = 1; }

         m_sectLength /= m_scanChanCount;
         if (m_sectLength == 0) { m_sectLength = 1024; }
      }
      virtual ~DN3Ai()
      {
      }

      // Legacy
      ErrorCode BfdAiRunOnce(bool asynchronous)
      {
         return m_module->BfdAiRunOnce(asynchronous);
      }

      // Legacy
      ErrorCode BfdAiScaleData(void *rawData, double *scaledData, uint32 dataCount, uint32 *channelOffset)
      {
         return m_module->BfdAiScaleData(rawData, scaledData, dataCount, channelOffset);
      }

   private:
      DaqModule *m_parent;
      int32      m_index;
      BDaqAi    *m_module;
   
      // Cached FAI settings
      uint32      m_sectLength;
      uint32      m_sectCount;
      uint32      m_scanChanCount;
      bool        m_useDN3SectDataCount;

      // Data buffer info
      uint32      m_dataSize;
      uint32      m_dataBufLen;
      uint8      *m_dataBuffer;
   };

   class DN3Ao : public DaqAoModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_OBJ; }
 
   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index 
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);

      // Override : DaqAoModule
      virtual ErrorCode BDAQCALL WriteSamples(uint32 chStart, uint32 chCount, void *rawData, double *scaledData);
      virtual ErrorCode BDAQCALL BfdAoPrepare(uint32 dataCount, void* &dataPtr);
      virtual ErrorCode BDAQCALL BfdAoRunOnce(bool asynchronous);
      virtual ErrorCode BDAQCALL BfdAoRun();
      virtual ErrorCode BDAQCALL BfdAoStop(uint32 action);
      virtual ErrorCode BDAQCALL BfdAoRelease();
      virtual ErrorCode BDAQCALL BfdAoScaleData(double *scaledData, void *rawData, uint32 count, uint32 *chanOffset);

      // Undocumented functions
      virtual ErrorCode BDAQCALL GetScaleTable(uint32 ch, uint32 &size, MapFuncPiece *tables);
      virtual ErrorCode BDAQCALL SetScaleTable(uint32 ch, uint32  size, MapFuncPiece *tables);

   public:
      DN3Ao(DaqModule *parent, int32 index, BDaqAo *module)
         : m_parent(parent), m_index(index), m_module(module)
      {
      }
      virtual ~DN3Ao()
      {
      }

   private:
      DaqModule *m_parent;
      int32      m_index;
      BDaqAo    *m_module;
   };

   class DN3Dio : public DaqDioModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_OBJ; }
 
   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index 
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);

      // Override : DaqDioModule
      virtual ErrorCode BDAQCALL DiReadPort(uint32 portStart, uint32 portCount, uint8 buffer[]);
      virtual ErrorCode BDAQCALL DiReadBit(uint32 port, uint32 bit, uint8 &data);
      virtual ErrorCode BDAQCALL DoWritePort(uint32 portStart, uint32 portCount, uint8 buffer[]);
      virtual ErrorCode BDAQCALL DoWriteBit(uint32 port, uint32 bit, uint8 data);
      virtual ErrorCode BDAQCALL DoReadPort(uint32 portStart, uint32 portCount, uint8 buffer[]);
      virtual ErrorCode BDAQCALL DoReadBit(uint32 port, uint32 bit, uint8 &data);
      virtual ErrorCode BDAQCALL DiSnapStart(EventId id, uint32 portStart, uint32 portCount, uint8 **dataPtr);
      virtual ErrorCode BDAQCALL DiSnapStop(EventId id);

      // Undocumented functions
      // 0 -- start, 1 -- stop, 2 -- feed
      virtual ErrorCode BDAQCALL WdtExec(int action);

   public:
      DN3Dio(DaqModule *parent, int32 index, BDaqDio *module)
         : m_parent(parent), m_index(index), m_module(module)
      {
      }
      virtual ~DN3Dio()
      {
      }

   private:
      DaqModule *m_parent;
      int32      m_index;
      BDaqDio   *m_module;
   };

   class DN3Cntr : public DaqCtModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_OBJ; }
 
   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index 
      virtual AccessMode BDAQCALL getMode();
      virtual ErrorCode  BDAQCALL PropAccess(PropertyId id, uint32 &size, void *buffer, uint32 flag);
      virtual ErrorCode  BDAQCALL EventGetHandle(EventId id, HANDLE &handle);
      virtual ErrorCode  BDAQCALL EventGetStatus(EventId id, EventStatus &status);
      virtual ErrorCode  BDAQCALL EventClearFlag(EventId id, EventStatus &flags);

      // Override : DaqCntrModule
      virtual ErrorCode BDAQCALL CtReset(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL CtRead(uint32 cntrStart, uint32 cntrCount, uint32 loadVal[], uint32 holdVal[]);
      virtual ErrorCode BDAQCALL CtStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL EcStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL EcRead(uint32 cntrStart, uint32 cntrCount, uint32 cntrValue[]);
      virtual ErrorCode BDAQCALL OsStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL TmrStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL FmStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL FmRead(uint32 cntrStart, uint32 cntrCount, double frequency[]);
      virtual ErrorCode BDAQCALL PiStart(uint32 cntrStart, uint32 groupCount);
      virtual ErrorCode BDAQCALL PiRead(uint32 cntrStart, uint32 groupCount, double hiPeriod[], double lowPeriod[]);
      virtual ErrorCode BDAQCALL PoStart(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL UdStart(uint32 cntrStart, uint32 cntrCount, uint32 resetCounter);
      virtual ErrorCode BDAQCALL UdResetValue(uint32 cntrStart, uint32 cntrCount);
      virtual ErrorCode BDAQCALL UdRead(uint32 cntrStart, uint32 cntrCount, uint32 cntrValue[]);
      virtual ErrorCode BDAQCALL CtSnapStart(EventId id, uint32 cntrStart, uint32 cntrCount, uint32 **loadValues, uint32 **holdValues);
      virtual ErrorCode BDAQCALL CtSnapStop(EventId id);
      virtual ErrorCode BDAQCALL CCmpSetTable(uint32 counter, uint32 dataCount, uint32 buffer[]);
      virtual ErrorCode BDAQCALL CCmpSetInterval(uint32 counter, uint32 firstValue, uint32 increment, uint32 count);
      virtual ErrorCode BDAQCALL CCmpClear(uint32 counter);
                         
   public:
      DN3Cntr(DaqModule *parent, int32 index, BDaqCntr *module)
         : m_parent(parent), m_index(index), m_module(module)
      {
      }
      virtual ~DN3Cntr()
      {
      }

   private:
      DaqModule *m_parent;
      int32      m_index;
      BDaqCntr  *m_module;
   };

   class DN3Fw : public DaqFwModule
   {
   public:
      virtual int32 BDAQCALL getIntfType() { return IntfType::DN3_OBJ; }

   public:
      // Override : DaqModule
      virtual int32      BDAQCALL getNumber();    // module index 
      virtual AccessMode BDAQCALL getMode();

      // Override : firmware
      virtual int32      BDAQCALL getCount();
      virtual uint32     BDAQCALL getDescription(int32 index, uint32 size, wchar_t buffer[]);
      virtual uint32     BDAQCALL getNotice(int32 index, uint32 size, wchar_t buffer[]);
      virtual uint32     BDAQCALL getVersionOnDevice(int32 index, uint32 size, wchar_t buffer[]);
      virtual uint32     BDAQCALL getVersionOnFile(int32 index, uint32 size, wchar_t buffer[]);
      virtual int32      BDAQCALL getOverwriteProgressing(int32 index);

      virtual ErrorCode  BDAQCALL LoadFwFile(int32 index, wchar_t const *filePath);
      virtual ErrorCode  BDAQCALL Overwrite(int32 index, FwAction action, wchar_t const *activationCode);
   public:
      DN3Fw(DaqModule *parent, int32 index, BDaqFw *module)
         : m_parent(parent), m_index(index), m_module(module)
      {
      }
      virtual ~DN3Fw()
      {
      }

   private:
      DaqModule *m_parent;
      int32      m_index;
      BDaqFw    *m_module;
   };
}

#endif 
