#ifndef _DAQNAVI_DEVICE_ENUMERATOR_H
#define _DAQNAVI_DEVICE_ENUMERATOR_H

struct DeviceEnumArgs {
   // current device information
   int32        DeviceNumber;
   wchar_t      *Description;
   DaqDevModule *Device;       // Note: the device will be closed after the delegate returned.
}; 

struct ModuleEnumArgs {
   // parent device information
   int32        DeviceNumber;
   wchar_t      *Description;
   DaqDevModule *Device; // Note: the device will be closed after the delegate returned.

   // current module information
   int          ModuleIndex;
   DaqModule    *Module;
};

typedef bool (BDAQCALL * DeviceEnumProc)(DeviceEnumArgs *args, void *userParam);
typedef bool (BDAQCALL * ModuleEnumProc)(ModuleEnumArgs *args, void *userParam);
class XDevEnumerator
{
public:
   // Enumerate all devices which are on the system
   static void EnumerateDevice(DeviceEnumProc userFunc, void *userParam)
   {
      EnumerateDevice(DaqAny, userFunc, userParam);
   }

   // Enumerate all devices which are on the system and support the specified function
   static void EnumerateDevice(ModuleType typeWanted, DeviceEnumProc userFunc, void *userParam)
   {
      wchar_t descBuff[256]; // device description buffer
      int32   active;
      int32   deviceNumber;

      int32   deviceIndex = 0;
      bool    keepGoing   = true;

      // the user doesn't care about the supported function of the device
      if (typeWanted == DaqAny) { typeWanted = DaqDevice; }

      while(keepGoing) {
         ErrorCode ret = XDevFactory::EnumDeviceInfo(deviceIndex++, active, deviceNumber, descBuff);
         // no more device
         if (deviceNumber == -1) { break; } 

         // device does not be on the system 
         if (ret != Success || active == 0) { continue; } 
         
         // open the device to do further check
         DaqDevModule *device = NULL;
         if (XDevFactory::OpenDevice(deviceNumber, ModeRead, device) == Success) {

            // Does the device support the function the user wanted?
            DaqModule *module = NULL;
            if (typeWanted == DaqDevice || device->GetModule(typeWanted, 0, module) == Success) {
               // Call the passed-in delegate to let the user do further action
               DeviceEnumArgs args = {deviceNumber, descBuff, device};
               keepGoing = userFunc(&args, userParam);
            }

            // close the device
            XDevFactory::CloseDevice(device);
         }
      } 
   }

   // Enumerate all modules which are on the system
   static void EnumerateModules(ModuleType typeWanted, ModuleEnumProc userFunc, void *userParam)
   {
      ModuleEnumCtx ctx = { typeWanted, userFunc, userParam };

      EnumerateDevice(typeWanted, (DeviceEnumProc)EnumDeviceCallBack, (void*)&ctx);
   }

private:
   struct ModuleEnumCtx
   {
      ModuleType      typeWanted;
      ModuleEnumProc  userFunc;
      void           *userParam;
   };

   static bool BDAQCALL EnumDeviceCallBack(DeviceEnumArgs *devArgs, void *userParam)
   {
      ModuleEnumCtx * ctx = (ModuleEnumCtx*)userParam;

      // save parent device information into our enumeration args
      ModuleEnumArgs modArgs = {devArgs->DeviceNumber, devArgs->Description, devArgs->Device};

      // enumerating all wanted modules
      DaqModule *module;
      for (int i = 0; true; ++i){
         if (devArgs->Device->GetModule(ctx->typeWanted, i, module) != Success) { 
            break; 
         }
         modArgs.ModuleIndex = i;
         modArgs.Module = module;
         if (!ctx->userFunc(&modArgs, ctx->userParam)) {
            return false; // break the enumeration
         }
      }
      return true;
   }
};

#endif
