#ifndef _DAQNAVI_XDEV_FACTORY
#define _DAQNAVI_XDEV_FACTORY

typedef ErrorCode (BDAQCALL *DN4FNOBJECTCREATE)(int32 deviceNumber, AccessMode mode, DaqDevModule **device);

class XDevFactory
{
   struct DriverEntry {
      int32   IntfType;
      int32   RefCount;
      HANDLE  LibHandle;
      union {
         DN3Api::DriverApis     *DrvApis;
         DN3Obj::FNOBJECTCREATE ObjCreate;
         DN4FNOBJECTCREATE      DN4ObjCreate;
      };
   };
   typedef std::map<std::string, DriverEntry *> DriverTable;

public:
   static ErrorCode OpenDevice(int32 deviceNumber, AccessMode mode, DaqDevModule *& device);
   static void      CloseDevice(DaqDevModule * device);
   static void      UnloadIdleDriver();

   static void *    GetDriverFunc(std::string const &driverName, std::string const funcName);

   // helper
   static ErrorCode GetDriverName(int32 deviceNumber, std::string &driverName);
   static ErrorCode EnumDeviceInfo(int32 deviceIndex, int32 &active, int32 &deviceNumber, wchar_t desc[MAX_DEVICE_DESC_LEN]);

private:
   XDevFactory() {}
   virtual ~XDevFactory() {}

   static XDevFactory & getInstance();
   static ErrorCode     LoadDriver(std::string const &driverName, DriverEntry * &entry);
   static void          UnloadDriver(DriverEntry *&entry);

private:
   MutexLock   m_lock;
   DriverTable m_drivers;
};

#endif
