/*******************************************************************************
              Copyright (c) 1983-2009 Advantech Co., Ltd.
********************************************************************************
THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY INFORMATION
WHICH IS THE PROPERTY OF ADVANTECH CORP., ANY DISCLOSURE, USE, OR REPRODUCTION,
WITHOUT WRITTEN AUTHORIZATION FROM ADVANTECH CORP., IS STRICTLY PROHIBITED. 

================================================================================
REVISION HISTORY
--------------------------------------------------------------------------------
$Log: $
--------------------------------------------------------------------------------
$NoKeywords:  $
*/

// ----------------------------------------------------------------------------
// File:        misc.cpp
// Module:      UserLib.lib
// Environment: User Mode
// Reference:   Microsoft WDK
// Description: common functions which are independent with any special 'function'
// ----------------------------------------------------------------------------
#include "std_decl.h"

//*************************************************************************
// add by jidong to fix up the code from ABJ.
// The code from ABJ has a minor bug that for the user frequency which can be
// gracefully divided into two pacer divisor, the codes may create two somehow
// strange value which has a minor different with the actual value.
//*************************************************************************
static bool CalcDivisorUseFactorization(
   uint32    divisor,
   uint16 & div1,
   uint16 & div2 )
{
   uint32 sqrtVal = (uint32)sqrt( (double)divisor );
   uint32 factor1;
   uint32 factor2;
   for( factor1 = sqrtVal; factor1 > 1; --factor1 )
   {
      if ( divisor % factor1 == 0 )
      {
         factor2 = divisor / factor1;
         if ( factor1 < 0x10000 && factor2 < 0x10000 )
         {
            div1 = (uint16)factor1;
            div2 = (uint16)factor2;
            return true;
         }
      }
   }
   return false;
}
//************************************************************************* 
// copied from ABJ PCI1747/MIC3747 DLL Driver: GetPacerRate
//************************************************************************* 
static void CalcDivisorUseMinimalError(
   IN  double   divisor, 
   OUT uint16 & div1,
   OUT uint16 & div2 )
{
   uint32  divisor1, divisor2;
   divisor1 = (uint32)(divisor/65535.0 + 0.5);
   if (divisor1 > 1L)
   {
      divisor1 = x_min(65535, (divisor1 + 1));
      divisor2 = x_min(65535, (uint32)(divisor/ divisor1));
   } else {
      divisor1 = 1; // set to 1 to avoid "divided by zero" when calculate divisor2

      double  derror = 1.0 ;
      double  dtemp;
      for (int i = 2; i <= (int)(sqrt(divisor)+0.5); ++i )
      {
         divisor2 = (uint32)(divisor/i +0.5);
         if (divisor2*i == (uint32)(divisor+0.5))
         {
            divisor1 = i;
            break;
         }
         dtemp = fabs((double)divisor2*i-divisor)/divisor;
         if ( derror > dtemp )
         {
            divisor1 = i;
            derror = dtemp;
         }
      }
      divisor2 = (uint32)(divisor/divisor1 +0.5);
   }

   div1 = (uint16)divisor1;
   div2 = (uint16)divisor2;
}
//*************************************************************************
// Name: UserFreqToCounterDivider
// Description:
//       Calculate the two 16-bit counter divider value for the user 
// specified frequency and the base clock frequency.
// Parameters: 
//  [in]baseFreq -- base clock frequency of the counter.
//  [in]userFreq -- the frequency the user wanted.
//  [out]div1, div2 -- the calculation result. 
//   Approximately, UserFreq = BaseClkFreq / (div1 * div2)
// return value: 
//    <None>
//************************************************************************* 
BDAQ_API
void BDAQCALL UserFreqToCounterDivider(
   IN  uint32    baseFreq,
   IN  double   userFreq, 
   OUT uint16 *  div1,
   OUT uint16 *  div2)
{
   double divisor = ((double)baseFreq) / userFreq;
   if (divisor < 2) {// if baseFreq < userFreq, divisor < 1, the calculated div1 or div2 maybe 0 
      *div1 = 1;
      *div2 = 2;
      return;
   }

   //
   // The best way is using factorization to calculate the divisor because
   // it has no error.
   uint32  divInt  = (uint32)divisor;
   if (divisor - divInt < FLT_EPSILON 
      && CalcDivisorUseFactorization(divInt, *div1, *div2)) {
      return;
   }

   //
   // If we can't calculate the divisor using factorization, we had to use
   // the 'minimal error' method
   CalcDivisorUseMinimalError(divisor, *div1, *div2);
}

//*************************************************************************
// Name: AiCalcLogicalChanCount
// Description:
//       Helper function to calculate the logical channel count.
// Parameters: 
//  [in]chType -- the array which stores all channels signal-connection type.
//  [in]count  -- the count of the SC type array .
// Return value: 
//    the logical channel count.
//************************************************************************* 
BDAQ_API
uint32 BDAQCALL AiCalcLogicalChanCount(
   IN uint8 const chType[], 
   IN uint32 count)
{
   uint32 logCount = count;
   for (uint32 i = 0; i < count; i += 2) { // Only check the even channel
      if (chType[i] == Differential) { 
         --logCount; 
      }
   }
   return logCount;
}
//*************************************************************************
// Name: VrgFindItemByGain
// Description:
//       Helper function to retrieve a value range information entry 
//    according with the specified gain code.
// Parameters: 
//  [in]gain   -- the gain code used to search.
//  [in]entries-- the value range info table to be searched.
//  [in]count  -- the count of the value range info table. 
//   
// return value: 
//    a const pointer to the AI value range info entry or NULL if not found.
//************************************************************************* 
BDAQ_API
PCVRG_INFO BDAQCALL VrgFindItemByGain(
   IN uint32    gain,
   IN PCVRG_INFO entries,
   IN uint32    count)
{
   for (uint32 i = 0; i < count; ++i) {
      if (entries[i].Gain == gain) {
         return &entries[i];
      }
   }
   return NULL;
}
//*************************************************************************
// Name: VrgFindItemByVrgType
// Description:
//       Helper function to retrieve a value range information entry 
//    according with the specified value range type.
// Parameters: 
//  [in]vrgType-- the value range type used to search.
//  [in]entries-- the value range info table to be searched.
//  [in]count  -- the count of the value range info table. 
//   
// return value: 
//    a const pointer to the AI value range info entry or NULL if not found.
//************************************************************************* 
BDAQ_API
PCVRG_INFO BDAQCALL VrgFindItemByVrgType(
   IN uint32    vrgType,
   IN PCVRG_INFO entries,
   IN uint32    count)
{
   for (uint32 i = 0; i < count; ++i) {
      if (entries[i].VrgType == vrgType) {
         return &entries[i];
      }
   }
   return NULL;
}

//*************************************************************************
// Name: SizeOfScalePolys
// Description:
//       Helper function to calculate the size of the specified polynomial polys. 
// 
// Parameters: 
//  [in]polys -- mapping function polynomial piece array
//  [in]count -- count of polynomial array.
// Return value: 
//    the size in bytes of the array.
//    A Zero indicates an error. Generally this due to a invalid
// count was passed to the function.
//************************************************************************* 
BDAQ_API
uint32 BDAQCALL SizeOfMapFuncPieces(
   IN PCMapFuncPiece polys, 
   IN uint32         count)
{
   uint32 size = 0;
#if defined(_WIN32) || defined(WIN32)
   __try
#endif
   {
      while (count--) {
         size += polys->Size;
         polys = (PMapFuncPiece)((char*)polys + polys->Size);
      }
   } 
#if defined(_WIN32) || defined(WIN32)
   __except (GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION ?
               EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH) {
      size = 0;
   }
#endif
   return size;
}

//*************************************************************************
// Name: CountOfScalePolys
// Description:
//       Helper function to calculate the item count of the specified polynomial polys . 
// 
// Parameters: 
//  [in]polys -- mapping function polynomial piece array
//  [in]size -- the size of polynomial array.
// Return value: 
//    the item count of the array.
//    A Zero indicates an error. Generally this due to a invalid
// table size was passed to the function.
//************************************************************************* 
BDAQ_API
uint32 BDAQCALL CountOfMapFuncPieces(
   IN PCMapFuncPiece polys, 
   IN uint32         size)
{
   uint32 count = 0;
#if defined(_WIN32) || defined(WIN32)
   __try
#endif
   {
      while(size) {
         ++count;
         size -= polys->Size;
         polys = (PCMapFuncPiece)((char*)polys + polys->Size);
      } 
   }
#if defined(_WIN32) || defined(WIN32)
   __except (GetExceptionCode() == EXCEPTION_ACCESS_VIOLATION ?
            EXCEPTION_EXECUTE_HANDLER : EXCEPTION_CONTINUE_SEARCH) {
       count = 0;
   }
#endif
   return count;
}


//*************************************************************************
// Name: AiBuildMapFunc
// Description:
//       Helper function to build map function of specified value range.
//
//*************************************************************************
BDAQ_API
void BDAQCALL AiBuildMapFunc(
   IN PCVRG_INFO    vrg,
   IN uint32        resInBit,
   OUT PSCALE_TABLE table)
{
   assert(vrg && table);
   table->VrgType    = vrg->VrgType;
   table->Gain       = vrg->Gain;
   table->PieceCount = 1;

   table->Pieces[0].Size      = sizeof(MapFuncPiece);
   table->Pieces[0].Degree    = 1;
   table->Pieces[0].UpperLimit= double(1ULL << resInBit);
   table->Pieces[0].Coef[0]   = vrg->RangeLow;
   table->Pieces[0].Coef[1]   = (vrg->RangeHigh - vrg->RangeLow) / double(1ULL << resInBit);
}

//*************************************************************************
// Name: AiBuildMapFuncByCalParam
// Description:
//       Helper function to build map function of specified value range
// according with the calibration parameters.
//
//*************************************************************************
BDAQ_API
void BDAQCALL AiBuildMapFuncByCalParam(
   IN PCVRG_INFO   vrg,
   IN uint32       resInBit,
   IN uint32       gainParam,
   IN uint32       offsetParam,
   IN double       scaleFactor,
   IN double       offsetFactor,
   OUT SCALE_TABLE *table)
{
   /*
   * Note: Data Conversion Equation is:
   * Bipolar:
   *    Volt = (data_binary - offsetParam) / (gainParam - offsetParam) * scaleFactor + offsetFactor
   *         = (data_binary - offsetParam) * scaleFactor / (gainParam - offsetParam) + offsetFactor
   *         = data_binary * scaleFactor / (gainParam - offsetParam) - offsetParam * scaleFactor / (gainParam - offsetParam) + offsetFactor
   * where:
   *    'scaleFactor / (gainParam - offsetParam)' is just the lsb. 
   *    generally the scaleFactor is '(vrg->rangeHigh - vrg->rangeLow)/2' for bipolar,
   *    '(vrg->rangeHigh - vrg->rangeLow)' for unipolar.
   *    but some devices maybe own its special scaling factor for each range.
   * so:
   *     Volt = data_binary * lsb + offsetFactor - offsetParam * lsb
   * SO, the polynomial is:
   * Coef[1] = lsb 
   * Coef[0] = offsetFactor - offsetParam * lsb
   */
   double lsb = scaleFactor / (gainParam - offsetParam); 

   assert(vrg && table);
   table->VrgType    = vrg->VrgType;
   table->Gain       = vrg->Gain;
   table->PieceCount = 1;

   table->Pieces[0].Size      = sizeof(MapFuncPiece);
   table->Pieces[0].Degree    = 1;
   table->Pieces[0].UpperLimit= double(1ULL << resInBit);
   table->Pieces[0].Coef[0]   = offsetFactor - (offsetParam * lsb);
   table->Pieces[0].Coef[1]   = lsb;
}

//*************************************************************************
// Name: AoBuildVoltageMapFunc
// Description:
//       Helper function to build map function of specified value range
// (built-in / external reference, voltage).
//
//*************************************************************************
BDAQ_API
void BDAQCALL AoBuildVoltageMapFunc(
   IN PCVRG_INFO     vrg,
   IN uint32         resInBit,
   IN uint32         extRefAntiPolar, /*ignored if not external reference*/
   IN double         extRefVoltage,   /*ignored if not external reference*/
   OUT PSCALE_TABLE  table)
{
   assert(vrg && table);

   double rangeLow;
   double rangeHigh;

   // the sign of coef[1], default 1 for internal reference voltage, for external reference sign = antiPolarSign * externalRefVoltage
   // If the AO external reference voltage is anti-polar,
   // for Unipolar output: the output range is (0, extV) @ -extV voltage, and (-extV,0) @ extV voltage
   // for Bipolar output: the output range is (-extV,extV) @ extV(binary 0 ==> extV), (-extV,extV) @ -extV(binary 0 ==> -extV)
   double sign = 1.0;
   double antiPolarSign = extRefAntiPolar ? -1.0 : 1.0;
   switch (vrg->VrgType)
   {
   case V_ExternalRefBipolar:
      rangeLow  = -fabs(extRefVoltage);
      rangeHigh = fabs(extRefVoltage);
      sign      = extRefVoltage * antiPolarSign > 0 ? 1 : -1;
      break;
   case V_ExternalRefUnipolar:
      if (extRefAntiPolar) {
         if (extRefVoltage < 0) {
            rangeLow = 0;
            rangeHigh= fabs(extRefVoltage);
         } else {
            rangeLow = -1 * extRefVoltage;
            rangeHigh= 0;
         }
      } else {
         rangeLow = x_min(0, extRefVoltage);
         rangeHigh= x_max(0, extRefVoltage);
      }
      sign = (extRefVoltage * antiPolarSign > 0) ? 1 : -1;
      break;
   default:
      rangeLow = vrg->RangeLow;
      rangeHigh= vrg->RangeHigh;
      break;
   }

   table->VrgType    = vrg->VrgType;
   table->Gain       = vrg->Gain;
   table->PieceCount = 2;

   /*/////////////////////////////////////////////////////////////////////////
   // Bipolar
   // Positive reference voltage
   //                 Y  (P2)   (P2')
   //                 | /----------
   //                 |/
   //                 /
   //                /|
   //               / |
   //              /  |
   // ------------/---0--------------> Voltage
   //           (P1)
   //
   // Negative reference voltage
   //         (P2) \  Y
   //               \ |
   //                \|
   //                 \
   //                 |\
   //                 | \
   //                 |  \
   // ----------------0---\-----------> Voltage
   //                    (P1)
   // map function poly 1: 0-0
   // map function poly 2: P1-P2
   //////////////////////////////////////////////////////////////////////////

   //////////////////////////////////////////////////////////////////////////
   // Unipolar
   // Positive reference voltage
   //                Binary
   //                 Y    (P2)   (P2')
   //                 |    /----------
   //                 |   /
   //                 |  /
   //                 | /
   //                 |/
   // ----------------0--------------> Voltage
   //                (P1)
   //
   // Negative reference voltage
   //                Binary
   //   (P1')   (P1)    Y   (P2)   (P2')
   //     -------\    |    /-------
   //             \   |   /
   //              \  |  /
   //               \ | /
   //                \|/
   // ----------------0--------------> Voltage
   // Case1: ExtRef is not AntiPolar
   //    RefVolt<0: 0-P1: antiPolarSign * RefVolt < 0
   //      map function poly 1: P1'-P1
   //      map function poly 2: P1-0
   //    RefVolt>0: 0-P2: antiPolarSign * RefVolt > 0
   //      map function poly 1: 0-0
   //      map function poly 2: 0-P2
   //
   // Case2: ExtRef is AntiPolar
   //    RefVolt<0: 0-P2: antiPolarSign * RefVolt > 0
   //      map function poly 1: 0-0
   //      map function poly 2: 0-P2
   //    RefVolt>0: 0-P1: antiPolarSign * RefVolt < 0
   //      map function poly 1: P1'-P1
   //      map function poly 2: P1-0
   //////////////////////////////////////////////////////////////////////////*/

   double const lsb = ANALOG_LSB_BITS(rangeLow, rangeHigh, resInBit);
   double const half_lsb = lsb / 2;

   MapFuncPiece *poly;

   // map function poly 1:
   poly            = &table->Pieces[0];
   poly->Size      = sizeof(MapFuncPiece);
   poly->Degree    = 1;
   poly->UpperLimit= rangeLow - sign * half_lsb;
   poly->Coef[0]   = (sign < 0) ? double((1ULL << resInBit) - 1): 0;
   poly->Coef[1]   = 0;

   // map function poly 2:
   poly            = &table->Pieces[1];
   poly->Size      = sizeof(MapFuncPiece);
   poly->Degree    = 1;
   poly->UpperLimit= rangeHigh - sign * half_lsb;
   poly->Coef[0]   = (vrg->VrgType == V_ExternalRefUnipolar) ? 0 : -(rangeLow / lsb);
   poly->Coef[1]   = sign * 1 / lsb;
}

//*************************************************************************
// Name: AoBuildDefaultCurrentMapFunc
// Description:
//       Helper function to build map function of specified value range
// (current type, e.g. 0~20ma or 4~20ma).
//
//*************************************************************************
BDAQ_API
void BDAQCALL AoBuildCurrentMapFunc(
   IN PCVRG_INFO     vrg,
   IN uint32         resInBit,
   IN uint32         offset,
   OUT PSCALE_TABLE  table)
{
   assert(vrg && table);

   double lsb = ANALOG_LSB_SPAN(vrg->RangeLow, vrg->RangeHigh, (1ULL << resInBit) - offset);
   double halfLsb = lsb / 2;

   table->VrgType    = vrg->VrgType;
   table->Gain       = vrg->Gain;
   table->PieceCount = 2;

   MapFuncPiece * poly;

   // map function poly 1:
   poly            = &table->Pieces[0];
   poly->Size      = sizeof(MapFuncPiece);
   poly->Degree    = 1;
   poly->UpperLimit= vrg->RangeLow - halfLsb;
   poly->Coef[0]   = offset;
   poly->Coef[1]   = 0;

   // map function poly 2:
   // bin_value = (bin_max - bin_offset) * (cur_value - cur_min) / (cur_max - cur_min) + bin_offset
   //           = (cur_value - cur_min) * ((bin_max - bin_offset) / (cur_max - cur_min)) + bin_offset
   // where '(bin_max - bin_offset) / (cur_max - cur_min)' is just the derivative of lsb, so
   // bin_value = (cur_value - cur_min) * 1 / lsb + bin_offset
   //           = cur_value * (1 / lsb) - cur_min * (1 / lsb) + bin_offset
   // so: coef[0] = bin_offset - cur_min * (1 / lsb)
   //     coef[1] = (1 / lsb)
   // Good Luck!
   poly            = &table->Pieces[1];
   poly->Size      = sizeof(MapFuncPiece);
   poly->Degree    = 1;
   poly->UpperLimit= vrg->RangeHigh - halfLsb;
   poly->Coef[0]   = -(vrg->RangeLow / lsb) + offset;
   poly->Coef[1]   = 1/lsb;
}

//*************************************************************************
// Name: AoBuildMapFuncByCalParamEx
// Description:
//       Helper function to build map function of specified value range
// (built-in, voltage, current).
//
//*************************************************************************
BDAQ_API
void BDAQCALL AoBuildMapFuncByCalParam(
   IN PCVRG_INFO     vrg,
   IN uint32         gainParam,
   IN uint32         offsetParam,
   IN double         scaleFactor,
   IN double         offsetFactor,
   IN double         swAdjust,
   OUT PSCALE_TABLE  table)
{
   /*
   * Note: Data Conversion Equation is:
   *    data_binary = [(data_scaled - offsetFactor)* (gainParam - offsetParam) / scaleFactor] + offsetParam
   *
   * where:
   *    '(gainParam - offsetParam) / scaleFactor' is just the derivative of lsb,
   *    generally the scaleFactor is '(vrg->rangeHigh - vrg->rangeLow)/2',
   *    but some devices maybe own its special scaling factor for each range. 
   * so:
   *     data_binary = [(data_scaled - offsetFactor) * 1 / lsb] + offset
   *                 = [data_sacled * 1 / lsb   ] - [offsetFactor * 1 /lsb] + offset
   *                 = [data_sacled * 1 / lsb   ] + offset - [offsetFactor * 1 /lsb]
   * SO, the polynomial is:
   * Coef[1] = 1/lsb 
   * Coef[0] = offsetParam - [offsetFacotr * 1 /lsb]
   */
   double lsb = scaleFactor/(gainParam - offsetParam);  // lsb of Unipolar
   MapFuncPiece *poly;

   table->VrgType    = vrg->VrgType;
   table->Gain       = vrg->Gain;
   table->PieceCount = 2;

   // map function poly 1:
   poly            = &table->Pieces[0];
   poly->Size      = sizeof(MapFuncPiece);
   poly->Degree    = 1;
   poly->UpperLimit= vrg->RangeLow + swAdjust - lsb / 2; // First polynomial's upper limit is the rangeLow minus half lsb
   poly->Coef[0]   = vrg->RangeLow / lsb + offsetParam + swAdjust / lsb - offsetFactor / lsb;
   poly->Coef[1]   = 0;

   // map function poly 2:
   poly            = &table->Pieces[1];
   poly->Size      = sizeof(MapFuncPiece);
   poly->Degree    = 1;
   poly->UpperLimit= vrg->RangeHigh;
   poly->Coef[0]   = offsetParam - offsetFactor / lsb;
   poly->Coef[1]   = 1 / lsb;
}

//*************************************************************************
// Name: DeviceGetDescription
// Description:
//       helper function to get device description. 
// Parameters: 
//  [in]deviceNumber -- the device number.
//  [in/out]length   -- the user buffer length 
//  [OUT]buffer -- the user buffer to receive the returned data.
// return value: 
//   error code
//************************************************************************* 
BDAQ_API
ErrorCode BDAQCALL DeviceGetDescription(
   IN uint32         deviceNumber, 
   IN OUT uint32     *length, 
   OUT OPTIONAL void *buffer )
{
   *length /= sizeof(wchar_t);
   ErrorCode ret = QueryDeviceFieldDB(deviceNumber, DFN_DESC, length, (wchar_t*)buffer);

   *length *= sizeof(wchar_t);
   return ret;
}

//*************************************************************************
// Name: DeviceSetDescription
// Description:
//       helper function to set device description. 
// Parameters: 
//  [in]devFd -- the device handle.
//  [in]length   -- the user buffer length 
//  [in]buffer -- the user buffer contains the data to be written out.
// return value: 
//    a error code.
//************************************************************************* 
BDAQ_API
ErrorCode BDAQCALL DeviceSetDescription(
   IN  uint32  deviceNumber, 
   IN  uint32  length, 
   IN  void    *buffer)
{
   wchar_t desc[MAX_DEVICE_DESC_LEN] = {0};

   //Limit the length to 64 character, according to the SPEC.
   length /= sizeof(wchar_t);
   if (length > MAX_DEVICE_DESC_LEN) {
      length = MAX_DEVICE_DESC_LEN;
   }

   //Fix bug: 11401
   //Convert the char '/' to '-', if any. The char '/' is reserved for driver and can't be used in description.
   //RtlCopyMemory(valInfo->Value, buffer, length);
   wchar_t const *src = (wchar_t *)buffer;
   wchar_t const *end = src + length;
   wchar_t       *dst = desc;
   for (; src < end; ++src, ++dst) {
      *dst = *src != L'/' ? *src : L'-';
   }

   //Set the stop character in case the buffer exceed the length limit
   desc[MAX_DEVICE_DESC_LEN - 1] = 0;
   return UpdateDeviceFieldDB(deviceNumber, DFN_DESC, desc);
}

//*************************************************************************
// Name: DeviceUpdateDescription
// Description:
//       helper function to update device description using the specified board id. 
// Parameters: 
//  [in]boardId -- the device board id.
//  [in]curDesc -- the current description to be updated
//  [out]newDesc-- the new description 
// return value: 
//    a error code.
//************************************************************************* 
BDAQ_API
void BDAQCALL DeviceUpdateDescription(
   IN  uint32        boardId, 
   IN  wchar_t const curDesc[], 
   OUT wchar_t       newDesc[])
{
   // Update the description according to the default format
#if defined(_WIN32) || defined(WIN32)
   wcsncpy_s(newDesc, MAX_DEVICE_DESC_LEN, curDesc, _TRUNCATE);
#else
   wcsncpy(newDesc, curDesc, MAX_DEVICE_DESC_LEN);
#endif

   wchar_t *bid_txt = wcsstr(newDesc, L"BID#");
   if (bid_txt) {
      bid_txt += 4;

      // Find for the rest of the original description following BID#xxx
      wchar_t const *rest = wcsstr(curDesc, L"BID#") + 4;
      while (isdigit(*rest)) { ++rest; }

      // Update board id
#if defined(_WIN32) || defined(WIN32)
      _snwprintf_s(bid_txt, newDesc + MAX_DEVICE_DESC_LEN - bid_txt, _TRUNCATE, L"%d%ls", boardId, rest);
#else
      swprintf(bid_txt, newDesc + MAX_DEVICE_DESC_LEN - bid_txt, L"%d%ls", boardId, rest);
#endif
   }
}

//*************************************************************************
// Name: DeviceCreatePnpEvent
// Description:
//       helper function to create device PnP event object
// Parameters: 
//  [in]evtId  -- the PnP event Id.
//  [in]prodId -- the product ID of the device
//  [in]devNum -- the device number of the device
// return value: 
//    a error code.
//************************************************************************* 
BDAQ_API
HANDLE BDAQCALL DeviceCreatePnpEvent(
   IN EventId evtId, 
   IN uint32  prodId, 
   IN uint32  devNum)
{
   char const *FMT_DEV_REMOVE = "Global\\dev_removed_0x%x_%d";
   char const *FMT_DEV_RECNNT = "Global\\dev_arrived_0x%x_%d";
   char const *fmt = NULL;

   switch (evtId)
   {
   default:                   return NULL;
   case EvtDeviceRemoved:     fmt = FMT_DEV_REMOVE; break;
   case EvtDeviceReconnected: fmt = FMT_DEV_RECNNT; break;
   }

   char name[EVENT_NAME_LEN_MAX + 1];
   name[EVENT_NAME_LEN_MAX] = 0;

#if defined(_WIN32) || defined(WIN32)
   _snprintf_s(name, EVENT_NAME_LEN_MAX, fmt, prodId, devNum);
   HANDLE h = CreateEventA(NULL, FALSE, FALSE, name);
#else
   snprintf(name, EVENT_NAME_LEN_MAX, fmt, prodId, devNum);
   HANDLE h = CreateEvent(NULL, FALSE, FALSE, name);
#endif

   return h;
}

//*************************************************************************
// Name: DeviceDbgReadRegisters
// Description:
//       Read a range of continuous registers from the specified start address.
// Parameters: 
//  [in]devFd -- the handle of the under-lying device.
//  [in]startAddr-- the start address to read from.
//  [in]length   -- the length to read.
//  [in]buffer   -- the user buffer to receive the returned data. 
//   
// return value: 
//    a error code.
//************************************************************************* 
static inline 
ErrorCode DeviceDbgReadPort(
   IN DEVFD  	devFd,
   IN uint64   startAddr,
   IN uint32   length,
   IN uint8	   *buffer)
{
   assert(length <= 4);
   DBG_REG_IO arg = {startAddr, length};
   ErrorCode ret = IoctlDevDbgReadRegister(devFd, &arg);
   if (ret == Success) {
      memcpy(buffer, &arg.Value, length);
   }
   return ret;
}

static ErrorCode DeviceDbgReadRegistersAligned(
   IN DEVFD  	devFd,
   IN uint64   startAddr,
   IN uint32   length,
   IN uint8	   *buffer)
{
   // Here the word 'aligned' means:
   // (startAddr & 0x3) == ((unsigned long)buffer & 0x3)
   // at this case no local buffer is needed.
   uint8 *end = buffer + length;
   uint64 mis_align;
   int    rd_len;

   ErrorCode ret = Success;

   // read one dword, one word or one byte,
   // according to the start address and the rest length
   while (buffer < end) {
      mis_align = startAddr & 0x3;
      if (!mis_align) {
         rd_len = x_min((int)(end - buffer), 4);
      } else {
         rd_len = x_min((int)(end - buffer), mis_align == 2 ? 2 : 1);
      }

      if ((ret = DeviceDbgReadPort(devFd, startAddr, rd_len, buffer)) != Success){
         break;
      }

      startAddr += rd_len;
      buffer    += rd_len;
   }

   return ret;
}

BDAQ_API
ErrorCode BDAQCALL DeviceDbgReadRegisters(
   IN DEVFD  	devFd,
   IN uint64   startAddr,
   IN uint32   length,
   IN void	   *buffer)
{
   ErrorCode ret;

   if ((startAddr & 0x3) == ((uint64)buffer & 0x3)) {
      ret = DeviceDbgReadRegistersAligned(devFd, startAddr, length, (uint8 *)buffer);
   } else {
      // align the startAddr and the buffer, then call the 'aligned' read method
      uint32 stack_buf[8 + 1]; // total 36 bytes
      uint8  *local, *aligned;
      if (length < 8 * sizeof(uint32)) {
         local = (uint8 *)stack_buf;
      } else {
         local = (uint8 *)malloc(length + 4);
         if (local == NULL){
            return ErrorMemoryNotEnough;
         }
      }

      aligned = local + (startAddr & 0x3);
      ret     = DeviceDbgReadRegistersAligned(devFd, startAddr, length, aligned);
      if (ret == Success){
         memcpy(buffer, aligned, length);
      }

      if (local != (uint8 *)stack_buf){
         free(local);
      }
   }

   return ret;
}

//*************************************************************************
// Name: DeviceDbgWriteRegisters
// Description:
//       Write a range of continuous registers from the specified start address.
// Parameters: 
//  [in]devFd -- the handle of the under-lying device.
//  [in]startAddr-- the start address to write to.
//  [in]length   -- the length to write.
//  [in]buffer   -- the user buffer contains the data to be written to device. 
//   
// return value: 
//    a error code.
//************************************************************************* 
static inline
ErrorCode DeviceDbgWritePort(
   IN DEVFD    devFd,
   IN uint64   startAddr,
   IN uint32   length, 
   IN uint8    *buffer)
{
   assert(length <= 4);

   DBG_REG_IO arg = {startAddr, length};
   memcpy(&arg.Value, buffer, length);
   return IoctlDevDbgWriteRegister(devFd, &arg);
}

static
ErrorCode DeviceDbgWriteRegistersAligned(
   IN DEVFD    devFd,
   IN uint64   startAddr,
   IN uint32   length, 
   IN uint8    *buffer)
{
   // Here the word 'aligned' means:
   // (startAddr & 0x3) == ((unsigned long)buffer & 0x3)
   // at this case no local buffer is needed.
   uint8 *end = buffer + length;
   uint64 mis_align;
   int    wr_len;

   ErrorCode ret = Success;

   // read one dword, one word or one byte,
   // according to the start address and the rest length
   while (buffer < end) {
      mis_align = startAddr & 0x3;
      if (!mis_align) {
         wr_len = x_min((int)(end - buffer), 4);
      } else {
         wr_len = x_min((int)(end - buffer), mis_align == 2 ? 2 : 1);
      }

      if ((ret = DeviceDbgWritePort(devFd, startAddr, wr_len, buffer)) != Success){
         break;
      }

      startAddr += wr_len;
      buffer    += wr_len;
   }

   return ret;
}

BDAQ_API
ErrorCode BDAQCALL DeviceDbgWriteRegisters(
   IN DEVFD   devFd,
   IN uint64  startAddr,
   IN uint32  length, 
   IN void    *buffer)
{
   ErrorCode ret;

   if ((startAddr & 0x3) == ((uint64)buffer & 0x3)) {
      ret = DeviceDbgWriteRegistersAligned(devFd, startAddr, length, (uint8 *)buffer);
   } else {
      // align the startAddr and the buffer, then call the 'aligned' read method
      uint32 stack_buf[8 + 1]; // total 36 bytes
      uint8  *local, *aligned;
      if (length < 8 * sizeof(uint32)) {
         local = (uint8 *)stack_buf;
      } else {
         local = (uint8 *)malloc(length + 4);
         if (local == NULL){
            return ErrorMemoryNotEnough;
         }
      }

      aligned = local + (startAddr & 0x3);
      memcpy(aligned, buffer, length);

      ret = DeviceDbgWriteRegistersAligned(devFd, startAddr, length, aligned);

      if (local != (uint8 *)stack_buf){
         free(local);
      }
   }

   return ret;
}
