/*******************************************************************************
              Copyright (c) 1983-2009 Advantech Co., Ltd.
********************************************************************************
THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY INFORMATION
WHICH IS THE PROPERTY OF ADVANTECH CORP., ANY DISCLOSURE, USE, OR REPRODUCTION,
WITHOUT WRITTEN AUTHORIZATION FROM ADVANTECH CORP., IS STRICTLY PROHIBITED. 

================================================================================
REVISION HISTORY
--------------------------------------------------------------------------------
$Log: $
--------------------------------------------------------------------------------
$NoKeywords:  $
*/

// ----------------------------------------------------------------------------
// File:        os_lnx.cpp
// Module:      UserLib.lib
// Environment: User Mode
// Reference:   Microsoft WDK
// Description: common functions which are independent with any special 'function'
// ----------------------------------------------------------------------------
#include "std_decl.h"

#if !defined(_WIN32) && !defined(WIN32)

//*************************************************************************
// Name: DeviceOpenKernDevice/DeviceCloseKernDevice
// Description:
//       Open/Close the kernel device according with the device number.
// Parameters: 
//  [in]devNumber -- The # of the device to open.
//  [in]accessMode -- the desired access accessMode to the device 
//   
// return value: 
//    the handle of device, if success, otherwise, return NULL.
// To get detail error info, call GetLastError.
//************************************************************************* 
BDAQ_API
ErrorCode BDAQCALL DeviceOpenKernDevice(
   IN uint32 devNumber, 
   IN uint32 accessMode, 
   IN uint32 szofKShared, 
   OUT DEVFD *devFd, 
   OUT void  **kshared)
{
   assert (devFd && kshared);
   *devFd = -1;
   *kshared= NULL;

   int fmode, pmode;
   if (accessMode == ModeRead){
      fmode = O_RDONLY;
      pmode = PROT_READ;
   } else {
      fmode = O_RDWR;
      pmode = PROT_READ | PROT_WRITE;
   }

   // Device number to device name
   char devname[256];
   uint32 length = 256;
   ErrorCode ret = QueryDeviceFieldADB(devNumber, DFN_DEVPATH, &length, devname);
   if (ret != Success) {
      return ErrorDeviceNotExist;
   }

   // Does device exist?
   struct stat stat_buf;
   if (stat(devname, &stat_buf)) { 
      return ErrorDeviceNotExist; 
   }

   // Open the device with proper privilege
   int fd = open(devname, fmode);
   if (fd < 0) { 
      return ErrorPrivilegeNotAvailable;
   }

   // make the file handle be 'write shared'
   if (accessMode == ModeWriteShared && IoctlDevMakeShared(fd, 1) != Success) {
      close(fd);
      return ErrorPrivilegeNotAvailable;
   }

   // Map the shared part of kernel device with proper privilege
   kshr_header *shared = (kshr_header *)mmap(NULL, szofKShared, pmode, MAP_SHARED, fd, 0);
   if (shared == MAP_FAILED){
      close(fd);
      return ErrorMemoryNotEnough;
   }

   if (shared->size != szofKShared){
      close(fd);
      return ErrorDriverVerMismatch;
   }

   *devFd = fd;
   *kshared = shared;
   return Success;
}

BDAQ_API
void BDAQCALL DeviceCloseKernDevice(IN uint32 szofKShared, IN DEVFD devFd, IN void *kshared)
{
   if (kshared != NULL && kshared != MAP_FAILED) {
      assert(szofKShared == ((kshr_header*)kshared)->size);

      munmap(kshared, szofKShared);
   }

   if (devFd != INVALID_HANDLE_VALUE) {
      close(devFd);
   }
}

//-----------------------------------------------------------------------
// Event APIs, for compatible with windows APIs
//-----------------------------------------------------------------------
__inline int GetKLibHandle()
{
   static int klib = -1;
   if (klib == -1) {
      klib = open("/dev/daq255", O_RDWR);
   }
   return klib;
}

BDAQ_API
HANDLE BDAQCALL CreateEvent(void *attr,int mrst, int initval, char *name)
{
   int klib = GetKLibHandle();
   NAMED_EVENT xbuf;

   xbuf.Handle = NULL;
   if (name && name[0]) {
      strncpy(xbuf.Name, name, EVENT_NAME_LEN_MAX);
      ioctl(klib, IOCTL_KLIB_CREATE_EVENT_NAMED, &xbuf);
   } else {
      ioctl(klib, IOCTL_KLIB_CREATE_EVENT, &xbuf.Handle);
   }
   return xbuf.Handle;
}

BDAQ_API
void BDAQCALL CloseHandle(HANDLE event)
{
   int klib = GetKLibHandle();
   ioctl(klib, IOCTL_KLIB_CLOSE_EVENT, event);
}

BDAQ_API
void BDAQCALL SetEvent(HANDLE event)
{
   int klib = GetKLibHandle();
   ioctl(klib, IOCTL_KLIB_SET_EVENT, event);
}

BDAQ_API
void BDAQCALL ResetEvent(HANDLE event)
{
   int klib = GetKLibHandle();
   ioctl(klib, IOCTL_KLIB_RESET_EVENT, event);
}

BDAQ_API
int BDAQCALL WaitForMultipleObjects(int count, HANDLE events[], int waitAll, int timeout)
{
   KLIB_WAIT_EVENT xbuf = {
      -1, timeout, waitAll, (uint32)count, {events}
   };
   int klib = GetKLibHandle();
   int ret;
   do{
      ret = ioctl(klib, IOCTL_KLIB_WAIT_EVENT, &xbuf);
   } while (ret < 0 && xbuf.Result == -EINTR);

   if (xbuf.Result >= 0) {
      ret = xbuf.Result;
   } else if (xbuf.Result == -1) {
      ret = WAIT_TIMEOUT;
   } else {
      ret = WAIT_FAILED;
   }
   return ret;
}

#endif
