/*******************************************************************************
              Copyright (c) 1983-2009 Advantech Co., Ltd.
********************************************************************************
THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY INFORMATION
WHICH IS THE PROPERTY OF ADVANTECH CORP., ANY DISCLOSURE, USE, OR REPRODUCTION,
WITHOUT WRITTEN AUTHORIZATION FROM ADVANTECH CORP., IS STRICTLY PROHIBITED. 

================================================================================
REVISION HISTORY
--------------------------------------------------------------------------------
$Log: $
--------------------------------------------------------------------------------
$NoKeywords:  $
*/

// ----------------------------------------------------------------------------
// File:        os_win.cpp
// Module:      UserLib.lib
// Environment: User Mode
// Reference:   Microsoft WDK
// Description: common functions which are independent with any special 'function'
// ----------------------------------------------------------------------------
#include "std_decl.h"

//-------------------------------------------------------------------------
// For Win32
//-------------------------------------------------------------------------
#if defined(_WIN32) || defined(WIN32)

#if !defined(_WIN32_WCE)
typedef struct _USERLIB_TLS {
   OVERLAPPED  ovlapIo; 
} USERLIB_TLS, *PUSERLIB_TLS;

static DWORD s_tlsIndex = TLS_OUT_OF_INDEXES; // TLS slot index assigned to the LIB.
#endif

static BOOL InitUserLibThreadCtx(DWORD tlsIndex);
static VOID DeinitUserLibThreadCtx(void);

BOOL APIENTRY DllMain(
#ifdef _WIN32_WCE
   HANDLE module, 
#else 
   HMODULE module,
#endif
   DWORD reason, 
   LPVOID reserved)
{
#if !defined(_WIN32_WCE)
   switch (reason)
   {
   case DLL_PROCESS_ATTACH: // The DLL is loading due to process initialization or a call to LoadLibrary. 
      if ((s_tlsIndex = TlsAlloc()) == TLS_OUT_OF_INDEXES) { // Allocate a TLS index.
         return FALSE; 
      }

      //
      // No break: Initialize the index for first thread. 
      // The attached process creates a new thread. 
   case DLL_THREAD_ATTACH:  // Initialize the TLS index for this thread.
      if (!InitUserLibThreadCtx(s_tlsIndex)) {
         return FALSE;
      }
      break; 
   case DLL_THREAD_DETACH:  // The thread of the attached process terminates.
      DeinitUserLibThreadCtx();
      break; 
   case DLL_PROCESS_DETACH: // DLL unload due to process termination or FreeLibrary. 
      DeinitUserLibThreadCtx();
      TlsFree(s_tlsIndex);  // Release the TLS index   
      break; 
   }
#else
   // WINCE, Do nothing!
#endif
   return TRUE;
}


//*************************************************************************
// Name: InitUserLibThreadCtx/DeinitUserLibThreadCtx
// Description:
//    initialize/deinitialize the TLS the lib used.
// Parameters: 
//    <none>
// return value: 
//    TRUE -- success, FALSE -- failure for initialize
//    <none> for deinitialize
//************************************************************************* 
static BOOL InitUserLibThreadCtx(DWORD tlsIndex)
{
#ifndef _WIN32_WCE
   TlsSetValue(tlsIndex, NULL);
   s_tlsIndex = tlsIndex;
   //
   // Note:
   // The thread context will be allocated when the thread calling the IOCTL method at first time.
   //
#endif
   return TRUE; // Nothing need do here
}

static VOID DeinitUserLibThreadCtx(void)
{
#ifndef _WIN32_WCE
   assert(s_tlsIndex != TLS_OUT_OF_INDEXES);
   PUSERLIB_TLS tls = (PUSERLIB_TLS)TlsGetValue(s_tlsIndex);
   if (tls)
   {
      if (tls->ovlapIo.hEvent)
      {
         CloseHandle(tls->ovlapIo.hEvent);
      }
      HeapFree(GetProcessHeap(), 0, tls);
      TlsSetValue(s_tlsIndex, NULL);
   }
#endif
}

//*************************************************************************
// Name: AllocUserLibThreadCtx
// Description:
//    allocate a user lib thread context for the calling thread.
// Parameters: 
//    [in]tlsIndex -- the TLS slot index assigned for the user lib. If this 
//                    parameter is not TLS_OUT_OF_INDEXES, the allocated context
//                    will be saved to the TLS slot too.
// return value: 
//    pointer to the context, NULL if failed.
//************************************************************************* 
#ifndef _WIN32_WCE
static PUSERLIB_TLS AllocUserLibThreadCtx(DWORD tlsIndex)
{
   PUSERLIB_TLS tls = (PUSERLIB_TLS)HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(USERLIB_TLS));
   if (tls)
   {
      tls->ovlapIo.hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
      if (!tls->ovlapIo.hEvent)
      {
         HeapFree(GetProcessHeap(), 0, tls);
         tls = NULL;
      }
   }
   if(tlsIndex != TLS_OUT_OF_INDEXES)
   {
      TlsSetValue(tlsIndex, tls);
   }
   return tls;
}
#endif

//*************************************************************************
// Name: GetFileVersion
// Description:
//       The function is used to get the specified file version.
// Parameters: [in]fileName -- name of the executable file.
//             [out]version -- version of the file. the buffer length of version
//                             should not be less that four byte.
// return value: 
//    TRUE -- success
//    FALSE -- failure
//
// copied from version check utility. :)
//************************************************************************* 
BDAQ_API
BOOL BDAQCALL GetExeFileVersion(
   IN  PCWSTR fileName, 
   OUT PWSTR  version)
{
   VS_FIXEDFILEINFO * verInfo;
   DWORD	             infoSize = 0;
   DWORD	             infoHnd  = 0;
   UINT	             infoLen  = 0;
   BOOL	             ret;

   infoSize = GetFileVersionInfoSize(fileName, &infoHnd);
   if (infoSize == 0)
   {
      return FALSE;
   }

   // allocate memory from stack for version data
   WCHAR * buffer = (WCHAR*)_alloca(infoSize);

   // read file version data
   GetFileVersionInfo(fileName, infoHnd, infoSize, buffer);

   // Get the file version firstly using the new methods. if failed
   // we will try again using the older methods.
   do{ 

      int codePage;
      int language;
      LPBYTE pData;

      // first get language and code page identifiers. we will use this to 
      // get file version.
      ret = VerQueryValue(buffer, L"\\VarFileInfo\\Translation", (void **)&pData, &infoLen);
      if (!ret || pData == NULL)
      {
         break;
      }
      language = *(int*)pData & 0xffff;
      codePage = *(int*)pData >> 16;    

      //build the value name of the file version. 
      WCHAR valueName[64];
      wsprintf(valueName, L"\\StringFileInfo\\%08X\\FileVersion", (language << 16) | codePage);

      //get the file version string
      ret = VerQueryValue(buffer, valueName, (void**)&pData, &infoLen);
      if (!ret || pData == NULL)
      {
         break;
      }

      lstrcpyn(version, (WCHAR*)pData, infoLen);
      return TRUE;

   } while (FALSE);

   // query file version value using the older methods.
   ret = VerQueryValue(buffer, L"\\", (void **)&verInfo, &infoLen);
   if (ret)
   {
      wsprintf(version, L"%d.%d.%d.%d", 
         (int)HIWORD(verInfo->dwFileVersionMS), (int)LOWORD(verInfo->dwFileVersionMS),
         (int)HIWORD(verInfo->dwFileVersionLS), (int)LOWORD(verInfo->dwFileVersionLS));
      return TRUE;
   }

   return FALSE;
}

//*************************************************************************
// Name: DeviceOpenKernDevice/DeviceCloseKernDevice
// Description:
//       Open/Close the kernel device according with the device number.
// Parameters: 
//  [in]devNumber -- The # of the device to open.
//  [in]accessMode -- the desired access accessMode to the device 
//   
// return value: 
//    the handle of device, if success, otherwise, return NULL.
// To get detail error info, call GetLastError.
//************************************************************************* 
BDAQ_API
ErrorCode BDAQCALL DeviceOpenKernDevice(
   IN uint32 devNumber, 
   IN uint32 accessMode, 
   IN uint32 szofKShared, 
   OUT DEVFD *devFd, 
   OUT void  **kshared)
{
   assert (devFd && kshared);
   *devFd   = INVALID_HANDLE_VALUE;
   *kshared = NULL;

   //
   // Get device interface name
   WCHAR symbolLink[256];
   uint32 length = 256;
   ErrorCode ret = QueryDeviceFieldDB(devNumber, DFN_DEVPATH, &length, symbolLink);

   if(ret != Success) { return ret; }
   
   uint32 desiredAccess;
   uint32 shareMode;
   if (accessMode == ModeRead) {
      desiredAccess = GENERIC_READ;
      shareMode     = FILE_SHARE_READ | FILE_SHARE_WRITE;
   } else {
      desiredAccess = GENERIC_READ | GENERIC_WRITE;
      shareMode     = FILE_SHARE_READ;
      if (accessMode == ModeWriteShared) {
         shareMode |= FILE_SHARE_WRITE;
      }
   }

#ifdef _WIN32_WCE
   int const fileFlag = FILE_ATTRIBUTE_NORMAL;
#else
   int const fileFlag = FILE_FLAG_OVERLAPPED;
#endif

   // Open the device with proper privilege
   HANDLE dev = CreateFile(symbolLink, desiredAccess, shareMode, NULL, OPEN_EXISTING, fileFlag, NULL);
   if (dev == INVALID_HANDLE_VALUE) {
      return SysErrorToBioError (GetLastError());
   }

   // Map the shared part of kernel device with proper privilege
   kshr_header *shared = NULL;

#ifdef _WIN32_WCE
   _tcscat(symbolLink, _T("_KShared"));

   HANDLE krnlMapHandle = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE | SEC_COMMIT, 0, szofKShared, symbolLink);
   if (GetLastError() != ERROR_ALREADY_EXISTS) {
      ret = ErrorDeviceNotExist;    //the handle is not shared memory handle
   } else {
      *shared = MapViewOfFile(krnlMapHandle, FILE_MAP_WRITE, 0, 0,0);
   }

   if (krnlMapHandle) { 
      CloseHandle(krnlMapHandle); 
   }      
#else
   DWORD size = sizeof( shared );
   ret = IoctlDevGetKShared(dev, (void **)&shared);
   if ( ret == Success )  {
      if ( !shared ) {
         ret = ErrorMemoryNotEnough;
      } else if ( shared->size != szofKShared ) {
         ret = ErrorDriverVerMismatch;
      }          
   }
#endif

   if ( ret != Success ) {
      CloseHandle(dev);
      return ret;
   }

   *devFd = dev;
   *kshared = shared;
   return Success;
}

BDAQ_API
void BDAQCALL DeviceCloseKernDevice(IN uint32 szofKShared, IN DEVFD devFd, IN void *kshared)
{
#ifdef _WIN32_WCE
   if (kshared) { UnmapViewOfFile(kshared); }
#endif

   if (devFd != INVALID_HANDLE_VALUE) {
      CloseHandle(devFd);
   }
}



//*************************************************************************
// Name: DeviceGetDllDriverVer
// Description:
//       get the dll driver version. 
// Parameters: 
//  [in]InstHdl -- the instance handle of the module.
//  [in]BufLen   -- the user buffer length 
//  [in]Buffer -- the user buffer to receive the returned data.
// return value: 
//    a error code.
//************************************************************************* 
BDAQ_API
ErrorCode BDAQCALL DeviceGetDllDriverVer(
   IN  HMODULE       instHdl, 
   IN OUT uint32     *length, 
   OUT OPTIONAL void *buffer)
{
   WCHAR filePath[MAX_PATH];
   GetModuleFileName(instHdl, filePath, MAX_PATH);

   WCHAR version[MAX_PATH];
   if (!GetExeFileVersion(filePath, version))
   {
      return ErrorConfigDataLost;
   }

   //
   // save version info into user buffer
   uint32 valLen = (lstrlen(version) + 1) * sizeof(WCHAR);
   ErrorCode ret = Success;
   if (buffer && *length < valLen)
   {
      ret = ErrorBufferTooSmall;
   } else {
      if (buffer)
      {
         lstrcpy((LPWSTR)buffer, version);
      }
   }
   *length = valLen; 
   return ret;
}

//*************************************************************************
// Name: DeviceGetKrnlDriverVer
// Description:
//       get the kernel(sys) driver version. 
// Parameters: 
//  [in]deviceNumber -- the device number.
//  [in]length   -- the user buffer length 
//  [in]buffer -- the user buffer to receive the returned data.
// return value: 
//    error code
//************************************************************************* 
BDAQ_API
ErrorCode BDAQCALL DeviceGetSysDriverVer(
#ifndef _WIN32_WCE
   IN  uint32         deviceNumber, 
#  else
   IN  PWSTR          templateKeyName, 
#  endif
   IN  OUT uint32     *length, 
   OUT OPTIONAL void  *buffer)
{
   WCHAR filePath[MAX_PATH];
   WCHAR value[MAX_PATH];
   DWORD valLen = MAX_PATH;

#ifndef _WIN32_WCE
   QueryDeviceFieldDB(deviceNumber, DFN_DRVNAME, (uint32*)&valLen, value);
   lstrcat(value, L"s.sys");
   
   // build the full qualified name of kernel driver
   GetSystemDirectory(filePath, MAX_PATH);
   lstrcat(filePath, L"\\Drivers\\");
   lstrcat(filePath, value);
#else
   HKEY  key;

   //
   // open the BIODAQ key to query the kernel key name
   LONG ret = RegOpenKeyEx(HKEY_LOCAL_MACHINE, templateKeyName, 0, KEY_QUERY_VALUE, &key);
   if(ret != ERROR_SUCCESS) {   return ErrorConfigDataLost; }

   //
   // query the kernel driver name
   valLen = sizeof(filePath);
   ret = RegQueryValueEx(key, ValImagePath, NULL, NULL, (BYTE*)filePath, &valLen);
   RegCloseKey(key);
   if(ret != ERROR_SUCCESS) { return ErrorConfigDataLost; }
#endif

   //
   // get the file version
   if (!GetExeFileVersion(filePath, value)) { return ErrorConfigDataLost; }

   //
   // save version info into user buffer
   ErrorCode err = Success;
   valLen = (lstrlen(value) + 1) * sizeof(WCHAR);
   if (buffer && *length >= valLen) { 
      wcscpy((LPWSTR)buffer, value);
   } else {
      if (buffer) { err = ErrorBufferTooSmall; }
   }
   *length = valLen;
   return err;
}

//*************************************************************************
// Name: SysErrorToBioError
// Description:
//       translate a system defined error code to bionic driver defined error code.
// Parameters: 
//  [in]sysErrorCode -- error code returned by system API
// return value: 
//    a bionic driver defined error code.
//************************************************************************* 
BDAQ_API
ErrorCode BDAQCALL SysErrorToBioError(uint32 sysErrorCode)
{
   // Not a system error Code. it must be a bionic driver error code we defined.
   if (sysErrorCode >= 0xA0000000) {
      return ErrorCode(sysErrorCode); 
   }

   ErrorCode ret;
   switch(sysErrorCode)
   {
   case ERROR_SUCCESS:             ret = Success;                 break;
   case ERROR_FILE_NOT_FOUND:      ret = ErrorDeviceNotExist;     break;
   case ERROR_MORE_DATA:           ret = ErrorBufferTooSmall;     break;
   case ERROR_INVALID_PARAMETER:   ret = ErrorBufferIsNull;       break; // the buffer is NULL!?
   case ERROR_NO_SYSTEM_RESOURCES: ret = ErrorDataLenExceedLimit; break; // the buffer is too large when calling ReadFile/WriteFile
   case ERROR_BAD_COMMAND:         ret = ErrorDeviceNotExist;     break; // the DeviceIocontrol return when the USB device is removed
   default:                        ret = ErrorUndefined;          break;
   }
   return ret;
}


//*************************************************************************
// Name: DeviceIoctlWrapper
// Description:
//    helper functions used to simplify the process of IOCTLs
// Parameters: 
//    krnlHdl - Handle to the kernel device object. 
//    ctlCode - IOCTL code
//    inBuf   - input buffer
//    inBufSize  - input buffer size
//    outBuf     - input buffer
//    outBufSize - output buffer size 
//    bytesReturned -- returned data size, in bytes.
// Return:
//    <error code>
//************************************************************************* 
BDAQ_API
ErrorCode BDAQCALL DeviceIoctlWrapper(
   IN HANDLE krnlHdl, 
   IN DWORD  ctlCode, 
   IN PVOID  inBuf, 
   IN DWORD  inBufSize, 
   IN PVOID  outBuf, 
   IN DWORD  outBufSize,
   OUT DWORD * bytesReturned) 
{
   //If inBuf is valid but the inBufSize set as 0, blue screen will occur 
   // when wdfRequestComplete is called. It is maybe the unmatched params made the OS confused
   //Add param check here, to avoid such situation occur
   if ((inBuf && inBufSize == 0) || (outBuf && outBufSize == 0)) {
      return Success;
   }
   if (!bytesReturned) {
      bytesReturned = &outBufSize;
   }

#ifdef _WIN32_WCE
   if (DeviceIoControl(krnlHdl, ctlCode, inBuf, inBufSize, outBuf, outBufSize, bytesReturned, NULL)) {
      return Success;
   }
#else
   assert(s_tlsIndex != TLS_OUT_OF_INDEXES);
   PUSERLIB_TLS tls = (PUSERLIB_TLS)TlsGetValue(s_tlsIndex);
   if (!tls){
      tls = AllocUserLibThreadCtx(s_tlsIndex);
      if (!tls) {// failed to allocate user lib thread context
         return ErrorMemoryNotEnough;
      }
   }
   assert(tls && tls->ovlapIo.hEvent);
   if (DeviceIoControl(krnlHdl, ctlCode, inBuf, inBufSize, outBuf, outBufSize, bytesReturned, &tls->ovlapIo)) {
      return Success;
   }

   if (ERROR_IO_PENDING == GetLastError()) {
      DWORD dataCount;
      if (GetOverlappedResult(krnlHdl, &tls->ovlapIo, &dataCount, TRUE)) {
         return Success;
      }
   } 
#endif
   return SysErrorToBioError(GetLastError());
}

//*************************************************************************
// Name: QueryDeviceIntfDetail
// Description:
//       Query the device interface details. The device interface detail contains a
//    path to the device driver that can be used to open the device driver.
// Parameters: 
//    devInfoSet  - Handle to the device information set. 
//    devInfoData - Pointer to an SP_DEVINFO_DATA structure that defines the device instance
//    devIntfGUID   - points to device interface GUID
//    detailData   - device interface details returned.
// return value: 
//    TRUE -- success
//    FALSE -- failure
//************************************************************************* 
#ifndef _WIN32_WCE
BDAQ_API
BOOL BDAQCALL QueryDeviceIntfDetail (
   IN HDEVINFO          devInfoSet,
   IN PSP_DEVINFO_DATA  devInfoData,
   IN GUID const      * devIntfGUID,
   OUT PSP_DEVICE_INTERFACE_DETAIL_DATA * detailData)
{
   // Get the device instance id (PnP string).  The first call will retrieve
   // the buffer length in characters.  fSuccess will be FALSE.
   DWORD deviceInstIdSize = 0;
   BOOL  success = SetupDiGetDeviceInstanceId (
                     devInfoSet, devInfoData,
                     NULL, 0, &deviceInstIdSize);
   // Check for error.
   if ((GetLastError () != ERROR_INSUFFICIENT_BUFFER) || (!deviceInstIdSize))
   {
      return FALSE;
   }

   // Allocate the buffer for the device instance ID (PnP string).
   PTSTR deviceInstID = (PTSTR)LocalAlloc (LPTR, deviceInstIdSize * sizeof (TCHAR));
   if (!deviceInstID) 
   {
      return FALSE;
   }

   // Now call again, this time with all parameters.
   success = SetupDiGetDeviceInstanceId (
                  devInfoSet, devInfoData,
                  deviceInstID, deviceInstIdSize, NULL);
   // Check for error.
   if (!success)
   {
      LocalFree (deviceInstID);
      return FALSE;
   }

   // Now we can get the handle to the dev info with interface.
   // We parse the device specifically for the XXX_GUID interface.
   HDEVINFO deviceIntfInfo = SetupDiGetClassDevs (
                              devIntfGUID, deviceInstID,
                              NULL,  DIGCF_DEVICEINTERFACE);
   // We don't need pDeviceInstanceID anymore.
   LocalFree (deviceInstID);

   // Check for error.
   if (deviceIntfInfo == INVALID_HANDLE_VALUE)
   {
      return FALSE;
   }

   // Go through the list of device interfaces of this device.
   // We assume that there is only one device interface and
   // we will store the device details in our private structure.
   SP_DEVICE_INTERFACE_DATA deviceIntfData;
   deviceIntfData.cbSize = sizeof (deviceIntfData);
   success = SetupDiEnumDeviceInterfaces (
               deviceIntfInfo, NULL,devIntfGUID, 0, &deviceIntfData);
   // Check for error.
   if (!success)
   {
      SetupDiDestroyDeviceInfoList (deviceIntfInfo);
      return FALSE;
   }

   // Get the details for this device interface.  The first call will retrieve
   // the buffer length in characters.  fSuccess will be FALSE.
   DWORD  deviceIntfDetailSize = 0;
   success = SetupDiGetDeviceInterfaceDetail (
                  deviceIntfInfo, &deviceIntfData,
                  NULL, 0, &deviceIntfDetailSize, NULL);
   // Check for error.
   if ((GetLastError () != ERROR_INSUFFICIENT_BUFFER) || (!deviceIntfDetailSize)) 
   {
      SetupDiDestroyDeviceInfoList (deviceIntfInfo);
      return FALSE;
   }

   // Allocate the buffer for the device interface detail data.
   *detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)LocalAlloc(LPTR, deviceIntfDetailSize);
   if (!(*detailData)) 
   {
      SetupDiDestroyDeviceInfoList (deviceIntfInfo);
      return FALSE;
   }
   // The size contains only the structure, not the additional path.
   (*detailData)->cbSize = sizeof (SP_DEVICE_INTERFACE_DETAIL_DATA);

   // Get the details for this device interface, this time with all parameters.
   success = SetupDiGetDeviceInterfaceDetail (
                  deviceIntfInfo, &deviceIntfData,
                  *detailData,  deviceIntfDetailSize,
                  NULL, NULL);

   // We don't need the handle anymore.
   SetupDiDestroyDeviceInfoList (deviceIntfInfo);

   if (!success) 
   {
      LocalFree(*detailData);
      *detailData = NULL;
      return FALSE;
   }
   return TRUE;
}
#endif
//*************************************************************************
// Name: QueryDeviceNumberByIntfDetail
// Description:
//       Query the device number via device interface details.
// Parameters: 
//     detailData   - device interface details data.
//    fieldOffsetInKObj - the device number field offset in the kernel device shared structure.
// Return:
//    -1 if failed, a non-zero positive value otherwise.
//************************************************************************* 
#ifndef _WIN32_WCE
BDAQ_API
uint32 BDAQCALL QueryDeviceNumberByIntfDetail(
   IN PSP_DEVICE_INTERFACE_DETAIL_DATA detailData, 
   IN uint32                            fieldOffsetInKObj)
{
   HANDLE devHdl = CreateFile (detailData->DevicePath,
                     GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE,
                     NULL, OPEN_EXISTING, 0, NULL);

   DEVICE_RECORD *rec = NULL;
   uint32       count = 10;   
   do {
      rec = new DEVICE_RECORD[count];
      if (rec == NULL) { return -1; }

      ErrorCode ret = QueryDeviceInfoDB(ByDevPath, detailData->DevicePath, &count, rec);
      if (ret != ErrorBufferTooSmall) {
         break;
      }
      delete [] rec;
      count += 10;
   } while (true);

   uint32 devNum = -1;
   for (unsigned i = 0; i < count; ++i) {
      if (rec[i].OnSystem) {
         devNum = rec[i].DeviceNumber;
         break;
      }
   }

   delete [] rec;
   return devNum;
}
#endif
//*************************************************************************
// Name: QueryDeviceNumberByIntfGUID
// Description:
//    Query the device number via device interface GUID.
// Parameters: 
//    devInfoSet  - Handle to the device information set. 
//    devInfoData - Pointer to an SP_DEVINFO_DATA structure that defines the device instance
//    devIntfGUID - pointer to the device interface GUID.
//    fieldOffsetInKObj - the device number field offset in the kernel device shared structure.
// Return:
//    -1 if failed, a non-zero positive value otherwise.
//************************************************************************* 
#ifndef _WIN32_WCE
BDAQ_API
uint32 BDAQCALL QueryDeviceNumberByIntfGUID(
   IN HDEVINFO          devInfoSet,
   IN PSP_DEVINFO_DATA  devInfoData,
   IN GUID const      * devIntfGUID,
   IN uint32             fieldOffsetInKObj)
{

   PSP_DEVICE_INTERFACE_DETAIL_DATA intfDetail;

   // Get the device interface detail which return a path to the device
   // driver that we need to open the device.
   if (!QueryDeviceIntfDetail(devInfoSet,devInfoData, devIntfGUID, &intfDetail)) 
   {
      return -1;
   }

   uint32 deviceNumber = QueryDeviceNumberByIntfDetail(intfDetail, fieldOffsetInKObj);

   // We don't need the device interface details any more, get rid of it now!
   LocalFree (intfDetail);

   return deviceNumber;
}
#endif


#endif

