/*******************************************************************************
              Copyright (c) 1983-2009 Advantech Co., Ltd.
********************************************************************************
THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY INFORMATION
WHICH IS THE PROPERTY OF ADVANTECH CORP., ANY DISCLOSURE, USE, OR REPRODUCTION,
WITHOUT WRITTEN AUTHORIZATION FROM ADVANTECH CORP., IS STRICTLY PROHIBITED. 

================================================================================
REVISION HISTORY
--------------------------------------------------------------------------------
$Log: $
--------------------------------------------------------------------------------
$NoKeywords:  $
*/

#ifndef DAQNAVI_USER_MODE_LIBRARY
#define DAQNAVI_USER_MODE_LIBRARY

#include <stdlib.h>
#include <string.h>
#include <bdaq_def.h>
#include <math.h>
#include "link_list.h"

#if (defined(_WIN32) || defined(WIN32)) && !defined(_WIN32_WCE)
#  include <setupapi.h>
#  include <malloc.h>
#  include <win32/ioctls.h>
#else
#  include <linux/ioctls.h>
#endif

#ifdef __cplusplus
#  define BDAQ_BEGIN_C_DECLARE extern "C" {
#  define BDAQ_END_C_DECLARE   }
#else
#  define BDAQ_BEGIN_C_DECLARE 
#  define BDAQ_END_C_DECLARE
#endif

//-------------------------------------------------------------------------
// macros used by user mode modules
//-------------------------------------------------------------------------

#if defined(_WIN32) || defined(WIN32)
//The following In/Out macro is used in DLL only, the kernel mode use AdxKeMemInX
#  define AdxMemInB(MemBase, Offset) (*(uint8*)((uint8*)MemBase + Offset))
#  define AdxMemInW(MemBase, Offset) (*(uint16*)((uint8*)MemBase + Offset))
#  define AdxMemInD(MemBase, Offset) (*(uint32*)((uint8*)MemBase + Offset))

#  define AdxMemOutB(MemBase, Offset, Data) (*(uint8*)((uint8*)MemBase + Offset) =  (uint8)Data)
#  define AdxMemOutW(MemBase, Offset, Data) (*(uint16*)((uint8*)MemBase + Offset) = (uint16)Data)
#  define AdxMemOutD(MemBase, Offset, Data) (*(uint32*)((uint8*)MemBase + Offset) = (uint32)Data)

   typedef ULONGLONG PHYSICAL_ADDRESS;
#endif

// --------------------------------------------------------------------------
// Macros to verify user parameters
// --------------------------------------------------------------------------
#define CHK_EXPR(expr, err)            do { if (!(expr)) return err; } while(0)
#define CHK_HANDLE(hdl)                CHK_EXPR(hdl != NULL, ErrorHandleNotValid)
#define CHK_PRVLG(expr)                CHK_EXPR(expr,  ErrorPrivilegeNotHeld)
#define CHK_DATA_PTR(ptr)              CHK_EXPR(ptr != NULL, ErrorBufferIsNull)
#define CHK_USER_BUF(buf)              CHK_EXPR(buf != NULL, ErrorBufferIsNull)
#define CHK_USER_BUF_LEN(buf, bufLen)  do{ if (buf == NULL) return ErrorBufferIsNull; if (!(bufLen)) return ErrorBufferTooSmall; } while(0)

#define SET_WARN(w, newWarn)           do{ if ((w) == Success) { w = newWarn; } } while(0)

// --------------------------------------------------------------------------
// Macro to calculate actual section length for buffered operation.
// --------------------------------------------------------------------------
#define CALC_ACTUAL_SECT_SIZE(section, samples) \
   (section > samples ? samples : (section ? section : (samples + 1) / 2))

// -------------------------------------------------------------------------------
// debug helper
// -------------------------------------------------------------------------------
#if defined(_DEBUG) || defined(DEBUG) || defined(DBG) || defined(DAQ_TRACING) || defined(BIO_TRACING)
#   if defined(_WIN32_WCE) || defined(__linux__)
#      define daq_trace(_x_)  printf _x_  
#   else
#     include <stdio.h>
#     include <stdarg.h>
#pragma warning(disable: 4996)
      inline void DbgPrint(char * fmt,...)
      {
         char msg[256];
         va_list args;
         va_start(args, fmt);
         _vsnprintf(msg, ARRAY_SIZE(msg), fmt, args);
         va_end(args);
         OutputDebugStringA(msg);
      }
#pragma warning(default: 4996)
#      define daq_trace(_x_)  DbgPrint _x_
#   endif
#else
#  define daq_trace(_x_)  do{}while(0)
#endif

#define BioTrace  daq_trace // for compatible with our linux driver

#if defined(_WIN32_WCE) || defined(__linux__)
#  define _malloca malloc
#  define _freea   free
#endif

//-------------------------------------------------------------------------
// 
//-------------------------------------------------------------------------
#if defined(_WIN32) || defined(WIN32)
   typedef HANDLE DEVFD;
#  define PAGE_READ  PAGE_READONLY
#  define PAGE_SIZE  getpagesize()

#  define x_valloc(addr, size, rw) VirtualAlloc(addr, size, MEM_COMMIT, rw);
#  define x_vlock(addr, size)      VirtualLock(addr, size)
#  define x_vfree(addr, size)      VirtualFree(addr, 0, MEM_RELEASE)

#  define x_msleep(ms)             Sleep(ms);

__inline uint32 getpagesize()
{
   static uint32 page_size = 0;
   if (!page_size) {
      SYSTEM_INFO info;
      GetSystemInfo(&info);
      page_size = info.dwPageSize;
   }
   return page_size;
}

#else
   typedef int    DEVFD;
#  define INVALID_HANDLE_VALUE -1
#  define MAXIMUM_WAIT_OBJECTS 64
#  define PAGE_READ            PROT_READ
#  define PAGE_READWRITE       (PROT_READ | PROT_WRITE)
#  define PAGE_SIZE            getpagesize()

#  define x_valloc(addr, size, rw) mmap(addr, size, rw,  MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
#  define x_vlock(addr, size)      // mlock(addr, size) : DO NOT call mlock family due to they can only be called under 'Supervisor' privilege.
#  define x_vfree(addr, size)      munmap(addr, size)

#  define x_msleep(ms)             usleep(ms * 1000);

#endif

//
// returns the input size rounded up to a multiple of the page size for the current platform.
#define ROUND_TO_PAGES(size)   (((size) + PAGE_SIZE - 1) & ~(PAGE_SIZE -1))

//-------------------------------------------------------------------------
// AI/AO function macros and structures
//-------------------------------------------------------------------------
//
// AI/AO Value range and gain code mapping table.
typedef struct VRG_INFO{
   uint32  VrgType;
   uint32  Gain;
   double  RangeLow;
   double  RangeHigh;
} VRG_INFO;
typedef VRG_INFO const * PCVRG_INFO;

//
// AI/AO scaling info table(polynomial for data conversion)
// !!!Internal use only!!!!
typedef struct SCALE_TABLE{
   uint32       VrgType;     // Predefined or user customized value range type.
   uint32       Gain;        // device specified gain code; 
   uint32       PieceCount;  // polynomial count.
   MapFuncPiece Pieces[1];   // variable length array, each of it is a scaling polynomial.
} SCALE_TABLE, *PSCALE_TABLE;
typedef SCALE_TABLE const * PCSCALE_TABLE;

// default piece count of map function
#define AI_DEF_PIECE_COUNT       1
#define AO_DEF_PIECE_COUNT       2

// Allocator and Deallocator of SCALING_TABLE
#define AllocScalingTable(size)  ((SCALE_TABLE *)new uint32[(size + 3)/4])
#define FreeScalingTable(table)  (delete [] table)

#define ANALOG_LSB_SPAN(low, high, bin_span)    (((high) - (low)) / (bin_span))
#define ANALOG_LSB_BITS(low, high, bits)        ANALOG_LSB_SPAN(low, high, double(1ULL<<bits))

/*-------------------------------------------------------------------
* Thermocouple type -- for internal using only!
*-------------------------------------------------------------------*/
typedef enum TempSensorType {
   TCType_J,      /* Thermocouple type J */
   TCType_K,      /* Thermocouple type K */
   TCType_S,      /* Thermocouple type S */
   TCType_T,      /* Thermocouple type T */
   TCType_B,      /* Thermocouple type B */
   TCType_R,      /* Thermocouple type R */
   TCType_E,      /* Thermocouple type E */
   TCType_N,      /* Thermocouple type N */
} TempSensorType;

/*-------------------------------------------------------------------
* Language type -- for internal using only!
*-------------------------------------------------------------------*/
typedef enum LanguageType {
   LT_EN_US,      /* Language for English */
   LT_ZH_CN,      /* Language for Simplified Chinese */
   LT_ZH_TW,      /* Language for Traditional Chinese */
} LanguageType;

//-----------------------------------------------------------------------------------------
// Macros & functions for Simplify property access.
//-----------------------------------------------------------------------------------------
//
// prototype for user customized Get/Set functions.
// ErrorCode T::Accessor(PropertyId id, uint32& size, void *buffer, uint32 opFlag);
// OpFlag : PropFlags
#  define BEGIN_PROPERTY_TABLE(PropFunc) \
   ErrorCode BDAQCALL PropFunc(PropertyId id, uint32 &size, void *buffer, uint32 flag) \
   {\
   switch (id) {

#define PROP_RO_VAL(propId, value) \
   case propId: return IsGet(flag) ? DefCopyT(value, buffer, size) : ErrorPropReadOnly;

#define PROP_RO_PTR(propId, ptr, length) \
   case propId: return IsGet(flag) ? DefCopyPtr(ptr, length, buffer, size) : ErrorPropReadOnly;

#define PROP_RO_ARR(propId, array) \
   PROP_RO_PTR(propId, array, sizeof(array))

#define PROP_CUSTOM(propId, func) \
   case propId: return func(propId, size, buffer, flag);

#define END_PROPERTY_TABLE(product, category) \
      default:  \
         if (IsGet(flag)) return PropReadValueDB(product, category, id, &size, buffer);\
         else             return ErrorPropNotSpted; \
      } \
   }

__inline ErrorCode DefCopyPtr(const void *src, uint32 srcLen, void *dst, uint32& dstLen)
{
   uint32 orgLen = dstLen;
   dstLen = srcLen;
   if (dst) {
      memcpy(dst, src, x_min(srcLen, orgLen));
      if (orgLen < dstLen) return ErrorBufferTooSmall;
   }
   return Success;
}

template<class T>
inline ErrorCode DefCopyT(T const & src, void *dst, uint32 &dstLen)
{
   return DefCopyPtr(&src, sizeof(T), dst, dstLen);
}

template<class Tin, class Tout>
inline ErrorCode DefCopyT(Tin const *src, uint32 srcLen, Tout *dst, uint32& dstLen)
{
   uint32 orgLen = dstLen;
   dstLen = (srcLen / sizeof(Tin)) * sizeof(Tout) ;
   if (dst) {
      Tin const *srcEnd = src + x_min(srcLen / sizeof(Tin), orgLen / sizeof(Tout));
      while (src < srcEnd) *dst++ = (Tout)*src++;
      if (orgLen < dstLen) return ErrorBufferTooSmall;
   }
   return Success;
}

#if defined(_WIN32) && !defined(_WIN64)
__inline ErrorCode DefCopyPtr(const void *src, ULONG srcLen, void *dst, ULONG& dstLen)
{
   return DefCopyPtr(src, (uint32)srcLen, dst, (uint32&)dstLen);
}
template<class T>
inline ErrorCode DefCopyT(T const & src, void *dst, ULONG &dstLen)
{
   return DefCopyPtr(&src, sizeof(T), dst, (uint32&)dstLen);
}
template<class Tin, class Tout>
inline ErrorCode DefCopyT(Tin const *src, ULONG srcLen, Tout *dst, ULONG &dstLen)
{
   return DefCopyT(src, (uint32)srcLen, dst, (uint32&)dstLen);
}
#endif

template<class T>
inline uint32 DefCopyString(T const *src, T *dst, uint32 dstLen)
{
   T *p = dst;

   for (; dstLen-- && (*p = *src); ++p, ++src) { /*Need do nothing*/ }

   // Counting the rest source chars, including the null-terminator 
   while (*src++) { ++p; }
   ++p;

   return uint32(p - dst);
}

//-----------------------------------------------------------------------------------------
// Helper function to get event index in the specified array.
//-----------------------------------------------------------------------------------------
template<class T>
inline int ArrayIndexOf(T const *array, uint32 len, T target)
{
   for (T const *curr = array; curr < array + len; ++curr){
      if (*curr == target) return (int)(curr - array);
   }
   return -1; // Not found.
}

template<>
inline int ArrayIndexOf(uint32 const *array, uint32 len, uint32 target)
{
   for (uint32 const *curr = array; curr < array + len; ++curr){
      if (*curr == target) return (int)(curr - array);
   }
   return -1; // Not found.
}

#define ARRAY_INDEX_OF(ar, target)  ArrayIndexOf(ar, ARRAY_SIZE(ar), target)

//-----------------------------------------------------------------------------------------
// Helper function to count the account of '1' in a number.
// A software implementation of ASM instruction: POPCNT, Population count
//-----------------------------------------------------------------------------------------
inline int soft_POPCNT(uint64 n)
{
   if (~n == 0) { return 64; }

   uint64 x = n - ((n >> 1) & 01333333333333333333333ULL) - ((n >> 2) & 01111111111111111111111ULL);
   x = ((x + (x >> 3)) & 0707070707070707070707ULL) % 63;
   if (!x && n) { 
      x = 63; 
   }
   return (int)x;
}

//-----------------------------------------------------------------------------------------
// Helper function to initialize the 'MathInterval' to default status.
//-----------------------------------------------------------------------------------------
inline void ResetMathInterval(MathInterval &x)
{
   x.Max = x.Min = 0;
   x.Type = (int32)LOBROB;
}

//-------------------------------------------------------------------------
// user mode lib APIs
//-------------------------------------------------------------------------
// Enum <--> String method
//-------------------------------------------------------------------------
BDAQ_BEGIN_C_DECLARE

BDAQ_API
ErrorCode BDAQCALL EnumToString(
   wchar_t const *enumTypeName,
   uint32        enumValue,
   uint32        enumStringLength,
   wchar_t       *enumString);

BDAQ_API
ErrorCode BDAQCALL StringToEnum(
   wchar_t const *enumTypeName,
   wchar_t const *enumString,
   uint32        *enumValue);

BDAQ_API
ErrorCode BDAQCALL EnumToStringEx(
   wchar_t const *enumTypeName,
   uint32        enumValue,
   LanguageType  type,
   uint32        enumStringLength,
   wchar_t       *enumString);

BDAQ_API
ErrorCode BDAQCALL QueryVrgInfoDB(
   uint32       vrg, 
   uint32       *unit, 
   MathInterval *range, 
   uint32       sizeofDesc, 
   wchar_t      *desc);

//---------------------------------------------------------------------
// property attribute DB field access
//
#define PROPATTR_ATTR  "Attribute"     // attribute: modal/nature, rw/ro
#define PROPATTR_TYPE  "Type"          // data type
#define PROPATTR_DESC  "Description"   // 
#define PROPATTR_TIP   "Tip"           // 

BDAQ_API
ErrorCode BDAQCALL PropIdToAttrDB(
   uint32     propId, 
   char const *field, 
   uint32     *strLen, 
   char       *strBuf);

BDAQ_API
ErrorCode BDAQCALL PropIdToAttrI32DB(
   uint32     propId,
   char const *field, 
   uint32     *value);

BDAQ_API
ErrorCode BDAQCALL PropSymToAttrDB(
   char const *symbol, 
   char const *field, 
   uint32     *strLen, 
   char       *strBuf);

BDAQ_API
ErrorCode BDAQCALL PropSymToAttrI32DB(
   char const *symbol,
   char const *field, 
   uint32     *value);

__inline 
uint32 BDAQCALL catofModule(ModuleType moduleType, uint32 moduleIndex)
{
   return moduleType | (moduleIndex << 16);
}

__inline
ModuleType BDAQCALL typeofCat(uint32 category, uint32 *moduleIndex)
{
   if (moduleIndex) { *moduleIndex = category >> 16; }
   return (ModuleType)(category & 0xFFFF); // module type
}

#define PROPACCESS_READONLY  0
#define PROPACCESS_READWRITE 1
typedef void (BDAQCALL * PropEnumCallback)(
   uint32 category, 
   uint32 id, 
   uint32 access,
   uint32 size, 
   void   *buffer,
   void   *userParam); 

BDAQ_API
ErrorCode BDAQCALL PropEnumerateDB(
   uint32           productId,
   int32            category,
   PropEnumCallback enumCallBack,
   void             *userParam);

BDAQ_API
ErrorCode BDAQCALL PropReadValueDB(
   uint32 productId, 
   uint32 category, 
   uint32 propId, 
   uint32 *bufSize, 
   void   *buffer);

BDAQ_API
ErrorCode BDAQCALL PropWriteValueDB(
   uint32 productId, 
   uint32 category, 
   uint32 propId, 
   uint32 bufSize, 
   void   *buffer);

BDAQ_API
ErrorCode BDAQCALL PropValueToString(
   uint32   propId, 
   uint32   valSize, 
   void    *valBuf,
   uint32  *strLen,
   wchar_t *strBuf);

BDAQ_API
ErrorCode BDAQCALL PropValueToStringA(
   uint32  propId, 
   uint32  valSize, 
   void   *valBuf,
   uint32 *strLen,
   char   *strBuf);

BDAQ_API
ErrorCode BDAQCALL PropStringToValue(
   uint32        propId, 
   wchar_t const *strBuf,
   uint32        *valSize, 
   void          *valBuf);

BDAQ_API           
ErrorCode BDAQCALL PropStringToValueA(
   uint32     propId, 
   char const *strBuf,
   uint32     *valSize, 
   void       *valBuf);

//---------------------------------------------------------------------
// device information DB access
//
BDAQ_API
ErrorCode BDAQCALL EnumDeviceInfoDB(
   int32 &deviceIndex, 
   int32 &active,
   int32 &deviceNumber, 
   char  desc[MAX_DEVICE_DESC_LEN]);

BDAQ_API
ErrorCode BDAQCALL FindDeviceNumberDB(
   DEVICE_HWINFO_EX const &hw, 
   int32                  *deviceNumber);

BDAQ_API
ErrorCode BDAQCALL AllocDeviceNumberDB(
   DEVICE_HWINFO_EX const &hw, 
#if defined(_WIN32) || defined(WIN32)
   wchar_t const *devicePath,
#else 
   char const    *devicePath,
#endif
   int32         *deviceNumber);

BDAQ_API
ErrorCode BDAQCALL AllocDeviceNumberDBForAudio(
   DEVICE_HWINFO_EX const &hw, 
#if defined(_WIN32) || defined(WIN32)
   wchar_t const *devicePath,
   wchar_t const *deviceDesc,
#else 
   char const    *devicePath,
   char const    *deviceDesc,
#endif
   int32         *deviceNumber);

BDAQ_API 
ErrorCode BDAQCALL FreeDeviceNumberDB(int32 deviceNumber);

BDAQ_API
void BDAQCALL ResyncDeviceInfoDB();

typedef enum QueryType {
   ListAll = 0,
   ByDevNumber,
   ByDevDesc,
   ByDevPath,
} QueryType;

typedef struct DEVICE_RECORD {
   int32  DeviceNumber;
   uint32 ProductId;
   uint32 OnSystem;
   uint32 DriverSPEC;
   char   DriverName[32];
   char   DeviceDesc[64];
   char   DevicePath[256];
} DEVICE_RECORD;

BDAQ_API
ErrorCode BDAQCALL QueryDeviceInfoDB(
   QueryType     type,
   wchar_t const *key,
   uint32        *count,
   DEVICE_RECORD *buffer);

BDAQ_API
ErrorCode BDAQCALL QueryDeviceInfoADB(
   QueryType     type,
   char const    *key,
   uint32        *count,
   DEVICE_RECORD *buffer);

// For ECAT devices
typedef struct DEVICE_RECORD_EX : public DEVICE_RECORD {
   // Extend information
   uint32        BoardId;
   uint32        BusSlot;
} DEVICE_RECORD_EX;

BDAQ_API
ErrorCode BDAQCALL QueryDeviceInfoDBEx(
   QueryType        type,
   wchar_t const    *key,
   uint32           *count,
   DEVICE_RECORD_EX *buffer);

BDAQ_API
ErrorCode BDAQCALL QueryDeviceInfoADBEx(
   QueryType        type,
   char const       *key,
   uint32           *count,
   DEVICE_RECORD_EX *buffer);

BDAQ_API 
ErrorCode BDAQCALL UpdateDeviceInfoDB(
   DEVICE_HWINFO_EX const &hw, 
#if defined(_WIN32) || defined(WIN32)
   wchar_t const *devicePath
#else 
   char const    *devicePath
#endif
  );

//---------------------------------------------------------------------
// device information DB field access
// field index & name
#define DFI_DEVNUM  0
#define DFN_DEVNUM  "DeviceNumber"     

#define DFI_PRODID  1    
#define DFN_PRODID  "ProductID"    

#define DFI_BOARDID 2 
#define DFN_BOARDID "BoardID"  

#define DFI_BUSSLOT 3 
#define DFN_BUSSLOT "BusSlot"  

#define DFI_ACTIVE  4
#define DFN_ACTIVE  "Active"

#define DFI_TERMBD  5
#define DFN_TERMBD  "TerminalBoard"

#define DFI_DRVSPEC 6
#define DFN_DRVSPEC "DriverSPEC"

#define DFI_DRVNAME 7
#define DFN_DRVNAME "DriverName"

#define DFI_DEVPATH 8
#define DFN_DEVPATH "DevicePath"

#define DFI_DESC    9
#define DFN_DESC    "Description"

#define DFI_PARAM   10
#define DFN_PARAM   "Parameter"

#define DFI_FUSIONID 11
#define DFN_FUSIONID "FusionID"

#define DFI_TREEPATH 12
#define DFN_TREEPATH "TreePath"

BDAQ_API
ErrorCode BDAQCALL QueryDeviceFieldDB(
   int32         deviceNumber, 
   char const    *field, 
   uint32        *strLen,
   wchar_t       *strBuf);

BDAQ_API
ErrorCode BDAQCALL QueryDeviceFieldADB(
   int32      deviceNumber, 
   char const *field, 
   uint32     *strLen,
   char       *strBuf);

BDAQ_API
ErrorCode BDAQCALL UpdateDeviceFieldDB(
   int32         deviceNumber, 
   char const    *field, 
   wchar_t const *value);

BDAQ_API
ErrorCode BDAQCALL UpdateDeviceFieldADB(
   int32      deviceNumber, 
   char const *field, 
   char const *value);

// helper method
BDAQ_API
ErrorCode BDAQCALL QueryDeviceFieldI32DB(
   int32      deviceNumber, 
   char const *field, 
   int32      *value);

BDAQ_API
ErrorCode BDAQCALL UpdateDeviceFieldI32DB(
   int32      deviceNumber, 
   char const *field, 
   int32      value);

// Access 'Misc' table(key/value)
BDAQ_API
ErrorCode BDAQCALL QueryMiscField(
   char const *key,
   uint32     *strLen,
   char       *strBuf);

//---------------------------------------------------------------------
// enum string information DB field access
//
#define ENUMSTR_SYMBOL  "Symbol"     
#define ENUMSTR_ID      "ID"        
#define ENUMSTR_TEXT    "Text"          

typedef void (BDAQCALL * EnumStrEnumCallback)(
   void   *buffer,
   void   *userParam); 

BDAQ_API
ErrorCode BDAQCALL EnumStrEnumerateDB(
   char const         *category,
   char const         *field,
   EnumStrEnumCallback userFunc,
   void                *userParam);

//---------------------------------------------------------------------
// supported product information DB field access
//
typedef struct PRODUCT_INFO {
   char const *Name;
   char const *Bus;
   char const *HWID;
   char const *Package; 
   char const *Model;
   char const *Description;
   char const *Manual;
   char const *Feature;
   char const *ManualPackage;
} PRODUCT_INFO;

//
// return value of the callback:
// 0 -- ok, continue to enumerate the next one
// 1 -- abort, finish the enumeration
typedef int (BDAQCALL * EnumSuppProductCallback)(
   PRODUCT_INFO *prodInfo,
   void         *userParam);

BDAQ_API
ErrorCode BDAQCALL EnumSupportedProductDB(
   EnumSuppProductCallback userFunc,
   void                    *userParam);

BDAQ_API
ErrorCode BDAQCALL QueryDeviceBusTypeDB(
   ProductId id,
   uint32   *strLen,
   char     *strBuf);

//
// helper: close the configuration DB
BDAQ_API
void BDAQCALL CloseDaqNaviConfigDB();

BDAQ_END_C_DECLARE

//-------------------------------------------------------------------------
// Miscellaneous Method
//-------------------------------------------------------------------------
BDAQ_BEGIN_C_DECLARE

BDAQ_API
void BDAQCALL UserFreqToCounterDivider(
   IN  uint32   baseFreq,
   IN  double   userFreq,
   OUT uint16   *div1,
   OUT uint16   *div2);

BDAQ_API
ErrorCode BDAQCALL DeviceOpenKernDevice(
   IN uint32 devNumber, 
   IN uint32 accessMode, 
   IN uint32 szofKShared, 
   OUT DEVFD *devFD, 
   OUT void  **kshared);

BDAQ_API
void BDAQCALL DeviceCloseKernDevice(
   IN uint32 szofKShared, 
   IN DEVFD  devFd, 
   IN void   *kshared);

BDAQ_API
ErrorCode BDAQCALL DeviceDbgReadRegisters(
   IN DEVFD  devHdl,
   IN uint64 startAddr,
   IN uint32 length,
   IN void   *buffer);

BDAQ_API
ErrorCode BDAQCALL DeviceDbgWriteRegisters(
   IN DEVFD  devHdl,
   IN uint64 startAddr,
   IN uint32 length, 
   IN void   *buffer);

BDAQ_API
ErrorCode BDAQCALL DeviceGetDescription(
   IN uint32         deviceNumber, 
   IN OUT uint32     *length, 
   OUT OPTIONAL void *buffer);

BDAQ_API
ErrorCode BDAQCALL DeviceSetDescription(
   IN  uint32 deviceNumber, 
   IN  uint32 length, 
   IN  void   *buffer);

// New: Rebuild device description
BDAQ_API
void BDAQCALL DeviceUpdateDescription(
   IN  uint32        boardId, 
   IN  wchar_t const curDesc[], 
   OUT wchar_t       newDesc[]);

// New: device PnP event
BDAQ_API
HANDLE BDAQCALL DeviceCreatePnpEvent(
   IN EventId evtId, 
   IN uint32  prodId, 
   IN uint32  devNum);

#if defined(_WIN32) || defined(WIN32)
   BDAQ_API
   ErrorCode BDAQCALL DeviceGetDllDriverVer(
      IN  HMODULE        instHdl, 
      IN OUT uint32      *length, 
      OUT OPTIONAL void  *buffer);

   BDAQ_API
   ErrorCode BDAQCALL DeviceGetSysDriverVer(
#  ifndef _WIN32_WCE
      IN  uint32          deviceNumber, 
#  else
      IN  PWSTR          templateKeyName, 
#  endif
      IN OUT uint32      *length, 
      OUT OPTIONAL void  *buffer);

#  define DeviceGetKrnlDriverVer DeviceGetSysDriverVer // For compatible

   BDAQ_API
   BOOL BDAQCALL GetExeFileVersion(IN PCWSTR fileName, OUT PWSTR version);

   BDAQ_API
   ErrorCode BDAQCALL SysErrorToBioError(IN uint32 sysErrorCode);

   BDAQ_API
   ErrorCode BDAQCALL DeviceIoctlWrapper(
      IN HANDLE krnlHdl, 
      IN DWORD  ctlCode, 
      IN PVOID  inBuf, 
      IN DWORD  inBufSize, 
      IN PVOID  outBuf, 
      IN DWORD  outBufSize,
      OUT DWORD * bytesReturned);

#  ifndef _WIN32_WCE
      BDAQ_API
      BOOL BDAQCALL QueryDeviceIntfDetail (
         IN HDEVINFO          devInfoSet,
         IN PSP_DEVINFO_DATA  devInfoData,
         IN GUID const      * devIntfGUID,
         OUT PSP_DEVICE_INTERFACE_DETAIL_DATA* detailData);

      BDAQ_API
      ULONG BDAQCALL  QueryDeviceNumberByIntfDetail(
         IN PSP_DEVICE_INTERFACE_DETAIL_DATA detailData,
         IN ULONG                            fieldOffsetInKObj);

      BDAQ_API
      ULONG BDAQCALL QueryDeviceNumberByIntfGUID(
         IN HDEVINFO          devInfoSet,
         IN PSP_DEVINFO_DATA  devInfoData,
         IN GUID const      * devIntfGUID,
         IN ULONG             fieldOffsetInKObj);
#  endif

#endif

BDAQ_API
PCVRG_INFO BDAQCALL VrgFindItemByGain(
   IN uint32    gain,
   IN PCVRG_INFO entries,
   IN uint32    count);

BDAQ_API
PCVRG_INFO BDAQCALL VrgFindItemByVrgType(
   IN uint32    vrgType,
   IN PCVRG_INFO entries,
   IN uint32    count);

BDAQ_API
uint32 BDAQCALL SizeOfMapFuncPieces(
   IN MapFuncPiece const *polys,
   IN uint32             count);

BDAQ_API
uint32 BDAQCALL CountOfMapFuncPieces(
   IN MapFuncPiece const *polys,
   IN uint32             size);

BDAQ_API
uint32 BDAQCALL AiCalcLogicalChanCount(
   IN uint8 const chType[], 
   IN uint32 count);

BDAQ_API
void BDAQCALL AiBuildMapFunc(
   IN PCVRG_INFO   vrg,
   IN uint32       resInBit,
   OUT SCALE_TABLE *table);

BDAQ_API
void BDAQCALL AoBuildVoltageMapFunc(
   IN PCVRG_INFO    vrg,
   IN uint32        resInBit,
   IN uint32        extRefAntiPolar, /*ignored if not external reference*/
   IN double        extRefVoltage,   /*ignored if not external reference*/
   OUT SCALE_TABLE  *table);

BDAQ_API
void BDAQCALL AoBuildCurrentMapFunc(
   IN PCVRG_INFO    vrg,
   IN uint32        resInBit,
   IN uint32        offset,
   OUT SCALE_TABLE  *table);

// New: Introduced since PCIe-1803
BDAQ_API
void BDAQCALL AiBuildMapFuncByCalParam(
   IN PCVRG_INFO   vrg,
   IN uint32       resInBit,
   IN uint32       gainParam,
   IN uint32       offsetParam,
   IN double       scaleFactor,
   IN double       offsetFactor,
   OUT SCALE_TABLE *table);

// New: Introduced since PCIe-1824
BDAQ_API
void BDAQCALL AoBuildMapFuncByCalParam(
   IN PCVRG_INFO     vrg,
   IN uint32         gainParam,
   IN uint32         offsetParam,
   IN double         scaleFactor,
   IN double         offsetFactor,
   IN double         swAdjust,
   OUT PSCALE_TABLE  table);

typedef struct RTD_INFO{
   uint32 VrgType;
   uint32 Gain;   
   int32  TempLow;
   int32  TempHigh;
   double ScaleFactor;
   double OffsetFactor;
   double ResisMin;
   double ResisZero;
   double ResisMax;
   double A0_Coef;
   double B0_Coef;
   double C0_Coef;
   double D0_Coef;
   double E0_Coef;
   double A1_Coef;
   double B1_Coef;
   double C1_Coef;
   double D1_Coef;
   double E1_Coef;
}RTD_INFO;
typedef RTD_INFO const * PCRTD_INFO;

// New: Introduced since iDaq-815
BDAQ_API
void BDAQCALL RtdBuildMapFuncByCalParam(    
    IN PCRTD_INFO   rtdInfoMap,
    IN uint32       gainParam,   
    IN uint32       offsetParam,        
    OUT SCALE_TABLE *table);

BDAQ_API
PCRTD_INFO BDAQCALL RtdFindItemByVrgType(
   IN uint32     vrgType,
   IN PCRTD_INFO entries,
   IN uint32     count);

// for compatible
#define AiBuildDefaultMapFunc         AiBuildMapFunc
#define AoBuildVoltageDefaultMapFunc  AoBuildVoltageMapFunc
#define AoBuildCurrentDefaultMapFunc  AoBuildCurrentMapFunc

BDAQ_END_C_DECLARE

//--------------------------------------------------------------------------
// Thermo functions
//--------------------------------------------------------------------------
BDAQ_BEGIN_C_DECLARE

BDAQ_API double BDAQCALL GetThermoTemperature(double thermoVolt, uint32 thermoType);
BDAQ_API double BDAQCALL GetCjcVoltage(double cjcTemperature, uint32 thermoType);
BDAQ_API double BDAQCALL ThermoTemperature(double fV1, uint32 type);
BDAQ_API double BDAQCALL TemperatureScales(double tmp, uint32 unit);
BDAQ_API double BDAQCALL GetCjcErrorVoltage(double cjc, uint32 type);
BDAQ_API double BDAQCALL GetActualThermoVolt(double thermoVolt, uint32 vrgType);
BDAQ_API double BDAQCALL GetActualThermoValue(double thermoValue, uint32 vrgType);

BDAQ_END_C_DECLARE

//-----------------------------------------------------------------------
// Event APIs, for compatible with windows APIs
//-----------------------------------------------------------------------
#if !defined(_WIN32) && !defined(WIN32)
BDAQ_BEGIN_C_DECLARE

#define MAXIMUM_WAIT_OBJECTS   64
#define WAIT_FAILED            ((uint32)-1)
#define WAIT_TIMEOUT           258
#define WAIT_ABANDONED_0       ((uint32)0x80)
#define WAIT_OBJECT_0          ((uint32)0x0)
#define INFINITE               -1

BDAQ_API HANDLE BDAQCALL CreateEvent(void *attr,int mrst, int initval, char *name);
BDAQ_API void BDAQCALL CloseHandle(HANDLE event);
BDAQ_API void BDAQCALL SetEvent(HANDLE event);
BDAQ_API void BDAQCALL ResetEvent(HANDLE event);
BDAQ_API int BDAQCALL WaitForMultipleObjects(int count, HANDLE events[], int waitAll, int timeout);

__inline int BDAQCALL WaitForSingleObject(HANDLE handle, int timeout)
{
   return WaitForMultipleObjects(1, &handle, 0, timeout);
}

BDAQ_END_C_DECLARE
#endif

//--------------------------------------------------------------------------
// inline functions
//--------------------------------------------------------------------------
//
// Helper method used to calculate the section count according to the sample rate and section length.
// szofElem -- size of the element in the buffer, i.e. AI_DATA_SIZE
// sampleRate -- total sample rate
// sectLength -- total section length
inline uint32 CalculateMaximumSectCount(uint32 szofElem, double sampleRate, uint32 sectLength)
{
   uint32 const MEM_THRESHOLD = 30 * 1024 * 1024;
   uint32 const MEM_THRESHOLD_DEV = MEM_THRESHOLD / szofElem;

   uint32 sectCount = uint32(sampleRate * 1 / sectLength); // default: 1s

   if (sectCount < 2) { // default: the min. sect count is 2
      sectCount = 2;
   }

   if (sectCount * sectLength < 4096) {
      sectCount = 4096 / sectLength;
   } 

   if (sectCount * sectLength > MEM_THRESHOLD_DEV) {
      sectCount = MEM_THRESHOLD_DEV / sectLength;
   }

   if (sectCount == 0) {
      sectCount = 1;
   } 

   return sectCount;
}

inline uint32 CalculateMaximumRecordCount(uint32 /*szofElem*/, double sampleRate, uint32 recordLength, uint32 const MAX_ALLOWED)
{
   uint32 recCount = (uint32)ceil(sampleRate * 1 / recordLength); // default: 1s
   return x_max(1, x_min(recCount, MAX_ALLOWED));                 // default: the min. record count is 1
}

inline uint32 CalculateMinimumRecordCount(double sampleRate, uint32 recordLength, uint32 const MAX_ALLOWED)
{
   uint32 rec_count = 2;

   while (rec_count <= MAX_ALLOWED) {
      double samp_time = recordLength * rec_count / sampleRate;
      if(samp_time >= 1.0) { // Sampling time >= 1s
         return rec_count;
      }

      rec_count++; 
   }

   return MAX_ALLOWED;
}

//--------------------------------------------------------------------------
// Data scale methods. BIN <--> Floating
inline double ScaleDataUsePolynomial(
   IN PCMapFuncPiece polys,
   IN uint32         count,
   IN double         inData)
{
   uint32 i;
   PCMapFuncPiece lastPiece = polys;
   //
   // find the map function piece according to the input data
   for (i = 0; i < count && inData > polys->UpperLimit; ++i) {
      lastPiece = polys;
      polys = (PCMapFuncPiece)((uint8*)polys + polys->Size);
   }
   //
   // the input data is out of range
   if (i >= count) {
      polys = lastPiece;              // Roll back to the last piece
      inData = lastPiece->UpperLimit;  // limit the input to a valid value
   }
   //
   // calculate the polynomial using method:
   // F(x) = (...(Cn * x + Cn-1) * x + Cn-2).....) * x + C1) * x + C0
   int              j = polys->Degree;
   double const *coef = polys->Coef;
   double      result = coef[j];
   do {
      result = result * inData + coef[--j];
   } while (j);
   return result;
}

//--------------------------------------------------------------------------
// Special:
// Data scale methods. Floating ---> Binary with data truncation.
inline uint32 ScaleDataUsePolynomialWithMask(
   IN PCMapFuncPiece polys,
   IN uint32         count,
   IN uint32         mask,
   IN double         inData)
{
   uint32 i;
   PCMapFuncPiece lastPiece = polys;
   //
   // find the map function piece according to the input data
   for (i = 0; i < count && inData > polys->UpperLimit; ++i) {
      lastPiece = polys;
      polys = (PCMapFuncPiece)((uint8*)polys + polys->Size);
   }
   //
   // the input data is out of range
   if (i >= count) {
      polys = lastPiece;              // Roll back to the last piece
      inData = lastPiece->UpperLimit;  // limit the input to a valid value
   }
   //
   // calculate the polynomial using method:
   // F(x) = (...(Cn * x + Cn-1) * x + Cn-2).....) * x + C1) * x + C0
   int              j = polys->Degree;
   double const *coef = polys->Coef;
   double      result = coef[j];
   do {
      result = result * inData + coef[--j];
   } while (j);

   int64 bin = int64(result);   
   if (bin < 0) {
      bin = 0;
   } else if (bin > mask) {
      bin = mask;
   } 
   
   return uint32(bin); // simple rounding
}

template<class Tin, class Tout>
inline uint32 ScaleDataToBinary(
   IN PSCALE_TABLE logCHVrg[], 
   IN uint32       logCHStart,
   IN uint32       logCHEnd,
   IN Tin          *inBuf,
   OUT Tout        *outBuf)
{
   for (; logCHStart < logCHEnd; ++logCHStart, ++inBuf, ++outBuf) {
      *outBuf = (Tout)(uint32)ScaleDataUsePolynomial(
                                 logCHVrg[logCHStart]->Pieces, 
                                 logCHVrg[logCHStart]->PieceCount, 
                                 *inBuf);
   }
   return logCHStart;
}

// Special:
// Data scale methods. Floating ---> Binary with data truncation.
template<class Tin, class Tout>
inline uint32 ScaleDataToBinaryWithMask(
   IN PSCALE_TABLE logCHVrg[], 
   IN uint32       logCHStart,
   IN uint32       logCHEnd,
   IN uint32       dataMask,
   IN Tin          *inBuf,
   OUT Tout        *outBuf)
{
   for (; logCHStart < logCHEnd; ++logCHStart, ++inBuf, ++outBuf) {
      *outBuf = (Tout)ScaleDataUsePolynomialWithMask(
                          logCHVrg[logCHStart]->Pieces, 
                          logCHVrg[logCHStart]->PieceCount, 
                          dataMask,
                          *inBuf);
   }
   return logCHStart;
}

template<class Tin, class Tout>
inline uint32 ScaleDataToFloating(
   IN PSCALE_TABLE logCHVrg[], 
   IN uint32       logCHStart,
   IN uint32       logCHEnd,
   IN Tin          *inBuf,
   OUT Tout        *outBuf)
{
   for (; logCHStart < logCHEnd; ++logCHStart, ++inBuf, ++outBuf) {
      *outBuf = (Tout)ScaleDataUsePolynomial(
                        logCHVrg[logCHStart]->Pieces,  
                        logCHVrg[logCHStart]->PieceCount, 
                        *inBuf);
   }
   return logCHStart;
}

template< class Tin, class Tout >
inline uint32 ScaleDataToFloatingWithMask (
   IN PSCALE_TABLE logCHVrg[], 
   IN uint32       logCHStart,
   IN uint32       logCHEnd,
   IN Tin          dataMask,
   IN Tin          *inBuf,
   OUT Tout        *outBuf)
{
   for (; logCHStart < logCHEnd; ++logCHStart, ++inBuf, ++outBuf) {
      *outBuf = (Tout)ScaleDataUsePolynomial(
                        logCHVrg[logCHStart]->Pieces, 
                        logCHVrg[logCHStart]->PieceCount,
                        *inBuf & dataMask);
   }
   return logCHStart;
}

//--------------------------------------------------------------------------
// VersionStringToInteger
//   helper functions to Convert a "xxx.yyy.zzz.www" version string to integer.
//
// Return: verion in integer
inline uint32 VersionStringToInteger(char const * ver_str)
{
   uint32 ver = 0;
   uint32   x = 0;

   do {
      for (x = 0; *ver_str; ++ver_str) {
         if (*ver_str == '.') {  
            ++ver_str;
            break;
         }

         x = x * 10 + (*ver_str - '0');
      }

      ver = (ver << 8) | (x & 0xff);
   } while (*ver_str);

   return ver;
}

#endif // DAQNAVI_USER_MODE_LIBRARY
