/*******************************************************************************
              Copyright (c) 1983-2009 Advantech Co., Ltd.
********************************************************************************
THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY INFORMATION
WHICH IS THE PROPERTY OF ADVANTECH CORP., ANY DISCLOSURE, USE, OR REPRODUCTION,
WITHOUT WRITTEN AUTHORIZATION FROM ADVANTECH CORP., IS STRICTLY PROHIBITED. 

================================================================================
REVISION HISTORY
--------------------------------------------------------------------------------
$Log: $
--------------------------------------------------------------------------------
$NoKeywords:  $
*/

#ifndef _DAQ_DEVICE_IOCTL_CODE_HELPER
#define _DAQ_DEVICE_IOCTL_CODE_HELPER

#if defined(_WIN32) || defined(WIN32) 
#  include <windows.h>
#  include <winioctl.h>
#  include <win32/ioctls.h>
#else
#  include <sys/ioctl.h>
#  include <linux/ioctls.h>
#endif

#include <assert.h>

//
// kernel driver interface abstract layer
//
#ifdef _MSC_VER
#  pragma warning(disable:4100)
#  pragma warning(disable:6011)
#endif

//
// Io control code
// ==================================================================================
// device function 
// ==================================================================================
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/
#if defined(_WIN32) || defined(WIN32) // Windows
//

inline ErrorCode IoctlDevGetHWInfo(DEVFD devFd, DEVICE_HWINFO_EX *info)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_GET_HWINFO, NULL, 0, info, sizeof(*info), NULL);
}

inline ErrorCode IoctlDevRegisterEvent(DEVFD devFd, EVENT_INFO *info)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_REGISTER_EVENT, info, sizeof(*info), NULL, 0, NULL);
}

inline ErrorCode IoctlDevUnregisterEvent(DEVFD devFd, EVENT_INFO *info)
{
   uint32 size = info ? sizeof(*info) : 0;
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_UNREGISTER_EVENT, info, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDevTriggerEvent(DEVFD devFd, uint32 eventId)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_TRIGGER_EVENT, &eventId, sizeof(uint32), NULL, 0, NULL);
}

// For compatible
#define IoctlDevNotifyPropChged(devFd) IoctlDevTriggerEvent(devFd, EvtPropertyChanged)

inline ErrorCode IoctlDevGetLocation(DEVFD devFd, wchar_t loc[], uint32 len)
{
   len *= sizeof(wchar_t);
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_GET_LOCATION, NULL, 0, loc, len, NULL);
}

inline ErrorCode IoctlDevGetBoardVer(DEVFD devFd, wchar_t ver[], uint32 len)
{
   len *= sizeof(wchar_t);
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_GET_BOARD_VER, NULL, 0, ver, len, NULL);
}

inline ErrorCode IoctlDevLocateDevice(DEVFD devFd, uint32 enabled)
{
   return DeviceIoctlWrapper(devFd,  IOCTL_DEV_LOCATE_DEVICE, &enabled, sizeof(uint32), NULL, 0, NULL);
}

inline ErrorCode IoctlDevSetBoardID(DEVFD devFd, uint32 id)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_SET_BOARDID, &id, sizeof(id), NULL, 0, NULL);
}

inline ErrorCode IoctlDevGetFreedBoardID(DEVFD devFd, uint32 id[], uint32 len)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_GET_FREED_BOARDID, NULL, 0, id, len * sizeof(uint32), NULL);
}

inline ErrorCode IoctlDevSetWdtMode(DEVFD devFd, uint32 mode)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_SET_WDT_MODE, &mode, sizeof(mode), NULL, 0, NULL);
}

inline ErrorCode IoctlDevDownloadFirmware(DEVFD devFd, uint8 *data, uint32 size, uint32 *result)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_DOWNLOAD_FW, data, size, result, sizeof(uint32), NULL);
}

// New : raw USB packet
inline ErrorCode IoctlDevDownloadFirmware(DEVFD devFd, void *xbuf, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_DOWNLOAD_FW, xbuf, size, xbuf, size, NULL);
}

inline ErrorCode IoctlDevReadPrivateRegion(DEVFD devFd, uint32 signature, uint8 *buffer, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_READ_PRV_RGN, &signature, sizeof(uint32), buffer, size, NULL);
}

inline ErrorCode IoctlDevWritePrivateRegion(DEVFD devFd, uint8 *data, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_WRITE_PRV_RGN, data, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDevDbgReadRegister(DEVFD devFd, DBG_REG_IO *arg)
{
   assert(arg && arg->Length <= sizeof(uint32));
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_DBG_REG_IN, arg, sizeof(*arg), &arg->Value, arg->Length, NULL);
}

inline ErrorCode IoctlDevDbgWriteRegister(DEVFD devFd, DBG_REG_IO *arg)
{
   assert(arg && arg->Length <= sizeof(uint32));
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_DBG_REG_OUT, arg, sizeof(*arg), NULL, 0, NULL);
}

inline ErrorCode IoctlDevDbgUsbIn(DEVFD devFd, DBG_USB_IO *arg)
{
   assert(arg);
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_DBG_USB_IN, arg, sizeof(*arg), arg->Data, arg->Length, NULL);
}

inline ErrorCode IoctlDevDbgUsbOut(DEVFD devFd, DBG_USB_IO *arg)
{
   uint32 size = SIZEOF_VLST(DBG_USB_IO, Data, arg->Length);
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_DBG_USB_OUT, arg, size, &arg->Result, sizeof(uint32), NULL);
}

// New : raw USB packet
inline ErrorCode IoctlDevDbgUsbIn(DEVFD devFd, void *xbuf, uint32 size)
{
   assert(xbuf);
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_DBG_USB_IN, xbuf, size, xbuf, size, NULL);
}

// New : raw USB packet
inline ErrorCode IoctlDevDbgUsbOut(DEVFD devFd, void *xbuf, uint32 size)
{
   assert(xbuf);
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_DBG_USB_OUT, xbuf, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDevGetKShared(DEVFD devFd, void **kshared)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_GET_KSHARED, NULL, 0, kshared, sizeof(void*), NULL);
}

inline ErrorCode IoctlDevGetMemMapped(DEVFD devFd, void *mems, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_GET_MEM_MAPPED, NULL, 0, mems, size, NULL);
}

inline ErrorCode IoctlDevGetDriverVer(DEVFD devFd, char *ver, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_GET_DRV_VER, NULL, 0, ver, size, NULL);
}

inline ErrorCode IoctlDevMakeShared(DEVFD devFd, uint32 shared)
{
   return ErrorFuncNotSpted;
}

inline ErrorCode IoctlDevSetWdtEnable(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_WDT_CMD, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDevGetWdtEnable(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 out_size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_WDT_CMD, arg, sizeof(*arg), arg, out_size, NULL);
}

inline ErrorCode IoctlDevClearWdtFlag(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_WDT_CMD, arg, sizeof(*arg), NULL, 0, NULL);
}

inline ErrorCode IoctlDevSetWdtInfo(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_WDT_CMD, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDevGetWdtInfo(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 out_size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_WDT_CMD, arg, sizeof(*arg), arg, out_size, NULL);
}

inline ErrorCode IoctlDevSetPfpDir(DEVFD devFd, void *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_SET_PFP_DIR, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDevSetRtEndpoint(DEVFD devFd, RT_EP_CFG* arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DEV_SET_REP_CFG, arg, size, NULL, 0, NULL);
}

//
#else // Linux, etc------------------------------------------------------------------
//

inline ErrorCode IoctlDevGetHWInfo(DEVFD devFd, DEVICE_HWINFO_EX *info)
{
   return ioctl(devFd, IOCTL_DEV_GET_HWINFO, info) ? ErrorFuncNotSpted : Success;
}

inline ErrorCode IoctlDevRegisterEvent(DEVFD devFd, EVENT_INFO *info)
{
   return ioctl(devFd, IOCTL_DEV_REGISTER_EVENT, info) ? ErrorMemoryNotEnough : Success;
}

inline ErrorCode IoctlDevUnregisterEvent(DEVFD devFd, EVENT_INFO *info)
{
   return ioctl(devFd, IOCTL_DEV_UNREGISTER_EVENT, info) ? ErrorParamNotSpted : Success;
}

inline ErrorCode IoctlDevTriggerEvent(DEVFD devFd, uint32 eventId)
{
   return ioctl(devFd, IOCTL_DEV_TRIGGER_EVENT, eventId) ? ErrorParamNotSpted : Success;
}

// For compatible
#define IoctlDevNotifyPropChged(devFd) IoctlDevTriggerEvent(devFd, EvtPropertyChanged)

inline ErrorCode IoctlDevGetLocation(DEVFD devFd, wchar_t loc[], uint32 len)
{
   assert(len >= 256);
   char xbuf[256];
   xbuf[0] = 0;
   if (ioctl(devFd, IOCTL_DEV_GET_LOCATION, xbuf)) {
      return ErrorUndefined;
   }

   mbstowcs(loc, xbuf, strlen(xbuf) + 1);
   return Success;
}

inline ErrorCode IoctlDevGetBoardVer(DEVFD devFd, wchar_t ver[], uint32 len)
{
   return ErrorFuncNotSpted;
}

inline ErrorCode IoctlDevLocateDevice(DEVFD devFd, uint32 enabled)
{
   return ioctl(devFd, IOCTL_DEV_LOCATE_DEVICE, (void*)(long)enabled) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevSetBoardID(DEVFD devFd, uint32 id)
{
   return ioctl(devFd, IOCTL_DEV_SET_BOARDID, (void*)(long)id) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevGetFreedBoardID(DEVFD devFd, uint32 id[], uint32 len)
{
   assert(len >= 16);
   return ioctl(devFd, IOCTL_DEV_GET_FREED_BOARDID, id) ? ErrorUndefined : Success;
}

inline ErrorCode IoctlDevSetWdtMode(DEVFD devFd, uint32 mode)
{
   return ioctl(devFd, IOCTL_DEV_SET_WDT_MODE, (void*)(long)mode) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevDownloadFirmware(DEVFD devFd, uint8 *data, uint32 size, uint32 *result)
{
   FW_DOWNLOAD xbuf = {size, {data}, {result}};
   return ioctl(devFd, IOCTL_DEV_DOWNLOAD_FW, &xbuf) ? ErrorDeviceIoTimeOut : Success;
}

// New : raw USB packet
inline ErrorCode IoctlDevDownloadFirmware(DEVFD devFd, void *xbuf, uint32 size)
{
   return ioctl(devFd, IOCTL_DEV_DOWNLOAD_FW, xbuf) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevReadPrivateRegion(DEVFD devFd, uint32 signature, uint8 *buffer, uint32 size)
{
   return ErrorFuncNotSpted;
}

inline ErrorCode IoctlDevWritePrivateRegion(DEVFD devFd, uint8 *data, uint32 size)
{
   return ErrorFuncNotSpted;
}

inline ErrorCode IoctlDevDbgReadRegister(DEVFD devFd, DBG_REG_IO *arg)
{
   assert (arg && arg->Length <= sizeof(uint32));
   return ioctl(devFd, IOCTL_DEV_DBG_REG_IN, arg) ? ErrorParamOutOfRange : Success;
}

inline ErrorCode IoctlDevDbgWriteRegister(DEVFD devFd, DBG_REG_IO *arg)
{
   assert(arg && arg->Length <= sizeof(uint32));
   return ioctl(devFd, IOCTL_DEV_DBG_REG_OUT, arg) ? ErrorParamOutOfRange : Success;
}

inline ErrorCode IoctlDevDbgUsbIn(DEVFD devFd, DBG_USB_IO *arg)
{
   assert(arg);
   return ioctl(devFd, IOCTL_DEV_DBG_USB_IN, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevDbgUsbOut(DEVFD devFd, DBG_USB_IO *arg)
{
   return ioctl(devFd, IOCTL_DEV_DBG_USB_OUT, arg) ? ErrorDeviceIoTimeOut : Success;
}

// New : raw USB packet
inline ErrorCode IoctlDevDbgUsbIn(DEVFD devFd, void *xbuf, uint32 size)
{
   assert(xbuf);
   return ioctl(devFd, IOCTL_DEV_DBG_USB_IN, xbuf) ? ErrorDeviceIoTimeOut : Success;
}

// New : raw USB packet
inline ErrorCode IoctlDevDbgUsbOut(DEVFD devFd, void *xbuf, uint32 size)
{
   assert(xbuf);
   return ioctl(devFd, IOCTL_DEV_DBG_USB_OUT, xbuf) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevGetKShared(DEVFD devFd, void **kshared)
{
   return ErrorFuncNotSpted;
}

inline ErrorCode IoctlDevGetMemMapped(DEVFD devFd, void *mems, uint32 size)
{
   return ErrorFuncNotSpted;
}

inline ErrorCode IoctlDevGetDriverVer(DEVFD devFd, char *ver, uint32 size)
{
   return ioctl(devFd, IOCTL_DEV_GET_DRV_VER, ver) ? ErrorBufferTooSmall : Success;
}

inline ErrorCode IoctlDevMakeShared(DEVFD devFd, uint32 shared)
{
   return ioctl(devFd, IOCTL_DEV_MAKE_SHARED, shared) ? ErrorPrivilegeNotAvailable : Success;
}

inline ErrorCode IoctlDevSetWdtEnable(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DEV_WDT_CMD, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevGetWdtEnable(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 out_size)
{
   return ioctl(devFd, IOCTL_DEV_WDT_CMD, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevClearWdtFlag(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DEV_WDT_CMD, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevSetWdtInfo(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DEV_WDT_CMD, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevGetWdtInfo(DEVFD devFd, WDT_EXEC_CMD* arg, uint32 out_size)
{
   return ioctl(devFd, IOCTL_DEV_WDT_CMD, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevSetPfpDir(DEVFD devFd, void *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DEV_SET_PFP_DIR, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDevSetRtEndpoint(DEVFD devFd, RT_EP_CFG* arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DEV_SET_REP_CFG, arg) ? ErrorDeviceIoTimeOut : Success;
}

//
#endif
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/

//========================================================================================
// AI IO control code
//========================================================================================
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/
#if defined(_WIN32) || defined(WIN32) // Windows
//

inline ErrorCode IoctlAiSetChan(DEVFD devFd, AI_SET_CH *arg)
{
   assert(arg);
   uint32 size = SIZEOF_VLST(AI_SET_CH, CHCfg, arg->PhyCHCount);
   return DeviceIoctlWrapper(devFd, IOCTL_AI_SET_CH, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlAiReadChan(DEVFD devFd, AI_READ_CH *arg, uint32 out_buf_size)
{
   assert(arg);
   return DeviceIoctlWrapper(devFd, IOCTL_AI_READ_CH, arg, sizeof(*arg), arg->Data, out_buf_size, NULL);
}

inline ErrorCode IoctlAiSetCjc(DEVFD devFd, uint32 cjc)
{
   return DeviceIoctlWrapper(devFd, IOCTL_AI_SET_CJC, &cjc, sizeof(uint32), NULL, 0, NULL);
}

inline ErrorCode IoctlAiReadCjc(DEVFD devFd, AI_CJC_VALUE *buffer, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_AI_READ_CJC, NULL, 0, buffer, size, NULL);
}

inline ErrorCode IoctlAiBurnDetect(DEVFD devFd, AI_BURN_DETECT *arg, uint32 out_buf_size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_AI_BURN_DETECT, arg, sizeof(*arg), arg->Status, out_buf_size, NULL);
}

inline ErrorCode IoctlAiUsbCali(DEVFD devFd, AI_USB_CALI *arg)
{
   return DeviceIoctlWrapper(devFd, IOCTL_AI_USB_CALI, arg, sizeof(*arg), arg->Result, sizeof(uint32), NULL);
}

inline ErrorCode IoctlFaiSetParam(DEVFD devFd, void *param, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FAI_SET_PARAM, param, size, NULL, 0, NULL);
}

inline ErrorCode IoctlFaiSetBuffer(DEVFD devFd, void *buffer, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FAI_SET_BUFFER,  NULL, 0, buffer, size, NULL);
}

inline ErrorCode IoctlFaiMapBuffer(DEVFD devFd, void **buffer, uint32 /*size*/, uint32 /*offset*/)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FAI_MAP_BUFFER,  NULL, 0, buffer, sizeof(*buffer), NULL);
}

inline ErrorCode IoctlFaiStart(DEVFD devFd, uint32 acqMode)
{
   FAI_EXEC_ARG arg = {FAI_CMD_START, acqMode};
   return DeviceIoctlWrapper(devFd, IOCTL_FAI_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlFaiStop(DEVFD devFd, uint32 stopFlag)
{
   FAI_EXEC_ARG arg = {FAI_CMD_STOP, stopFlag};
   return DeviceIoctlWrapper(devFd, IOCTL_FAI_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlFaiResumeDma(DEVFD devFd, uint32 resumeInfo)
{
   FAI_EXEC_ARG arg = {FAI_CMD_RESUMEDMA, resumeInfo};
   return DeviceIoctlWrapper(devFd, IOCTL_FAI_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

//
#else // Linux, etc------------------------------------------------------------------
//

inline ErrorCode IoctlAiSetChan(DEVFD devFd, AI_SET_CH *arg)
{
   return ioctl(devFd, IOCTL_AI_SET_CH, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlAiReadChan(DEVFD devFd, AI_READ_CH *arg, uint32 out_buf_size)
{
   return ioctl(devFd, IOCTL_AI_READ_CH, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlAiSetCjc(DEVFD devFd, uint32 cjc)
{
   return ioctl(devFd, IOCTL_AI_SET_CJC, (void*)(long)cjc) ? ErrorParamNotSpted : Success;
}

inline ErrorCode IoctlAiReadCjc(DEVFD devFd, AI_CJC_VALUE *buffer, uint32 size)
{
   return ioctl(devFd, IOCTL_AI_READ_CJC, buffer) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlAiBurnDetect(DEVFD devFd, AI_BURN_DETECT *arg, uint32 out_buf_size)
{
   return ioctl(devFd, IOCTL_AI_BURN_DETECT, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlAiUsbCali(DEVFD devFd, AI_USB_CALI *arg)
{
   return ioctl(devFd, IOCTL_AI_USB_CALI, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFaiSetParam(DEVFD devFd, void *param, uint32 size)
{
   return ioctl(devFd, IOCTL_FAI_SET_PARAM, param) ? ErrorFuncBusy : Success;
}

inline ErrorCode IoctlFaiSetBuffer(DEVFD devFd, void *buffer, uint32 size)
{
   return ioctl(devFd, IOCTL_FAI_SET_BUFFER, buffer) ? ErrorMemoryNotEnough : Success;
}

inline ErrorCode IoctlFaiMapBuffer(DEVFD devFd, void **buffer, uint32 size, uint32 offset)
{
   *buffer = mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_SHARED, devFd, offset);
   return *buffer && *buffer != MAP_FAILED ? Success : ErrorMemoryNotEnough;
}

inline ErrorCode IoctlFaiStart(DEVFD devFd, uint32 acqMode)
{
   FAI_EXEC_ARG arg = {FAI_CMD_START, {acqMode}};
   return ioctl(devFd, IOCTL_FAI_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFaiStop(DEVFD devFd, uint32 stopFlag)
{
   FAI_EXEC_ARG arg = {FAI_CMD_STOP, {stopFlag}};
   return ioctl(devFd, IOCTL_FAI_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFaiResumeDma(DEVFD devFd, uint32 resumeInfo)
{
   FAI_EXEC_ARG arg = {FAI_CMD_RESUMEDMA, {resumeInfo}};
   return ioctl(devFd, IOCTL_FAI_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

//
#endif
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/

//========================================================================================
// AO IO control code
//========================================================================================
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/
#if defined(_WIN32) || defined(WIN32) // Windows
//

inline ErrorCode IoctlAoSetChan(DEVFD devFd, AO_SET_CH *arg)
{
   assert(arg);
   uint32 size = SIZEOF_VLST(AO_SET_CH, Gain, arg->CHCount);
   return DeviceIoctlWrapper(devFd, IOCTL_AO_SET_CH, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlAoWriteChan(DEVFD devFd, AO_WRITE_CH *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_AO_WRITE_CH, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlFaoSetParam(DEVFD devFd, void *param, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FAO_SET_PARAM, param, size, NULL, 0, NULL);
}

inline ErrorCode IoctlFaoSetBuffer(DEVFD devFd, void *buffer, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FAO_SET_BUFFER, NULL, 0, buffer, size, NULL);
}

inline ErrorCode IoctlFaoStart(DEVFD devFd, uint32 acqMode)
{
   FAO_EXEC_ARG arg = {FAO_CMD_START, acqMode};
   return DeviceIoctlWrapper(devFd, IOCTL_FAO_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlFaoStop(DEVFD devFd, uint32 stopFlag)
{
   FAO_EXEC_ARG arg = {FAO_CMD_STOP, stopFlag};
   return DeviceIoctlWrapper(devFd, IOCTL_FAO_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

//
#else // Linux, etc------------------------------------------------------------------
//

inline ErrorCode IoctlAoSetChan(DEVFD devFd, AO_SET_CH *arg)
{
   return ioctl(devFd, IOCTL_AO_SET_CH, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlAoWriteChan(DEVFD devFd, AO_WRITE_CH *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_AO_WRITE_CH, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFaoSetParam(DEVFD devFd, void *param, uint32 size)
{
   return ioctl(devFd, IOCTL_FAO_SET_PARAM, param) ? ErrorFuncBusy : Success;
}

inline ErrorCode IoctlFaoSetBuffer(DEVFD devFd, void *buffer, uint32 size)
{
   return ioctl(devFd, IOCTL_FAO_SET_BUFFER, buffer) ? ErrorMemoryNotEnough : Success;
}

inline ErrorCode IoctlFaoStart(DEVFD devFd, uint32 acqMode)
{
   FAO_EXEC_ARG arg = {FAO_CMD_START, {acqMode}};
   return ioctl(devFd, IOCTL_FAO_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFaoStop(DEVFD devFd, uint32 stopFlag)
{
   FAO_EXEC_ARG arg = {FAO_CMD_STOP, {stopFlag}};
   return ioctl(devFd, IOCTL_FAO_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

//
#endif
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/

//========================================================================================
// DIO IO control code
//========================================================================================
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/
#if defined(_WIN32) || defined(WIN32) // Windows
//

inline ErrorCode IoctlDioSetPort(DEVFD devFd, DIO_SET_PORT *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_SET_PORT, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDioReadDiPort(DEVFD devFd, DIO_READ_PORT *arg, uint32 out_buf_size)
{
   assert(arg && arg->PortCount <= out_buf_size);
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_READ_DI_PORT, arg, sizeof(*arg), arg->Data, out_buf_size, NULL);
}

inline ErrorCode IoctlDioWriteDoPort(DEVFD devFd, DIO_WRITE_PORT *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_WRITE_DO_PORT, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDioReadDoPort(DEVFD devFd, DIO_READ_PORT *arg, uint32 out_buf_size)
{
   assert(arg && arg->PortCount <= out_buf_size);
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_READ_DO_PORT, arg, sizeof(*arg), arg->Data, out_buf_size, NULL);
}

inline ErrorCode IoctlDioWriteDoBit(DEVFD devFd, DIO_WRITE_BIT *arg)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_WRITE_DO_BIT, arg, sizeof(*arg), NULL, 0, NULL);
}

inline ErrorCode IoctlDioSetInt(DEVFD devFd, DIO_SET_INT *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_SET_INT, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDioStartDiSnap(DEVFD devFd, uint32 eventId, uint32 portStart, uint32 portCount)
{
   DIO_EXEC_DISNAP arg = {DISNAP_CMD_START, eventId, portStart, portCount};
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_EXEC_DISNAP, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlDioStopDiSnap(DEVFD devFd, uint32 eventId)
{
   DIO_EXEC_DISNAP arg = {DISNAP_CMD_STOP, eventId};
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_EXEC_DISNAP, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlDioEnableDoFreeze(DEVFD devFd, uint32 enable)
{
   DIO_EXEC_DOFRZ arg = { DOFRZ_CMD_ENABLE };
   if (!enable) { arg.Command = DOFRZ_CMD_DISABLE; }

   return DeviceIoctlWrapper(devFd, IOCTL_DIO_EXEC_DOFRZ, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlDioPollDoFreezeState(DEVFD devFd, uint32 *state)
{
   DIO_EXEC_DOFRZ arg = { DOFRZ_CMD_POLLSTATE };
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_EXEC_DOFRZ, &arg, sizeof(arg), state, sizeof(uint32), NULL);
}

inline ErrorCode IoctlDioSetPowerOnCfg(DEVFD devFd, DIO_SET_POCFG *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_SET_POCFG, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlDioExecDoWtd(DEVFD devFd, DIO_EXEC_DOWTD *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_DIO_EXEC_DOWTD, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlFdiSetParam(DEVFD devFd, void *param, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FDI_SET_PARAM, param, size, NULL, 0, NULL);
}

inline ErrorCode IoctlFdiSetBuffer(DEVFD devFd, void *buffer, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FDI_SET_BUFFER,  NULL, 0, buffer, size, NULL);
}

inline ErrorCode IoctlFdiMapBuffer(DEVFD devFd, void **buffer, uint32 /*size*/, uint32 /*offset*/)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FDI_MAP_BUFFER,  NULL, 0, buffer, sizeof(*buffer), NULL);
}

inline ErrorCode IoctlFdiStart(DEVFD devFd, uint32 acqMode)
{
   FDI_EXEC_ARG arg = {FDI_CMD_START, acqMode};
   return DeviceIoctlWrapper(devFd, IOCTL_FDI_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlFdiStop(DEVFD devFd, uint32 stopFlag)
{
   FDI_EXEC_ARG arg = {FDI_CMD_STOP, stopFlag};
   return DeviceIoctlWrapper(devFd, IOCTL_FDI_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlFdoSetParam(DEVFD devFd, void *param, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FDO_SET_PARAM, param, size, NULL, 0, NULL);
}

inline ErrorCode IoctlFdoSetBuffer(DEVFD devFd, void *buffer, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FDO_SET_BUFFER, NULL, 0, buffer, size, NULL);
}

inline ErrorCode IoctlFdoStart(DEVFD devFd, uint32 acqMode)
{
   FDO_EXEC_ARG arg = {FDO_CMD_START, acqMode};
   return DeviceIoctlWrapper(devFd, IOCTL_FDO_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlFdoStop(DEVFD devFd, uint32 stopFlag)
{
   FDO_EXEC_ARG arg = {FDO_CMD_STOP, stopFlag};
   return DeviceIoctlWrapper(devFd, IOCTL_FDO_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

//
#else // Linux, etc------------------------------------------------------------------
//

inline ErrorCode IoctlDioSetPort(DEVFD devFd, DIO_SET_PORT *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DIO_SET_PORT, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioReadDiPort(DEVFD devFd, DIO_READ_PORT *arg, uint32 out_buf_size)
{
   assert(arg && arg->PortCount <= out_buf_size);
   return ioctl(devFd, IOCTL_DIO_READ_DI_PORT, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioWriteDoPort(DEVFD devFd, DIO_WRITE_PORT *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DIO_WRITE_DO_PORT, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioReadDoPort(DEVFD devFd, DIO_READ_PORT *arg, uint32 out_buf_size)
{
   assert(arg && arg->PortCount <= out_buf_size);
   return ioctl(devFd, IOCTL_DIO_READ_DO_PORT, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioWriteDoBit(DEVFD devFd, DIO_WRITE_BIT *arg)
{
   return ioctl(devFd, IOCTL_DIO_WRITE_DO_BIT, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioSetInt(DEVFD devFd, DIO_SET_INT *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DIO_SET_INT, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioStartDiSnap(DEVFD devFd, uint32 eventId, uint32 portStart, uint32 portCount)
{
   DIO_EXEC_DISNAP arg = {DISNAP_CMD_START, eventId, portStart, portCount};
   return ioctl(devFd, IOCTL_DIO_EXEC_DISNAP, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioStopDiSnap(DEVFD devFd, uint32 eventId)
{
   DIO_EXEC_DISNAP arg = {DISNAP_CMD_STOP, eventId};
   return ioctl(devFd, IOCTL_DIO_EXEC_DISNAP, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioEnableDoFreeze(DEVFD devFd, uint32 enable)
{
   DIO_EXEC_DOFRZ arg = { DOFRZ_CMD_ENABLE };
   if (!enable) { arg.Command = DOFRZ_CMD_DISABLE; }

   return ioctl(devFd, IOCTL_DIO_EXEC_DOFRZ, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioPollDoFreezeState(DEVFD devFd, uint32 *state)
{
   DIO_EXEC_DOFRZ arg = { DOFRZ_CMD_POLLSTATE };
   return ioctl(devFd, IOCTL_DIO_EXEC_DOFRZ, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioSetPowerOnCfg(DEVFD devFd, DIO_SET_POCFG *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DIO_SET_POCFG, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlDioExecDoWtd(DEVFD devFd, DIO_EXEC_DOWTD *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_DIO_EXEC_DOWTD, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFdiSetParam(DEVFD devFd, void *param, uint32 size)
{
   return ioctl(devFd, IOCTL_FDI_SET_PARAM, param) ? ErrorFuncBusy : Success;
}

inline ErrorCode IoctlFdiSetBuffer(DEVFD devFd, void *buffer, uint32 size)
{
   return ioctl(devFd, IOCTL_FDI_SET_BUFFER, buffer) ? ErrorMemoryNotEnough : Success;
}

inline ErrorCode IoctlFdiMapBuffer(DEVFD devFd, void **buffer, uint32 size, uint32 offset)
{
   *buffer = mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_SHARED, devFd, offset);
   return *buffer && *buffer != MAP_FAILED ? Success : ErrorMemoryNotEnough;
}

inline ErrorCode IoctlFdiStart(DEVFD devFd, uint32 acqMode)
{
   FDI_EXEC_ARG arg = {FDI_CMD_START, {acqMode}};
   return ioctl(devFd, IOCTL_FDI_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFdiStop(DEVFD devFd, uint32 stopFlag)
{
   FDI_EXEC_ARG arg = {FDI_CMD_STOP, {stopFlag}};
   return ioctl(devFd, IOCTL_FDI_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFdoSetParam(DEVFD devFd, void *param, uint32 size)
{
   return ioctl(devFd, IOCTL_FDO_SET_PARAM, param) ? ErrorFuncBusy : Success;
}

inline ErrorCode IoctlFdoSetBuffer(DEVFD devFd, void *buffer, uint32 size)
{
   return ioctl(devFd, IOCTL_FDO_SET_BUFFER, buffer) ? ErrorMemoryNotEnough : Success;
}

inline ErrorCode IoctlFdoStart(DEVFD devFd, uint32 acqMode)
{
   FDO_EXEC_ARG arg = {FDO_CMD_START, {acqMode}};
   return ioctl(devFd, IOCTL_FDO_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFdoStop(DEVFD devFd, uint32 stopFlag)
{
   FDO_EXEC_ARG arg = {FDO_CMD_STOP, {stopFlag}};
   return ioctl(devFd, IOCTL_FDO_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

//
#endif
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/

//========================================================================================
// Counter/Timer IO control code
//========================================================================================
//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/
#if defined(_WIN32) || defined(WIN32) // Windows
//

inline ErrorCode IoctlCntrConfig(DEVFD devFd, CT_SET_CFG *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_CT_SET_CFG, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlCntrStart(DEVFD devFd, CT_START *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_CT_START, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlCntrRead(DEVFD devFd, CT_READ *arg, uint32 out_buf_size)
{
   assert(arg);
   return DeviceIoctlWrapper(devFd, IOCTL_CT_READ, arg, sizeof(*arg), arg->Data, out_buf_size, NULL);
}

inline ErrorCode IoctlCntrReset(DEVFD devFd, CT_RESET *arg)
{
   return DeviceIoctlWrapper(devFd, IOCTL_CT_RESET, arg, sizeof(*arg), NULL, 0, NULL);
}

inline ErrorCode IoctlCntrStartSnap(DEVFD devFd, uint32 eventId, uint32 cntrStart, uint32 cntrCount)
{
   CT_EXEC_SNAP arg = {CTSNAP_CMD_START, eventId, cntrStart, cntrCount};
   return DeviceIoctlWrapper(devFd, IOCTL_CT_EXEC_SNAP, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlCntrStopSnap(DEVFD devFd, uint32 eventId)
{
   CT_EXEC_SNAP arg = {CTSNAP_CMD_STOP, eventId};
   return DeviceIoctlWrapper(devFd, IOCTL_CT_EXEC_SNAP, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlCntrCCmpConfig(DEVFD devFd, CT_CCMP_CFG *arg, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_CT_CCMP_CFG, arg, size, NULL, 0, NULL);
}

inline ErrorCode IoctlFctSetParam(DEVFD devFd, void *param, uint32 size)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FCT_SET_PARAM, param, size, NULL, 0, NULL);
}

inline ErrorCode IoctlFctSetBuffer(DEVFD devFd, void *buffer, uint32 size, uint32 cntr)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FCT_SET_BUFFER,  &cntr, sizeof(uint32), buffer, size, NULL);
}

inline ErrorCode IoctlFctMapBuffer(DEVFD devFd, void **buffer, uint32 cntr, uint32 /*size*/, uint32 /*offset*/)
{
   return DeviceIoctlWrapper(devFd, IOCTL_FCT_MAP_BUFFER,  &cntr, sizeof(uint32), buffer, sizeof(*buffer), NULL);
}

inline ErrorCode IoctlFctStart(DEVFD devFd, uint32 acqMode)
{
   FCT_EXEC_ARG arg = {FCT_CMD_START, acqMode};
   return DeviceIoctlWrapper(devFd, IOCTL_FCT_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

inline ErrorCode IoctlFctStop(DEVFD devFd, uint32 stopFlag)
{
   FCT_EXEC_ARG arg = {FCT_CMD_STOP, stopFlag};
   return DeviceIoctlWrapper(devFd, IOCTL_FCT_EXEC, &arg, sizeof(arg), NULL, 0, NULL);
}

//
#else // Linux, etc------------------------------------------------------------------
//

inline ErrorCode IoctlCntrConfig(DEVFD devFd, CT_SET_CFG *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_CT_SET_CFG, arg) ? ErrorParamNotSpted : Success;
}

inline ErrorCode IoctlCntrStart(DEVFD devFd, CT_START *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_CT_START, arg) ? ErrorFuncBusy : Success;
}

inline ErrorCode IoctlCntrRead(DEVFD devFd, CT_READ *arg, uint32 out_buf_size)
{
   assert(arg);
   return ioctl(devFd, IOCTL_CT_READ, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlCntrReset(DEVFD devFd, CT_RESET *arg)
{
   return ioctl(devFd, IOCTL_CT_RESET, arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlCntrStartSnap(DEVFD devFd, uint32 eventId, uint32 cntrStart, uint32 cntrCount)
{
   CT_EXEC_SNAP arg = {CTSNAP_CMD_START, eventId, cntrStart, cntrCount};
   return ioctl(devFd, IOCTL_CT_EXEC_SNAP, &arg) ? ErrorParamNotSpted : Success;
}

inline ErrorCode IoctlCntrStopSnap(DEVFD devFd, uint32 eventId)
{
   CT_EXEC_SNAP arg = {CTSNAP_CMD_STOP, eventId};
   return ioctl(devFd, IOCTL_CT_EXEC_SNAP, &arg) ? ErrorParamNotSpted : Success;
}

inline ErrorCode IoctlCntrCCmpConfig(DEVFD devFd, CT_CCMP_CFG *arg, uint32 size)
{
   return ioctl(devFd, IOCTL_CT_CCMP_CFG, arg) ? ErrorMemoryNotEnough : Success;
}

inline ErrorCode IoctlFctSetParam(DEVFD devFd, void *param, uint32 size)
{
   return ioctl(devFd, IOCTL_FCT_SET_PARAM, param) ? ErrorFuncBusy : Success;
}

inline ErrorCode IoctlFctSetBuffer(DEVFD devFd, void *buffer, uint32 size, uint32 cntr)
{
   FCT_BUFFER arg = {cntr, size, buffer};
   return ioctl(devFd, IOCTL_FCT_SET_BUFFER, &arg) ? ErrorMemoryNotEnough : Success;
}

inline ErrorCode IoctlFctMapBuffer(DEVFD devFd, void **buffer, uint32 ch, uint32 size, uint32 offset)
{
   *buffer = mmap(NULL, size, PROT_READ|PROT_WRITE, MAP_SHARED, devFd, offset);
   return *buffer && *buffer != MAP_FAILED ? Success : ErrorMemoryNotEnough;
}

inline ErrorCode IoctlFctStart(DEVFD devFd, uint32 acqMode)
{
   FCT_EXEC_ARG arg = {FCT_CMD_START, {acqMode}};
   return ioctl(devFd, IOCTL_FCT_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

inline ErrorCode IoctlFctStop(DEVFD devFd, uint32 stopFlag)
{
   FCT_EXEC_ARG arg = {FCT_CMD_STOP, {stopFlag}};
   return ioctl(devFd, IOCTL_FCT_EXEC, &arg) ? ErrorDeviceIoTimeOut : Success;
}

//
#endif

//\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\/
#ifdef _MSC_VER
#  pragma warning(default:4100)
#  pragma warning(default:6011)
#endif

#endif
