#ifndef DAQNAVI4_UM_KERN_STUB
#define DAQNAVI4_UM_KERN_STUB

template<class KSHARED>
class DaqKernStub
{
public:
   DaqKernStub() : m_devFd(INVALID_HANDLE_VALUE), m_kshared(NULL), m_cached_kshr(NULL)
   {
   }

   virtual ~DaqKernStub()
   {
      Cleanup();
   }

   ErrorCode Initialize(int32 devNumber, AccessMode mode)
   {
      assert(m_devFd == INVALID_HANDLE_VALUE && m_kshared == NULL);

      // Open the file handle to the kernel device
      ErrorCode ret = DeviceOpenKernDevice(devNumber, mode, sizeof(KSHARED), &m_devFd, (void**)&m_kshared);
      if (ret != Success) { return ret; }

      m_devNumber = devNumber;
      m_mode = mode;

      return ret;
   }

   void Cleanup()
   {
      DeviceCloseKernDevice(sizeof(KSHARED), m_devFd, (void*)m_kshared);

      m_devFd     = INVALID_HANDLE_VALUE;
      m_kshared   = NULL;
      m_mode      = ModeRead;
      m_devNumber = -1;

      if (m_cached_kshr) {
         delete m_cached_kshr;
         m_cached_kshr = NULL;
      }
   }

   // DN4 device surprise remove & reconnect
   ErrorCode OnDeviceStateChanged(int32 newState)
   {
      // The device is removed surprisingly!
      if (newState == DevState::Removed) {
         
         // backup the current device context
         assert(m_kshared);
         if (m_cached_kshr == NULL) {
            m_cached_kshr = new KSHARED;
         }
         memcpy(m_cached_kshr, m_kshared, sizeof(KSHARED));

         // Close the kernel handle so the kernel device can be unloaded
         DeviceCloseKernDevice(sizeof(KSHARED), m_devFd, (void*)m_kshared);
         m_devFd = INVALID_HANDLE_VALUE;

         // To avoid app crashed, set the m_kshared to our cached kshared data.
         m_kshared = m_cached_kshr;

         return Success;
      }

      // The device is reconnected, reopen the device.
      if (newState == DevState::Arrived) {
         KSHARED *kshr = NULL;
         ErrorCode ret = DeviceOpenKernDevice(m_devNumber, m_mode, sizeof(KSHARED), &m_devFd, (void**)&kshr);
         m_kshared = kshr;
         return ret;
      }

      return ErrorFuncNotSpted;
   }

   operator   DEVFD()        { return m_devFd;       }
   DEVFD      getDevFd()     { return m_devFd;       }
   int32      getDevNumber() { return m_devNumber;   }
   AccessMode getMode()      { return m_mode;        }
   KSHARED *  getShared()    { return m_kshared;     }
   KSHARED *  getCachedKShr(){ return m_cached_kshr; }

   void FreeCachedKShr()
   {
      if (m_cached_kshr) {
         delete m_cached_kshr;
         m_cached_kshr = NULL;
      }
   }
   
protected:
   int32       m_devNumber;
   AccessMode  m_mode;
   DEVFD       m_devFd;
   KSHARED    *m_kshared;
   KSHARED    *m_cached_kshr;
};

#endif
