//
// hack method : using timeBeginPeriod to get a high resolution timer.
// (I can't do this begin the API hadn't exported to kernel driver).
// the timer is created in kernel driver and being used to measure the freq
// Be careful:
// You must match each call to timeBeginPeriod with a call to timeEndPeriod,
// specifying the same minimum resolution in both calls.
// To simplify the coding, A helper class is supplied to do the work
// 
#ifndef _DAQNAVI_SYS_TIMER_HACKER_H
#define _DAQNAVI_SYS_TIMER_HACKER_H

#define MMTIMER_RES_MS      2 // MMTimer resolution  

template<uint32 tmr_period>
class SysTimerHacker
{
public:
   SysTimerHacker() : m_setCount(0) 
   {
   }

   ~SysTimerHacker() 
   {  
      while(m_setCount) { 
         End(); 
      }
   }

#if defined(_WIN32) || defined(WIN32)
   bool Begin()
   {
      // the hi-resolution timer had been started. 
      if (m_setCount > 0)  { return TRUE; }

      MMRESULT ret = timeBeginPeriod(tmr_period);
      if (ret == TIMERR_NOERROR) {
         AtomicIncrement(&m_setCount);
      } 
      return ret == TIMERR_NOERROR;
   }
   void End()
   {
      if (m_setCount > 0) {
         AtomicDecrement(&m_setCount);
         timeEndPeriod(tmr_period);
      }
   }
#else
   bool Begin() 
   { 
      return true; 
   }
   void End()
   { 
   }
#endif

protected:
   int32 m_setCount;
};

#endif