#ifndef _DAQNAVI_XBUFFER_H
#define _DAQNAVI_XBUFFER_H

class XBuffer
{
   enum {SIZE_MASK = 255};
public:
   XBuffer()
   { 
      m_size   = sizeof(M_DEF_BUF)/sizeof(M_DEF_BUF[0]);
      m_buffer = M_DEF_BUF;
   }

   virtual ~XBuffer()
   {
      if (m_buffer != M_DEF_BUF) {
         free(m_buffer);
      }
   }

   /* size = 0 : return current allocated buffer*/
   char * Buffer(unsigned size = 0) 
   {
      if (size <= m_size) { return m_buffer; }

      // allocate a new bigger buffer
      m_size = (size + SIZE_MASK) & ~SIZE_MASK;

      // free the old buffer
      if (m_buffer != M_DEF_BUF) { free(m_buffer); }

      // allocate a new one
      m_buffer = (char *)malloc(m_size);

      return m_buffer;
   }

   int Size() 
   {
      return m_size;
   }

   /*fill the buffer with the specified format & data*/
   int Format(char const *format, ...)
   {
      assert(format != NULL);

      va_list args;
      va_start(args, format);
      return Format(format, args);
   }

   int Format(char const *format, va_list args)
   {
      assert(format != NULL);

      int len = 0;
      do {
#        if defined(_WIN32) || defined(WIN32)
         len = _vsnprintf(Buffer(), Size() - 1, format, args);
#        else
         len = vsnprintf(Buffer(), Size() - 1, format, args);
#        endif

         // build string ok
         if (len != -1) { break; }

         // Need a bigger buffer than the current one
         if (Buffer(Size() + 256) == NULL) { break; }
      } while (true);

      return len;
   }

private:
   char       M_DEF_BUF[256];
   unsigned   m_size;
   char       *m_buffer;
};

#endif  // __cplusplus
