/*
* BDaqCtrl.cpp
*
*  Created on: 2011-8-11
*      Author: rocky
*/
// --------------------------------------------------
// DaqNavi4 CSCL Implementation Headers
// --------------------------------------------------
#include <std_decl.h>

#include <ai/ai_features.h>
#include <ai/ai_channel.h>
#include <ai/cjc_setting.h>
#include <ai/ai_ctrl_base.h>
#include <ai/instant_ai_ctrl.h>
#include <ai/buffered_ai_ctrl.h>

#include <ao/ao_features.h>
#include <ao/ao_channel.h>
#include <ao/ao_ctrl_base.h>
#include <ao/instant_ao_ctrl.h>
#include <ao/buffered_ao_ctrl.h>

#include <dio/dio_features.h>
#include <dio/dio_port.h>
#include <dio/scan_port.h>
#include <dio/dio_trigger.h>
#include <dio/dio_convert_clock.h>
#include <dio/dio_ctrl_base.h>
#include <dio/di_int_channel.h>
#include <dio/di_cosint_port.h>
#include <dio/di_pmint_port.h>
#include <dio/instant_di_ctrl.h>
#include <dio/instant_do_ctrl.h>

#include <cntr/cntr_channel.h>
#include <cntr/cntr_features.h>
#include <cntr/cntr_ctrl_base.h>
#include <cntr/event_counter_ctrl.h>
#include <cntr/freq_meter_ctrl.h>
#include <cntr/oneshot_ctrl.h>
#include <cntr/pw_meter_ctrl.h>
#include <cntr/pw_modulator_ctrl.h>
#include <cntr/timer_pulse_ctrl.h>
#include <cntr/ud_counter_ctrl.h>

#include <algorithm>

#include "daqnavi3_api.h"

// --------------------------------------------------
// DaqNavi3 CSCL Headers
// --------------------------------------------------
// Only Need DaqNavi3 Class Definition
#undef _BDAQ_TYPES_DEFINED
#define _BDAQ_TYPES_DEFINED

#undef _BDAQ_TYPES_ONLY
#undef _BDAQ_COM_STYLE_CLASS_LIB
#undef _BDAQ_NO_NAMESPACE
#undef BEGIN_NAMEAPCE_AUTOMATION_BDAQ
#undef END_NAMEAPCE_AUTOMATION_BDAQ  

#define BEGIN_NAMEAPCE_AUTOMATION_BDAQ namespace DN3CSCL {
#define END_NAMEAPCE_AUTOMATION_BDAQ  }
#define  _BIONIC_DAQ_DLL
#include "daqnavi3_cscl.h"

// *****************************************************************************************
//
// DaqNavi3 CSCL implementation. For Legacy APPs ONLY!
//
// *****************************************************************************************
namespace DN3CSCL {
   /********************************************************************************/
   /* DeviceCtrlBase                                                               */
   /********************************************************************************/
   template<class T, class TBase>
   class DeviceCtrlBaseImpl : public TBase
   {
   public:
      /*------------------------------------------------------------------------------*/
      /* method                                                                       */
      /*------------------------------------------------------------------------------*/
      virtual void BDAQCALL Dispose() { Cleanup(); delete (T*)this; }
      virtual void BDAQCALL Cleanup() { ctrl_impl().Cleanup(); }                                                
      virtual ErrorCode BDAQCALL UpdateProperties() 
      { 
         return  dev_impl() ? dev_impl()->Refresh() : ErrorFuncNotInited; 
      }                                  
      /*------------------------------------------------------------------------------*/
      /* event                                                                        */
      /*------------------------------------------------------------------------------*/
      virtual void BDAQCALL addRemovedListener(DeviceEventListener & handler)
      { 
         ctrl_impl().addEventHandler(EvtDeviceRemoved, (DaqEventProc)OnDeviceEvent, &handler);
      }
      virtual void BDAQCALL removeRemovedListener(DeviceEventListener & handler) 
      { 
         ctrl_impl().removeEventHandler(EvtDeviceRemoved, (DaqEventProc)OnDeviceEvent, &handler);
      }
      virtual void BDAQCALL addReconnectedListener(DeviceEventListener & handler)
      { 
         ctrl_impl().addEventHandler(EvtDeviceReconnected, (DaqEventProc)OnDeviceEvent, &handler);
      }
      virtual void BDAQCALL removeReconnectedListener(DeviceEventListener & handler) 
      { 
         ctrl_impl().removeEventHandler(EvtDeviceReconnected, (DaqEventProc)OnDeviceEvent, &handler);
      }
      virtual void BDAQCALL addPropertyChangedListener(DeviceEventListener & handler)
      { 
         ctrl_impl().addEventHandler(EvtPropertyChanged, (DaqEventProc)OnDeviceEvent, &handler);
      }
      virtual void BDAQCALL removePropertyChangedListener(DeviceEventListener & handler) 
      { 
         ctrl_impl().removeEventHandler(EvtPropertyChanged, (DaqEventProc)OnDeviceEvent, &handler);
      }

      /*------------------------------------------------------------------------------*/
      /* property                                                                     */
      /*------------------------------------------------------------------------------*/
      virtual void BDAQCALL getSelectedDevice(DeviceInformation &x) 
      { 
         ctrl_impl().getSelectedDevice(*(::DeviceInformation *)&x);
      }
      virtual ErrorCode BDAQCALL setSelectedDevice(DeviceInformation const &x) 
      { 
         return ctrl_impl().setSelectedDevice(*(::DeviceInformation const*)&x);
      }
      virtual bool BDAQCALL getInitialized()    { return ctrl_impl().getInitialized();    }
      virtual bool BDAQCALL getCanEditProperty(){ return ctrl_impl().CanEditProperty();   }
      virtual HANDLE BDAQCALL getDevice()       { return (HANDLE)dev_impl()->getDevice(); }
      virtual HANDLE BDAQCALL getModule()       { return (HANDLE)ctrl_impl().getModule(); } 
      virtual ICollection<DeviceTreeNode>* BDAQCALL getSupportedDevices() 
      { 
         return (ICollection<DeviceTreeNode>*)ctrl_impl().getSupportedDevices(); 
      }
      virtual ICollection<AccessMode> * BDAQCALL getSupportedModes()
      {
         return (ICollection<AccessMode> *)ctrl_impl().getSupportedModes();
      }

      /*------------------------------------------------------------------------------*/
      /* DeviceCtrlBaseExt                                                            */
      /*------------------------------------------------------------------------------*/
      /* method                                                                       */
      /*------------------------------------------------------------------------------*/
      virtual ErrorCode BDAQCALL ReadRegister(int32 startAddress, int32 length, void *data)
      {
         return  dev_impl() ? AdxDeviceReadPorts(getDevice(), startAddress, length, data) : ErrorFuncNotInited; 
      }
      virtual ErrorCode BDAQCALL WriteRegister(int32 startAddress, int32  length, void *data)
      {
         return  dev_impl() ? AdxDeviceWritePorts(getDevice(), startAddress, length, data) : ErrorFuncNotInited; 
      }
      virtual ErrorCode BDAQCALL ReadPrivateRegion(int32 signature, int32 length, uint8 *data)
      {
         return  dev_impl() ? dev_impl()->ReadPrivateRegion(signature, length, data) : ErrorFuncNotInited; 
      }
      virtual ErrorCode BDAQCALL WritePrivateRegion(int32 signature, int32 length, uint8 *data)
      {
         return  dev_impl() ? dev_impl()->WritePrivateRegion(signature, length, data) : ErrorFuncNotInited; 
      }
      /*------------------------------------------------------------------------------*/
      /* property                                                                     */
      /*------------------------------------------------------------------------------*/
      virtual ProductId BDAQCALL getProductId()
      {
         return  dev_impl() ? dev_impl()->getProductId() : ProductId(-1); 
      }
      virtual int32 BDAQCALL getBoardId()
      {
         return dev_impl() ? dev_impl()->getBoardId() : -1;
      }
      virtual void BDAQCALL getBoardVersion(int32 length, wchar_t *ver)
      { 
         if (dev_impl()) dev_impl()->getBoardVersion(length, ver); 
      }
      virtual void BDAQCALL getDriverVersion(int32 length, wchar_t *ver)
      { 
         if (dev_impl()) dev_impl()->getDriverVersion(length, ver); 
      }
      virtual void BDAQCALL getDllVersion(int32 length, wchar_t *ver)
      { 
         if (dev_impl()) dev_impl()->getDllVersion(length, ver); 
      }
      virtual void BDAQCALL getLocation(int32 length, wchar_t *loc)
      { 
         if (dev_impl()) dev_impl()->getLocation(length, loc); 
      }
      virtual int32 BDAQCALL getPrivateRegionLength()
      {
         return dev_impl() ? dev_impl()->getPrivateRegionLength() : 0;
      }
      virtual int32 BDAQCALL getHotResetPreventable()
      {
         return dev_impl() ? dev_impl()->getHotResetPreventable() : 0;
      }
      virtual ICollection<int32>* BDAQCALL getBaseAddresses()
      {
         return dev_impl() ? (ICollection<int32>*)dev_impl()->getBaseAddresses() : NULL;
      }
      virtual ICollection<int32>* BDAQCALL getInterrupts()
      {
         return dev_impl() ? (ICollection<int32>*)dev_impl()->getInterrupts() : NULL;
      }
      virtual ICollection<TerminalBoard>* BDAQCALL getSupportedTerminalBoard()
      {
         return dev_impl() ? (ICollection<TerminalBoard>*)dev_impl()->getSupportedTerminalBoard() : NULL;
      }
      virtual ICollection<EventId>* BDAQCALL getSupportedEvents()
      {
         return dev_impl() ? (ICollection<EventId>*)dev_impl()->getSupportedEvents() : NULL;
      }
      virtual TerminalBoard BDAQCALL getTerminalBoard()
      {
         return dev_impl() ? dev_impl()->getTerminalBoard() : TerminalBoard(0);
      }
      virtual ErrorCode BDAQCALL setTerminalBoard(TerminalBoard value)
      {
         return dev_impl() ? dev_impl()->setTerminalBoard(value) : ErrorFuncNotInited;
      }
      virtual int32 BDAQCALL getLoadingTimeInit()
      {
         return 0;
      }
      virtual ErrorCode BDAQCALL setLoadingTimeInit(int32 value)
      {
         return ErrorPropNotSpted;
      }

   private:
      static void BDAQCALL OnDeviceEvent(void * sender, ::DeviceEventArgs *args, void *userParam)
      {
         DeviceEventArgs dn3Args = { {(int32)args->Id} };
         DeviceEventListener *handler = static_cast<DeviceEventListener *>(userParam);
         handler->DeviceEvent(sender, &dn3Args);
      }

   private: // helper
      ::DaqCtrlBaseImpl& ctrl_impl() { return ((T *)this)->m_impl; }
      ::DeviceCtrlImpl*  dev_impl()  { return (::DeviceCtrlImpl*)ctrl_impl().getDevice(); }
   };

   /********************************************************************************/
   /* AI related classes                                                           */
   /********************************************************************************/
   class AiFeaturesImpl : public AiFeatures
   {
   public:
      // ADC features
      virtual int32 BDAQCALL getResolution() { return m_impl->getResolution(); }
      virtual int32 BDAQCALL getDataSize()   { return m_impl->getDataSize(); } 
      virtual int32 BDAQCALL getDataMask()   { return m_impl->getDataMask(); } 

      // channel features
      virtual int32                        BDAQCALL getChannelCountMax()   { return m_impl->getChannelCountMax(); } 
      virtual AiChannelType                BDAQCALL getChannelType()       { return m_impl->getChannelType(); } 
      virtual bool                         BDAQCALL getOverallValueRange() { return m_impl->getOverallValueRange(); } 
      virtual bool                         BDAQCALL getThermoSupported()   { return m_impl->getThermoSupported(); } 
      virtual ICollection<ValueRange>*     BDAQCALL getValueRanges()       { return (ICollection<ValueRange>*)m_impl->getValueRanges(); } 
      virtual ICollection<BurnoutRetType>* BDAQCALL getBurnoutReturnTypes(){ return (ICollection<BurnoutRetType>*)m_impl->getBurnoutReturnTypes(); } 

      // CJC features
      virtual ICollection<int32>*         BDAQCALL getCjcChannels() { return (ICollection<int32>*)m_impl->getCjcChannels(); } 

      // buffered ai->basic features
      virtual bool                        BDAQCALL getBufferedAiSupported() { return m_impl->getBufferedAiSupported(); } 
      virtual SamplingMethod              BDAQCALL getSamplingMethod()      { return m_impl->getSamplingMethod(); } 
      virtual int32                       BDAQCALL getChannelStartBase()    { return m_impl->getChannelStartBase(); } 
      virtual int32                       BDAQCALL getChannelCountBase()    { return m_impl->getChannelCountBase(); } 

      // buffered ai->conversion clock features
      virtual ICollection<SignalDrop>*    BDAQCALL getConvertClockSources() { return (ICollection<SignalDrop>*)m_impl->getConvertClockSources(); } 
      virtual MathInterval                BDAQCALL getConvertClockRange()   
      { 
         MathInterval x;
         m_impl->getConvertClockRange(x); 
         return x;
      } 

      // buffered ai->burst scan
      virtual bool                        BDAQCALL getBurstScanSupported() { return m_impl->getBurstScanSupported(); } 
      virtual ICollection<SignalDrop>*    BDAQCALL getScanClockSources()   { return (ICollection<SignalDrop>*)m_impl->getScanClockSources()  ; } 
      virtual int32                       BDAQCALL getScanCountMax()       { return m_impl->getScanCountMax()      ; } 
      virtual MathInterval                BDAQCALL getScanClockRange()     
      { 
         MathInterval x;
         m_impl->getScanClockRange(x); 
         return x;
      } 

      // buffered ai->trigger features
      virtual bool                        BDAQCALL getTriggerSupported()  { return m_impl->getTriggerCount() > 0; } 
      virtual int32                       BDAQCALL getTriggerCount()      { return m_impl->getTriggerCount(); } 
      virtual ICollection<SignalDrop>*    BDAQCALL getTriggerSources()    { return (ICollection<SignalDrop>*)m_impl->getTriggerSources(0); } 
      virtual ICollection<TriggerAction>* BDAQCALL getTriggerActions()    { return (ICollection<TriggerAction>*)m_impl->getTriggerActions(0); } 
      virtual MathInterval                BDAQCALL getTriggerDelayRange() 
      { 
         MathInterval x;
         m_impl->getTriggerDelayRange(0, x); 
         return x;
      } 

      // add trigger 1 features
      virtual bool                        BDAQCALL getTrigger1Supported()  { return m_impl->getTriggerCount() > 1; } 
      virtual ICollection<SignalDrop>*    BDAQCALL getTrigger1Sources()    { return (ICollection<SignalDrop>*)m_impl->getTriggerSources(1); } 
      virtual ICollection<TriggerAction>* BDAQCALL getTrigger1Actions()    { return (ICollection<TriggerAction>*)m_impl->getTriggerActions(1); } 
      virtual MathInterval                BDAQCALL getTrigger1DelayRange() 
      { 
         MathInterval x;
         m_impl->getTriggerDelayRange(1, x); 
         return x;
      } 

      // new: coupling & IEPE
      virtual ICollection<CouplingType> * BDAQCALL getCouplingTypes()       { return (ICollection<CouplingType> *)m_impl->getCouplingTypes(); }
      virtual ICollection<IepeType> *     BDAQCALL getIepeTypes()           { return (ICollection<IepeType> *)m_impl->getIepeTypes();         }

   public:
      ::AiFeaturesImpl * m_impl;
   };

   template<class T, class TBase>
   class AiCtrlBaseImpl : public DeviceCtrlBaseImpl<T, TBase>
   {
   public:
      /*------------------------------------------------------------------------------*/
      /* property                                                                     */
      /*------------------------------------------------------------------------------*/
      virtual AiFeatures * BDAQCALL getFeatures()         { return &m_features;      }
      virtual int32        BDAQCALL getChannelCount()     { return ctrl_impl().getChannelCount(); }
      virtual AiChannelCollection* BDAQCALL getChannels() 
      { 
         return (AiChannelCollection*)ctrl_impl().getChannels(); 
      }

   public:
      AiCtrlBaseImpl() { m_features.m_impl = (::AiFeaturesImpl *)ctrl_impl().getFeatures(); }

   private: // helper
      ::AiCtrlBaseImpl& ctrl_impl() { return static_cast<T *>(this)->m_impl; }

   private:
      AiFeaturesImpl m_features;
   };

   class InstantAiCtrlCppWrapper : public AiCtrlBaseImpl<InstantAiCtrlCppWrapper, InstantAiCtrl>
   {
   public:
      //--------------------------------------------------------------------------------
      // method
      //--------------------------------------------------------------------------------
      virtual ErrorCode BDAQCALL ReadAny(int32 chStart, int32 chCount, void * dataRaw, double * dataScaled)
      {
         return m_impl.ReadAny(chStart, chCount, dataRaw, dataScaled);
      }

      //--------------------------------------------------------------------------------
      // property
      //--------------------------------------------------------------------------------
      virtual CjcSetting* BDAQCALL getCjc()  {  return (CjcSetting*)m_impl.getCjc();  }

   public:
      InstantAiCtrlCppWrapper() { m_impl.setOuter(this); }
      virtual ~InstantAiCtrlCppWrapper() { }

   public:
      ::InstantAiCtrlImpl m_impl;
   };

   class BufferedAiCtrlCppWrapper : public AiCtrlBaseImpl<BufferedAiCtrlCppWrapper, BufferedAiCtrl>
   {
   public:
      //--------------------------------------------------------------------------------
      // event
      //--------------------------------------------------------------------------------
      virtual void BDAQCALL addDataReadyListener(BfdAiEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAiDataReady, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL removeDataReadyListener(BfdAiEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAiDataReady, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL addOverrunListener(BfdAiEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAiOverrun, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL removeOverrunListener(BfdAiEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAiOverrun, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL addCacheOverflowListener(BfdAiEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAiCacheOverflow, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL removeCacheOverflowListener(BfdAiEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAiCacheOverflow, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL addStoppedListener(BfdAiEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAiStopped, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL removeStoppedListener(BfdAiEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAiStopped, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL addBurnOutListener(BfdAiEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAiBurnOut, (DaqEventProc)OnAiEvent, &listener);
      }
      virtual void BDAQCALL removeBurnOutListener(BfdAiEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAiBurnOut, (DaqEventProc)OnAiEvent, &listener);
      }

      //--------------------------------------------------------------------------------
      // method
      //--------------------------------------------------------------------------------
      virtual ErrorCode BDAQCALL Prepare() {	return m_impl.Prepare();}
      virtual ErrorCode BDAQCALL RunOnce() {	return m_impl.RunOnce();}
      virtual ErrorCode BDAQCALL Start()	 {	return m_impl.Start();	}
      virtual ErrorCode BDAQCALL Stop()	 {	return m_impl.Stop();	}
      virtual void      BDAQCALL Release() { return m_impl.Release();}

      virtual ErrorCode BDAQCALL GetDataI16(int32  count, int16 *  rawData)
      {
         return m_impl.GetData(sizeof(uint16), count, rawData);
      }
      virtual ErrorCode BDAQCALL GetDataI32(int32 count, int32 * rawData)
      {
         return m_impl.GetData(sizeof(uint32), count, rawData);
      }
      virtual ErrorCode BDAQCALL GetDataF64(int32 count, double * scaledData)
      {
         return m_impl.GetData(sizeof(double), count, scaledData);
      }
      virtual ErrorCode BDAQCALL GetEventStatus(EventId id, int32 & status)
      {
         EventStatus es = {sizeof(es)};
         m_impl.ModEvent().GetStatus(id, es);
         if (id == EvtBufferedAiDataReady) {// Round down the data ready count to times of scan channel count.
            int32 scanCHCount = m_impl.getScanChannel()->getChannelCount();
            if (!scanCHCount) { scanCHCount = 1; }
            status = (es.AiDataReady.DataCount / scanCHCount) * scanCHCount;
         } else {
            status = es.General.lParam;
         }
         return Success;
      }

      //--------------------------------------------------------------------------------
      // property
      //--------------------------------------------------------------------------------
      virtual void*         BDAQCALL getBuffer()             { return m_impl.getBuffer();		                  }
      virtual int32         BDAQCALL getBufferCapacity()	    { return m_impl.getBufferCapacity();		         }
      virtual ControlState  BDAQCALL getState()	             { return (ControlState)m_impl.getState();		   }
      virtual ScanChannel*  BDAQCALL getScanChannel()	       { return (ScanChannel*)m_impl.getScanChannel();	}
      virtual ConvertClock* BDAQCALL getConvertClock()       { return (ConvertClock*)m_impl.getConvertClock();	}
      virtual ScanClock*    BDAQCALL getScanClock()          { return (ScanClock*)m_impl.getScanClock();		   }
      virtual Trigger*      BDAQCALL getTrigger()		       { return (Trigger*)m_impl.getTrigger(0);		      }
      virtual Trigger*      BDAQCALL getTrigger1()		       { return (Trigger*)m_impl.getTrigger(1);		      }
      virtual bool          BDAQCALL getStreaming()          { return m_impl.getStreaming();                   }
      virtual ErrorCode     BDAQCALL setStreaming(bool value){ return m_impl.setStreaming(value);              }

   private:
      static void BDAQCALL OnAiEvent(void *sender, ::BfdAiEventArgs *args, void *userParam)
      {
         BfdAiEventArgs dn3Args = { args->Offset, args->Count };
         BfdAiEventListener *handler = static_cast<BfdAiEventListener *>(userParam);
         handler->BfdAiEvent(sender, &dn3Args);
      }

   public:
      BufferedAiCtrlCppWrapper() { m_impl.setOuter(this); }
      virtual ~BufferedAiCtrlCppWrapper() { }

   public:
      ::BufferedAiCtrlImpl m_impl;
   };

   /********************************************************************************/
   /* AO related classes                                                       */
   /********************************************************************************/
   class AoFeaturesImpl : public AoFeatures
   {
   public:
      // DAC features
      virtual int32 BDAQCALL getResolution() { return m_impl->getResolution(); }
      virtual int32 BDAQCALL getDataSize()   { return m_impl->getDataSize(); } 
      virtual int32 BDAQCALL getDataMask()   { return m_impl->getDataMask(); } 

      // channel features
      virtual int32                       BDAQCALL getChannelCountMax()     { return m_impl->getChannelCountMax(); } 
      virtual ICollection<ValueRange>*    BDAQCALL getValueRanges()         { return (ICollection<ValueRange>*)m_impl->getValueRanges(); } 
      virtual bool                        BDAQCALL getExternalRefAntiPolar(){ return m_impl->getExternalRefAntiPolar(); } 
      virtual MathInterval                BDAQCALL getExternalRefRange()    
      { 
         MathInterval x;
         m_impl->getExternalRefRange(x); 
         return x;
      } 

      // buffered ao->basic features
      virtual bool                        BDAQCALL getBufferedAoSupported() { return m_impl->getBufferedAoSupported(); } 
      virtual SamplingMethod              BDAQCALL getSamplingMethod()      { return m_impl->getSamplingMethod(); } 
      virtual int32                       BDAQCALL getChannelStartBase()    { return m_impl->getChannelStartBase(); } 
      virtual int32                       BDAQCALL getChannelCountBase()    { return m_impl->getChannelCountBase(); } 

      // buffered ao->conversion clock features
      virtual ICollection<SignalDrop>*    BDAQCALL getConvertClockSources() { return (ICollection<SignalDrop>*)m_impl->getConvertClockSources(); } 
      virtual MathInterval                BDAQCALL getConvertClockRange()   
      { 
         MathInterval x;
         m_impl->getConvertClockRange(x); 
         return x;
      }  

      // buffered ao->trigger features
      virtual bool                        BDAQCALL getTriggerSupported()  { return m_impl->getTriggerCount() > 0; } 
      virtual int32                       BDAQCALL getTriggerCount()      { return m_impl->getTriggerCount(); } 
      virtual ICollection<SignalDrop>*    BDAQCALL getTriggerSources()    { return (ICollection<SignalDrop>*)m_impl->getTriggerSources(0); } 
      virtual ICollection<TriggerAction>* BDAQCALL getTriggerActions()    { return (ICollection<TriggerAction>*)m_impl->getTriggerActions(0); } 
      virtual MathInterval                BDAQCALL getTriggerDelayRange() 
      { 
         MathInterval x;
         m_impl->getTriggerDelayRange(0, x); 
         return x;
      }  

      // add trigger 1 features
      virtual bool                        BDAQCALL getTrigger1Supported()  { return m_impl->getTriggerCount() > 1; } 
      virtual ICollection<SignalDrop>*    BDAQCALL getTrigger1Sources()    { return (ICollection<SignalDrop>*)m_impl->getTriggerSources(1); } 
      virtual ICollection<TriggerAction>* BDAQCALL getTrigger1Actions()    { return (ICollection<TriggerAction>*)m_impl->getTriggerActions(1); } 
      virtual MathInterval                BDAQCALL getTrigger1DelayRange() 
      { 
         MathInterval x;
         m_impl->getTriggerDelayRange(1, x); 
         return x;
      } 

   public:
      ::AoFeaturesImpl * m_impl;
   };

   template<class T, class TBase>
   class AoCtrlBaseImpl : public DeviceCtrlBaseImpl<T, TBase>
   {
   public:
      /*------------------------------------------------------------------------------*/
      /* property                                                                     */
      /*------------------------------------------------------------------------------*/
      virtual AoFeatures * BDAQCALL getFeatures()
      {
         return &m_features;
      }
      virtual AoChannelCollection* BDAQCALL getChannels()
      {
         return (AoChannelCollection*)ctrl_impl().getChannels();
      }
      virtual int32 BDAQCALL getChannelCount()
      {
         return ctrl_impl().getChannelCount();
      }
      virtual double BDAQCALL getExtRefValueForUnipolar()
      {
         return ctrl_impl().getExtRefValueForUnipolar();
      }
      virtual ErrorCode BDAQCALL setExtRefValueForUnipolar(double value)
      {
         return ctrl_impl().setExtRefValueForUnipolar(value);
      }
      virtual double BDAQCALL getExtRefValueForBipolar()
      {
         return ctrl_impl().getExtRefValueForBipolar();
      }
      virtual ErrorCode BDAQCALL setExtRefValueForBipolar(double value)
      {
         return ctrl_impl().setExtRefValueForBipolar(value);
      }

   public:
      AoCtrlBaseImpl() { m_features.m_impl = (::AoFeaturesImpl *)ctrl_impl().getFeatures(); }

   private: // helper
      ::AoCtrlBaseImpl& ctrl_impl() { return static_cast<T *>(this)->m_impl; }

   private:
      AoFeaturesImpl m_features;
   };

   class InstantAoCtrlCppWrapper : public AoCtrlBaseImpl<InstantAoCtrlCppWrapper, InstantAoCtrl>
   {
   public:
      //--------------------------------------------------------------------------------
      // method
      //--------------------------------------------------------------------------------
      virtual ErrorCode BDAQCALL WriteAny(int32 chStart, int32 chCount, void * dataRaw, double * dataScaled)
      {
         return m_impl.WriteAny(chStart, chCount, dataRaw, dataScaled);
      }

   public:
      InstantAoCtrlCppWrapper() { m_impl.setOuter(this); }
      virtual ~InstantAoCtrlCppWrapper() { }

   public:
      ::InstantAoCtrlImpl m_impl;
   };

   class BufferedAoCtrlCppWrapper : public AoCtrlBaseImpl<BufferedAoCtrlCppWrapper, BufferedAoCtrl>
   {
   public:
      //--------------------------------------------------------------------------------
      // event
      //--------------------------------------------------------------------------------
      virtual void BDAQCALL addDataTransmittedListener(BfdAoEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAoDataTransmitted, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL removeDataTransmittedListener(BfdAoEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAoDataTransmitted, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL addUnderrunListener(BfdAoEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAoUnderrun, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL removeUnderrunListener(BfdAoEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAoUnderrun, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL addCacheEmptiedListener(BfdAoEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAoCacheEmptied, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL removeCacheEmptiedListener(BfdAoEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAoCacheEmptied, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL addTransitStoppedListener(BfdAoEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAoTransStopped, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL removeTransitStoppedListener(BfdAoEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAoTransStopped, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL addStoppedListener(BfdAoEventListener & listener)
      {
         m_impl.addEventHandler(EvtBufferedAoStopped, (DaqEventProc)OnAoEvent, &listener);
      }
      virtual void BDAQCALL removeStoppedListener(BfdAoEventListener & listener)
      {
         m_impl.removeEventHandler(EvtBufferedAoStopped, (DaqEventProc)OnAoEvent, &listener);
      }
 
      //--------------------------------------------------------------------------------
      // method
      //--------------------------------------------------------------------------------
      virtual ErrorCode BDAQCALL Prepare()         { return m_impl.Prepare();    }
      virtual ErrorCode BDAQCALL RunOnce()         { return m_impl.RunOnce();    }
      virtual ErrorCode BDAQCALL Start()           { return m_impl.Start();      }
      virtual ErrorCode BDAQCALL Stop(int32 action){ return m_impl.Stop(action); }
      virtual void      BDAQCALL Release()         { return m_impl.Release();    }

      virtual ErrorCode BDAQCALL SetDataI16(int32  count, int16 *  rawData)
      {
         return m_impl.SetData(sizeof(uint16), count, rawData);
      }
      virtual ErrorCode BDAQCALL SetDataI32(int32 count, int32 * rawData)
      {
         return m_impl.SetData(sizeof(uint32), count, rawData);
      }
      virtual ErrorCode BDAQCALL SetDataF64(int32 count, double * scaledData)
      {
         return m_impl.SetData(sizeof(double), count, scaledData);
      }
 
      //--------------------------------------------------------------------------------
      // property
      //--------------------------------------------------------------------------------
      virtual void*         BDAQCALL getBuffer()             { return m_impl.getBuffer();                      }
      virtual int32         BDAQCALL getBufferCapacity()     { return m_impl.getBufferCapacity();              }
      virtual ControlState  BDAQCALL getState()              { return (ControlState)m_impl.getState();         }
      virtual ScanChannel*  BDAQCALL getScanChannel()        { return (ScanChannel*)m_impl.getScanChannel();   }
      virtual ConvertClock* BDAQCALL getConvertClock()       { return (ConvertClock*)m_impl.getConvertClock(); }
      virtual Trigger*      BDAQCALL getTrigger()            { return (Trigger*)m_impl.getTrigger(0);          }
      virtual Trigger*      BDAQCALL getTrigger1()           { return (Trigger*)m_impl.getTrigger(1);          }
      virtual bool          BDAQCALL getStreaming()          { return m_impl.getStreaming();                   }
      virtual ErrorCode     BDAQCALL setStreaming(bool value){ return m_impl.setStreaming(value);              }

   private:
      static void BDAQCALL OnAoEvent(void *sender, ::BfdAoEventArgs *args, void *userParam)
      {
         BfdAoEventArgs dn3Args = { args->Offset, args->Count };
         BfdAoEventListener *handler = static_cast<BfdAoEventListener *>(userParam);
         handler->BfdAoEvent(sender, &dn3Args);
      }

   public:
      BufferedAoCtrlCppWrapper() { m_impl.setOuter(this); }
      virtual ~BufferedAoCtrlCppWrapper() { }

   public:
      ::BufferedAoCtrlImpl m_impl;
   };

   /********************************************************************************/
   /* DIO related classes                                                       */
   /********************************************************************************/
   class DiFeaturesImpl : public DiFeatures
   {
   public:
      // port features
      virtual bool                BDAQCALL getPortProgrammable()  { return m_impl->getPortProgrammable();} 
      virtual int32               BDAQCALL getPortCount()         { return m_impl->getPortCount();       } 
      virtual ICollection<uint8>* BDAQCALL getPortsType()         { return (ICollection<uint8>*)m_impl->getPortsType();       } 
      virtual bool                BDAQCALL getDiSupported()       { return m_impl->getDiSupported();     } 
      virtual bool                BDAQCALL getDoSupported()       { return m_impl->getDoSupported();     } 

      // channel features
      virtual int32               BDAQCALL getChannelCountMax()   { return m_impl->getChannelCountMax(); } 

      virtual ICollection<uint8>* BDAQCALL getDataMask()          { return (ICollection<uint8>*)m_impl->getDiDataMask(); } 

      // di noise filter features
      virtual bool                BDAQCALL getNoiseFilterSupported()      { return m_impl->getDiNoiseFilterSupported(); } 
      virtual ICollection<uint8>* BDAQCALL getNoiseFilterOfChannels()     { return (ICollection<uint8>*)m_impl->getDiNoiseFilterOfChannels(); } 
      virtual MathInterval        BDAQCALL getNoiseFilterBlockTimeRange() 
      { 
         MathInterval x;
         m_impl->getDiNoiseFilterBlockTimeRange(x); 
         return x;
      } 

      // di interrupt features
      virtual bool                        BDAQCALL getDiintSupported()     { return m_impl->getDiintSupported();     } 
      virtual bool                        BDAQCALL getDiintGateSupported() { return m_impl->getDiintGateSupported(); } 
      virtual bool                        BDAQCALL getDiCosintSupported()  { return m_impl->getDiCosintSupported();  } 
      virtual bool                        BDAQCALL getDiPmintSupported()   { return m_impl->getDiPmintSupported();   } 
      virtual ICollection<ActiveSignal>*  BDAQCALL getDiintTriggerEdges()  { return (ICollection<ActiveSignal>*)m_impl->getDiintTriggerEdges();  } 
      virtual ICollection<uint8>*         BDAQCALL getDiintOfChannels()    { return (ICollection<uint8>*)m_impl->getDiintOfChannels();    } 
      virtual ICollection<uint8>*         BDAQCALL getDiintGateOfChannels(){ return (ICollection<uint8>*)m_impl->getDiintGateOfChannels();} 
      virtual ICollection<uint8>*         BDAQCALL getDiCosintOfPorts()    { return (ICollection<uint8>*)m_impl->getDiCosintOfPorts();    } 
      virtual ICollection<uint8>*         BDAQCALL getDiPmintOfPorts()     { return (ICollection<uint8>*)m_impl->getDiPmintOfPorts();     } 
      virtual ICollection<int32>*         BDAQCALL getSnapEventSources()   { return (ICollection<int32>*)m_impl->getDiSnapEventSources(); } 

      // buffered di->basic features
      virtual bool                        BDAQCALL getBufferedDiSupported() { return false;        } 
      virtual SamplingMethod              BDAQCALL getSamplingMethod()      { return Simultaneous; } 

      // buffered di->conversion clock features
      virtual ICollection<SignalDrop>*    BDAQCALL getConvertClockSources() { return NULL; } 
      virtual MathInterval                BDAQCALL getConvertClockRange()   { MathInterval x = {0}; return x; } 

      // buffered di->burst scan
      virtual bool                        BDAQCALL getBurstScanSupported()  { return false; } 
      virtual ICollection<SignalDrop>*    BDAQCALL getScanClockSources()    { return NULL;  } 
      virtual MathInterval                BDAQCALL getScanClockRange()      { MathInterval x = {0}; return x; } 
      virtual int32                       BDAQCALL getScanCountMax()        { return 0; } 

      // buffered di->trigger features
      virtual bool                        BDAQCALL getTriggerSupported() { return false; } 
      virtual int32                       BDAQCALL getTriggerCount()     { return 0; } 
      virtual ICollection<SignalDrop>*    BDAQCALL getTriggerSources()   { return NULL; } 
      virtual ICollection<TriggerAction>* BDAQCALL getTriggerActions()   { return NULL; } 
      virtual MathInterval                BDAQCALL getTriggerDelayRange(){ MathInterval x = {0}; return x; } 

   public:
      ::DioFeaturesImpl * m_impl;
   };

   class DoFeaturesImpl : public DoFeatures
   {
   public:
      // port features
      virtual bool                BDAQCALL getPortProgrammable()  { return m_impl->getPortProgrammable();} 
      virtual int32               BDAQCALL getPortCount()         { return m_impl->getPortCount();       } 
      virtual ICollection<uint8>* BDAQCALL getPortsType()         { return (ICollection<uint8>*)m_impl->getPortsType();       } 
      virtual bool                BDAQCALL getDiSupported()       { return m_impl->getDiSupported();     } 
      virtual bool                BDAQCALL getDoSupported()       { return m_impl->getDoSupported();     } 

      // channel features
      virtual int32               BDAQCALL getChannelCountMax()   { return m_impl->getChannelCountMax(); } 
      virtual ICollection<uint8>* BDAQCALL getDataMask()          { return (ICollection<uint8>*)m_impl->getDoDataMask(); } 

      // do freeze features
      virtual ICollection<SignalDrop>*    BDAQCALL getDoFreezeSignalSources() { return (ICollection<SignalDrop>*)m_impl->getDoFreezeSignalSources(); } 

      // do reflect Wdt features
      virtual MathInterval                BDAQCALL getDoReflectWdtFeedIntervalRange() 
      { 
         MathInterval x;
         m_impl->getDoReflectWdtFeedIntervalRange(x); 
         return x;
      } 

      // buffered do->basic features
      virtual bool                        BDAQCALL getBufferedDoSupported() { return false;        }
      virtual SamplingMethod              BDAQCALL getSamplingMethod()      { return Simultaneous; }

      // buffered do->conversion clock features
      virtual ICollection<SignalDrop>*    BDAQCALL getConvertClockSources() { return NULL; } 
      virtual MathInterval                BDAQCALL getConvertClockRange()   { MathInterval x = {0}; return x; } 

      // buffered do->burst scan
      virtual bool                        BDAQCALL getBurstScanSupported() { return false; }  
      virtual ICollection<SignalDrop>*    BDAQCALL getScanClockSources()   { return NULL;  }  
      virtual MathInterval                BDAQCALL getScanClockRange()     { MathInterval x = {0}; return x; } 
      virtual int32                       BDAQCALL getScanCountMax()       { return 0; }  

      // buffered do->trigger features
      virtual bool                        BDAQCALL getTriggerSupported() { return false; }  
      virtual int32                       BDAQCALL getTriggerCount()     { return 0; }  
      virtual ICollection<SignalDrop>*    BDAQCALL getTriggerSources()   { return NULL; }  
      virtual ICollection<TriggerAction>* BDAQCALL getTriggerActions()   { return NULL; }  
      virtual MathInterval                BDAQCALL getTriggerDelayRange(){ MathInterval x = {0}; return x; }  

   public:
      ::DioFeaturesImpl * m_impl;
   };

   template<class T, class TBase>
   class DiCtrlBaseImpl : public DeviceCtrlBaseImpl<T, TBase>
   {
   public:
      /*------------------------------------------------------------------------------*/
      /* property                                                                     */
      /*------------------------------------------------------------------------------*/
      virtual int32 BDAQCALL getPortCount()
      {
         return ctrl_impl().getPortCount();
      }

      virtual ICollection<PortDirection>* BDAQCALL getPortDirection()
      {
         return (ICollection<PortDirection>*)ctrl_impl().getPorts();
      }

      virtual DiFeatures * BDAQCALL getFeatures()
      {
         return &m_features;
      }

      ICollection<NoiseFilterChannel>* BDAQCALL getNoiseFilter()
      {
         return (ICollection<NoiseFilterChannel>*)static_cast<T *>(this)->m_impl.getNoiseFilter();
      }
   public:
      DiCtrlBaseImpl() { m_features.m_impl = (::DioFeaturesImpl *)ctrl_impl().getFeatures(); }

   private: // helper
      ::DioCtrlBaseImpl& ctrl_impl() { return static_cast<T *>(this)->m_impl; }

   private:
      DiFeaturesImpl m_features;
   };

   template<class T, class TBase>
   class DoCtrlBaseImpl : public DeviceCtrlBaseImpl<T, TBase>
   {
   public:
      /*------------------------------------------------------------------------------*/
      /* property                                                                     */
      /*------------------------------------------------------------------------------*/
      virtual int32 BDAQCALL getPortCount()
      {
         return ctrl_impl().getPortCount();
      }

      virtual ICollection<PortDirection>* BDAQCALL getPortDirection()
      {
         return (ICollection<PortDirection>*)ctrl_impl().getPorts();
      }

      virtual DoFeatures * BDAQCALL getFeatures()
      {
         return &m_features;
      }

   public:
      DoCtrlBaseImpl() { m_features.m_impl = (::DioFeaturesImpl *)ctrl_impl().getFeatures(); }

   private: // helper
      ::DioCtrlBaseImpl& ctrl_impl() { return static_cast<T *>(this)->m_impl; }

   private:
      DoFeaturesImpl m_features;
   };

   class InstantDiCtrlCppWrapper : public DiCtrlBaseImpl<InstantDiCtrlCppWrapper, InstantDiCtrl>
   {
   public:
      //--------------------------------------------------------------------------------
      // event
      //--------------------------------------------------------------------------------
      virtual void BDAQCALL addInterruptListener(DiSnapEventListener & listener)
      {
         m_impl.addEventHandler(EvtDiintChannel000, (DaqEventProc)OnDiEvent, &listener);

      }
      virtual void BDAQCALL removeInterruptListener(DiSnapEventListener & listener)
      {
         m_impl.removeEventHandler(EvtDiintChannel000, (DaqEventProc)OnDiEvent, &listener);

      }
      virtual void BDAQCALL addChangeOfStateListener(DiSnapEventListener & listener)
      {
         m_impl.addEventHandler(EvtDiCosintPort000, (DaqEventProc)OnDiEvent, &listener);

      }
      virtual void BDAQCALL removeChangeOfStateListener(DiSnapEventListener & listener)
      {
         m_impl.removeEventHandler(EvtDiCosintPort000, (DaqEventProc)OnDiEvent, &listener);

      }
      virtual void BDAQCALL addPatternMatchListener(DiSnapEventListener & listener)
      {
         m_impl.addEventHandler(EvtDiPmintPort000, (DaqEventProc)OnDiEvent, &listener);

      }
      virtual void BDAQCALL removePatternMatchListener(DiSnapEventListener & listener)
      {
         m_impl.removeEventHandler(EvtDiPmintPort000, (DaqEventProc)OnDiEvent, &listener);
      }

      //--------------------------------------------------------------------------------
      // method
      //--------------------------------------------------------------------------------
      virtual ErrorCode BDAQCALL ReadAny(int32 portStart, int32 portCount, uint8 data[])
      {
         return m_impl.ReadAny(portStart, portCount, data);
      }
      virtual ErrorCode BDAQCALL ReadBit(int32 port, int32 bit, uint8* data)
      {
         return m_impl.ReadBit(port, bit, data);
      }

      virtual ErrorCode BDAQCALL SnapStart()
      {
         return m_impl.SnapStart();
      }
      virtual ErrorCode BDAQCALL SnapStop()
      {
         return m_impl.SnapStop();
      }

      //--------------------------------------------------------------------------------
      // property
      //--------------------------------------------------------------------------------
      virtual ICollection<DiintChannel>* BDAQCALL getDiintChannels()
      {
         return (ICollection<DiintChannel>*)m_impl.getDiintChannels();
      }
      virtual ICollection<DiCosintPort>* BDAQCALL getDiCosintPorts()
      {
         return (ICollection<DiCosintPort>*)m_impl.getDiCosintPorts();
      }
      virtual ICollection<DiPmintPort>*  BDAQCALL getDiPmintPorts()
      {
         return (ICollection<DiPmintPort>*)m_impl.getDiPmintPorts();
      }

      // new property
      virtual double BDAQCALL getNoiseFilterBlockTime()
      {
         return m_impl.getNoiseFilterBlockTime();
      }
      virtual ErrorCode BDAQCALL setNoiseFilterBlockTime(double value)
      {
         return m_impl.setNoiseFilterBlockTime(value);
      }

   private:
      static void BDAQCALL OnDiEvent(void *sender, ::DiSnapEventArgs *args, void *userParam)
      {
         DiSnapEventArgs dn3Args;
         dn3Args.SrcNum = args->SrcNum;
         dn3Args.Length = args->Length;
         if (dn3Args.Length) { 
            assert(dn3Args.Length < MAX_DIO_PORT_COUNT);
            memcpy(dn3Args.PortData, args->PortData, dn3Args.Length);
         }
         DiSnapEventListener *handler = static_cast<DiSnapEventListener *>(userParam);
         handler->DiSnapEvent(sender, &dn3Args);
      }

   public:
      InstantDiCtrlCppWrapper() { m_impl.setOuter(this); }
      virtual ~InstantDiCtrlCppWrapper() { }

   public:
      ::InstantDiCtrlImpl m_impl;
   };

   class InstantDoCtrlCppWrapper : public DoCtrlBaseImpl<InstantDoCtrlCppWrapper, InstantDoCtrl>
   {
   public:
      //--------------------------------------------------------------------------------
      // method
      //--------------------------------------------------------------------------------
      virtual ErrorCode BDAQCALL WriteAny(int32 portStart, int32 portCount, uint8 data[])
      {
         return m_impl.WriteAny(portStart, portCount, data);
      }
      virtual ErrorCode BDAQCALL ReadAny(int32 portStart, int32 portCount, uint8 data[])
      {
         return m_impl.ReadAny(portStart, portCount, data);
      }
      virtual ErrorCode BDAQCALL WriteBit(int32 port, int32 bit, uint8 data)
      {
         return m_impl.WriteBit(port, bit, data);
      }
      virtual ErrorCode BDAQCALL ReadBit(int32 port, int32 bit, uint8* data)
      {
         return m_impl.ReadBit(port, bit, data);
      }

      //--------------------------------------------------------------------------------
      // property
      //--------------------------------------------------------------------------------

   public:
      InstantDoCtrlCppWrapper() { m_impl.setOuter(this); }
      virtual ~InstantDoCtrlCppWrapper() { }

   public:
      ::InstantDoCtrlImpl m_impl;
   };

   /********************************************************************************/
   /* Counter related classes                                                      */
   /********************************************************************************/
   template<class T, class TBase>
   class CntrCtrlBaseImpl : public DeviceCtrlBaseImpl<T, TBase>
   {
   public:
      /*------------------------------------------------------------------------------*/
      /* property                                                                     */
      /*------------------------------------------------------------------------------*/
      virtual int32         BDAQCALL getChannel()            { return ctrl_impl().getChannelStart();   }
      virtual ErrorCode     BDAQCALL setChannel(int32 ch)    { return ctrl_impl().setChannelStart(ch); }
      virtual bool          BDAQCALL getEnabled()            { return ctrl_impl().getEnabled();        } 
      virtual ErrorCode     BDAQCALL setEnabled(bool enabled){ return ctrl_impl().setEnabled(enabled); }
      virtual bool          BDAQCALL getRunning()            { return ctrl_impl().getRunning();        }
      virtual NoiseFilterChannel* BDAQCALL getNoiseFilter()  
      { 
         return (NoiseFilterChannel*)ctrl_impl().getNoiseFilter(); 
      } 

      // new property
      virtual double BDAQCALL getNoiseFilterBlockTime()
      {
         return  ctrl_impl().getNoiseFilterBlockTime();
      }
      virtual ErrorCode BDAQCALL setNoiseFilterBlockTime(double value)
      {
         return  ctrl_impl().setNoiseFilterBlockTime(value);
      }
   public:
      CntrCtrlBaseImpl() { }

   private: // helper
      ::CntrCtrlBaseImpl& ctrl_impl() { return static_cast<T *>(this)->m_impl; }
   };

   class EventCounterFeaturesImpl : public EventCounterFeatures
   {
   public:
      // channel features
      virtual int32 BDAQCALL getChannelCountMax() { return m_impl->getChannelCountMax(); }
      virtual int32 BDAQCALL getResolution()      { return m_impl->getResolution();      }
      virtual int32 BDAQCALL getDataSize()        { return m_impl->getDataSize();        }
      virtual CounterCapabilityIndexer* BDAQCALL getCapabilities()
      {
         return (CounterCapabilityIndexer*)m_impl->getCapabilities();
      }

      // noise filter features
      virtual bool                BDAQCALL getNoiseFilterSupported() { return m_impl->getNoiseFilterSupported(); }
      virtual ICollection<uint8>* BDAQCALL getNoiseFilterOfChannels(){ return (ICollection<uint8>*)m_impl->getNoiseFilterOfChannels();}
      virtual MathInterval        BDAQCALL getNoiseFilterBlockTimeRange()
      {
         MathInterval x;
         m_impl->getNoiseFilterBlockTimeRange(x);
         return x;
      }

   public:
      ::CntrFeaturesImpl * m_impl;
   };

   class EventCounterCtrlCppWrapper : public CntrCtrlBaseImpl<EventCounterCtrlCppWrapper, EventCounterCtrl>
   {
   public:
      virtual EventCounterFeatures* BDAQCALL getFeatures() { return &m_features; }
      virtual int32  BDAQCALL getValue()                   
      { 
         int32 data = 0;
         m_impl.Read(1, &data);
         return data;
      }

   public:
      EventCounterCtrlCppWrapper() 
      { 
         m_impl.setOuter(this); 
         m_features.m_impl =  (::CntrFeaturesImpl *)m_impl.getFeatures();
      }

      virtual ~EventCounterCtrlCppWrapper() { }

   public:
      ::EventCounterCtrlImpl m_impl;
      EventCounterFeaturesImpl m_features;
   };

   class FreqMeterFeaturesImpl : public FreqMeterFeatures
   {
   public:
      // channel features
      virtual int32 BDAQCALL getChannelCountMax() { return m_impl->getChannelCountMax(); }
      virtual int32 BDAQCALL getResolution()      { return m_impl->getResolution();      }
      virtual int32 BDAQCALL getDataSize()        { return m_impl->getDataSize();        }
      virtual CounterCapabilityIndexer* BDAQCALL getCapabilities()
      {
         return (CounterCapabilityIndexer*)m_impl->getCapabilities();
      }

      // noise filter features
      virtual bool                BDAQCALL getNoiseFilterSupported() { return m_impl->getNoiseFilterSupported(); }
      virtual ICollection<uint8>* BDAQCALL getNoiseFilterOfChannels(){ return (ICollection<uint8>*)m_impl->getNoiseFilterOfChannels();}
      virtual MathInterval        BDAQCALL getNoiseFilterBlockTimeRange()
      {
         MathInterval x;
         m_impl->getNoiseFilterBlockTimeRange(x);
         return x;
      }

      virtual ICollection<FreqMeasureMethod>* BDAQCALL getFmMethods()
      {
         return (ICollection<FreqMeasureMethod>*)m_impl->getFmMethods();
      }
   public:
      ::CntrFeaturesImpl * m_impl;
   };

   class FreqMeterCtrlCppWrapper : public CntrCtrlBaseImpl<FreqMeterCtrlCppWrapper, FreqMeterCtrl>
   {
   public:
      virtual FreqMeterFeatures* BDAQCALL getFeatures() { return &m_features; }

      virtual FreqMeasureMethod  BDAQCALL getMethod()
      {
         FmChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->getFmMethod() : FreqMeasureMethod(0);
      }
      virtual ErrorCode BDAQCALL setMethod(FreqMeasureMethod value)
      {
         FmChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->setFmMethod(value) : ErrorFuncNotInited;
      }
      virtual double BDAQCALL getCollectionPeriod()
      {
         FmChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->getCollectionPeriod() : FreqMeasureMethod(0);
      }
      virtual ErrorCode BDAQCALL setCollectionPeriod(double value)
      {
         FmChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->setCollectionPeriod(value) : ErrorFuncNotInited;
      }

      virtual double  BDAQCALL getValue()                   
      { 
         double data = 0;
         m_impl.Read(1, &data);
         return data;
      }

      // Helper
      FmChannelImpl * GetObject(int32 index)
      {
         ArrayImpl<FmChannelImpl> *chs = m_impl.getChannels();
         if (chs && unsigned(chs->getLength()) > unsigned(index)) {
            return &chs->Item(index);
         }
         return NULL;
      }

   public:
      FreqMeterCtrlCppWrapper() 
      { 
         m_impl.setOuter(this); 
         m_features.m_impl =  (::CntrFeaturesImpl *)m_impl.getFeatures();
      }

      virtual ~FreqMeterCtrlCppWrapper() { }

   public:
      ::FreqMeterCtrlImpl m_impl;
      FreqMeterFeaturesImpl m_features;
   };

   class OneShotFeaturesImpl : public OneShotFeatures
   {
   public:
      // channel features
      virtual int32 BDAQCALL getChannelCountMax() { return m_impl->getChannelCountMax(); }
      virtual int32 BDAQCALL getResolution()      { return m_impl->getResolution();      }
      virtual int32 BDAQCALL getDataSize()        { return m_impl->getDataSize();        }
      virtual CounterCapabilityIndexer* BDAQCALL getCapabilities()
      {
         return (CounterCapabilityIndexer*)m_impl->getCapabilities();
      }

      // noise filter features
      virtual bool                BDAQCALL getNoiseFilterSupported() { return m_impl->getNoiseFilterSupported(); }
      virtual ICollection<uint8>* BDAQCALL getNoiseFilterOfChannels(){ return (ICollection<uint8>*)m_impl->getNoiseFilterOfChannels();}
      virtual MathInterval        BDAQCALL getNoiseFilterBlockTimeRange()
      {
         MathInterval x;
         m_impl->getNoiseFilterBlockTimeRange(x);
         return x;
      }

      virtual bool         BDAQCALL getOneShotEventSupported() { return false; }
      virtual MathInterval BDAQCALL getDelayCountRange()
      {
         MathInterval x;
         m_impl->getOsDelayCountRange(x);
         return x;
      }

   public:
      ::CntrFeaturesImpl * m_impl;
   };

   class OneShotCtrlCppWrapper : public CntrCtrlBaseImpl<OneShotCtrlCppWrapper, OneShotCtrl>
   {
   public:
      virtual void BDAQCALL addOneShotListener(CntrEventListener & listener)
      {
         m_impl.addEventHandler(EvtCntOneShot0, (DaqEventProc)OnCntrEvent, &listener);
      }
      virtual void BDAQCALL removeOneShotListener(CntrEventListener & listener)
      {
         m_impl.removeEventHandler(EvtCntOneShot0, (DaqEventProc)OnCntrEvent, &listener);
      }

      virtual OneShotFeatures* BDAQCALL getFeatures() 
      { 
         return &m_features; 
      }

      virtual int32 BDAQCALL getDelayCount()
      {
         OsChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->getDelayCount() : 0;
      }
      virtual ErrorCode BDAQCALL setDelayCount(int32 value)
      {
         OsChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->setDelayCount(value) : ErrorFuncNotInited;
      }

      // Helper
      OsChannelImpl * GetObject(int32 index)
      {
         ArrayImpl<OsChannelImpl> *chs = m_impl.getChannels();
         if (chs && unsigned(chs->getLength()) > unsigned(index)) {
            return &chs->Item(index);
         }
         return NULL;
      }

   private:
      static void BDAQCALL OnCntrEvent(void *sender, ::CntrEventArgs *args, void *userParam)
      {
         CntrEventArgs dn3Args = { args->Channel };
         CntrEventListener *handler = static_cast<CntrEventListener *>(userParam);
         handler->CntrEvent(sender, &dn3Args);
      }

   public:
      OneShotCtrlCppWrapper() 
      { 
         m_impl.setOuter(this); 
         m_features.m_impl =  (::CntrFeaturesImpl *)m_impl.getFeatures();
      }

      virtual ~OneShotCtrlCppWrapper() { }

   public:
      ::OneShotCtrlImpl m_impl;
      OneShotFeaturesImpl m_features;
   };

   class TimerPulseFeaturesImpl : public TimerPulseFeatures
   {
   public:
      // channel features
      virtual int32 BDAQCALL getChannelCountMax() { return m_impl->getChannelCountMax(); }
      virtual int32 BDAQCALL getResolution()      { return m_impl->getResolution();      }
      virtual int32 BDAQCALL getDataSize()        { return m_impl->getDataSize();        }
      virtual CounterCapabilityIndexer* BDAQCALL getCapabilities()
      {
         return (CounterCapabilityIndexer*)m_impl->getCapabilities();
      }

      // noise filter features
      virtual bool                BDAQCALL getNoiseFilterSupported() { return m_impl->getNoiseFilterSupported(); }
      virtual ICollection<uint8>* BDAQCALL getNoiseFilterOfChannels(){ return (ICollection<uint8>*)m_impl->getNoiseFilterOfChannels();}
      virtual MathInterval        BDAQCALL getNoiseFilterBlockTimeRange()
      {
         MathInterval x;
         m_impl->getNoiseFilterBlockTimeRange(x);
         return x;
      }

      virtual bool         BDAQCALL getTimerEventSupported() { return false; }
      virtual MathInterval BDAQCALL getTimerFrequencyRange()
      {
         MathInterval x;
         m_impl->getTmrFrequencyRange(x);
         return x;
      }

   public:
      ::CntrFeaturesImpl * m_impl;
   };

   class TimerPulseCtrlCppWrapper : public CntrCtrlBaseImpl<TimerPulseCtrlCppWrapper, TimerPulseCtrl>
   {
   public:
      virtual void BDAQCALL addTimerTickListener(CntrEventListener & listener)
      {
         m_impl.addEventHandler(EvtCntTimer0, (DaqEventProc)OnCntrEvent, &listener);
      }
      virtual void BDAQCALL removeTimerTickListener(CntrEventListener & listener)
      {
         m_impl.removeEventHandler(EvtCntTimer0, (DaqEventProc)OnCntrEvent, &listener);
      }

      virtual TimerPulseFeatures* BDAQCALL getFeatures() 
      { 
         return &m_features; 
      }

      virtual double BDAQCALL getFrequency()
      {
         TmrChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->getFrequency() : 0;
      }
      virtual ErrorCode BDAQCALL setFrequency(double value)
      {
         TmrChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->setFrequency(value) : ErrorFuncNotInited;
      }

      // Helper
      TmrChannelImpl * GetObject(int32 index)
      {
         ArrayImpl<TmrChannelImpl> *chs = m_impl.getChannels();
         if (chs && unsigned(chs->getLength()) > unsigned(index)) {
            return &chs->Item(index);
         }
         return NULL;
      }

   private:
      static void BDAQCALL OnCntrEvent(void *sender, ::CntrEventArgs *args, void *userParam)
      {
         CntrEventArgs dn3Args = { args->Channel };
         CntrEventListener *handler = static_cast<CntrEventListener *>(userParam);
         handler->CntrEvent(sender, &dn3Args);
      }

   public:
      TimerPulseCtrlCppWrapper() 
      { 
         m_impl.setOuter(this); 
         m_features.m_impl =  (::CntrFeaturesImpl *)m_impl.getFeatures();
      }

      virtual ~TimerPulseCtrlCppWrapper() { }

   public:
      ::TimerPulseCtrlImpl m_impl;
      TimerPulseFeaturesImpl m_features;
   };

   class PwMeterFeaturesImpl : public PwMeterFeatures
   {
   public:
      // channel features
      virtual int32 BDAQCALL getChannelCountMax() { return m_impl->getChannelCountMax(); }
      virtual int32 BDAQCALL getResolution()      { return m_impl->getResolution();      }
      virtual int32 BDAQCALL getDataSize()        { return m_impl->getDataSize();        }
      virtual CounterCapabilityIndexer* BDAQCALL getCapabilities()
      {
         return (CounterCapabilityIndexer*)m_impl->getCapabilities();
      }

      // noise filter features
      virtual bool                BDAQCALL getNoiseFilterSupported() { return m_impl->getNoiseFilterSupported(); }
      virtual ICollection<uint8>* BDAQCALL getNoiseFilterOfChannels(){ return (ICollection<uint8>*)m_impl->getNoiseFilterOfChannels();}
      virtual MathInterval        BDAQCALL getNoiseFilterBlockTimeRange()
      {
         MathInterval x;
         m_impl->getNoiseFilterBlockTimeRange(x);
         return x;
      }

      virtual bool BDAQCALL getOverflowEventSupported() { return false; }
      virtual ICollection<CounterCascadeGroup>* BDAQCALL getPwmCascadeGroup()
      {
         return (ICollection<CounterCascadeGroup>*)m_impl->getPiCascadeGroup();
      }

   public:
      ::CntrFeaturesImpl * m_impl;
   };

   class PwMeterCtrlCppWrapper : public CntrCtrlBaseImpl<PwMeterCtrlCppWrapper, PwMeterCtrl>
   {
   public:
      virtual void BDAQCALL addOverflowListener(CntrEventListener & listener)
      {
         m_impl.addEventHandler(EvtCntPwmInOverflow0, (DaqEventProc)OnCntrEvent, &listener);
      }
      virtual void BDAQCALL removeOverflowListener(CntrEventListener & listener)
      {
         m_impl.removeEventHandler(EvtCntPwmInOverflow0, (DaqEventProc)OnCntrEvent, &listener);
      }

      virtual PwMeterFeatures* BDAQCALL getFeatures() 
      { 
         return &m_features; 
      }

      virtual void BDAQCALL getValue(PulseWidth &width)
      {
         m_impl.Read(1, (::PulseWidth *)&width);
      }

   private:
      static void BDAQCALL OnCntrEvent(void *sender, ::CntrEventArgs *args, void *userParam)
      {
         CntrEventArgs dn3Args = { args->Channel };
         CntrEventListener *handler = static_cast<CntrEventListener *>(userParam);
         handler->CntrEvent(sender, &dn3Args);
      }

   public:
      PwMeterCtrlCppWrapper() 
      { 
         m_impl.setOuter(this); 
         m_features.m_impl =  (::CntrFeaturesImpl *)m_impl.getFeatures();
      }

      virtual ~PwMeterCtrlCppWrapper() { }

   public:
      ::PwMeterCtrlImpl m_impl;
      PwMeterFeaturesImpl m_features;
   };

   class PwModulatorFeaturesImpl : public PwModulatorFeatures
   {
   public:
      // channel features
      virtual int32 BDAQCALL getChannelCountMax() { return m_impl->getChannelCountMax(); }
      virtual int32 BDAQCALL getResolution()      { return m_impl->getResolution();      }
      virtual int32 BDAQCALL getDataSize()        { return m_impl->getDataSize();        }
      virtual CounterCapabilityIndexer* BDAQCALL getCapabilities()
      {
         return (CounterCapabilityIndexer*)m_impl->getCapabilities();
      }

      // noise filter features
      virtual bool                BDAQCALL getNoiseFilterSupported() { return m_impl->getNoiseFilterSupported(); }
      virtual ICollection<uint8>* BDAQCALL getNoiseFilterOfChannels(){ return (ICollection<uint8>*)m_impl->getNoiseFilterOfChannels();}
      virtual MathInterval        BDAQCALL getNoiseFilterBlockTimeRange()
      {
         MathInterval x;
         m_impl->getNoiseFilterBlockTimeRange(x);
         return x;
      }

      virtual MathInterval BDAQCALL getHiPeriodRange()
      {
         MathInterval x;
         m_impl->getPoHiPeriodRange(x);
         return x;
      }
      virtual MathInterval BDAQCALL getLoPeriodRange()
      {
         MathInterval x;
         m_impl->getPoLoPeriodRange(x);
         return x;
      }

   public:
      ::CntrFeaturesImpl * m_impl;
   };

   class PwModulatorCtrlCppWrapper : public CntrCtrlBaseImpl<PwModulatorCtrlCppWrapper, PwModulatorCtrl>
   {
   public:
      virtual PwModulatorFeatures* BDAQCALL getFeatures() 
      { 
         return &m_features; 
      }

      virtual void BDAQCALL getPulseWidth(PulseWidth &width)
      {
         PoChannelImpl *ch = GetObject(getChannel());
         if (ch) ch->getPulseWidth(*(::PulseWidth *)&width);
      }
      virtual ErrorCode BDAQCALL setPulseWidth(PulseWidth const &width)
      {
         PoChannelImpl *ch = GetObject(getChannel());
         return ch? ch->setPulseWidth(*(::PulseWidth const *)&width) : ErrorFuncNotInited;
      }

      // Helper
      PoChannelImpl * GetObject(int32 index)
      {
         ArrayImpl<PoChannelImpl> *chs = m_impl.getChannels();
         if (chs && unsigned(chs->getLength()) > unsigned(index)) {
            return &chs->Item(index);
         }
         return NULL;
      }

   public:
      PwModulatorCtrlCppWrapper() 
      { 
         m_impl.setOuter(this); 
         m_features.m_impl =  (::CntrFeaturesImpl *)m_impl.getFeatures();
      }

      virtual ~PwModulatorCtrlCppWrapper() { }

   public:
      ::PwModulatorCtrlImpl m_impl;
      PwModulatorFeaturesImpl m_features;
   };

   class UdCounterFeaturesImpl : public UdCounterFeatures
   {
   public:
      // channel features
      virtual int32 BDAQCALL getChannelCountMax() { return m_impl->getChannelCountMax(); }
      virtual int32 BDAQCALL getResolution()      { return m_impl->getResolution();      }
      virtual int32 BDAQCALL getDataSize()        { return m_impl->getDataSize();        }
      virtual CounterCapabilityIndexer* BDAQCALL getCapabilities()
      {
         return (CounterCapabilityIndexer*)m_impl->getCapabilities();
      }

      // noise filter features
      virtual bool                BDAQCALL getNoiseFilterSupported() { return m_impl->getNoiseFilterSupported(); }
      virtual ICollection<uint8>* BDAQCALL getNoiseFilterOfChannels(){ return (ICollection<uint8>*)m_impl->getNoiseFilterOfChannels();}
      virtual MathInterval        BDAQCALL getNoiseFilterBlockTimeRange()
      {
         MathInterval x;
         m_impl->getNoiseFilterBlockTimeRange(x);
         return x;
      }

      virtual ICollection<SignalCountingType>* BDAQCALL getCountingTypes()
      {
         return (ICollection<SignalCountingType>*)m_impl->getUdCountingTypes();
      }
      virtual ICollection<int32>* BDAQCALL getInitialValues()
      {
         return (ICollection<int32>*)m_impl->getUdInitialValues();
      }
      virtual ICollection<int32>* BDAQCALL getSnapEventSources()
      {
         return (ICollection<int32>*)m_impl->getUdSnapEventSources();
      }

   public:
      ::CntrFeaturesImpl * m_impl;
   };

   class UdCounterCtrlCppWrapper : public CntrCtrlBaseImpl<UdCounterCtrlCppWrapper, UdCounterCtrl>
   {
   public:
      // event
      virtual void BDAQCALL addUdCntrEventListener(UdCntrEventListener & listener)
      {
         m_impl.addEventHandler(EvtUdIndex0, (DaqEventProc)OnUdCntrEvent, &listener);
      }
      virtual void BDAQCALL removeUdCntrEventListener(UdCntrEventListener & listener)
      {
         m_impl.removeEventHandler(EvtUdIndex0, (DaqEventProc)OnUdCntrEvent, &listener);
      }

      // method
      virtual ErrorCode BDAQCALL SnapStart(int32 srcId) { return m_impl.SnapStart(srcId); }
      virtual ErrorCode BDAQCALL SnapStop(int32 srcId)  { return m_impl.SnapStop(srcId);  }

      virtual ErrorCode BDAQCALL CompareSetTable(int32 count, int32 *table) 
      {
         return m_impl.CompareSetTable(m_impl.getChannelStart(), count, table);
      }
      virtual ErrorCode BDAQCALL CompareSetInterval(int32 start, int32 increment,int32 count)
      {
         return m_impl.CompareSetInterval(m_impl.getChannelStart(), start, increment, count);
      }
      virtual ErrorCode BDAQCALL CompareClear() 
      { 
         return m_impl.CompareClear(m_impl.getChannelStart()); 
      }
      virtual ErrorCode BDAQCALL ValueReset()
      {
         return m_impl.ValueReset();
      }

      // property
      virtual UdCounterFeatures* BDAQCALL getFeatures() 
      { 
         return &m_features; 
      }

      virtual int32 BDAQCALL getValue()
      {
         int32 data = 0;
         m_impl.Read(1, &data);
         return data;
      }
      virtual CountingType BDAQCALL getCountingType()
      {
         UdChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->getCountingType() : CountingType(0);
      }
      virtual ErrorCode BDAQCALL setCountingType(CountingType value)
      {
         UdChannelImpl *ch = GetObject(getChannel());
         return ch? ch->setCountingType(value) : ErrorFuncNotInited;
      }
      virtual int32 BDAQCALL getInitialValue()
      {
         UdChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->getInitialValue() : 0;
      }
      virtual ErrorCode BDAQCALL setInitialValue(int32 value)
      {
         UdChannelImpl *ch = GetObject(getChannel());
         return ch? ch->setInitialValue(value) : ErrorFuncNotInited;
      }
      virtual int32 BDAQCALL getResetTimesByIndex()
      {
         UdChannelImpl *ch = GetObject(getChannel());
         return ch ? ch->getResetTimesByIndex() : 0;
      }
      virtual ErrorCode BDAQCALL setResetTimesByIndex(int32 value)
      {
         UdChannelImpl *ch = GetObject(getChannel());
         return ch? ch->setResetTimesByIndex(value) : ErrorFuncNotInited;
      }

      // Helper
      UdChannelImpl * GetObject(int32 index)
      {
         ArrayImpl<UdChannelImpl> *chs = m_impl.getChannels();
         if (chs && unsigned(chs->getLength()) > unsigned(index)) {
            return &chs->Item(index);
         }
         return NULL;
      }

   private:
      static void BDAQCALL OnUdCntrEvent(void *sender, ::UdCntrEventArgs *args, void *userParam)
      {
         UdCntrEventArgs dn3Args;
         dn3Args.SrcId = args->Id;
         dn3Args.Length = args->Length;
         if (dn3Args.Length) { 
            assert(dn3Args.Length < MAX_DIO_PORT_COUNT);
            memcpy(dn3Args.Data, args->Data, dn3Args.Length);
         }

         UdCntrEventListener *handler = static_cast<UdCntrEventListener *>(userParam);
         handler->UdCntrEvent(sender, &dn3Args);
      }

   public:
      UdCounterCtrlCppWrapper() 
      { 
         m_impl.setOuter(this); 
         m_features.m_impl =  (::CntrFeaturesImpl *)m_impl.getFeatures();
      }

      virtual ~UdCounterCtrlCppWrapper() { }

   public:
      ::UdCounterCtrlImpl m_impl;
      UdCounterFeaturesImpl m_features;
   };

   extern "C" {
      // *****************************************************************************************
      // InstantAiCtrl Cpp Wrapper factory method
      // *****************************************************************************************
      InstantAiCtrl* BDAQCALL AdxInstantAiCtrlCreate()
      {
         return new InstantAiCtrlCppWrapper();
      }

      // *****************************************************************************************
      // BufferedAiCtrl Cpp Wrapper factory method
      // *****************************************************************************************
      BufferedAiCtrl* BDAQCALL AdxBufferedAiCtrlCreate()
      {
         return new BufferedAiCtrlCppWrapper();
      }

      // *****************************************************************************************
      //
      // InstantAoCtrl Cpp Wrapper factory method
      //
      // *****************************************************************************************
      InstantAoCtrl* BDAQCALL AdxInstantAoCtrlCreate()
      {
         return new InstantAoCtrlCppWrapper();
      }

      // *****************************************************************************************
      //
      // BufferedAoCtrl Cpp Wrapper factory method
      //
      // *****************************************************************************************
      BufferedAoCtrl* BDAQCALL AdxBufferedAoCtrlCreate()
      {
         return new BufferedAoCtrlCppWrapper();
      }

      // *****************************************************************************************
      // InstantDiCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      InstantDiCtrl* BDAQCALL AdxInstantDiCtrlCreate()
      {
         return new InstantDiCtrlCppWrapper();
      }

      // *****************************************************************************************
      // InstantDoCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      InstantDoCtrl* BDAQCALL AdxInstantDoCtrlCreate()
      {
         return new InstantDoCtrlCppWrapper();
      }

      // *****************************************************************************************
      // EventCounterCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      EventCounterCtrl* BDAQCALL AdxEventCounterCtrlCreate()
      {
         return new EventCounterCtrlCppWrapper();
      }

      // *****************************************************************************************
      // FreqMeterCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      FreqMeterCtrl* BDAQCALL AdxFreqMeterCtrlCreate()
      {
         return new FreqMeterCtrlCppWrapper();
      }

      // *****************************************************************************************
      // OneShotCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      OneShotCtrl* BDAQCALL AdxOneShotCtrlCreate()
      {
         return new OneShotCtrlCppWrapper();
      }

      // *****************************************************************************************
      // PwMeterCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      PwMeterCtrl* BDAQCALL AdxPwMeterCtrlCreate()
      {
         return new PwMeterCtrlCppWrapper();
      }

      // *****************************************************************************************
      // PwModulatorCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      PwModulatorCtrl* BDAQCALL AdxPwModulatorCtrlCreate()
      {
         return new PwModulatorCtrlCppWrapper();
      }

      // *****************************************************************************************
      // TimerPulseCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      TimerPulseCtrl* BDAQCALL AdxTimerPulseCtrlCreate()
      {
         return new TimerPulseCtrlCppWrapper();
      }

      // *****************************************************************************************
      // UdCounterCtrl Cpp Wrapper and its factory method
      // *****************************************************************************************
      UdCounterCtrl* BDAQCALL AdxUdCounterCtrlCreate()
      {
         return new UdCounterCtrlCppWrapper();
      }
   }
}

// *****************************************************************************************
//
// BDAQ CSCL ANSI C Wrapper
//
// *****************************************************************************************
namespace DN3CSCL {
// ----------------------------------------------------------
// common classes : ICollection
// ----------------------------------------------------------
class ICollectionImplStub
{
public:
   virtual void BDAQCALL Dispose() = 0;   // destroy the instance
   virtual int32 BDAQCALL getCount() = 0;
   virtual void * BDAQCALL getItem(int32 index) = 0;
};

extern "C" {
   void BDAQCALL ICollection_Dispose(ICollectionImplStub *obj)              
   {
      obj->Dispose();            
   }
   int32 BDAQCALL ICollection_getCount(ICollectionImplStub *obj)             
   {
      return obj->getCount();    
   }
   void* BDAQCALL ICollection_getItem(ICollectionImplStub *obj, int32 index) 
   {
      return obj->getItem(index);
   }

   // ----------------------------------------------------------
   // common classes : AnalogChannel
   // ----------------------------------------------------------
   int32 BDAQCALL AnalogChannel_getChannel(AnalogChannel *obj)                      
   {
      return obj->getChannel();         
   }
   ValueRange BDAQCALL AnalogChannel_getValueRange(AnalogChannel *obj)                   
   {
      return obj->getValueRange();      
   }
   ErrorCode BDAQCALL AnalogChannel_setValueRange(AnalogChannel *obj, ValueRange value) 
   {
      return obj->setValueRange(value); 
   }

   // ----------------------------------------------------------
   // common classes : AnalogInputChannel
   // ----------------------------------------------------------
   int32 BDAQCALL AnalogInputChannel_getChannel(AnalogInputChannel *obj)                              
   {
      return obj->getChannel();             
   } 
   ValueRange BDAQCALL AnalogInputChannel_getValueRange(AnalogInputChannel *obj)                           
   {
      return obj->getValueRange();          
   }
   ErrorCode BDAQCALL AnalogInputChannel_setValueRange(AnalogInputChannel *obj, ValueRange value)         
   {
      return obj->setValueRange(value);     
   }
   AiSignalType BDAQCALL AnalogInputChannel_getSignalType(AnalogInputChannel *obj)                           
   {
      return obj->getSignalType();          
   }
   ErrorCode BDAQCALL AnalogInputChannel_setSignalType(AnalogInputChannel *obj, AiSignalType value)       
   {
      return obj->setSignalType(value);     
   }
   BurnoutRetType BDAQCALL AnalogInputChannel_getBurnoutRetType(AnalogInputChannel *obj)                       
   {
      return obj->getBurnoutRetType();      
   }
   ErrorCode BDAQCALL AnalogInputChannel_setBurnoutRetType(AnalogInputChannel *obj, BurnoutRetType value) 
   {
      return obj->setBurnoutRetType(value); 
   }
   double BDAQCALL AnalogInputChannel_getBurnoutRetValue(AnalogInputChannel *obj)                      
   {
      return obj->getBurnoutRetValue();     
   }
   ErrorCode BDAQCALL AnalogInputChannel_setBurnoutRetValue(AnalogInputChannel *obj, double value)        
   {
      return obj->setBurnoutRetValue(value);
   }

   // ----------------------------------------------------------
   // common classes : CjcSetting
   // ----------------------------------------------------------
   int32 BDAQCALL CjcSetting_getChannel(CjcSetting *obj)               
   {
      return obj->getChannel();    
   }
   ErrorCode BDAQCALL CjcSetting_setChannel(CjcSetting *obj, int32 ch)     
   {
      return obj->setChannel(ch);  
   }
   double BDAQCALL CjcSetting_getValue(CjcSetting *obj)                 
   {
      return obj->getValue();      
   }
   ErrorCode BDAQCALL CjcSetting_setValue(CjcSetting *obj, double value)   
   {
      return obj->setValue(value); 
   }

   // ----------------------------------------------------------
   // common classes : ScanChannel
   // ----------------------------------------------------------
   int32 BDAQCALL ScanChannel_getChannelStart(ScanChannel *obj)               
   {
      return obj->getChannelStart();      
   }
   ErrorCode BDAQCALL ScanChannel_setChannelStart(ScanChannel *obj, int32 value)  
   {
      return obj->setChannelStart(value); 
   }
   int32 BDAQCALL ScanChannel_getChannelCount(ScanChannel *obj)               
   {
      return obj->getChannelCount();      
   }
   ErrorCode BDAQCALL ScanChannel_setChannelCount(ScanChannel *obj, int32 value)  
   {
      return obj->setChannelCount(value); 
   }
   int32 BDAQCALL ScanChannel_getSamples(ScanChannel *obj)                    
   {
      return obj->getSamples();           
   }
   ErrorCode BDAQCALL ScanChannel_setSamples(ScanChannel *obj, int32 value)       
   {
      return obj->setSamples(value);      
   }
   int32 BDAQCALL ScanChannel_getIntervalCount(ScanChannel *obj)              
   {
      return obj->getIntervalCount();     
   }
   ErrorCode BDAQCALL ScanChannel_setIntervalCount(ScanChannel *obj, int32 value) 
   {
      return obj->setIntervalCount(value);
   }

   // ----------------------------------------------------------
   // common classes : ConvertClock
   // ----------------------------------------------------------
   SignalDrop BDAQCALL ConvertClock_getSource(ConvertClock *obj)                   
   {
      return obj->getSource();      
   }
   ErrorCode BDAQCALL ConvertClock_setSource(ConvertClock *obj, SignalDrop value) 
   {
      return obj->setSource(value); 
   }
   double BDAQCALL ConvertClock_getRate(ConvertClock *obj)                     
   {
      return obj->getRate();        
   }
   ErrorCode BDAQCALL ConvertClock_setRate(ConvertClock *obj, double value)       
   {
      return obj->setRate(value);   
   }

   // ----------------------------------------------------------
   // common classes : ScanClock
   // ----------------------------------------------------------
   SignalDrop BDAQCALL ScanClock_getSource(ScanClock *obj)                   
   {
      return obj->getSource();         
   }
   ErrorCode BDAQCALL ScanClock_setSource(ScanClock *obj, SignalDrop value) 
   {
      return obj->setSource(value);    
   }
   double BDAQCALL ScanClock_getRate(ScanClock *obj)                     
   {
      return obj->getRate();           
   }
   ErrorCode BDAQCALL ScanClock_setRate(ScanClock *obj, double value)       
   {
      return obj->setRate(value);      
   }
   int32 BDAQCALL ScanClock_getScanCount(ScanClock *obj)                
   {
      return obj->getScanCount();      
   }
   ErrorCode BDAQCALL ScanClock_setScanCount(ScanClock *obj, int32 value)   
   {
      return obj->setScanCount(value); 
   }

   // ----------------------------------------------------------
   // common classes : Trigger
   // ----------------------------------------------------------
   SignalDrop BDAQCALL Trigger_getSource(Trigger *obj)                     
   {
      return obj->getSource();         
   }
   ErrorCode BDAQCALL Trigger_setSource(Trigger *obj,SignalDrop value)    
   {
      return obj->setSource(value);    
   }
   ActiveSignal BDAQCALL Trigger_getEdge(Trigger *obj)                       
   {
      return obj->getEdge();           
   }
   ErrorCode BDAQCALL Trigger_setEdge(Trigger *obj, ActiveSignal value)   
   {
      return obj->setEdge(value);      
   }
   double BDAQCALL Trigger_getLevel(Trigger *obj)                      
   {
      return obj->getLevel();          
   }
   ErrorCode BDAQCALL Trigger_setLevel(Trigger *obj, double value)        
   {
      return obj->setLevel(value);     
   }
   TriggerAction BDAQCALL Trigger_getAction(Trigger *obj)                     
   {
      return obj->getAction();         
   }
   ErrorCode BDAQCALL Trigger_setAction(Trigger *obj, TriggerAction value)
   {
      return obj->setAction(value);    
   }
   int32 BDAQCALL Trigger_getDelayCount(Trigger *obj)                 
   {
      return obj->getDelayCount();     
   }
   ErrorCode BDAQCALL Trigger_setDelayCount(Trigger *obj, int32 value)    
   {
      return obj->setDelayCount(value);
   }

   // ----------------------------------------------------------
   // common classes : PortDirection
   // ----------------------------------------------------------
   int32 BDAQCALL PortDirection_getPort(PortDirection *obj)                       
   {
      return obj->getPort();           
   }
   DioPortDir BDAQCALL PortDirection_getDirection(PortDirection *obj)                  
   {
      return obj->getDirection();      
   }
   ErrorCode BDAQCALL PortDirection_setDirection(PortDirection *obj, DioPortDir value)
   {
      return obj->setDirection(value); 
   }

   // ----------------------------------------------------------
   // common classes : NoiseFilterChannel
   // ----------------------------------------------------------
   int32 BDAQCALL NoiseFilterChannel_getChannel(NoiseFilterChannel *obj)            
   {
      return obj->getChannel();      
   }
   bool BDAQCALL NoiseFilterChannel_getEnabled(NoiseFilterChannel *obj)            
   {
      return obj->getEnabled();      
   }
   ErrorCode BDAQCALL NoiseFilterChannel_setEnabled(NoiseFilterChannel *obj, bool value)
   {
      return obj->setEnabled(value); 
   }

   // ----------------------------------------------------------
   // common classes : DiintChannel
   // ----------------------------------------------------------
   int32 BDAQCALL DiintChannel_getChannel(DiintChannel *obj)                     
   {
      return obj->getChannel();      
   }
   bool BDAQCALL DiintChannel_getEnabled(DiintChannel *obj)                     
   {
      return obj->getEnabled();      
   }
   ErrorCode BDAQCALL DiintChannel_setEnabled(DiintChannel *obj, bool value)         
   {
      return obj->setEnabled(value); 
   }
   bool BDAQCALL DiintChannel_getGated(DiintChannel *obj)                       
   {
      return obj->getGated();        
   }
   ErrorCode BDAQCALL DiintChannel_setGated(DiintChannel *obj, bool value)           
   {
      return obj->setGated(value);   
   }
   ActiveSignal BDAQCALL DiintChannel_getTrigEdge(DiintChannel *obj)                    
   {
      return obj->getTrigEdge();     
   }
   ErrorCode BDAQCALL DiintChannel_setTrigEdge(DiintChannel *obj, ActiveSignal value)
   {
      return obj->setTrigEdge(value);
   }

   // ----------------------------------------------------------
   // common classes : DiCosintPort
   // ----------------------------------------------------------
   int32 BDAQCALL DiCosintPort_getPort(DiCosintPort *obj)               
   {
      return obj->getPort();      
   }
   uint8 BDAQCALL DiCosintPort_getMask(DiCosintPort *obj)               
   {
      return obj->getMask();      
   }
   ErrorCode BDAQCALL DiCosintPort_setMask(DiCosintPort *obj, uint8 value)  
   {
      return obj->setMask(value); 
   }

   // ----------------------------------------------------------
   // common classes : DiPmintPort
   // ----------------------------------------------------------
   int32 BDAQCALL DiPmintPort_getPort(DiPmintPort *obj)                 
   {
      return obj->getPort();         
   }
   uint8 BDAQCALL DiPmintPort_getMask(DiPmintPort *obj)                 
   {
      return obj->getMask();         
   }
   ErrorCode BDAQCALL DiPmintPort_setMask(DiPmintPort *obj, uint8 value)    
   {
      return obj->setMask(value);    
   }
   uint8 BDAQCALL DiPmintPort_getPattern(DiPmintPort *obj)              
   {
      return obj->getPattern();      
   }
   ErrorCode BDAQCALL DiPmintPort_setPattern(DiPmintPort *obj, uint8 value) 
   {
      return obj->setPattern(value); 
   }

   // ----------------------------------------------------------
   // common classes : ScanPort
   // ----------------------------------------------------------
   int32 BDAQCALL ScanPort_getPortStart(ScanPort *obj)                  
   {
      return obj->getPortStart();         
   }
   ErrorCode BDAQCALL ScanPort_setPortStart(ScanPort *obj, int32 value)     
   {
      return obj->setPortStart(value);    
   }
   int32 BDAQCALL ScanPort_getPortCount(ScanPort *obj)                  
   {
      return obj->getPortCount();         
   }
   ErrorCode BDAQCALL ScanPort_setPortCount(ScanPort *obj, int32 value)     
   {
      return obj->setPortCount(value);    
   }
   int32 BDAQCALL ScanPort_getSamples(ScanPort *obj)                    
   {
      return obj->getSamples();           
   }
   ErrorCode BDAQCALL ScanPort_setSamples(ScanPort *obj, int32 value)       
   {
      return obj->setSamples(value);      
   }
   int32 BDAQCALL ScanPort_getIntervalCount(ScanPort *obj)              
   {
      return obj->getIntervalCount();     
   }
   ErrorCode BDAQCALL ScanPort_setIntervalCount(ScanPort *obj, int32 value) 
   {
      return obj->setIntervalCount(value);
   }

   // ----------------------------------------------------------
   // ctrl class : AI related
   // ----------------------------------------------------------
   // AI features
   // ADC features
   int32 BDAQCALL AiFeatures_getResolution(AiFeatures *obj)                  
   {
      return obj->getResolution();    
   }
   int32 BDAQCALL AiFeatures_getDataSize(AiFeatures *obj)                    
   {
      return obj->getDataSize();      
   }
   int32 BDAQCALL AiFeatures_getDataMask(AiFeatures *obj)                    
   {
      return obj->getDataMask();      
   }

   // channel features                                                        
   int32 BDAQCALL AiFeatures_getChannelCountMax(AiFeatures *obj)     
   {
      return obj->getChannelCountMax();    
   }
   AiChannelType BDAQCALL AiFeatures_getChannelType(AiFeatures *obj)         
   {
      return obj->getChannelType();        
   }
   int8 BDAQCALL AiFeatures_getOverallValueRange(AiFeatures *obj)   
   {
      return obj->getOverallValueRange();  
   }
   int8 BDAQCALL AiFeatures_getThermoSupported(AiFeatures *obj)     
   {
      return obj->getThermoSupported();    
   }
   ICollection<ValueRange>* BDAQCALL AiFeatures_getValueRanges(AiFeatures *obj) 
   {
      return obj->getValueRanges(); 
   }
   ICollection<BurnoutRetType>* BDAQCALL AiFeatures_getBurnoutReturnTypes(AiFeatures *obj)  
   {
      return obj->getBurnoutReturnTypes(); 
   }

   // CJC features
   ICollection<int32>* BDAQCALL AiFeatures_getCjcChannels(AiFeatures *obj)         
   {
      return obj->getCjcChannels(); 
   }

   // buffered ai->basic features
   int8 BDAQCALL AiFeatures_getBufferedAiSupported(AiFeatures *obj)
   {
      return obj->getBufferedAiSupported(); 
   }
   SamplingMethod BDAQCALL AiFeatures_getSamplingMethod(AiFeatures *obj)     
   {
      return obj->getSamplingMethod();      
   }
   int32 BDAQCALL AiFeatures_getChannelStartBase(AiFeatures *obj)   
   {
      return obj->getChannelStartBase();    
   }
   int32 BDAQCALL AiFeatures_getChannelCountBase(AiFeatures *obj)   
   {
      return obj->getChannelCountBase();    
   }

   // buffered ai->conversion clock features
   ICollection<SignalDrop>* BDAQCALL AiFeatures_getConvertClockSources(AiFeatures *obj)
   {
      return obj->getConvertClockSources(); 
   }
   void BDAQCALL AiFeatures_getConvertClockRange(AiFeatures *obj, MathInterval *value)  
   {
      *value = obj->getConvertClockRange();   
   }

   // buffered ai->burst scan
   int8 BDAQCALL AiFeatures_getBurstScanSupported(AiFeatures *obj) 
   {
      return obj->getBurstScanSupported(); 
   }
   ICollection<SignalDrop>* BDAQCALL AiFeatures_getScanClockSources(AiFeatures *obj)   
   {
      return obj->getScanClockSources();   
   }
   void BDAQCALL AiFeatures_getScanClockRange(AiFeatures *obj, MathInterval *value)     
   {
      *value = obj->getScanClockRange();     
   }
   int32 BDAQCALL AiFeatures_getScanCountMax(AiFeatures *obj)       
   {
      return obj->getScanCountMax();       
   }

   // buffered ai->trigger features
   int8 BDAQCALL AiFeatures_getTriggerSupported(AiFeatures *obj)   
   {
      return obj->getTriggerSupported();  
   }
   int32 BDAQCALL AiFeatures_getTriggerCount(AiFeatures *obj)       
   {
      return obj->getTriggerCount();      
   }
   ICollection<SignalDrop>* BDAQCALL AiFeatures_getTriggerSources(AiFeatures *obj)     
   {
      return obj->getTriggerSources();    
   }
   ICollection<TriggerAction>* BDAQCALL AiFeatures_getTriggerActions(AiFeatures *obj)     
   {
      return obj->getTriggerActions();    
   }
   void BDAQCALL AiFeatures_getTriggerDelayRange(AiFeatures *obj, MathInterval *value)  
   {
      *value = obj->getTriggerDelayRange(); 
   }

   // buffered ai->trigger1 features
   int8 BDAQCALL AiFeatures_getTrigger1Supported(AiFeatures *obj)  
   {
      return obj->getTrigger1Supported(); 
   }
   ICollection<SignalDrop>* BDAQCALL AiFeatures_getTrigger1Sources(AiFeatures *obj)    
   {
      return obj->getTrigger1Sources();   
   }
   ICollection<TriggerAction>* BDAQCALL AiFeatures_getTrigger1Actions(AiFeatures *obj)    
   {
      return obj->getTrigger1Actions();   
   }
   void BDAQCALL AiFeatures_getTrigger1DelayRange(AiFeatures *obj, MathInterval *value) 
   {
      *value = obj->getTrigger1DelayRange();
   }

   // ----------------------------------------------------------
   // InstantAiCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL InstantAiCtrl_Dispose(InstantAiCtrl *obj)                                                       
   {
      obj->Dispose();                              
   }
   void BDAQCALL InstantAiCtrl_Cleanup(InstantAiCtrl *obj)                                                       
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL InstantAiCtrl_UpdateProperties(InstantAiCtrl *obj)                                              
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL InstantAiCtrl_addRemovedListener(InstantAiCtrl *obj, DeviceEventListener * listener)            
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL InstantAiCtrl_removeRemovedListener(InstantAiCtrl *obj, DeviceEventListener * listener)         
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL InstantAiCtrl_addReconnectedListener(InstantAiCtrl *obj, DeviceEventListener * listener)        
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL InstantAiCtrl_removeReconnectedListener(InstantAiCtrl *obj, DeviceEventListener * listener)     
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL InstantAiCtrl_addPropertyChangedListener(InstantAiCtrl *obj, DeviceEventListener * listener)    
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL InstantAiCtrl_removePropertyChangedListener(InstantAiCtrl *obj, DeviceEventListener * listener) 
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL InstantAiCtrl_getSelectedDevice(InstantAiCtrl *obj, DeviceInformation *x)                       
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL InstantAiCtrl_setSelectedDevice(InstantAiCtrl *obj, DeviceInformation const *x)                 
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL InstantAiCtrl_getInitialized(InstantAiCtrl *obj)                                                
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL InstantAiCtrl_getCanEditProperty(InstantAiCtrl *obj)                                            
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL InstantAiCtrl_getDevice(InstantAiCtrl *obj)                                                     
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL InstantAiCtrl_getModule(InstantAiCtrl *obj)                                                     
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL InstantAiCtrl_getSupportedDevices(InstantAiCtrl *obj)                           
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL InstantAiCtrl_getSupportedModes(InstantAiCtrl *obj)                             
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from AiCtrlBase */
   AiFeatures* BDAQCALL InstantAiCtrl_getFeatures(InstantAiCtrl *obj)                                                   
   {
      return obj->getFeatures();                   
   }
   ICollection<AnalogInputChannel>* BDAQCALL InstantAiCtrl_getChannels(InstantAiCtrl *obj)                                                   
   {
      return obj->getChannels();                   
   }
   int32 BDAQCALL InstantAiCtrl_getChannelCount(InstantAiCtrl *obj)                                               
   {
      return obj->getChannelCount();               
   }
   /* InstantAiCtrl methods */
   ErrorCode BDAQCALL InstantAiCtrl_ReadAny(InstantAiCtrl *obj, int32 chStart, int32 chCount, void *dataRaw, double *dataScaled) 
   {
      return obj->ReadAny(chStart, chCount, dataRaw, dataScaled);  
   }
   CjcSetting* BDAQCALL InstantAiCtrl_getCjc(InstantAiCtrl *obj)                                                                   
   {
      return obj->getCjc();             
   }

   // ----------------------------------------------------------
   // BufferedAiCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL BufferedAiCtrl_Dispose(BufferedAiCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL BufferedAiCtrl_Cleanup(BufferedAiCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL BufferedAiCtrl_UpdateProperties(BufferedAiCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL BufferedAiCtrl_addRemovedListener(BufferedAiCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL BufferedAiCtrl_removeRemovedListener(BufferedAiCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL BufferedAiCtrl_addReconnectedListener(BufferedAiCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL BufferedAiCtrl_removeReconnectedListener(BufferedAiCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL BufferedAiCtrl_addPropertyChangedListener(BufferedAiCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL BufferedAiCtrl_removePropertyChangedListener(BufferedAiCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL BufferedAiCtrl_getSelectedDevice(BufferedAiCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL BufferedAiCtrl_setSelectedDevice(BufferedAiCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL BufferedAiCtrl_getInitialized(BufferedAiCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL BufferedAiCtrl_getCanEditProperty(BufferedAiCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL BufferedAiCtrl_getDevice(BufferedAiCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL BufferedAiCtrl_getModule(BufferedAiCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL BufferedAiCtrl_getSupportedDevices(BufferedAiCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL BufferedAiCtrl_getSupportedModes(BufferedAiCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from AiCtrlBase */                                                                                    
   AiFeatures* BDAQCALL BufferedAiCtrl_getFeatures(BufferedAiCtrl *obj)                                                  
   {
      return obj->getFeatures();                   
   }
   ICollection<AnalogInputChannel>* BDAQCALL BufferedAiCtrl_getChannels(BufferedAiCtrl *obj)                                                  
   {
      return obj->getChannels();                   
   }
   int32 BDAQCALL BufferedAiCtrl_getChannelCount(BufferedAiCtrl *obj)                                              
   {
      return obj->getChannelCount();               
   }
   /* BufferedAiCtrl methods */
   // event
   void BDAQCALL BufferedAiCtrl_addDataReadyListener(BufferedAiCtrl *obj, BfdAiEventListener *listener)           
   {
      obj->addDataReadyListener(*listener);         
   }
   void BDAQCALL BufferedAiCtrl_removeDataReadyListener(BufferedAiCtrl *obj, BfdAiEventListener *listener)        
   {
      obj->removeDataReadyListener(*listener);      
   }
   void BDAQCALL BufferedAiCtrl_addOverrunListener(BufferedAiCtrl *obj, BfdAiEventListener *listener)             
   {
      obj->addOverrunListener(*listener);           
   }
   void BDAQCALL BufferedAiCtrl_removeOverrunListener(BufferedAiCtrl *obj, BfdAiEventListener *listener)          
   {
      obj->removeOverrunListener(*listener);        
   }
   void BDAQCALL BufferedAiCtrl_addCacheOverflowListener(BufferedAiCtrl *obj, BfdAiEventListener *listener)       
   {
      obj->addCacheOverflowListener(*listener);     
   }
   void BDAQCALL BufferedAiCtrl_removeCacheOverflowListener(BufferedAiCtrl *obj, BfdAiEventListener *listener)    
   {
      obj->removeCacheOverflowListener(*listener);  
   }
   void BDAQCALL BufferedAiCtrl_addStoppedListener(BufferedAiCtrl *obj, BfdAiEventListener *listener)             
   {
      obj->addStoppedListener(*listener);           
   }
   void BDAQCALL BufferedAiCtrl_removeStoppedListener(BufferedAiCtrl *obj, BfdAiEventListener *listener)          
   {
      obj->removeStoppedListener(*listener);        
   }
   // method
   ErrorCode BDAQCALL BufferedAiCtrl_Prepare(BufferedAiCtrl *obj)                                                      
   {
      return obj->Prepare();                       
   }
   ErrorCode BDAQCALL BufferedAiCtrl_RunOnce(BufferedAiCtrl *obj)                                                      
   {
      return obj->RunOnce();                       
   }
   ErrorCode BDAQCALL BufferedAiCtrl_Start(BufferedAiCtrl *obj)                                                        
   {
      return obj->Start();                         
   }
   ErrorCode BDAQCALL BufferedAiCtrl_Stop(BufferedAiCtrl *obj)                                                         
   {
      return obj->Stop();                          
   }
   ErrorCode BDAQCALL BufferedAiCtrl_GetDataI16(BufferedAiCtrl *obj, int32 count, int16 rawData[])                     
   {
      return obj->GetDataI16(count, rawData);      
   }
   ErrorCode BDAQCALL BufferedAiCtrl_GetDataI32(BufferedAiCtrl *obj, int32 count, int32 rawData[])                     
   {
      return obj->GetDataI32(count, rawData);      
   }
   ErrorCode BDAQCALL BufferedAiCtrl_GetDataF64(BufferedAiCtrl *obj, int32 count, double scaledData[])                 
   {
      return obj->GetDataF64(count, scaledData);   
   }
   void BDAQCALL BufferedAiCtrl_Release(BufferedAiCtrl *obj)                                                      
   {
      obj->Release();                              
   }
   // property
   void* BDAQCALL BufferedAiCtrl_getBuffer(BufferedAiCtrl *obj)                                                   
   {
      return obj->getBuffer();                     
   }
   int32 BDAQCALL BufferedAiCtrl_getBufferCapacity(BufferedAiCtrl *obj)                                           
   {
      return obj->getBufferCapacity();             
   }
   ControlState BDAQCALL BufferedAiCtrl_getState(BufferedAiCtrl *obj)                                                    
   {
      return obj->getState();                      
   }
   ScanChannel* BDAQCALL BufferedAiCtrl_getScanChannel(BufferedAiCtrl *obj)                                              
   {
      return obj->getScanChannel();                
   }
   ConvertClock* BDAQCALL BufferedAiCtrl_getConvertClock(BufferedAiCtrl *obj)                                             
   {
      return obj->getConvertClock();               
   }
   ScanClock* BDAQCALL BufferedAiCtrl_getScanClock(BufferedAiCtrl *obj)                                                
   {
      return obj->getScanClock();                  
   }
   Trigger* BDAQCALL BufferedAiCtrl_getTrigger(BufferedAiCtrl *obj)                                                  
   {
      return obj->getTrigger();                    
   }
   bool BDAQCALL BufferedAiCtrl_getStreaming(BufferedAiCtrl *obj)                                                
   {
      return obj->getStreaming();                  
   }
   ErrorCode BDAQCALL BufferedAiCtrl_setStreaming(BufferedAiCtrl *obj, bool value)                                    
   {
      return obj->setStreaming(value);             
   }
   // method
   ErrorCode BDAQCALL BufferedAiCtrl_GetEventStatus(BufferedAiCtrl *obj, EventId id, int32 *status)                   
   {
      return obj->GetEventStatus(id, *status);      
   }
   // property
   Trigger* BDAQCALL BufferedAiCtrl_getTrigger1(BufferedAiCtrl *obj)                                                 
   {
      return obj->getTrigger1();                   
   }

   // ----------------------------------------------------------
   // ctrl class : AO related
   // ----------------------------------------------------------
   // AO features
   // DAC features                                                               
   int32 BDAQCALL AoFeatures_getResolution(AoFeatures *obj)           
   {
      return obj->getResolution();           
   }
   int32 BDAQCALL AoFeatures_getDataSize(AoFeatures *obj)             
   {
      return obj->getDataSize();             
   }
   int32 BDAQCALL AoFeatures_getDataMask(AoFeatures *obj)             
   {
      return obj->getDataMask();             
   }

   // channel features                                                           
   int32 BDAQCALL AoFeatures_getChannelCountMax(AoFeatures *obj)      
   {
      return obj->getChannelCountMax();      
   }
   ICollection<ValueRange>* BDAQCALL AoFeatures_getValueRanges(AoFeatures *obj)          
   {
      return obj->getValueRanges();          
   }
   int8 BDAQCALL AoFeatures_getExternalRefAntiPolar(AoFeatures *obj) 
   {
      return obj->getExternalRefAntiPolar(); 
   }
   void BDAQCALL AoFeatures_getExternalRefRange(AoFeatures *obj, MathInterval *value)     
   {
      *value = obj->getExternalRefRange();     
   }

   // buffered ao->basic features                                                
   int8 BDAQCALL AoFeatures_getBufferedAoSupported(AoFeatures *obj)  
   {
      return obj->getBufferedAoSupported();  
   }
   SamplingMethod BDAQCALL AoFeatures_getSamplingMethod(AoFeatures *obj)       
   {
      return obj->getSamplingMethod();       
   }
   int32 BDAQCALL AoFeatures_getChannelStartBase(AoFeatures *obj)     
   {
      return obj->getChannelStartBase();     
   }
   int32 BDAQCALL AoFeatures_getChannelCountBase(AoFeatures *obj)     
   {
      return obj->getChannelCountBase();     
   }

   // buffered ao->conversion clock features                                     
   ICollection<SignalDrop>* BDAQCALL AoFeatures_getConvertClockSources(AoFeatures *obj)  
   {
      return obj->getConvertClockSources();  
   }
   void BDAQCALL AoFeatures_getConvertClockRange(AoFeatures *obj, MathInterval *value)    
   {
      *value = obj->getConvertClockRange();    
   }

   // buffered ao->trigger features                                              
   int8 BDAQCALL AoFeatures_getTriggerSupported(AoFeatures *obj)     
   {
      return obj->getTriggerSupported();     
   }
   int32 BDAQCALL AoFeatures_getTriggerCount(AoFeatures *obj)         
   {
      return obj->getTriggerCount();         
   }
   ICollection<SignalDrop>* BDAQCALL AoFeatures_getTriggerSources(AoFeatures *obj)       
   {
      return obj->getTriggerSources();       
   }
   ICollection<TriggerAction>* BDAQCALL AoFeatures_getTriggerActions(AoFeatures *obj)       
   {
      return obj->getTriggerActions();       
   }
   void BDAQCALL AoFeatures_getTriggerDelayRange(AoFeatures *obj, MathInterval *value)    
   {
      *value = obj->getTriggerDelayRange();    
   }

   // buffered ao->trigger1 features                                             
   int8 BDAQCALL AoFeatures_getTrigger1Supported(AoFeatures *obj)    
   {
      return obj->getTrigger1Supported();    
   }
   ICollection<SignalDrop>* BDAQCALL AoFeatures_getTrigger1Sources(AoFeatures *obj)      
   {
      return obj->getTrigger1Sources();      
   }
   ICollection<TriggerAction>* BDAQCALL AoFeatures_getTrigger1Actions(AoFeatures *obj)      
   {
      return obj->getTrigger1Actions();      
   }
   MathInterval BDAQCALL AoFeatures_getTrigger1DelayRange(AoFeatures *obj)   
   {
      return obj->getTrigger1DelayRange();   
   }

   // ----------------------------------------------------------
   // InstantAoCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL InstantAoCtrl_Dispose(InstantAoCtrl *obj)                                                       
   {
      obj->Dispose();                              
   }
   void BDAQCALL InstantAoCtrl_Cleanup(InstantAoCtrl *obj)                                                       
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL InstantAoCtrl_UpdateProperties(InstantAoCtrl *obj)                                              
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL InstantAoCtrl_addRemovedListener(InstantAoCtrl *obj, DeviceEventListener * listener)            
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL InstantAoCtrl_removeRemovedListener(InstantAoCtrl *obj, DeviceEventListener * listener)         
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL InstantAoCtrl_addReconnectedListener(InstantAoCtrl *obj, DeviceEventListener * listener)        
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL InstantAoCtrl_removeReconnectedListener(InstantAoCtrl *obj, DeviceEventListener * listener)     
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL InstantAoCtrl_addPropertyChangedListener(InstantAoCtrl *obj, DeviceEventListener * listener)    
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL InstantAoCtrl_removePropertyChangedListener(InstantAoCtrl *obj, DeviceEventListener * listener) 
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL InstantAoCtrl_getSelectedDevice(InstantAoCtrl *obj, DeviceInformation *x)                       
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL InstantAoCtrl_setSelectedDevice(InstantAoCtrl *obj, DeviceInformation const *x)                 
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL InstantAoCtrl_getInitialized(InstantAoCtrl *obj)                                                
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL InstantAoCtrl_getCanEditProperty(InstantAoCtrl *obj)                                            
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL InstantAoCtrl_getDevice(InstantAoCtrl *obj)                                                     
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL InstantAoCtrl_getModule(InstantAoCtrl *obj)                                                     
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL InstantAoCtrl_getSupportedDevices(InstantAoCtrl *obj)                                           
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL InstantAoCtrl_getSupportedModes(InstantAoCtrl *obj)                                             
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from AiCtrlBase */
   AoFeatures* BDAQCALL InstantAoCtrl_getFeatures(InstantAoCtrl *obj)                                                   
   {
      return obj->getFeatures();                   
   }
   ICollection<AnalogChannel>* BDAQCALL InstantAoCtrl_getChannels(InstantAoCtrl *obj)                                                   
   {
      return obj->getChannels();                   
   }
   int32 BDAQCALL InstantAoCtrl_getChannelCount(InstantAoCtrl *obj)                                               
   {
      return obj->getChannelCount();               
   }
   double BDAQCALL InstantAoCtrl_getExtRefValueForUnipolar(InstantAoCtrl *obj)                                     
   {
      return obj->getExtRefValueForUnipolar();     
   }
   ErrorCode BDAQCALL InstantAoCtrl_setExtRefValueForUnipolar(InstantAoCtrl *obj, double value)                       
   {
      return obj->setExtRefValueForUnipolar(value);
   }
   double BDAQCALL InstantAoCtrl_getExtRefValueForBipolar(InstantAoCtrl *obj)                                      
   {
      return obj->getExtRefValueForBipolar();      
   }
   ErrorCode BDAQCALL InstantAoCtrl_setExtRefValueForBipolar(InstantAoCtrl *obj, double value)                        
   {
      return obj->setExtRefValueForBipolar(value); 
   }
   /* InstantAoCtrl methods */
   ErrorCode BDAQCALL InstantAoCtrl_WriteAny(InstantAoCtrl *obj, int32 chStart, int32 chCount, void *dataRaw, double *dataScaled) 
   {
      return obj->WriteAny(chStart, chCount, dataRaw, dataScaled);  
   }

   // ----------------------------------------------------------
   // BufferedAoCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL BufferedAoCtrl_Dispose(BufferedAoCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL BufferedAoCtrl_Cleanup(BufferedAoCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL BufferedAoCtrl_UpdateProperties(BufferedAoCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL BufferedAoCtrl_addRemovedListener(BufferedAoCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL BufferedAoCtrl_removeRemovedListener(BufferedAoCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL BufferedAoCtrl_addReconnectedListener(BufferedAoCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL BufferedAoCtrl_removeReconnectedListener(BufferedAoCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL BufferedAoCtrl_addPropertyChangedListener(BufferedAoCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL BufferedAoCtrl_removePropertyChangedListener(BufferedAoCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL BufferedAoCtrl_getSelectedDevice(BufferedAoCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL BufferedAoCtrl_setSelectedDevice(BufferedAoCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL BufferedAoCtrl_getInitialized(BufferedAoCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL BufferedAoCtrl_getCanEditProperty(BufferedAoCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL BufferedAoCtrl_getDevice(BufferedAoCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL BufferedAoCtrl_getModule(BufferedAoCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL BufferedAoCtrl_getSupportedDevices(BufferedAoCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL BufferedAoCtrl_getSupportedModes(BufferedAoCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from AiCtrlBase */                                                                                    
   AoFeatures* BDAQCALL BufferedAoCtrl_getFeatures(BufferedAoCtrl *obj)                                                  
   {
      return obj->getFeatures();                   
   }
   ICollection<AnalogChannel>* BDAQCALL BufferedAoCtrl_getChannels(BufferedAoCtrl *obj)                                                  
   {
      return obj->getChannels();                   
   }
   int32 BDAQCALL BufferedAoCtrl_getChannelCount(BufferedAoCtrl *obj)                                              
   {
      return obj->getChannelCount();               
   }
   double BDAQCALL BufferedAoCtrl_getExtRefValueForUnipolar(InstantAoCtrl *obj)                                      
   {
      return obj->getExtRefValueForUnipolar();     
   }
   ErrorCode BDAQCALL BufferedAoCtrl_setExtRefValueForUnipolar(InstantAoCtrl *obj, double value)                        
   {
      return obj->setExtRefValueForUnipolar(value);
   }
   double BDAQCALL BufferedAoCtrl_getExtRefValueForBipolar(InstantAoCtrl *obj)                                       
   {
      return obj->getExtRefValueForBipolar();      
   }
   ErrorCode BDAQCALL BufferedAoCtrl_setExtRefValueForBipolar(InstantAoCtrl *obj, double value)                         
   {
      return obj->setExtRefValueForBipolar(value); 
   }
   /* BufferedAoCtrl methods */
   // event
   void BDAQCALL BufferedAoCtrl_addDataTransmittedListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)     
   {
      obj->addDataTransmittedListener(*listener);   
   }
   void BDAQCALL BufferedAoCtrl_removeDataTransmittedListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)  
   {
      obj->removeDataTransmittedListener(*listener);
   }
   void BDAQCALL BufferedAoCtrl_addUnderrunListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)            
   {
      obj->addUnderrunListener(*listener);          
   }
   void BDAQCALL BufferedAoCtrl_removeUnderrunListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)         
   {
      obj->removeUnderrunListener(*listener);       
   }
   void BDAQCALL BufferedAoCtrl_addCacheEmptiedListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)        
   {
      obj->addCacheEmptiedListener(*listener);      
   }
   void BDAQCALL BufferedAoCtrl_removeCacheEmptiedListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)     
   {
      obj->removeCacheEmptiedListener(*listener);   
   }
   void BDAQCALL BufferedAoCtrl_addTransitStoppedListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)      
   {
      obj->addTransitStoppedListener(*listener);    
   }
   void BDAQCALL BufferedAoCtrl_removeTransitStoppedListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)   
   {
      obj->removeTransitStoppedListener(*listener); 
   }
   void BDAQCALL BufferedAoCtrl_addStoppedListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)             
   {
      obj->addStoppedListener(*listener);           
   }
   void BDAQCALL BufferedAoCtrl_removeStoppedListener(BufferedAoCtrl *obj, BfdAoEventListener *listener)          
   {
      obj->removeStoppedListener(*listener);        
   }
   // method
   ErrorCode BDAQCALL BufferedAoCtrl_Prepare(BufferedAoCtrl *obj)                                                      
   {
      return obj->Prepare();                       
   }
   ErrorCode BDAQCALL BufferedAoCtrl_RunOnce(BufferedAoCtrl *obj)                                                      
   {
      return obj->RunOnce();                       
   }
   ErrorCode BDAQCALL BufferedAoCtrl_Start(BufferedAoCtrl *obj)                                                        
   {
      return obj->Start();                         
   }
   ErrorCode BDAQCALL BufferedAoCtrl_Stop(BufferedAoCtrl *obj, int32 action)                                           
   {
      return obj->Stop(action);                    
   }
   ErrorCode BDAQCALL BufferedAoCtrl_SetDataI16(BufferedAoCtrl *obj, int32 count, int16 rawData[])                     
   {
      return obj->SetDataI16(count, rawData);      
   }
   ErrorCode BDAQCALL BufferedAoCtrl_SetDataI32(BufferedAoCtrl *obj, int32 count, int32 rawData[])                     
   {
      return obj->SetDataI32(count, rawData);      
   }
   ErrorCode BDAQCALL BufferedAoCtrl_SetDataF64(BufferedAoCtrl *obj, int32 count, double scaledData[])                 
   {
      return obj->SetDataF64(count, scaledData);   
   }
   void BDAQCALL BufferedAoCtrl_Release(BufferedAoCtrl *obj)                                                      
   {
      obj->Release();                              
   }
   // property
   void* BDAQCALL BufferedAoCtrl_getBuffer(BufferedAoCtrl *obj)                                                   
   {
      return obj->getBuffer();                     
   }
   int32 BDAQCALL BufferedAoCtrl_getBufferCapacity(BufferedAoCtrl *obj)                                           
   {
      return obj->getBufferCapacity();             
   }
   ControlState BDAQCALL BufferedAoCtrl_getState(BufferedAoCtrl *obj)                                                    
   {
      return obj->getState();                      
   }
   ScanChannel* BDAQCALL BufferedAoCtrl_getScanChannel(BufferedAoCtrl *obj)                                              
   {
      return obj->getScanChannel();                
   }
   ConvertClock* BDAQCALL BufferedAoCtrl_getConvertClock(BufferedAoCtrl *obj)                                             
   {
      return obj->getConvertClock();               
   }
   Trigger* BDAQCALL BufferedAoCtrl_getTrigger(BufferedAoCtrl *obj)                                                  
   {
      return obj->getTrigger();                    
   }
   bool BDAQCALL BufferedAoCtrl_getStreaming(BufferedAoCtrl *obj)                                                
   {
      return obj->getStreaming();                  
   }
   ErrorCode BDAQCALL BufferedAoCtrl_setStreaming(BufferedAoCtrl *obj, bool value)                                    
   {
      return obj->setStreaming(value);             
   }
   Trigger* BDAQCALL BufferedAoCtrl_getTrigger1(BufferedAoCtrl *obj)                                                 
   {
      return obj->getTrigger1();                   
   }

   // ----------------------------------------------------------
   // ctrl class : DIO related classes
   // ----------------------------------------------------------
   // DI features
   int8 BDAQCALL DiFeatures_getPortProgrammable(DiFeatures *obj)              
   {
      return obj->getPortProgrammable();          
   }
   int32 BDAQCALL DiFeatures_getPortCount(DiFeatures *obj)                     
   {
      return obj->getPortCount();                 
   }
   ICollection<uint8>* BDAQCALL DiFeatures_getPortsType(DiFeatures *obj)                     
   {
      return obj->getPortsType();                 
   }
   int8 BDAQCALL DiFeatures_getDiSupported(DiFeatures *obj)                   
   {
      return obj->getDiSupported();               
   }
   int8 BDAQCALL DiFeatures_getDoSupported(DiFeatures *obj)                   
   {
      return obj->getDoSupported();               
   }
   int32 BDAQCALL DiFeatures_getChannelCountMax(DiFeatures *obj)               
   {
      return obj->getChannelCountMax();           
   }

   ICollection<uint8>* BDAQCALL DiFeatures_getDataMask(DiFeatures *obj)                      
   {
      return obj->getDataMask();                  
   }

   // di noise filter features                                                            
   int8 BDAQCALL DiFeatures_getNoiseFilterSupported(DiFeatures *obj)          
   {
      return obj->getNoiseFilterSupported();      
   }
   ICollection<uint8>* BDAQCALL DiFeatures_getNoiseFilterOfChannels(DiFeatures *obj)         
   {
      return obj->getNoiseFilterOfChannels();     
   }
   void BDAQCALL DiFeatures_getNoiseFilterBlockTimeRange(DiFeatures *obj, MathInterval *value)     
   {
      *value = obj->getNoiseFilterBlockTimeRange(); 
   }

   // di interrupt features                                                               
   int8 BDAQCALL DiFeatures_getDiintSupported(DiFeatures *obj)                
   {
      return obj->getDiintSupported();            
   }
   int8 BDAQCALL DiFeatures_getDiintGateSupported(DiFeatures *obj)            
   {
      return obj->getDiintGateSupported();        
   }
   int8 BDAQCALL DiFeatures_getDiCosintSupported(DiFeatures *obj)             
   {
      return obj->getDiCosintSupported();         
   }
   int8 BDAQCALL DiFeatures_getDiPmintSupported(DiFeatures *obj)              
   {
      return obj->getDiPmintSupported();          
   }
   ICollection<ActiveSignal>* BDAQCALL DiFeatures_getDiintTriggerEdges(DiFeatures *obj)             
   {
      return obj->getDiintTriggerEdges();         
   }
   ICollection<uint8>* BDAQCALL DiFeatures_getDiintOfChannels(DiFeatures *obj)               
   {
      return obj->getDiintOfChannels();           
   }
   ICollection<uint8>* BDAQCALL DiFeatures_getDiintGateOfChannels(DiFeatures *obj)           
   {
      return obj->getDiintGateOfChannels();       
   }
   ICollection<uint8>* BDAQCALL DiFeatures_getDiCosintOfPorts(DiFeatures *obj)               
   {
      return obj->getDiCosintOfPorts();           
   }
   ICollection<uint8>* BDAQCALL DiFeatures_getDiPmintOfPorts(DiFeatures *obj)                
   {
      return obj->getDiPmintOfPorts();            
   }
   ICollection<int32>* BDAQCALL DiFeatures_getSnapEventSources(DiFeatures *obj)              
   {
      return obj->getSnapEventSources();          
   }

   // buffered di->basic features                                                         
   int8 BDAQCALL DiFeatures_getBufferedDiSupported(DiFeatures *obj)           
   {
      return obj->getBufferedDiSupported();       
   }
   SamplingMethod BDAQCALL DiFeatures_getSamplingMethod(DiFeatures *obj)                
   {
      return obj->getSamplingMethod();            
   }

   // buffered di->conversion clock features                                              
   ICollection<SignalDrop>* BDAQCALL DiFeatures_getConvertClockSources(DiFeatures *obj)           
   {
      return obj->getConvertClockSources();       
   }
   void BDAQCALL DiFeatures_getConvertClockRange(DiFeatures *obj, MathInterval *value)             
   {
      *value = obj->getConvertClockRange();         
   }

   // buffered di->burst scan                                                             
   int8 BDAQCALL DiFeatures_getBurstScanSupported(DiFeatures *obj)            
   {
      return obj->getBurstScanSupported();        
   }
   ICollection<SignalDrop>* BDAQCALL DiFeatures_getScanClockSources(DiFeatures *obj)              
   {
      return obj->getScanClockSources();          
   }
   void BDAQCALL DiFeatures_getScanClockRange(DiFeatures *obj, MathInterval *value)                
   {
      *value = obj->getScanClockRange();            
   }
   int32 BDAQCALL DiFeatures_getScanCountMax(DiFeatures *obj)                  
   {
      return obj->getScanCountMax();              
   }

   // buffered di->trigger features                                                       
   int8 BDAQCALL DiFeatures_getTriggerSupported(DiFeatures *obj)              
   {
      return obj->getTriggerSupported();          
   }
   int32 BDAQCALL DiFeatures_getTriggerCount(DiFeatures *obj)                  
   {
      return obj->getTriggerCount();              
   }
   ICollection<SignalDrop>* BDAQCALL DiFeatures_getTriggerSources(DiFeatures *obj)                
   {
      return obj->getTriggerSources();            
   }
   ICollection<TriggerAction>* BDAQCALL DiFeatures_getTriggerActions(DiFeatures *obj)                
   {
      return obj->getTriggerActions();            
   }
   void BDAQCALL DiFeatures_getTriggerDelayRange(DiFeatures *obj, MathInterval *value)             
   {
      *value = obj->getTriggerDelayRange();         
   }

   // ----------------------------------------------------------
   // InstantDiCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL InstantDiCtrl_Dispose(InstantDiCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL InstantDiCtrl_Cleanup(InstantDiCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL InstantDiCtrl_UpdateProperties(InstantDiCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL InstantDiCtrl_addRemovedListener(InstantDiCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL InstantDiCtrl_removeRemovedListener(InstantDiCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL InstantDiCtrl_addReconnectedListener(InstantDiCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL InstantDiCtrl_removeReconnectedListener(InstantDiCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL InstantDiCtrl_addPropertyChangedListener(InstantDiCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL InstantDiCtrl_removePropertyChangedListener(InstantDiCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL InstantDiCtrl_getSelectedDevice(InstantDiCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL InstantDiCtrl_setSelectedDevice(InstantDiCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL InstantDiCtrl_getInitialized(InstantDiCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL InstantDiCtrl_getCanEditProperty(InstantDiCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL InstantDiCtrl_getDevice(InstantDiCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL InstantDiCtrl_getModule(InstantDiCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL InstantDiCtrl_getSupportedDevices(InstantDiCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL InstantDiCtrl_getSupportedModes(InstantDiCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from DioCtrlBase */
   int32 BDAQCALL InstantDiCtrl_getPortCount(InstantDiCtrl *obj)                                                 
   {
      return obj->getPortCount();                  
   }
   ICollection<PortDirection>* BDAQCALL InstantDiCtrl_getPortDirection(InstantDiCtrl *obj)                                             
   {
      return obj->getPortDirection();              
   }
   /* Methods derived from DiCtrlBase */ 
   DiFeatures* BDAQCALL InstantDiCtrl_getFeatures(InstantDiCtrl *obj)                                                  
   {
      return obj->getFeatures();                   
   }
   ICollection<NoiseFilterChannel>* BDAQCALL InstantDiCtrl_getNoiseFilter(InstantDiCtrl *obj)                                               
   {
      return obj->getNoiseFilter();                
   }
   /* Instant DI methods */
   // event
   void BDAQCALL InstantDiCtrl_addInterruptListener(InstantDiCtrl *obj, DiSnapEventListener * listener)         
   {
      obj->addInterruptListener(*listener);         
   }
   void BDAQCALL InstantDiCtrl_removeInterruptListener(InstantDiCtrl *obj, DiSnapEventListener * listener)      
   {
      obj->removeInterruptListener(*listener);      
   }
   void BDAQCALL InstantDiCtrl_addChangeOfStateListener(InstantDiCtrl *obj, DiSnapEventListener * listener)     
   {
      obj->addChangeOfStateListener(*listener);     
   }
   void BDAQCALL InstantDiCtrl_removeChangeOfStateListener(InstantDiCtrl *obj, DiSnapEventListener * listener)  
   {
      obj->removeChangeOfStateListener(*listener);  
   }
   void BDAQCALL InstantDiCtrl_addPatternMatchListener(InstantDiCtrl *obj, DiSnapEventListener * listener)      
   {
      obj->addPatternMatchListener(*listener);      
   }
   void BDAQCALL InstantDiCtrl_removePatternMatchListener(InstantDiCtrl *obj, DiSnapEventListener * listener)   
   {
      obj->removePatternMatchListener(*listener);   
   }
   // method                                                                                                              // method
   ErrorCode BDAQCALL InstantDiCtrl_ReadAny(InstantDiCtrl *obj, int32 portStart, int32 portCount, uint8 data[])      
   {
      return obj->ReadAny(portStart, portCount, data); 
   }
   ErrorCode BDAQCALL InstantDiCtrl_ReadBit(InstantDiCtrl *obj, int32 port, int32 bit, uint8* data)      
   {
      return obj->ReadBit(port, bit, data); 
   }
   ErrorCode BDAQCALL InstantDiCtrl_SnapStart(InstantDiCtrl *obj)                                                    
   {
      return obj->SnapStart();                     
   }
   ErrorCode BDAQCALL InstantDiCtrl_SnapStop(InstantDiCtrl *obj)                                                     
   {
      return obj->SnapStop();                      
   }
   // property                                                                                                            // property
   ICollection<DiintChannel>* BDAQCALL InstantDiCtrl_getDiintChannels(InstantDiCtrl *obj)                                             
   {
      return obj->getDiintChannels();              
   }
   ICollection<DiCosintPort>* BDAQCALL InstantDiCtrl_getDiCosintPorts(InstantDiCtrl *obj)                                             
   {
      return obj->getDiCosintPorts();              
   }
   ICollection<DiPmintPort>* BDAQCALL InstantDiCtrl_getDiPmintPorts(InstantDiCtrl *obj)                                              
   {
      return obj->getDiPmintPorts();               
   }

   // ----------------------------------------------------------
   // BufferedDiCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL BufferedDiCtrl_Dispose(BufferedDiCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL BufferedDiCtrl_Cleanup(BufferedDiCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL BufferedDiCtrl_UpdateProperties(BufferedDiCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL BufferedDiCtrl_addRemovedListener(BufferedDiCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL BufferedDiCtrl_removeRemovedListener(BufferedDiCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL BufferedDiCtrl_addReconnectedListener(BufferedDiCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL BufferedDiCtrl_removeReconnectedListener(BufferedDiCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL BufferedDiCtrl_addPropertyChangedListener(BufferedDiCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL BufferedDiCtrl_removePropertyChangedListener(BufferedDiCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL BufferedDiCtrl_getSelectedDevice(BufferedDiCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL BufferedDiCtrl_setSelectedDevice(BufferedDiCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL BufferedDiCtrl_getInitialized(BufferedDiCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL BufferedDiCtrl_getCanEditProperty(BufferedDiCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL BufferedDiCtrl_getDevice(BufferedDiCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL BufferedDiCtrl_getModule(BufferedDiCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL BufferedDiCtrl_getSupportedDevices(BufferedDiCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL BufferedDiCtrl_getSupportedModes(BufferedDiCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from DioCtrlBase */
   int32 BDAQCALL BufferedDiCtrl_getPortCount(BufferedDiCtrl *obj)                                                 
   {
      return obj->getPortCount();                  
   }
   ICollection<PortDirection>* BDAQCALL BufferedDiCtrl_getPortDirection(BufferedDiCtrl *obj)                                             
   {
      return obj->getPortDirection();              
   }
   /* Methods derived from DiCtrlBase */ 
   DiFeatures* BDAQCALL BufferedDiCtrl_getFeatures(BufferedDiCtrl *obj)                                                  
   {
      return obj->getFeatures();                   
   }
   ICollection<NoiseFilterChannel>* BDAQCALL BufferedDiCtrl_getNoiseFilter(BufferedDiCtrl *obj)                                               
   {
      return obj->getNoiseFilter();                
   }
   /* Buffered DI methods */
   // event
   void BDAQCALL BufferedDiCtrl_addDataReadyListener(BufferedDiCtrl *obj, BfdDiEventListener *listener)           
   {
      obj->addDataReadyListener(*listener);         
   }
   void BDAQCALL BufferedDiCtrl_removeDataReadyListener(BufferedDiCtrl *obj, BfdDiEventListener *listener)        
   {
      obj->removeDataReadyListener(*listener);      
   }
   void BDAQCALL BufferedDiCtrl_addOverrunListener(BufferedDiCtrl *obj, BfdDiEventListener *listener)             
   {
      obj->addOverrunListener(*listener);           
   }
   void BDAQCALL BufferedDiCtrl_removeOverrunListener(BufferedDiCtrl *obj, BfdDiEventListener *listener)          
   {
      obj->removeOverrunListener(*listener);        
   }
   void BDAQCALL BufferedDiCtrl_addCacheOverflowListener(BufferedDiCtrl *obj, BfdDiEventListener *listener)       
   {
      obj->addCacheOverflowListener(*listener);     
   }
   void BDAQCALL BufferedDiCtrl_removeCacheOverflowListener(BufferedDiCtrl *obj, BfdDiEventListener *listener)    
   {
      obj->removeCacheOverflowListener(*listener);  
   }
   void BDAQCALL BufferedDiCtrl_addStoppedListener(BufferedDiCtrl *obj, BfdDiEventListener *listener)             
   {
      obj->addStoppedListener(*listener);           
   }
   void BDAQCALL BufferedDiCtrl_removeStoppedListener(BufferedDiCtrl *obj, BfdDiEventListener *listener)          
   {
      obj->removeStoppedListener(*listener);        
   }
   // method
   ErrorCode BDAQCALL BufferedDiCtrl_Prepare(BufferedDiCtrl *obj)                                                      
   {
      return obj->Prepare();                       
   }
   ErrorCode BDAQCALL BufferedDiCtrl_RunOnce(BufferedDiCtrl *obj)                                                      
   {
      return obj->RunOnce();                       
   }
   ErrorCode BDAQCALL BufferedDiCtrl_Start(BufferedDiCtrl *obj)                                                        
   {
      return obj->Start();                         
   }
   ErrorCode BDAQCALL BufferedDiCtrl_Stop(BufferedDiCtrl *obj)                                                         
   {
      return obj->Stop();                          
   }
   ErrorCode BDAQCALL BufferedDiCtrl_GetData(BufferedDiCtrl *obj, int32 count, uint8 data[])                           
   {
      return obj->GetData(count, data);            
   }
   void BDAQCALL BufferedDiCtrl_Release(BufferedDiCtrl *obj)                                                      
   {
      obj->Release();                              
   }
   // property
   void* BDAQCALL BufferedDiCtrl_getBuffer(BufferedDiCtrl *obj)                                                   
   {
      return obj->getBuffer();                     
   }
   int32 BDAQCALL BufferedDiCtrl_getBufferCapacity(BufferedDiCtrl *obj)                                           
   {
      return obj->getBufferCapacity();             
   }
   ControlState BDAQCALL BufferedDiCtrl_getState(BufferedDiCtrl *obj)                                                    
   {
      return obj->getState();                      
   }
   ScanPort* BDAQCALL BufferedDiCtrl_getScanPort(BufferedDiCtrl *obj)                                                 
   {
      return obj->getScanPort();                   
   }
   ConvertClock* BDAQCALL BufferedDiCtrl_getConvertClock(BufferedDiCtrl *obj)                                             
   {
      return obj->getConvertClock();               
   }
   ScanClock* BDAQCALL BufferedDiCtrl_getScanClock(BufferedDiCtrl *obj)                                                
   {
      return obj->getScanClock();                  
   }
   Trigger* BDAQCALL BufferedDiCtrl_getTrigger(BufferedDiCtrl *obj)                                                  
   {
      return obj->getTrigger();                    
   }
   bool BDAQCALL BufferedDiCtrl_getStreaming(BufferedDiCtrl *obj)                                                
   {
      return obj->getStreaming();                  
   }
   ErrorCode BDAQCALL BufferedDiCtrl_setStreaming(BufferedDiCtrl *obj, bool value)                                    
   {
      return obj->setStreaming(value);             
   }

   // ----------------------------------------------------------
   // DO features
   int8 BDAQCALL DoFeatures_getPortProgrammable(DoFeatures *obj)              
   {
      return obj->getPortProgrammable();             
   }
   int32 BDAQCALL DoFeatures_getPortCount(DoFeatures *obj)                     
   {
      return obj->getPortCount();                    
   }
   ICollection<uint8>* BDAQCALL DoFeatures_getPortsType(DoFeatures *obj)                     
   {
      return obj->getPortsType();                    
   }
   int8 BDAQCALL DoFeatures_getDiSupported(DoFeatures *obj)                   
   {
      return obj->getDiSupported();                  
   }
   int8 BDAQCALL DoFeatures_getDoSupported(DoFeatures *obj)                   
   {
      return obj->getDoSupported();                  
   }
   int32 BDAQCALL DoFeatures_getChannelCountMax(DoFeatures *obj)               
   {
      return obj->getChannelCountMax();              
   }

   ICollection<uint8>* BDAQCALL DoFeatures_getDataMask(DoFeatures *obj)                      
   {
      return obj->getDataMask();                     
   }
   // do freeze features                                                                  
   ICollection<SignalDrop>* BDAQCALL DoFeatures_getDoFreezeSignalSources(DoFeatures *obj)         
   {
      return obj->getDoFreezeSignalSources();        
   }

   // do reflect Wdt features                                                             
   void BDAQCALL DoFeatures_getDoReflectWdtFeedIntervalRange(DoFeatures *obj, MathInterval *value) 
   {
      *value = obj->getDoReflectWdtFeedIntervalRange();
   }

   // buffered do->basic features                                                         
   int8 BDAQCALL DoFeatures_getBufferedDoSupported(DoFeatures *obj)           
   {
      return obj->getBufferedDoSupported();          
   }
   SamplingMethod BDAQCALL DoFeatures_getSamplingMethod(DoFeatures *obj)                
   {
      return obj->getSamplingMethod();               
   }

   // buffered do->conversion clock features                                              
   ICollection<SignalDrop>* BDAQCALL DoFeatures_getConvertClockSources(DoFeatures *obj)           
   {
      return obj->getConvertClockSources();          
   }
   void BDAQCALL DoFeatures_getConvertClockRange(DoFeatures *obj, MathInterval *value)             
   {
      *value = obj->getConvertClockRange();            
   }

   // buffered do->burst scan                                                             
   int8 BDAQCALL DoFeatures_getBurstScanSupported(DoFeatures *obj)            
   {
      return obj->getBurstScanSupported();           
   }
   ICollection<SignalDrop>* BDAQCALL DoFeatures_getScanClockSources(DoFeatures *obj)              
   {
      return obj->getScanClockSources();             
   }
   void BDAQCALL DoFeatures_getScanClockRange(DoFeatures *obj, MathInterval *value)                
   {
      *value = obj->getScanClockRange();               
   }
   int32 BDAQCALL DoFeatures_getScanCountMax(DoFeatures *obj)                  
   {
      return obj->getScanCountMax();                 
   }

   // buffered do->trigger features                                                       
   int8 BDAQCALL DoFeatures_getTriggerSupported(DoFeatures *obj)              
   {
      return obj->getTriggerSupported();             
   }
   int32 BDAQCALL DoFeatures_getTriggerCount(DoFeatures *obj)                  
   {
      return obj->getTriggerCount();                 
   }
   ICollection<SignalDrop>* BDAQCALL DoFeatures_getTriggerSources(DoFeatures *obj)                
   {
      return obj->getTriggerSources();               
   }
   ICollection<TriggerAction>* BDAQCALL DoFeatures_getTriggerActions(DoFeatures *obj)                
   {
      return obj->getTriggerActions();               
   }
   void BDAQCALL DoFeatures_getTriggerDelayRange(DoFeatures *obj, MathInterval *value)             
   {
      *value = obj->getTriggerDelayRange();            
   }

   // ----------------------------------------------------------
   // InstantDoCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL InstantDoCtrl_Dispose(InstantDoCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL InstantDoCtrl_Cleanup(InstantDoCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL InstantDoCtrl_UpdateProperties(InstantDoCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL InstantDoCtrl_addRemovedListener(InstantDoCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL InstantDoCtrl_removeRemovedListener(InstantDoCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL InstantDoCtrl_addReconnectedListener(InstantDoCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL InstantDoCtrl_removeReconnectedListener(InstantDoCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL InstantDoCtrl_addPropertyChangedListener(InstantDoCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL InstantDoCtrl_removePropertyChangedListener(InstantDoCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL InstantDoCtrl_getSelectedDevice(InstantDoCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL InstantDoCtrl_setSelectedDevice(InstantDoCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL InstantDoCtrl_getInitialized(InstantDoCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL InstantDoCtrl_getCanEditProperty(InstantDoCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL InstantDoCtrl_getDevice(InstantDoCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL InstantDoCtrl_getModule(InstantDoCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL InstantDoCtrl_getSupportedDevices(InstantDoCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL InstantDoCtrl_getSupportedModes(InstantDoCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from DioCtrlBase */
   int32 BDAQCALL InstantDoCtrl_getPortCount(InstantDoCtrl *obj)                                                 
   {
      return obj->getPortCount();                  
   }
   ICollection<PortDirection>* BDAQCALL InstantDoCtrl_getPortDirection(InstantDoCtrl *obj)                                             
   {
      return obj->getPortDirection();              
   }
   /* Methods derived from DoCtrlBase */ 
   DoFeatures* BDAQCALL InstantDoCtrl_getFeatures(InstantDoCtrl *obj)                                                  
   {
      return obj->getFeatures();                   
   }
   /* Instant DO methods */
   ErrorCode BDAQCALL InstantDoCtrl_WriteAny(InstantDoCtrl *obj, int32 portStart, int32 portCount, uint8 data[])     
   {
      return obj->WriteAny(portStart, portCount, data); 
   }
   ErrorCode BDAQCALL InstantDoCtrl_ReadAny(InstantDoCtrl *obj, int32 portStart, int32 portCount, uint8 data[])      
   {
      return obj->ReadAny(portStart, portCount, data);  
   }
   ErrorCode BDAQCALL InstantDoCtrl_WriteBit(InstantDoCtrl *obj, int32 port, int32 bit, uint8 data)     
   {
      return obj->WriteBit(port, bit, data); 
   }
   ErrorCode BDAQCALL InstantDoCtrl_ReadBit(InstantDoCtrl *obj, int32 port, int32 bit, uint8* data)      
   {
      return obj->ReadBit(port, bit, data);  
   }

   // ----------------------------------------------------------
   // BufferedDoCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL BufferedDoCtrl_Dispose(BufferedDoCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL BufferedDoCtrl_Cleanup(BufferedDoCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL BufferedDoCtrl_UpdateProperties(BufferedDoCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL BufferedDoCtrl_addRemovedListener(BufferedDoCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL BufferedDoCtrl_removeRemovedListener(BufferedDoCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL BufferedDoCtrl_addReconnectedListener(BufferedDoCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL BufferedDoCtrl_removeReconnectedListener(BufferedDoCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL BufferedDoCtrl_addPropertyChangedListener(BufferedDoCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL BufferedDoCtrl_removePropertyChangedListener(BufferedDoCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL BufferedDoCtrl_getSelectedDevice(BufferedDoCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL BufferedDoCtrl_setSelectedDevice(BufferedDoCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL BufferedDoCtrl_getInitialized(BufferedDoCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL BufferedDoCtrl_getCanEditProperty(BufferedDoCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL BufferedDoCtrl_getDevice(BufferedDoCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL BufferedDoCtrl_getModule(BufferedDoCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL BufferedDoCtrl_getSupportedDevices(BufferedDoCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL BufferedDoCtrl_getSupportedModes(BufferedDoCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from DioCtrlBase */
   int32 BDAQCALL BufferedDoCtrl_getPortCount(BufferedDoCtrl *obj)                                                 
   {
      return obj->getPortCount();                  
   }
   ICollection<PortDirection>* BDAQCALL BufferedDoCtrl_getPortDirection(BufferedDoCtrl *obj)                                             
   {
      return obj->getPortDirection();              
   }
   /* Methods derived from DoCtrlBase */ 
   DoFeatures* BDAQCALL BufferedDoCtrl_getFeatures(BufferedDoCtrl *obj)                                                  
   {
      return obj->getFeatures();                   
   }
   /* Buffered DO methods */
   // event
   void BDAQCALL BufferedDoCtrl_addDataTransmittedListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)     
   {
      obj->addDataTransmittedListener(*listener);   
   }
   void BDAQCALL BufferedDoCtrl_removeDataTransmittedListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)  
   {
      obj->removeDataTransmittedListener(*listener);
   }
   void BDAQCALL BufferedDoCtrl_addUnderrunListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)            
   {
      obj->addUnderrunListener(*listener);          
   }
   void BDAQCALL BufferedDoCtrl_removeUnderrunListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)         
   {
      obj->removeUnderrunListener(*listener);       
   }
   void BDAQCALL BufferedDoCtrl_addCacheEmptiedListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)        
   {
      obj->addCacheEmptiedListener(*listener);      
   }
   void BDAQCALL BufferedDoCtrl_removeCacheEmptiedListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)     
   {
      obj->removeCacheEmptiedListener(*listener);   
   }
   void BDAQCALL BufferedDoCtrl_addTransitStoppedListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)      
   {
      obj->addTransitStoppedListener(*listener);    
   }
   void BDAQCALL BufferedDoCtrl_removeTransitStoppedListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)   
   {
      obj->removeTransitStoppedListener(*listener); 
   }
   void BDAQCALL BufferedDoCtrl_addStoppedListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)             
   {
      obj->addStoppedListener(*listener);           
   }
   void BDAQCALL BufferedDoCtrl_removeStoppedListener(BufferedDoCtrl *obj, BfdDoEventListener *listener)          
   {
      obj->removeStoppedListener(*listener);        
   }
   // method
   ErrorCode BDAQCALL BufferedDoCtrl_Prepare(BufferedDoCtrl *obj)                                                      
   {
      return obj->Prepare();                       
   }
   ErrorCode BDAQCALL BufferedDoCtrl_RunOnce(BufferedDoCtrl *obj)                                                      
   {
      return obj->RunOnce();                       
   }
   ErrorCode BDAQCALL BufferedDoCtrl_Start(BufferedDoCtrl *obj)                                                        
   {
      return obj->Start();                         
   }
   ErrorCode BDAQCALL BufferedDoCtrl_Stop(BufferedDoCtrl *obj, int32 action)                                           
   {
      return obj->Stop(action);                    
   }
   ErrorCode BDAQCALL BufferedDoCtrl_SetData(BufferedDoCtrl *obj, int32 count, uint8 data[])                           
   {
      return obj->SetData(count, data);            
   }
   void BDAQCALL BufferedDoCtrl_Release(BufferedDoCtrl *obj)                                                      
   {
      obj->Release();                              
   }
   // property
   void* BDAQCALL BufferedDoCtrl_getBuffer(BufferedDoCtrl *obj)                                                   
   {
      return obj->getBuffer();                     
   }
   int32 BDAQCALL BufferedDoCtrl_getBufferCapacity(BufferedDoCtrl *obj)                                           
   {
      return obj->getBufferCapacity();             
   }
   ControlState BDAQCALL BufferedDoCtrl_getState(BufferedDoCtrl *obj)                                                    
   {
      return obj->getState();                      
   }
   ScanPort* BDAQCALL BufferedDoCtrl_getScanPort(BufferedDoCtrl *obj)                                                 
   {
      return obj->getScanPort();                   
   }
   ConvertClock* BDAQCALL BufferedDoCtrl_getConvertClock(BufferedDoCtrl *obj)                                             
   {
      return obj->getConvertClock();               
   }
   Trigger* BDAQCALL BufferedDoCtrl_getTrigger(BufferedDoCtrl *obj)                                                  
   {
      return obj->getTrigger();                    
   }
   bool BDAQCALL BufferedDoCtrl_getStreaming(BufferedDoCtrl *obj)                                                
   {
      return obj->getStreaming();                  
   }
   ErrorCode BDAQCALL BufferedDoCtrl_setStreaming(BufferedDoCtrl *obj, bool value)                                    
   {
      return obj->setStreaming(value);             
   }

   // ----------------------------------------------------------
   // ctrl class : Counter related
   // ----------------------------------------------------------
   // Counter Capability Indexer
   void BDAQCALL CounterCapabilityIndexer_Dispose(CounterCapabilityIndexer *obj)                  
   {
      obj->Dispose();       
   }
   int32 BDAQCALL CounterCapabilityIndexer_getCount(CounterCapabilityIndexer *obj)                 
   {
      return obj->getCount();      
   }
   ICollection<CounterCapability>* BDAQCALL CounterCapabilityIndexer_getItem(CounterCapabilityIndexer *obj, int32 channel)   
   {
      return obj->getItem(channel);
   }

   // ----------------------------------------------------------
   // Event Counter features
   int32 BDAQCALL EventCounterFeatures_getChannelCountMax(EventCounterFeatures *obj)                               
   {
      return obj->getChannelCountMax();                
   }
   int32 BDAQCALL EventCounterFeatures_getResolution(EventCounterFeatures *obj)                                    
   {
      return obj->getResolution();                     
   }
   int32 BDAQCALL EventCounterFeatures_getDataSize(EventCounterFeatures *obj)                                      
   {
      return obj->getDataSize();                       
   }
   CounterCapabilityIndexer* BDAQCALL EventCounterFeatures_getCapabilities(EventCounterFeatures *obj)                     
   {
      return obj->getCapabilities();                   
   }
   int8 BDAQCALL EventCounterFeatures_getNoiseFilterSupported(EventCounterFeatures *obj)                          
   {
      return obj->getNoiseFilterSupported();           
   }
   ICollection<uint8>* BDAQCALL EventCounterFeatures_getNoiseFilterOfChannels(EventCounterFeatures *obj)                         
   {
      return obj->getNoiseFilterOfChannels();          
   }
   void BDAQCALL EventCounterFeatures_getNoiseFilterBlockTimeRange(EventCounterFeatures *obj, MathInterval *value)
   {
      *value = obj->getNoiseFilterBlockTimeRange();        
   } 

   // ----------------------------------------------------------
   // EventCounterCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL EventCounterCtrl_Dispose(EventCounterCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL EventCounterCtrl_Cleanup(EventCounterCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL EventCounterCtrl_UpdateProperties(EventCounterCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL EventCounterCtrl_addRemovedListener(EventCounterCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL EventCounterCtrl_removeRemovedListener(EventCounterCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL EventCounterCtrl_addReconnectedListener(EventCounterCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL EventCounterCtrl_removeReconnectedListener(EventCounterCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL EventCounterCtrl_addPropertyChangedListener(EventCounterCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL EventCounterCtrl_removePropertyChangedListener(EventCounterCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL EventCounterCtrl_getSelectedDevice(EventCounterCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL EventCounterCtrl_setSelectedDevice(EventCounterCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL EventCounterCtrl_getInitialized(EventCounterCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL EventCounterCtrl_getCanEditProperty(EventCounterCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL EventCounterCtrl_getDevice(EventCounterCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL EventCounterCtrl_getModule(EventCounterCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL EventCounterCtrl_getSupportedDevices(EventCounterCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL EventCounterCtrl_getSupportedModes(EventCounterCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from CntrCtrlBase */
   int32 BDAQCALL EventCounterCtrl_getChannel(EventCounterCtrl *obj)                                                   
   {
      return obj->getChannel();                    
   }
   ErrorCode BDAQCALL EventCounterCtrl_setChannel(EventCounterCtrl *obj, int32 ch)                                         
   {
      return obj->setChannel(ch);                  
   }
   bool BDAQCALL EventCounterCtrl_getEnabled(EventCounterCtrl *obj)                                                   
   {
      return obj->getEnabled();                    
   }
   ErrorCode BDAQCALL EventCounterCtrl_setEnabled(EventCounterCtrl *obj, bool enabled)                                     
   {
      return obj->setEnabled(enabled);             
   }
   int8 BDAQCALL EventCounterCtrl_getRunning(EventCounterCtrl *obj)                                                   
   {
      return obj->getRunning();                    
   }
   /* Methods derived from CntrCtrlExt */
   NoiseFilterChannel* BDAQCALL EventCounterCtrl_getNoiseFilter(EventCounterCtrl *obj)                                        
   {
      return obj->getNoiseFilter();                
   }
   /* Event counter methods */
   EventCounterFeatures* BDAQCALL EventCounterCtrl_getFeatures(EventCounterCtrl *obj)                                         
   {
      return obj->getFeatures();                   
   }
   int32 BDAQCALL EventCounterCtrl_getValue(EventCounterCtrl *obj)                                            
   {
      return obj->getValue();                      
   }

   // ----------------------------------------------------------
   // Frequency meter features
   int32 BDAQCALL FreqMeterFeatures_getChannelCountMax(FreqMeterFeatures *obj)                               
   {
      return obj->getChannelCountMax();                
   }
   int32 BDAQCALL FreqMeterFeatures_getResolution(FreqMeterFeatures *obj)                                    
   {
      return obj->getResolution();                     
   }
   int32 BDAQCALL FreqMeterFeatures_getDataSize(FreqMeterFeatures *obj)                                      
   {
      return obj->getDataSize();                       
   }
   CounterCapabilityIndexer* BDAQCALL FreqMeterFeatures_getCapabilities(FreqMeterFeatures *obj)                     
   {
      return obj->getCapabilities();                   
   }
   int8 BDAQCALL FreqMeterFeatures_getNoiseFilterSupported(FreqMeterFeatures *obj)                          
   {
      return obj->getNoiseFilterSupported();           
   }
   ICollection<uint8>* BDAQCALL FreqMeterFeatures_getNoiseFilterOfChannels(FreqMeterFeatures *obj)                         
   {
      return obj->getNoiseFilterOfChannels();          
   }
   void BDAQCALL FreqMeterFeatures_getNoiseFilterBlockTimeRange(FreqMeterFeatures *obj, MathInterval *value)
   {
      *value = obj->getNoiseFilterBlockTimeRange();        
   } 
   ICollection<FreqMeasureMethod>* BDAQCALL FreqMeterFeatures_getFmMethods(FreqMeterFeatures *obj)                                     
   {
      return obj->getFmMethods();                      
   }

   // ----------------------------------------------------------
   // FreqMeterCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL FreqMeterCtrl_Dispose(FreqMeterCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL FreqMeterCtrl_Cleanup(FreqMeterCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL FreqMeterCtrl_UpdateProperties(FreqMeterCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL FreqMeterCtrl_addRemovedListener(FreqMeterCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL FreqMeterCtrl_removeRemovedListener(FreqMeterCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL FreqMeterCtrl_addReconnectedListener(FreqMeterCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL FreqMeterCtrl_removeReconnectedListener(FreqMeterCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL FreqMeterCtrl_addPropertyChangedListener(FreqMeterCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL FreqMeterCtrl_removePropertyChangedListener(FreqMeterCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL FreqMeterCtrl_getSelectedDevice(FreqMeterCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL FreqMeterCtrl_setSelectedDevice(FreqMeterCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL FreqMeterCtrl_getInitialized(FreqMeterCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL FreqMeterCtrl_getCanEditProperty(FreqMeterCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL FreqMeterCtrl_getDevice(FreqMeterCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL FreqMeterCtrl_getModule(FreqMeterCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL FreqMeterCtrl_getSupportedDevices(FreqMeterCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL FreqMeterCtrl_getSupportedModes(FreqMeterCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from CntrCtrlBase */
   int32 BDAQCALL FreqMeterCtrl_getChannel(FreqMeterCtrl *obj)                                                   
   {
      return obj->getChannel();                    
   }
   ErrorCode BDAQCALL FreqMeterCtrl_setChannel(FreqMeterCtrl *obj, int32 ch)                                         
   {
      return obj->setChannel(ch);                  
   }
   bool BDAQCALL FreqMeterCtrl_getEnabled(FreqMeterCtrl *obj)                                                   
   {
      return obj->getEnabled();                    
   }
   ErrorCode BDAQCALL FreqMeterCtrl_setEnabled(FreqMeterCtrl *obj, bool enabled)                                     
   {
      return obj->setEnabled(enabled);             
   }
   int8 BDAQCALL FreqMeterCtrl_getRunning(FreqMeterCtrl *obj)                                                   
   {
      return obj->getRunning();                    
   }
   /* Methods derived from CntrCtrlExt */
   NoiseFilterChannel* BDAQCALL FreqMeterCtrl_getNoiseFilter(FreqMeterCtrl *obj)                                        
   {
      return obj->getNoiseFilter();                
   }
   /* Frequency meter methods */
   FreqMeterFeatures* BDAQCALL FreqMeterCtrl_getFeatures(FreqMeterCtrl *obj)                                            
   {
      return obj->getFeatures();                   
   }
   double BDAQCALL FreqMeterCtrl_getValue(FreqMeterCtrl *obj)                                               
   {
      return obj->getValue();                      
   }
   FreqMeasureMethod BDAQCALL FreqMeterCtrl_getMethod(FreqMeterCtrl *obj)                                              
   {
      return obj->getMethod();                     
   }
   ErrorCode BDAQCALL FreqMeterCtrl_setMethod(FreqMeterCtrl *obj, FreqMeasureMethod value)                     
   {
      return obj->setMethod(value);                
   }
   double BDAQCALL FreqMeterCtrl_getCollectionPeriod(FreqMeterCtrl *obj)                                    
   {
      return obj->getCollectionPeriod();           
   }
   ErrorCode BDAQCALL FreqMeterCtrl_setCollectionPeriod(FreqMeterCtrl *obj, double value)                      
   {
      return obj->setCollectionPeriod(value);      
   }

   // ----------------------------------------------------------
   // One shot features
   int32 BDAQCALL OneShotFeatures_getChannelCountMax(OneShotFeatures *obj)                               
   {
      return obj->getChannelCountMax();                 
   }
   int32 BDAQCALL OneShotFeatures_getResolution(OneShotFeatures *obj)                                    
   {
      return obj->getResolution();                      
   }
   int32 BDAQCALL OneShotFeatures_getDataSize(OneShotFeatures *obj)                                      
   {
      return obj->getDataSize();                        
   }
   CounterCapabilityIndexer* BDAQCALL OneShotFeatures_getCapabilities(OneShotFeatures *obj)                     
   {
      return obj->getCapabilities();                    
   }
   int8 BDAQCALL OneShotFeatures_getNoiseFilterSupported(OneShotFeatures *obj)                          
   {
      return obj->getNoiseFilterSupported();            
   }
   ICollection<uint8>* BDAQCALL OneShotFeatures_getNoiseFilterOfChannels(OneShotFeatures *obj)                         
   {
      return obj->getNoiseFilterOfChannels();           
   }
   void BDAQCALL OneShotFeatures_getNoiseFilterBlockTimeRange(OneShotFeatures *obj, MathInterval *value)
   {
      *value = obj->getNoiseFilterBlockTimeRange();         
   } 
   void BDAQCALL OneShotFeatures_getDelayCountRange(OneShotFeatures *obj, MathInterval *value)         
   {
      *value = obj->getDelayCountRange();                   
   } 

   // ----------------------------------------------------------
   // OneShotCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL OneShotCtrl_Dispose(OneShotCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL OneShotCtrl_Cleanup(OneShotCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL OneShotCtrl_UpdateProperties(OneShotCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL OneShotCtrl_addRemovedListener(OneShotCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL OneShotCtrl_removeRemovedListener(OneShotCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL OneShotCtrl_addReconnectedListener(OneShotCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL OneShotCtrl_removeReconnectedListener(OneShotCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL OneShotCtrl_addPropertyChangedListener(OneShotCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL OneShotCtrl_removePropertyChangedListener(OneShotCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL OneShotCtrl_getSelectedDevice(OneShotCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL OneShotCtrl_setSelectedDevice(OneShotCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL OneShotCtrl_getInitialized(OneShotCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL OneShotCtrl_getCanEditProperty(OneShotCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL OneShotCtrl_getDevice(OneShotCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL OneShotCtrl_getModule(OneShotCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL OneShotCtrl_getSupportedDevices(OneShotCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL OneShotCtrl_getSupportedModes(OneShotCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from CntrCtrlBase */
   int32 BDAQCALL OneShotCtrl_getChannel(OneShotCtrl *obj)                                                   
   {
      return obj->getChannel();                    
   }
   ErrorCode BDAQCALL OneShotCtrl_setChannel(OneShotCtrl *obj, int32 ch)                                         
   {
      return obj->setChannel(ch);                  
   }
   bool BDAQCALL OneShotCtrl_getEnabled(OneShotCtrl *obj)                                                   
   {
      return obj->getEnabled();                    
   }
   ErrorCode BDAQCALL OneShotCtrl_setEnabled(OneShotCtrl *obj, bool enabled)                                     
   {
      return obj->setEnabled(enabled);             
   }
   int8 BDAQCALL OneShotCtrl_getRunning(OneShotCtrl *obj)                                                   
   {
      return obj->getRunning();                    
   }
   /* Methods derived from CntrCtrlExt */
   NoiseFilterChannel* BDAQCALL OneShotCtrl_getNoiseFilter(OneShotCtrl *obj)                                        
   {
      return obj->getNoiseFilter();                
   }
   /* one shot methods */
   void BDAQCALL OneShotCtrl_addOneShotListener(OneShotCtrl *obj, CntrEventListener * listener)           
   {
      obj->addOneShotListener(*listener);           
   }
   void BDAQCALL OneShotCtrl_removeOneShotListener(OneShotCtrl *obj, CntrEventListener * listener)        
   {
      obj->removeOneShotListener(*listener);        
   }
   OneShotFeatures* BDAQCALL OneShotCtrl_getFeatures(OneShotCtrl *obj)                                              
   {
      return obj->getFeatures();                   
   }
   int32 BDAQCALL OneShotCtrl_getDelayCount(OneShotCtrl *obj)                                                
   {
      return obj->getDelayCount();                 
   }
   ErrorCode BDAQCALL OneShotCtrl_setDelayCount(OneShotCtrl *obj, int32 value)                                   
   {
      return obj->setDelayCount(value);            
   }

   // ----------------------------------------------------------
   // Timer/Pulse features
   int32 BDAQCALL TimerPulseFeatures_getChannelCountMax(TimerPulseFeatures *obj)                               
   {
      return obj->getChannelCountMax();          
   }
   int32 BDAQCALL TimerPulseFeatures_getResolution(TimerPulseFeatures *obj)                                    
   {
      return obj->getResolution();               
   }
   int32 BDAQCALL TimerPulseFeatures_getDataSize(TimerPulseFeatures *obj)                                      
   {
      return obj->getDataSize();                 
   }
   CounterCapabilityIndexer* BDAQCALL TimerPulseFeatures_getCapabilities(TimerPulseFeatures *obj)                     
   {
      return obj->getCapabilities();             
   }
   int8 BDAQCALL TimerPulseFeatures_getNoiseFilterSupported(TimerPulseFeatures *obj)                          
   {
      return obj->getNoiseFilterSupported();     
   }
   ICollection<uint8>* BDAQCALL TimerPulseFeatures_getNoiseFilterOfChannels(TimerPulseFeatures *obj)                         
   {
      return obj->getNoiseFilterOfChannels();    
   }
   void BDAQCALL TimerPulseFeatures_getNoiseFilterBlockTimeRange(TimerPulseFeatures *obj, MathInterval *value)
   {
      *value = obj->getNoiseFilterBlockTimeRange();  
   } 
   void BDAQCALL TimerPulseFeatures_getTimerFrequencyRange(TimerPulseFeatures *obj, MathInterval *value)      
   {
      *value = obj->getTimerFrequencyRange();        
   }
   int8 BDAQCALL TimerPulseFeatures_getTimerEventSupported(TimerPulseFeatures *obj)                           
   {
      return obj->getTimerEventSupported();      
   }

   // ----------------------------------------------------------
   // TimerPulseCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL TimerPulseCtrl_Dispose(TimerPulseCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL TimerPulseCtrl_Cleanup(TimerPulseCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL TimerPulseCtrl_UpdateProperties(TimerPulseCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL TimerPulseCtrl_addRemovedListener(TimerPulseCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL TimerPulseCtrl_removeRemovedListener(TimerPulseCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL TimerPulseCtrl_addReconnectedListener(TimerPulseCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL TimerPulseCtrl_removeReconnectedListener(TimerPulseCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL TimerPulseCtrl_addPropertyChangedListener(TimerPulseCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL TimerPulseCtrl_removePropertyChangedListener(TimerPulseCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL TimerPulseCtrl_getSelectedDevice(TimerPulseCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL TimerPulseCtrl_setSelectedDevice(TimerPulseCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL TimerPulseCtrl_getInitialized(TimerPulseCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL TimerPulseCtrl_getCanEditProperty(TimerPulseCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL TimerPulseCtrl_getDevice(TimerPulseCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL TimerPulseCtrl_getModule(TimerPulseCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL TimerPulseCtrl_getSupportedDevices(TimerPulseCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL TimerPulseCtrl_getSupportedModes(TimerPulseCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from CntrCtrlBase */
   int32 BDAQCALL TimerPulseCtrl_getChannel(TimerPulseCtrl *obj)                                                   
   {
      return obj->getChannel();                    
   }
   ErrorCode BDAQCALL TimerPulseCtrl_setChannel(TimerPulseCtrl *obj, int32 ch)                                         
   {
      return obj->setChannel(ch);                  
   }
   bool BDAQCALL TimerPulseCtrl_getEnabled(TimerPulseCtrl *obj)                                                   
   {
      return obj->getEnabled();                    
   }
   ErrorCode BDAQCALL TimerPulseCtrl_setEnabled(TimerPulseCtrl *obj, bool enabled)                                     
   {
      return obj->setEnabled(enabled);             
   }
   int8 BDAQCALL TimerPulseCtrl_getRunning(TimerPulseCtrl *obj)                                                   
   {
      return obj->getRunning();                    
   }
   /* Methods derived from CntrCtrlExt */
   NoiseFilterChannel* BDAQCALL TimerPulseCtrl_getNoiseFilter(TimerPulseCtrl *obj)                                        
   {
      return obj->getNoiseFilter();                
   }
   /* one shot methods */
   void BDAQCALL TimerPulseCtrl_addTimerTickListener(TimerPulseCtrl *obj, CntrEventListener * listener)         
   {
      obj->addTimerTickListener(*listener);         
   }
   void BDAQCALL TimerPulseCtrl_removeTimerTickListener(TimerPulseCtrl *obj, CntrEventListener * listener)      
   {
      obj->removeTimerTickListener(*listener);      
   }
   TimerPulseFeatures* BDAQCALL TimerPulseCtrl_getFeatures(TimerPulseCtrl *obj)                                           
   {
      return obj->getFeatures();                   
   }
   double BDAQCALL TimerPulseCtrl_getFrequency(TimerPulseCtrl *obj)                                                 
   {
      return obj->getFrequency();                  
   }
   ErrorCode BDAQCALL TimerPulseCtrl_setFrequency(TimerPulseCtrl *obj, double value)                                   
   {
      return obj->setFrequency(value);             
   }

   // ----------------------------------------------------------
   // Pulse width meter features
   int32 BDAQCALL PwMeterFeatures_getChannelCountMax(PwMeterFeatures *obj)                               
   {
      return obj->getChannelCountMax();          
   }
   int32 BDAQCALL PwMeterFeatures_getResolution(PwMeterFeatures *obj)                                    
   {
      return obj->getResolution();               
   }
   int32 BDAQCALL PwMeterFeatures_getDataSize(PwMeterFeatures *obj)                                      
   {
      return obj->getDataSize();                 
   }
   CounterCapabilityIndexer* BDAQCALL PwMeterFeatures_getCapabilities(PwMeterFeatures *obj)                     
   {
      return obj->getCapabilities();             
   }
   int8 BDAQCALL PwMeterFeatures_getNoiseFilterSupported(PwMeterFeatures *obj)                          
   {
      return obj->getNoiseFilterSupported();     
   }
   ICollection<uint8>* BDAQCALL PwMeterFeatures_getNoiseFilterOfChannels(PwMeterFeatures *obj)                  
   {
      return obj->getNoiseFilterOfChannels();    
   }
   void BDAQCALL PwMeterFeatures_getNoiseFilterBlockTimeRange(PwMeterFeatures *obj, MathInterval *value)
   {
      *value = obj->getNoiseFilterBlockTimeRange();  
   } 
   ICollection<CounterCascadeGroup>* BDAQCALL PwMeterFeatures_getPwmCascadeGroup(PwMeterFeatures *obj)          
   {
      return obj->getPwmCascadeGroup();          
   }
   int8 BDAQCALL PwMeterFeatures_getOverflowEventSupported(PwMeterFeatures *obj)                        
   {
      return obj->getOverflowEventSupported();   
   }

   // ----------------------------------------------------------
   // PwMeterCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL PwMeterCtrl_Dispose(PwMeterCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL PwMeterCtrl_Cleanup(PwMeterCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL PwMeterCtrl_UpdateProperties(PwMeterCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL PwMeterCtrl_addRemovedListener(PwMeterCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL PwMeterCtrl_removeRemovedListener(PwMeterCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL PwMeterCtrl_addReconnectedListener(PwMeterCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL PwMeterCtrl_removeReconnectedListener(PwMeterCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL PwMeterCtrl_addPropertyChangedListener(PwMeterCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL PwMeterCtrl_removePropertyChangedListener(PwMeterCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL PwMeterCtrl_getSelectedDevice(PwMeterCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL PwMeterCtrl_setSelectedDevice(PwMeterCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL PwMeterCtrl_getInitialized(PwMeterCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL PwMeterCtrl_getCanEditProperty(PwMeterCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL PwMeterCtrl_getDevice(PwMeterCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL PwMeterCtrl_getModule(PwMeterCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL PwMeterCtrl_getSupportedDevices(PwMeterCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL PwMeterCtrl_getSupportedModes(PwMeterCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from CntrCtrlBase */
   int32 BDAQCALL PwMeterCtrl_getChannel(PwMeterCtrl *obj)                                                   
   {
      return obj->getChannel();                    
   }
   ErrorCode BDAQCALL PwMeterCtrl_setChannel(PwMeterCtrl *obj, int32 ch)                                         
   {
      return obj->setChannel(ch);                  
   }
   bool BDAQCALL PwMeterCtrl_getEnabled(PwMeterCtrl *obj)                                                   
   {
      return obj->getEnabled();                    
   }
   ErrorCode BDAQCALL PwMeterCtrl_setEnabled(PwMeterCtrl *obj, bool enabled)                                     
   {
      return obj->setEnabled(enabled);             
   }
   int8 BDAQCALL PwMeterCtrl_getRunning(PwMeterCtrl *obj)                                                   
   {
      return obj->getRunning();                    
   }
   /* Methods derived from CntrCtrlExt */
   NoiseFilterChannel* BDAQCALL PwMeterCtrl_getNoiseFilter(PwMeterCtrl *obj)                                        
   {
      return obj->getNoiseFilter();                
   }
   /* one shot methods */
   void BDAQCALL PwMeterCtrl_addOverflowListener(PwMeterCtrl *obj, CntrEventListener * listener)          
   {
      obj->addOverflowListener(*listener);          
   }
   void BDAQCALL PwMeterCtrl_removeOverflowListener(PwMeterCtrl *obj, CntrEventListener * listener)       
   {
      obj->removeOverflowListener(*listener);       
   }
   PwMeterFeatures* BDAQCALL PwMeterCtrl_getFeatures(PwMeterCtrl *obj)                                              
   {
      return obj->getFeatures();                   
   }
   void BDAQCALL PwMeterCtrl_getValue(PwMeterCtrl *obj, PulseWidth *width)                                  
   {
      obj->getValue(*width);                        
   }

   // ----------------------------------------------------------
   // Pulse width modulator features
   int32 BDAQCALL PwModulatorFeatures_getChannelCountMax(PwModulatorFeatures *obj)                               
   {
      return obj->getChannelCountMax();          
   }
   int32 BDAQCALL PwModulatorFeatures_getResolution(PwModulatorFeatures *obj)                                    
   {
      return obj->getResolution();               
   }
   int32 BDAQCALL PwModulatorFeatures_getDataSize(PwModulatorFeatures *obj)                                      
   {
      return obj->getDataSize();                 
   }
   CounterCapabilityIndexer* BDAQCALL PwModulatorFeatures_getCapabilities(PwModulatorFeatures *obj)                     
   {
      return obj->getCapabilities();             
   }
   int8 BDAQCALL PwModulatorFeatures_getNoiseFilterSupported(PwModulatorFeatures *obj)                          
   {
      return obj->getNoiseFilterSupported();     
   }
   ICollection<uint8>* BDAQCALL PwModulatorFeatures_getNoiseFilterOfChannels(PwModulatorFeatures *obj)                         
   {
      return obj->getNoiseFilterOfChannels();    
   }
   void BDAQCALL PwModulatorFeatures_getNoiseFilterBlockTimeRange(PwModulatorFeatures *obj, MathInterval *value)
   {
      *value = obj->getNoiseFilterBlockTimeRange();  
   } 
   void BDAQCALL PwModulatorFeatures_getHiPeriodRange(PwModulatorFeatures *obj, MathInterval *value)            
   {
      *value = obj->getHiPeriodRange();              
   } 
   void BDAQCALL PwModulatorFeatures_getLoPeriodRange(PwModulatorFeatures *obj, MathInterval *value)            
   {
      *value = obj->getLoPeriodRange();              
   } 

   // ----------------------------------------------------------
   // PwModulatorCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL PwModulatorCtrl_Dispose(PwModulatorCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL PwModulatorCtrl_Cleanup(PwModulatorCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL PwModulatorCtrl_UpdateProperties(PwModulatorCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL PwModulatorCtrl_addRemovedListener(PwModulatorCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL PwModulatorCtrl_removeRemovedListener(PwModulatorCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL PwModulatorCtrl_addReconnectedListener(PwModulatorCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL PwModulatorCtrl_removeReconnectedListener(PwModulatorCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL PwModulatorCtrl_addPropertyChangedListener(PwModulatorCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL PwModulatorCtrl_removePropertyChangedListener(PwModulatorCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL PwModulatorCtrl_getSelectedDevice(PwModulatorCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL PwModulatorCtrl_setSelectedDevice(PwModulatorCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL PwModulatorCtrl_getInitialized(PwModulatorCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL PwModulatorCtrl_getCanEditProperty(PwModulatorCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL PwModulatorCtrl_getDevice(PwModulatorCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL PwModulatorCtrl_getModule(PwModulatorCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL PwModulatorCtrl_getSupportedDevices(PwModulatorCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL PwModulatorCtrl_getSupportedModes(PwModulatorCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from CntrCtrlBase */
   int32 BDAQCALL PwModulatorCtrl_getChannel(PwModulatorCtrl *obj)                                                   
   {
      return obj->getChannel();                    
   }
   ErrorCode BDAQCALL PwModulatorCtrl_setChannel(PwModulatorCtrl *obj, int32 ch)                                         
   {
      return obj->setChannel(ch);                  
   }
   bool BDAQCALL PwModulatorCtrl_getEnabled(PwModulatorCtrl *obj)                                                   
   {
      return obj->getEnabled();                    
   }
   ErrorCode BDAQCALL PwModulatorCtrl_setEnabled(PwModulatorCtrl *obj, bool enabled)                                     
   {
      return obj->setEnabled(enabled);             
   }
   int8 BDAQCALL PwModulatorCtrl_getRunning(PwModulatorCtrl *obj)                                                   
   {
      return obj->getRunning();                    
   }
   /* Methods derived from CntrCtrlExt */
   NoiseFilterChannel* BDAQCALL PwModulatorCtrl_getNoiseFilter(PwModulatorCtrl *obj)                                        
   {
      return obj->getNoiseFilter();                
   }
   /* Pulse width modulator methods */
   PwModulatorFeatures* BDAQCALL PwModulatorCtrl_getFeatures(PwModulatorCtrl *obj)                                          
   {
      return obj->getFeatures();                   
   }
   void BDAQCALL PwModulatorCtrl_getPulseWidth(PwModulatorCtrl *obj, PulseWidth *width)                             
   {
      obj->getPulseWidth(*width);                   
   } 
   ErrorCode BDAQCALL PwModulatorCtrl_setPulseWidth(PwModulatorCtrl *obj, PulseWidth *width)                             
   {
      return obj->setPulseWidth(*width);                   
   } 

   // ----------------------------------------------------------
   // Up-Down counter features
   int32 BDAQCALL UdCounterFeatures_getChannelCountMax(UdCounterFeatures *obj)                               
   {
      return obj->getChannelCountMax();          
   }
   int32 BDAQCALL UdCounterFeatures_getResolution(UdCounterFeatures *obj)                                    
   {
      return obj->getResolution();               
   }
   int32 BDAQCALL UdCounterFeatures_getDataSize(UdCounterFeatures *obj)                                      
   {
      return obj->getDataSize();                 
   }
   CounterCapabilityIndexer* BDAQCALL UdCounterFeatures_getCapabilities(UdCounterFeatures *obj)                     
   {
      return obj->getCapabilities();             
   }
   int8 BDAQCALL UdCounterFeatures_getNoiseFilterSupported(UdCounterFeatures *obj)                          
   {
      return obj->getNoiseFilterSupported();     
   }
   ICollection<uint8>* BDAQCALL UdCounterFeatures_getNoiseFilterOfChannels(UdCounterFeatures *obj)                         
   {
      return obj->getNoiseFilterOfChannels();    
   }
   void BDAQCALL UdCounterFeatures_getNoiseFilterBlockTimeRange(UdCounterFeatures *obj, MathInterval *value)
   {
      *value = obj->getNoiseFilterBlockTimeRange();  
   } 
   ICollection<SignalCountingType>* BDAQCALL UdCounterFeatures_getCountingTypes(UdCounterFeatures *obj)                                 
   {
      return obj->getCountingTypes();            
   }
   ICollection<int32>* BDAQCALL UdCounterFeatures_getInitialValues(UdCounterFeatures *obj)                                 
   {
      return obj->getInitialValues();            
   }
   ICollection<int32>* BDAQCALL UdCounterFeatures_getSnapEventSources(UdCounterFeatures *obj)                              
   {
      return obj->getSnapEventSources();         
   }

   // ----------------------------------------------------------
   // UdCounterCtrl
   /* Methods derived from DeviceCtrlBase */
   void BDAQCALL UdCounterCtrl_Dispose(UdCounterCtrl *obj)                                                      
   {
      obj->Dispose();                              
   }
   void BDAQCALL UdCounterCtrl_Cleanup(UdCounterCtrl *obj)                                                      
   {
      obj->Cleanup();                              
   }
   ErrorCode BDAQCALL UdCounterCtrl_UpdateProperties(UdCounterCtrl *obj)                                             
   {
      return obj->UpdateProperties();              
   }
   void BDAQCALL UdCounterCtrl_addRemovedListener(UdCounterCtrl *obj, DeviceEventListener * listener)           
   {
      obj->addRemovedListener(*listener);           
   }
   void BDAQCALL UdCounterCtrl_removeRemovedListener(UdCounterCtrl *obj, DeviceEventListener * listener)        
   {
      obj->removeRemovedListener(*listener);        
   }
   void BDAQCALL UdCounterCtrl_addReconnectedListener(UdCounterCtrl *obj, DeviceEventListener * listener)       
   {
      obj->addReconnectedListener(*listener);       
   }
   void BDAQCALL UdCounterCtrl_removeReconnectedListener(UdCounterCtrl *obj, DeviceEventListener * listener)    
   {
      obj->removeReconnectedListener(*listener);    
   }
   void BDAQCALL UdCounterCtrl_addPropertyChangedListener(UdCounterCtrl *obj, DeviceEventListener * listener)   
   {
      obj->addPropertyChangedListener(*listener);   
   }
   void BDAQCALL UdCounterCtrl_removePropertyChangedListener(UdCounterCtrl *obj, DeviceEventListener * listener)
   {
      obj->removePropertyChangedListener(*listener);
   }
   void BDAQCALL UdCounterCtrl_getSelectedDevice(UdCounterCtrl *obj, DeviceInformation *x)                      
   {
      obj->getSelectedDevice(*x);                   
   }
   ErrorCode BDAQCALL UdCounterCtrl_setSelectedDevice(UdCounterCtrl *obj, DeviceInformation const *x)                
   {
      return obj->setSelectedDevice(*x);            
   }
   int8 BDAQCALL UdCounterCtrl_getInitialized(UdCounterCtrl *obj)                                               
   {
      return obj->getInitialized();                
   }
   int8 BDAQCALL UdCounterCtrl_getCanEditProperty(UdCounterCtrl *obj)                                           
   {
      return obj->getCanEditProperty();            
   }
   HANDLE BDAQCALL UdCounterCtrl_getDevice(UdCounterCtrl *obj)                                                    
   {
      return obj->getDevice();                     
   }
   HANDLE BDAQCALL UdCounterCtrl_getModule(UdCounterCtrl *obj)                                                    
   {
      return obj->getModule();                     
   }
   ICollection<DeviceTreeNode>* BDAQCALL UdCounterCtrl_getSupportedDevices(UdCounterCtrl *obj)                                          
   {
      return obj->getSupportedDevices();           
   }
   ICollection<AccessMode>* BDAQCALL UdCounterCtrl_getSupportedModes(UdCounterCtrl *obj)                                            
   {
      return obj->getSupportedModes();             
   }
   /* Methods derived from CntrCtrlBase */
   int32 BDAQCALL UdCounterCtrl_getChannel(UdCounterCtrl *obj)                                                   
   {
      return obj->getChannel();                    
   }
   ErrorCode BDAQCALL UdCounterCtrl_setChannel(UdCounterCtrl *obj, int32 ch)                                         
   {
      return obj->setChannel(ch);                  
   }
   bool BDAQCALL UdCounterCtrl_getEnabled(UdCounterCtrl *obj)                                                   
   {
      return obj->getEnabled();                    
   }
   ErrorCode BDAQCALL UdCounterCtrl_setEnabled(UdCounterCtrl *obj, bool enabled)                                     
   {
      return obj->setEnabled(enabled);             
   }
   int8 BDAQCALL UdCounterCtrl_getRunning(UdCounterCtrl *obj)                                                   
   {
      return obj->getRunning();                    
   }
   /* Methods derived from CntrCtrlExt */
   NoiseFilterChannel* BDAQCALL UdCounterCtrl_getNoiseFilter(UdCounterCtrl *obj)                                        
   {
      return obj->getNoiseFilter();                
   }
   /* one shot methods */
   void BDAQCALL UdCounterCtrl_addUdCntrEventListener(UdCounterCtrl *obj, UdCntrEventListener * listener)       
   {
      obj->addUdCntrEventListener(*listener);       
   }
   void BDAQCALL UdCounterCtrl_removeUdCntrEventListener(UdCounterCtrl *obj, UdCntrEventListener * listener)    
   {
      obj->removeUdCntrEventListener(*listener);    
   }
   ErrorCode BDAQCALL UdCounterCtrl_SnapStart(UdCounterCtrl *obj, int32 srcId)                                       
   {
      return obj->SnapStart(srcId);                           
   }
   ErrorCode BDAQCALL UdCounterCtrl_SnapStop(UdCounterCtrl *obj, int32 srcId)                                        
   {
      return obj->SnapStop(srcId);                            
   }
   ErrorCode BDAQCALL UdCounterCtrl_CompareSetTable(UdCounterCtrl *obj, int32 count, int32 *table)                   
   {
      return obj->CompareSetTable(count, table);              
   }
   ErrorCode BDAQCALL UdCounterCtrl_CompareSetInterval(UdCounterCtrl *obj, int32 start, int32 increment,int32 count) 
   {
      return obj->CompareSetInterval(start, increment, count);
   }
   ErrorCode BDAQCALL UdCounterCtrl_CompareClear(UdCounterCtrl *obj)                                                 
   {
      return obj->CompareClear();                             
   }
   ErrorCode BDAQCALL UdCounterCtrl_ValueReset(UdCounterCtrl *obj)                                                   
   {
      return obj->ValueReset();                               
   }

   UdCounterFeatures* BDAQCALL UdCounterCtrl_getFeatures(UdCounterCtrl *obj)                                            
   {
      return obj->getFeatures();                              
   }
   int32 BDAQCALL UdCounterCtrl_getValue(UdCounterCtrl *obj)                                               
   {
      return obj->getValue();                                 
   }
   SignalCountingType BDAQCALL UdCounterCtrl_getCountingType(UdCounterCtrl *obj)                                        
   {
      return obj->getCountingType();                          
   }
   ErrorCode BDAQCALL UdCounterCtrl_setCountingType(UdCounterCtrl *obj, SignalCountingType value)              
   {
      return obj->setCountingType(value);                     
   }
   int32 BDAQCALL UdCounterCtrl_getInitialValue(UdCounterCtrl *obj)                                        
   {
      return obj->getInitialValue();                          
   }
   ErrorCode BDAQCALL UdCounterCtrl_setInitialValue(UdCounterCtrl *obj, int32 value)                           
   {
      return obj->setInitialValue(value);                     
   }
   int32 BDAQCALL UdCounterCtrl_getResetTimesByIndex(UdCounterCtrl *obj)                                   
   {
      return obj->getResetTimesByIndex();                     
   }
   ErrorCode BDAQCALL UdCounterCtrl_setResetTimesByIndex(UdCounterCtrl *obj, int32 value)                      
   {
      return obj->setResetTimesByIndex(value);                
   }
}

#if defined(_WIN32) || defined(WIN32)

// ---------------------------------------------------------------
// BDAQ CSCL ANSI-C interface function table
// 
// NOTE:
//     MUST NOT RE-ORDER OR REMOVE the elements of the table.
//     YOU  CAN JUST APPEND new element at the tail of the table
// ---------------------------------------------------------------

FARPROC BDAQ_CSCL_ANSI_C_APIs[] = {
   // ICollection -------------------------------------------------------------------------
   (FARPROC)ICollection_Dispose,                               // index: 0
   (FARPROC)ICollection_getCount,                              // index: 1
   (FARPROC)ICollection_getItem,                               // index: 2
   // AnalogChannel -----------------------------------------------------------------------
   (FARPROC)AnalogChannel_getChannel,                          // index: 3
   (FARPROC)AnalogChannel_getValueRange,                       // index: 4
   (FARPROC)AnalogChannel_setValueRange,                       // index: 5
   // AnalogInputChannel ------------------------------------------------------------------
   (FARPROC)AnalogInputChannel_getChannel,                     // index: 6
   (FARPROC)AnalogInputChannel_getValueRange,                  // index: 7
   (FARPROC)AnalogInputChannel_setValueRange,                  // index: 8
   (FARPROC)AnalogInputChannel_getSignalType,                  // index: 9
   (FARPROC)AnalogInputChannel_setSignalType,                  // index: 10
   (FARPROC)AnalogInputChannel_getBurnoutRetType,              // index: 11
   (FARPROC)AnalogInputChannel_setBurnoutRetType,              // index: 12
   (FARPROC)AnalogInputChannel_getBurnoutRetValue,             // index: 13
   (FARPROC)AnalogInputChannel_setBurnoutRetValue,             // index: 14
   // CjcSetting -------------------------------------------------------------------------
   (FARPROC)CjcSetting_getChannel,                             // index: 15
   (FARPROC)CjcSetting_setChannel,                             // index: 16
   (FARPROC)CjcSetting_getValue,                               // index: 17
   (FARPROC)CjcSetting_setValue,                               // index: 18
   // ScanChannel -------------------------------------------------------------------------
   (FARPROC)ScanChannel_getChannelStart,                       // index: 19
   (FARPROC)ScanChannel_setChannelStart,                       // index: 20
   (FARPROC)ScanChannel_getChannelCount,                       // index: 21
   (FARPROC)ScanChannel_setChannelCount,                       // index: 22
   (FARPROC)ScanChannel_getSamples,                            // index: 23
   (FARPROC)ScanChannel_setSamples,                            // index: 24
   (FARPROC)ScanChannel_getIntervalCount,                      // index: 25
   (FARPROC)ScanChannel_setIntervalCount,                      // index: 26
   // ConvertClock ------------------------------------------------------------------------
   (FARPROC)ConvertClock_getSource,                            // index: 27
   (FARPROC)ConvertClock_setSource,                            // index: 28
   (FARPROC)ConvertClock_getRate,                              // index: 29
   (FARPROC)ConvertClock_setRate,                              // index: 30
   // ScanClock -------------------------------------------------------------------------
   (FARPROC)ScanClock_getSource,                               // index: 31
   (FARPROC)ScanClock_setSource,                               // index: 32
   (FARPROC)ScanClock_getRate,                                 // index: 33
   (FARPROC)ScanClock_setRate,                                 // index: 34
   (FARPROC)ScanClock_getScanCount,                            // index: 35
   (FARPROC)ScanClock_setScanCount,                            // index: 36
   // Trigger -------------------------------------------------------------------------
   (FARPROC)Trigger_getSource,                                 // index: 37
   (FARPROC)Trigger_setSource,                                 // index: 38
   (FARPROC)Trigger_getEdge,                                   // index: 39
   (FARPROC)Trigger_setEdge,                                   // index: 40
   (FARPROC)Trigger_getLevel,                                  // index: 41
   (FARPROC)Trigger_setLevel,                                  // index: 42
   (FARPROC)Trigger_getAction,                                 // index: 43
   (FARPROC)Trigger_setAction,                                 // index: 44
   (FARPROC)Trigger_getDelayCount,                             // index: 45
   (FARPROC)Trigger_setDelayCount,                             // index: 46
   // PortDirection -----------------------------------------------------------------------
   (FARPROC)PortDirection_getPort,                             // index: 47
   (FARPROC)PortDirection_getDirection,                        // index: 48
   (FARPROC)PortDirection_setDirection,                        // index: 49
   // NoiseFilterChannel ------------------------------------------------------------------
   (FARPROC)NoiseFilterChannel_getChannel,                     // index: 50
   (FARPROC)NoiseFilterChannel_getEnabled,                     // index: 51
   (FARPROC)NoiseFilterChannel_setEnabled,                     // index: 52
   // DiintChannel ------------------------------------------------------------------------
   (FARPROC)DiintChannel_getChannel,                           // index: 53
   (FARPROC)DiintChannel_getEnabled,                           // index: 54
   (FARPROC)DiintChannel_setEnabled,                           // index: 55
   (FARPROC)DiintChannel_getGated,                             // index: 56
   (FARPROC)DiintChannel_setGated,                             // index: 57
   (FARPROC)DiintChannel_getTrigEdge,                          // index: 58
   (FARPROC)DiintChannel_setTrigEdge,                          // index: 59
   // DiCosintPort ------------------------------------------------------------------------
   (FARPROC)DiCosintPort_getPort,                              // index: 60
   (FARPROC)DiCosintPort_getMask,                              // index: 61
   (FARPROC)DiCosintPort_setMask,                              // index: 62
   // DiPmintPort -------------------------------------------------------------------------
   (FARPROC)DiPmintPort_getPort,                               // index: 63
   (FARPROC)DiPmintPort_getMask,                               // index: 64
   (FARPROC)DiPmintPort_setMask,                               // index: 65
   (FARPROC)DiPmintPort_getPattern,                            // index: 66
   (FARPROC)DiPmintPort_setPattern,                            // index: 67
   // ScanPort -------------------------------------------------------------------------
   (FARPROC)ScanPort_getPortStart,                             // index: 68
   (FARPROC)ScanPort_setPortStart,                             // index: 69
   (FARPROC)ScanPort_getPortCount,                             // index: 70
   (FARPROC)ScanPort_setPortCount,                             // index: 71
   (FARPROC)ScanPort_getSamples,                               // index: 72
   (FARPROC)ScanPort_setSamples,                               // index: 73
   (FARPROC)ScanPort_getIntervalCount,                         // index: 74
   (FARPROC)ScanPort_setIntervalCount,                         // index: 75
   // AI features -------------------------------------------------------------------------
   (FARPROC)AiFeatures_getResolution,                          // index: 76
   (FARPROC)AiFeatures_getDataSize,                            // index: 77
   (FARPROC)AiFeatures_getDataMask,                            // index: 78
   (FARPROC)AiFeatures_getChannelCountMax,                     // index: 79
   (FARPROC)AiFeatures_getChannelType,                         // index: 80
   (FARPROC)AiFeatures_getOverallValueRange,                   // index: 81
   (FARPROC)AiFeatures_getThermoSupported,                     // index: 82
   (FARPROC)AiFeatures_getValueRanges,                         // index: 83
   (FARPROC)AiFeatures_getBurnoutReturnTypes,                  // index: 84
   (FARPROC)AiFeatures_getCjcChannels,                         // index: 85
   (FARPROC)AiFeatures_getBufferedAiSupported,                 // index: 86
   (FARPROC)AiFeatures_getSamplingMethod,                      // index: 87
   (FARPROC)AiFeatures_getChannelStartBase,                    // index: 88
   (FARPROC)AiFeatures_getChannelCountBase,                    // index: 89
   (FARPROC)AiFeatures_getConvertClockSources,                 // index: 90
   (FARPROC)AiFeatures_getConvertClockRange,                   // index: 91
   (FARPROC)AiFeatures_getBurstScanSupported,                  // index: 92
   (FARPROC)AiFeatures_getScanClockSources,                    // index: 93
   (FARPROC)AiFeatures_getScanClockRange,                      // index: 94
   (FARPROC)AiFeatures_getScanCountMax,                        // index: 95
   (FARPROC)AiFeatures_getTriggerSupported,                    // index: 96
   (FARPROC)AiFeatures_getTriggerCount,                        // index: 97
   (FARPROC)AiFeatures_getTriggerSources,                      // index: 98
   (FARPROC)AiFeatures_getTriggerActions,                      // index: 99
   (FARPROC)AiFeatures_getTriggerDelayRange,                   // index: 100
   (FARPROC)AiFeatures_getTrigger1Supported,                   // index: 101
   (FARPROC)AiFeatures_getTrigger1Sources,                     // index: 102
   (FARPROC)AiFeatures_getTrigger1Actions,                     // index: 103
   (FARPROC)AiFeatures_getTrigger1DelayRange,                  // index: 104
   // InstantAICtrl -----------------------------------------------------------------------
   (FARPROC)InstantAiCtrl_Dispose,                             // index: 105
   (FARPROC)InstantAiCtrl_Cleanup,                             // index: 106
   (FARPROC)InstantAiCtrl_UpdateProperties,                    // index: 107
   (FARPROC)InstantAiCtrl_addRemovedListener,                  // index: 108
   (FARPROC)InstantAiCtrl_removeRemovedListener,               // index: 109
   (FARPROC)InstantAiCtrl_addReconnectedListener,              // index: 110
   (FARPROC)InstantAiCtrl_removeReconnectedListener,           // index: 111
   (FARPROC)InstantAiCtrl_addPropertyChangedListener,          // index: 112
   (FARPROC)InstantAiCtrl_removePropertyChangedListener,       // index: 113
   (FARPROC)InstantAiCtrl_getSelectedDevice,                   // index: 114
   (FARPROC)InstantAiCtrl_setSelectedDevice,                   // index: 115
   (FARPROC)InstantAiCtrl_getInitialized,                      // index: 116
   (FARPROC)InstantAiCtrl_getCanEditProperty,                  // index: 117
   (FARPROC)InstantAiCtrl_getDevice,                           // index: 118
   (FARPROC)InstantAiCtrl_getModule,                           // index: 119
   (FARPROC)InstantAiCtrl_getSupportedDevices,                 // index: 120
   (FARPROC)InstantAiCtrl_getSupportedModes,                   // index: 121
   (FARPROC)InstantAiCtrl_getFeatures,                         // index: 122
   (FARPROC)InstantAiCtrl_getChannels,                         // index: 123
   (FARPROC)InstantAiCtrl_getChannelCount,                     // index: 124
   (FARPROC)InstantAiCtrl_ReadAny,                             // index: 125
   (FARPROC)InstantAiCtrl_getCjc,                              // index: 126
   // BufferedAiCtrl ----------------------------------------------------------------------
   (FARPROC)BufferedAiCtrl_Dispose,                            // index: 127
   (FARPROC)BufferedAiCtrl_Cleanup,                            // index: 128
   (FARPROC)BufferedAiCtrl_UpdateProperties,                   // index: 129
   (FARPROC)BufferedAiCtrl_addRemovedListener,                 // index: 130
   (FARPROC)BufferedAiCtrl_removeRemovedListener,              // index: 131
   (FARPROC)BufferedAiCtrl_addReconnectedListener,             // index: 132
   (FARPROC)BufferedAiCtrl_removeReconnectedListener,          // index: 133
   (FARPROC)BufferedAiCtrl_addPropertyChangedListener,         // index: 134
   (FARPROC)BufferedAiCtrl_removePropertyChangedListener,      // index: 135
   (FARPROC)BufferedAiCtrl_getSelectedDevice,                  // index: 136
   (FARPROC)BufferedAiCtrl_setSelectedDevice,                  // index: 137
   (FARPROC)BufferedAiCtrl_getInitialized,                     // index: 138
   (FARPROC)BufferedAiCtrl_getCanEditProperty,                 // index: 139
   (FARPROC)BufferedAiCtrl_getDevice,                          // index: 140
   (FARPROC)BufferedAiCtrl_getModule,                          // index: 141
   (FARPROC)BufferedAiCtrl_getSupportedDevices,                // index: 142
   (FARPROC)BufferedAiCtrl_getSupportedModes,                  // index: 143
   (FARPROC)BufferedAiCtrl_getFeatures,                        // index: 144
   (FARPROC)BufferedAiCtrl_getChannels,                        // index: 145
   (FARPROC)BufferedAiCtrl_getChannelCount,                    // index: 146
   (FARPROC)BufferedAiCtrl_addDataReadyListener,               // index: 147
   (FARPROC)BufferedAiCtrl_removeDataReadyListener,            // index: 148
   (FARPROC)BufferedAiCtrl_addOverrunListener,                 // index: 149
   (FARPROC)BufferedAiCtrl_removeOverrunListener,              // index: 150
   (FARPROC)BufferedAiCtrl_addCacheOverflowListener,           // index: 151
   (FARPROC)BufferedAiCtrl_removeCacheOverflowListener,        // index: 152
   (FARPROC)BufferedAiCtrl_addStoppedListener,                 // index: 153
   (FARPROC)BufferedAiCtrl_removeStoppedListener,              // index: 154
   (FARPROC)BufferedAiCtrl_Prepare,                            // index: 155
   (FARPROC)BufferedAiCtrl_RunOnce,                            // index: 156
   (FARPROC)BufferedAiCtrl_Start,                              // index: 157
   (FARPROC)BufferedAiCtrl_Stop,                               // index: 158
   (FARPROC)BufferedAiCtrl_GetDataI16,                         // index: 159
   (FARPROC)BufferedAiCtrl_GetDataI32,                         // index: 160
   (FARPROC)BufferedAiCtrl_GetDataF64,                         // index: 161
   (FARPROC)BufferedAiCtrl_Release,                            // index: 162
   (FARPROC)BufferedAiCtrl_getBuffer,                          // index: 163
   (FARPROC)BufferedAiCtrl_getBufferCapacity,                  // index: 164
   (FARPROC)BufferedAiCtrl_getState,                           // index: 165
   (FARPROC)BufferedAiCtrl_getScanChannel,                     // index: 166
   (FARPROC)BufferedAiCtrl_getConvertClock,                    // index: 167
   (FARPROC)BufferedAiCtrl_getScanClock,                       // index: 168
   (FARPROC)BufferedAiCtrl_getTrigger,                         // index: 169
   (FARPROC)BufferedAiCtrl_getStreaming,                       // index: 170
   (FARPROC)BufferedAiCtrl_setStreaming,                       // index: 171
   (FARPROC)BufferedAiCtrl_GetEventStatus,                     // index: 172
   (FARPROC)BufferedAiCtrl_getTrigger1,                        // index: 173
   // AO features -------------------------------------------------------------------------
   (FARPROC)AoFeatures_getResolution,                          // index: 174
   (FARPROC)AoFeatures_getDataSize,                            // index: 175
   (FARPROC)AoFeatures_getDataMask,                            // index: 176
   (FARPROC)AoFeatures_getChannelCountMax,                     // index: 177
   (FARPROC)AoFeatures_getValueRanges,                         // index: 178
   (FARPROC)AoFeatures_getExternalRefAntiPolar,                // index: 179
   (FARPROC)AoFeatures_getExternalRefRange,                    // index: 180
   (FARPROC)AoFeatures_getBufferedAoSupported,                 // index: 181
   (FARPROC)AoFeatures_getSamplingMethod,                      // index: 182
   (FARPROC)AoFeatures_getChannelStartBase,                    // index: 183
   (FARPROC)AoFeatures_getChannelCountBase,                    // index: 184
   (FARPROC)AoFeatures_getConvertClockSources,                 // index: 185
   (FARPROC)AoFeatures_getConvertClockRange,                   // index: 186
   (FARPROC)AoFeatures_getTriggerSupported,                    // index: 187
   (FARPROC)AoFeatures_getTriggerCount,                        // index: 188
   (FARPROC)AoFeatures_getTriggerSources,                      // index: 189
   (FARPROC)AoFeatures_getTriggerActions,                      // index: 190
   (FARPROC)AoFeatures_getTriggerDelayRange,                   // index: 191
   (FARPROC)AoFeatures_getTrigger1Supported,                   // index: 192
   (FARPROC)AoFeatures_getTrigger1Sources,                     // index: 193
   (FARPROC)AoFeatures_getTrigger1Actions,                     // index: 194
   (FARPROC)AoFeatures_getTrigger1DelayRange,                  // index: 195
   // InstantAoCtrl -----------------------------------------------------------------------
   (FARPROC)InstantAoCtrl_Dispose,                             // index: 196
   (FARPROC)InstantAoCtrl_Cleanup,                             // index: 197
   (FARPROC)InstantAoCtrl_UpdateProperties,                    // index: 198
   (FARPROC)InstantAoCtrl_addRemovedListener,                  // index: 199
   (FARPROC)InstantAoCtrl_removeRemovedListener,               // index: 200
   (FARPROC)InstantAoCtrl_addReconnectedListener,              // index: 201
   (FARPROC)InstantAoCtrl_removeReconnectedListener,           // index: 202
   (FARPROC)InstantAoCtrl_addPropertyChangedListener,          // index: 203
   (FARPROC)InstantAoCtrl_removePropertyChangedListener,       // index: 204
   (FARPROC)InstantAoCtrl_getSelectedDevice,                   // index: 205
   (FARPROC)InstantAoCtrl_setSelectedDevice,                   // index: 206
   (FARPROC)InstantAoCtrl_getInitialized,                      // index: 207
   (FARPROC)InstantAoCtrl_getCanEditProperty,                  // index: 208
   (FARPROC)InstantAoCtrl_getDevice,                           // index: 209
   (FARPROC)InstantAoCtrl_getModule,                           // index: 210
   (FARPROC)InstantAoCtrl_getSupportedDevices,                 // index: 211
   (FARPROC)InstantAoCtrl_getSupportedModes,                   // index: 212
   (FARPROC)InstantAoCtrl_getFeatures,                         // index: 213
   (FARPROC)InstantAoCtrl_getChannels,                         // index: 214
   (FARPROC)InstantAoCtrl_getChannelCount,                     // index: 215
   (FARPROC)InstantAoCtrl_getExtRefValueForUnipolar,           // index: 216
   (FARPROC)InstantAoCtrl_setExtRefValueForUnipolar,           // index: 217
   (FARPROC)InstantAoCtrl_getExtRefValueForBipolar,            // index: 218
   (FARPROC)InstantAoCtrl_setExtRefValueForBipolar,            // index: 219
   (FARPROC)InstantAoCtrl_WriteAny,                            // index: 220
   // BufferedAoCtrl ----------------------------------------------------------------------
   (FARPROC)BufferedAoCtrl_Dispose,                            // index: 221
   (FARPROC)BufferedAoCtrl_Cleanup,                            // index: 222
   (FARPROC)BufferedAoCtrl_UpdateProperties,                   // index: 223
   (FARPROC)BufferedAoCtrl_addRemovedListener,                 // index: 224
   (FARPROC)BufferedAoCtrl_removeRemovedListener,              // index: 225
   (FARPROC)BufferedAoCtrl_addReconnectedListener,             // index: 226
   (FARPROC)BufferedAoCtrl_removeReconnectedListener,          // index: 227
   (FARPROC)BufferedAoCtrl_addPropertyChangedListener,         // index: 228
   (FARPROC)BufferedAoCtrl_removePropertyChangedListener,      // index: 229
   (FARPROC)BufferedAoCtrl_getSelectedDevice,                  // index: 230
   (FARPROC)BufferedAoCtrl_setSelectedDevice,                  // index: 231
   (FARPROC)BufferedAoCtrl_getInitialized,                     // index: 232
   (FARPROC)BufferedAoCtrl_getCanEditProperty,                 // index: 233
   (FARPROC)BufferedAoCtrl_getDevice,                          // index: 234
   (FARPROC)BufferedAoCtrl_getModule,                          // index: 235
   (FARPROC)BufferedAoCtrl_getSupportedDevices,                // index: 236
   (FARPROC)BufferedAoCtrl_getSupportedModes,                  // index: 237
   (FARPROC)BufferedAoCtrl_getFeatures,                        // index: 238
   (FARPROC)BufferedAoCtrl_getChannels,                        // index: 239
   (FARPROC)BufferedAoCtrl_getChannelCount,                    // index: 240
   (FARPROC)BufferedAoCtrl_getExtRefValueForUnipolar,          // index: 241
   (FARPROC)BufferedAoCtrl_setExtRefValueForUnipolar,          // index: 242
   (FARPROC)BufferedAoCtrl_getExtRefValueForBipolar,           // index: 243
   (FARPROC)BufferedAoCtrl_setExtRefValueForBipolar,           // index: 244
   (FARPROC)BufferedAoCtrl_addDataTransmittedListener,         // index: 245
   (FARPROC)BufferedAoCtrl_removeDataTransmittedListener,      // index: 246
   (FARPROC)BufferedAoCtrl_addUnderrunListener,                // index: 247
   (FARPROC)BufferedAoCtrl_removeUnderrunListener,             // index: 248
   (FARPROC)BufferedAoCtrl_addCacheEmptiedListener,            // index: 249
   (FARPROC)BufferedAoCtrl_removeCacheEmptiedListener,         // index: 250
   (FARPROC)BufferedAoCtrl_addTransitStoppedListener,          // index: 251
   (FARPROC)BufferedAoCtrl_removeTransitStoppedListener,       // index: 252
   (FARPROC)BufferedAoCtrl_addStoppedListener,                 // index: 253
   (FARPROC)BufferedAoCtrl_removeStoppedListener,              // index: 254
   (FARPROC)BufferedAoCtrl_Prepare,                            // index: 255
   (FARPROC)BufferedAoCtrl_RunOnce,                            // index: 256
   (FARPROC)BufferedAoCtrl_Start,                              // index: 257
   (FARPROC)BufferedAoCtrl_Stop,                               // index: 258
   (FARPROC)BufferedAoCtrl_SetDataI16,                         // index: 259
   (FARPROC)BufferedAoCtrl_SetDataI32,                         // index: 260
   (FARPROC)BufferedAoCtrl_SetDataF64,                         // index: 261
   (FARPROC)BufferedAoCtrl_Release,                            // index: 262
   (FARPROC)BufferedAoCtrl_getBuffer,                          // index: 263
   (FARPROC)BufferedAoCtrl_getBufferCapacity,                  // index: 264
   (FARPROC)BufferedAoCtrl_getState,                           // index: 265
   (FARPROC)BufferedAoCtrl_getScanChannel,                     // index: 266
   (FARPROC)BufferedAoCtrl_getConvertClock,                    // index: 267
   (FARPROC)BufferedAoCtrl_getTrigger,                         // index: 268
   (FARPROC)BufferedAoCtrl_getStreaming,                       // index: 269
   (FARPROC)BufferedAoCtrl_setStreaming,                       // index: 270
   (FARPROC)BufferedAoCtrl_getTrigger1,                        // index: 271
   // DI features -------------------------------------------------------------------------
   (FARPROC)DiFeatures_getPortProgrammable,                    // index: 272
   (FARPROC)DiFeatures_getPortCount,                           // index: 273
   (FARPROC)DiFeatures_getPortsType,                           // index: 274
   (FARPROC)DiFeatures_getDiSupported,                         // index: 275
   (FARPROC)DiFeatures_getDoSupported,                         // index: 276
   (FARPROC)DiFeatures_getChannelCountMax,                     // index: 277
   (FARPROC)DiFeatures_getDataMask,                            // index: 278
   (FARPROC)DiFeatures_getNoiseFilterSupported,                // index: 279
   (FARPROC)DiFeatures_getNoiseFilterOfChannels,               // index: 280
   (FARPROC)DiFeatures_getNoiseFilterBlockTimeRange,           // index: 281
   (FARPROC)DiFeatures_getDiintSupported,                      // index: 282
   (FARPROC)DiFeatures_getDiintGateSupported,                  // index: 283
   (FARPROC)DiFeatures_getDiCosintSupported,                   // index: 284
   (FARPROC)DiFeatures_getDiPmintSupported,                    // index: 285
   (FARPROC)DiFeatures_getDiintTriggerEdges,                   // index: 286
   (FARPROC)DiFeatures_getDiintOfChannels,                     // index: 287
   (FARPROC)DiFeatures_getDiintGateOfChannels,                 // index: 288
   (FARPROC)DiFeatures_getDiCosintOfPorts,                     // index: 289
   (FARPROC)DiFeatures_getDiPmintOfPorts,                      // index: 290
   (FARPROC)DiFeatures_getSnapEventSources,                    // index: 291
   (FARPROC)DiFeatures_getBufferedDiSupported,                 // index: 292
   (FARPROC)DiFeatures_getSamplingMethod,                      // index: 293
   (FARPROC)DiFeatures_getConvertClockSources,                 // index: 294
   (FARPROC)DiFeatures_getConvertClockRange,                   // index: 295
   (FARPROC)DiFeatures_getBurstScanSupported,                  // index: 296
   (FARPROC)DiFeatures_getScanClockSources,                    // index: 297
   (FARPROC)DiFeatures_getScanClockRange,                      // index: 298
   (FARPROC)DiFeatures_getScanCountMax,                        // index: 299
   (FARPROC)DiFeatures_getTriggerSupported,                    // index: 300
   (FARPROC)DiFeatures_getTriggerCount,                        // index: 301
   (FARPROC)DiFeatures_getTriggerSources,                      // index: 302
   (FARPROC)DiFeatures_getTriggerActions,                      // index: 303
   (FARPROC)DiFeatures_getTriggerDelayRange,                   // index: 304
   // InstantDiCtrl -----------------------------------------------------------------------
   (FARPROC)InstantDiCtrl_Dispose,                             // index: 305
   (FARPROC)InstantDiCtrl_Cleanup,                             // index: 306
   (FARPROC)InstantDiCtrl_UpdateProperties,                    // index: 307
   (FARPROC)InstantDiCtrl_addRemovedListener,                  // index: 308
   (FARPROC)InstantDiCtrl_removeRemovedListener,               // index: 309
   (FARPROC)InstantDiCtrl_addReconnectedListener,              // index: 310
   (FARPROC)InstantDiCtrl_removeReconnectedListener,           // index: 311
   (FARPROC)InstantDiCtrl_addPropertyChangedListener,          // index: 312
   (FARPROC)InstantDiCtrl_removePropertyChangedListener,       // index: 313
   (FARPROC)InstantDiCtrl_getSelectedDevice,                   // index: 314
   (FARPROC)InstantDiCtrl_setSelectedDevice,                   // index: 315
   (FARPROC)InstantDiCtrl_getInitialized,                      // index: 316
   (FARPROC)InstantDiCtrl_getCanEditProperty,                  // index: 317
   (FARPROC)InstantDiCtrl_getDevice,                           // index: 318
   (FARPROC)InstantDiCtrl_getModule,                           // index: 319
   (FARPROC)InstantDiCtrl_getSupportedDevices,                 // index: 320
   (FARPROC)InstantDiCtrl_getSupportedModes,                   // index: 321
   (FARPROC)InstantDiCtrl_getPortCount,                        // index: 322
   (FARPROC)InstantDiCtrl_getPortDirection,                    // index: 323
   (FARPROC)InstantDiCtrl_getFeatures,                         // index: 324
   (FARPROC)InstantDiCtrl_getNoiseFilter,                      // index: 325
   (FARPROC)InstantDiCtrl_addInterruptListener,                // index: 326
   (FARPROC)InstantDiCtrl_removeInterruptListener,             // index: 327
   (FARPROC)InstantDiCtrl_addChangeOfStateListener,            // index: 328
   (FARPROC)InstantDiCtrl_removeChangeOfStateListener,         // index: 329
   (FARPROC)InstantDiCtrl_addPatternMatchListener,             // index: 330
   (FARPROC)InstantDiCtrl_removePatternMatchListener,          // index: 331
   (FARPROC)InstantDiCtrl_ReadAny,                             // index: 332
   (FARPROC)InstantDiCtrl_SnapStart,                           // index: 333
   (FARPROC)InstantDiCtrl_SnapStop,                            // index: 334
   (FARPROC)InstantDiCtrl_getDiintChannels,                    // index: 335
   (FARPROC)InstantDiCtrl_getDiCosintPorts,                    // index: 336
   (FARPROC)InstantDiCtrl_getDiPmintPorts,                     // index: 337
   // BufferedDiCtrl ----------------------------------------------------------------------
   (FARPROC)BufferedDiCtrl_Dispose,                            // index: 338
   (FARPROC)BufferedDiCtrl_Cleanup,                            // index: 339
   (FARPROC)BufferedDiCtrl_UpdateProperties,                   // index: 340
   (FARPROC)BufferedDiCtrl_addRemovedListener,                 // index: 341
   (FARPROC)BufferedDiCtrl_removeRemovedListener,              // index: 342
   (FARPROC)BufferedDiCtrl_addReconnectedListener,             // index: 343
   (FARPROC)BufferedDiCtrl_removeReconnectedListener,          // index: 344
   (FARPROC)BufferedDiCtrl_addPropertyChangedListener,         // index: 345
   (FARPROC)BufferedDiCtrl_removePropertyChangedListener,      // index: 346
   (FARPROC)BufferedDiCtrl_getSelectedDevice,                  // index: 347
   (FARPROC)BufferedDiCtrl_setSelectedDevice,                  // index: 348
   (FARPROC)BufferedDiCtrl_getInitialized,                     // index: 349
   (FARPROC)BufferedDiCtrl_getCanEditProperty,                 // index: 350
   (FARPROC)BufferedDiCtrl_getDevice,                          // index: 351
   (FARPROC)BufferedDiCtrl_getModule,                          // index: 352
   (FARPROC)BufferedDiCtrl_getSupportedDevices,                // index: 353
   (FARPROC)BufferedDiCtrl_getSupportedModes,                  // index: 354
   (FARPROC)BufferedDiCtrl_getPortCount,                       // index: 355
   (FARPROC)BufferedDiCtrl_getPortDirection,                   // index: 356
   (FARPROC)BufferedDiCtrl_getFeatures,                        // index: 357
   (FARPROC)BufferedDiCtrl_getNoiseFilter,                     // index: 358
   (FARPROC)BufferedDiCtrl_addDataReadyListener,               // index: 359
   (FARPROC)BufferedDiCtrl_removeDataReadyListener,            // index: 360
   (FARPROC)BufferedDiCtrl_addOverrunListener,                 // index: 361
   (FARPROC)BufferedDiCtrl_removeOverrunListener,              // index: 362
   (FARPROC)BufferedDiCtrl_addCacheOverflowListener,           // index: 363
   (FARPROC)BufferedDiCtrl_removeCacheOverflowListener,        // index: 364
   (FARPROC)BufferedDiCtrl_addStoppedListener,                 // index: 365
   (FARPROC)BufferedDiCtrl_removeStoppedListener,              // index: 366
   (FARPROC)BufferedDiCtrl_Prepare,                            // index: 367
   (FARPROC)BufferedDiCtrl_RunOnce,                            // index: 368
   (FARPROC)BufferedDiCtrl_Start,                              // index: 369
   (FARPROC)BufferedDiCtrl_Stop,                               // index: 370
   (FARPROC)BufferedDiCtrl_GetData,                            // index: 371
   (FARPROC)BufferedDiCtrl_Release,                            // index: 372
   (FARPROC)BufferedDiCtrl_getBuffer,                          // index: 373
   (FARPROC)BufferedDiCtrl_getBufferCapacity,                  // index: 374
   (FARPROC)BufferedDiCtrl_getState,                           // index: 375
   (FARPROC)BufferedDiCtrl_getScanPort,                        // index: 376
   (FARPROC)BufferedDiCtrl_getConvertClock,                    // index: 377
   (FARPROC)BufferedDiCtrl_getScanClock,                       // index: 378
   (FARPROC)BufferedDiCtrl_getTrigger,                         // index: 379
   (FARPROC)BufferedDiCtrl_getStreaming,                       // index: 380
   (FARPROC)BufferedDiCtrl_setStreaming,                       // index: 381
   // DO features -------------------------------------------------------------------------
   (FARPROC)DoFeatures_getPortProgrammable,                    // index: 382
   (FARPROC)DoFeatures_getPortCount,                           // index: 383
   (FARPROC)DoFeatures_getPortsType,                           // index: 384
   (FARPROC)DoFeatures_getDiSupported,                         // index: 385
   (FARPROC)DoFeatures_getDoSupported,                         // index: 386
   (FARPROC)DoFeatures_getChannelCountMax,                     // index: 387
   (FARPROC)DoFeatures_getDataMask,                            // index: 388
   (FARPROC)DoFeatures_getDoFreezeSignalSources,               // index: 389
   (FARPROC)DoFeatures_getDoReflectWdtFeedIntervalRange,       // index: 390
   (FARPROC)DoFeatures_getBufferedDoSupported,                 // index: 391
   (FARPROC)DoFeatures_getSamplingMethod,                      // index: 392
   (FARPROC)DoFeatures_getConvertClockSources,                 // index: 393
   (FARPROC)DoFeatures_getConvertClockRange,                   // index: 394
   (FARPROC)DoFeatures_getBurstScanSupported,                  // index: 395
   (FARPROC)DoFeatures_getScanClockSources,                    // index: 396
   (FARPROC)DoFeatures_getScanClockRange,                      // index: 397
   (FARPROC)DoFeatures_getScanCountMax,                        // index: 398
   (FARPROC)DoFeatures_getTriggerSupported,                    // index: 399
   (FARPROC)DoFeatures_getTriggerCount,                        // index: 400
   (FARPROC)DoFeatures_getTriggerSources,                      // index: 401
   (FARPROC)DoFeatures_getTriggerActions,                      // index: 402
   (FARPROC)DoFeatures_getTriggerDelayRange,                   // index: 403
   // InstantDoCtrl -----------------------------------------------------------------------
   (FARPROC)InstantDoCtrl_Dispose,                             // index: 404
   (FARPROC)InstantDoCtrl_Cleanup,                             // index: 405
   (FARPROC)InstantDoCtrl_UpdateProperties,                    // index: 406
   (FARPROC)InstantDoCtrl_addRemovedListener,                  // index: 407
   (FARPROC)InstantDoCtrl_removeRemovedListener,               // index: 408
   (FARPROC)InstantDoCtrl_addReconnectedListener,              // index: 409
   (FARPROC)InstantDoCtrl_removeReconnectedListener,           // index: 410
   (FARPROC)InstantDoCtrl_addPropertyChangedListener,          // index: 411
   (FARPROC)InstantDoCtrl_removePropertyChangedListener,       // index: 412
   (FARPROC)InstantDoCtrl_getSelectedDevice,                   // index: 413
   (FARPROC)InstantDoCtrl_setSelectedDevice,                   // index: 414
   (FARPROC)InstantDoCtrl_getInitialized,                      // index: 415
   (FARPROC)InstantDoCtrl_getCanEditProperty,                  // index: 416
   (FARPROC)InstantDoCtrl_getDevice,                           // index: 417
   (FARPROC)InstantDoCtrl_getModule,                           // index: 418
   (FARPROC)InstantDoCtrl_getSupportedDevices,                 // index: 419
   (FARPROC)InstantDoCtrl_getSupportedModes,                   // index: 420
   (FARPROC)InstantDoCtrl_getPortCount,                        // index: 421
   (FARPROC)InstantDoCtrl_getPortDirection,                    // index: 422
   (FARPROC)InstantDoCtrl_getFeatures,                         // index: 423
   (FARPROC)InstantDoCtrl_WriteAny,                            // index: 424
   (FARPROC)InstantDoCtrl_ReadAny,                             // index: 425
   // BufferedDoCtrl -----------------------------------------------------------------------
   (FARPROC)BufferedDoCtrl_Dispose,                            // index: 426
   (FARPROC)BufferedDoCtrl_Cleanup,                            // index: 427
   (FARPROC)BufferedDoCtrl_UpdateProperties,                   // index: 428
   (FARPROC)BufferedDoCtrl_addRemovedListener,                 // index: 429
   (FARPROC)BufferedDoCtrl_removeRemovedListener,              // index: 430
   (FARPROC)BufferedDoCtrl_addReconnectedListener,             // index: 431
   (FARPROC)BufferedDoCtrl_removeReconnectedListener,          // index: 432
   (FARPROC)BufferedDoCtrl_addPropertyChangedListener,         // index: 433
   (FARPROC)BufferedDoCtrl_removePropertyChangedListener,      // index: 434
   (FARPROC)BufferedDoCtrl_getSelectedDevice,                  // index: 435
   (FARPROC)BufferedDoCtrl_setSelectedDevice,                  // index: 436
   (FARPROC)BufferedDoCtrl_getInitialized,                     // index: 437
   (FARPROC)BufferedDoCtrl_getCanEditProperty,                 // index: 438
   (FARPROC)BufferedDoCtrl_getDevice,                          // index: 439
   (FARPROC)BufferedDoCtrl_getModule,                          // index: 440
   (FARPROC)BufferedDoCtrl_getSupportedDevices,                // index: 441
   (FARPROC)BufferedDoCtrl_getSupportedModes,                  // index: 442
   (FARPROC)BufferedDoCtrl_getPortCount,                       // index: 443
   (FARPROC)BufferedDoCtrl_getPortDirection,                   // index: 444
   (FARPROC)BufferedDoCtrl_getFeatures,                        // index: 445
   (FARPROC)BufferedDoCtrl_addDataTransmittedListener,         // index: 446
   (FARPROC)BufferedDoCtrl_removeDataTransmittedListener,      // index: 447
   (FARPROC)BufferedDoCtrl_addUnderrunListener,                // index: 448
   (FARPROC)BufferedDoCtrl_removeUnderrunListener,             // index: 449
   (FARPROC)BufferedDoCtrl_addCacheEmptiedListener,            // index: 450
   (FARPROC)BufferedDoCtrl_removeCacheEmptiedListener,         // index: 451
   (FARPROC)BufferedDoCtrl_addTransitStoppedListener,          // index: 452
   (FARPROC)BufferedDoCtrl_removeTransitStoppedListener,       // index: 453
   (FARPROC)BufferedDoCtrl_addStoppedListener,                 // index: 454
   (FARPROC)BufferedDoCtrl_removeStoppedListener,              // index: 455
   (FARPROC)BufferedDoCtrl_Prepare,                            // index: 456
   (FARPROC)BufferedDoCtrl_RunOnce,                            // index: 457
   (FARPROC)BufferedDoCtrl_Start,                              // index: 458
   (FARPROC)BufferedDoCtrl_Stop,                               // index: 459
   (FARPROC)BufferedDoCtrl_SetData,                            // index: 460
   (FARPROC)BufferedDoCtrl_Release,                            // index: 461
   (FARPROC)BufferedDoCtrl_getBuffer,                          // index: 462
   (FARPROC)BufferedDoCtrl_getBufferCapacity,                  // index: 463
   (FARPROC)BufferedDoCtrl_getState,                           // index: 464
   (FARPROC)BufferedDoCtrl_getScanPort,                        // index: 465
   (FARPROC)BufferedDoCtrl_getConvertClock,                    // index: 466
   (FARPROC)BufferedDoCtrl_getTrigger,                         // index: 467
   (FARPROC)BufferedDoCtrl_getStreaming,                       // index: 468
   (FARPROC)BufferedDoCtrl_setStreaming,                       // index: 469
   // Counter Capability Indexer ------------------------------------------------------------
   (FARPROC)CounterCapabilityIndexer_Dispose,                  // index: 470
   (FARPROC)CounterCapabilityIndexer_getCount,                 // index: 471
   (FARPROC)CounterCapabilityIndexer_getItem,                  // index: 472
   // Event Counter Features ----------------------------------------------------------------
   (FARPROC)EventCounterFeatures_getChannelCountMax,           // index: 473
   (FARPROC)EventCounterFeatures_getResolution,                // index: 474
   (FARPROC)EventCounterFeatures_getDataSize,                  // index: 475
   (FARPROC)EventCounterFeatures_getCapabilities,              // index: 476
   (FARPROC)EventCounterFeatures_getNoiseFilterSupported,      // index: 477
   (FARPROC)EventCounterFeatures_getNoiseFilterOfChannels,     // index: 478
   (FARPROC)EventCounterFeatures_getNoiseFilterBlockTimeRange, // index: 479
   // EventCounterCtrl ----------------------------------------------------------------
   (FARPROC)EventCounterCtrl_Dispose,                          // index: 480
   (FARPROC)EventCounterCtrl_Cleanup,                          // index: 481
   (FARPROC)EventCounterCtrl_UpdateProperties,                 // index: 482
   (FARPROC)EventCounterCtrl_addRemovedListener,               // index: 483
   (FARPROC)EventCounterCtrl_removeRemovedListener,            // index: 484
   (FARPROC)EventCounterCtrl_addReconnectedListener,           // index: 485
   (FARPROC)EventCounterCtrl_removeReconnectedListener,        // index: 486
   (FARPROC)EventCounterCtrl_addPropertyChangedListener,       // index: 487
   (FARPROC)EventCounterCtrl_removePropertyChangedListener,    // index: 488
   (FARPROC)EventCounterCtrl_getSelectedDevice,                // index: 489
   (FARPROC)EventCounterCtrl_setSelectedDevice,                // index: 490
   (FARPROC)EventCounterCtrl_getInitialized,                   // index: 491
   (FARPROC)EventCounterCtrl_getCanEditProperty,               // index: 492
   (FARPROC)EventCounterCtrl_getDevice,                        // index: 493
   (FARPROC)EventCounterCtrl_getModule,                        // index: 494
   (FARPROC)EventCounterCtrl_getSupportedDevices,              // index: 495
   (FARPROC)EventCounterCtrl_getSupportedModes,                // index: 496
   (FARPROC)EventCounterCtrl_getChannel,                       // index: 497
   (FARPROC)EventCounterCtrl_setChannel,                       // index: 498
   (FARPROC)EventCounterCtrl_getEnabled,                       // index: 499
   (FARPROC)EventCounterCtrl_setEnabled,                       // index: 500
   (FARPROC)EventCounterCtrl_getRunning,                       // index: 501
   (FARPROC)EventCounterCtrl_getNoiseFilter,                   // index: 502
   (FARPROC)EventCounterCtrl_getFeatures,                      // index: 503
   (FARPROC)EventCounterCtrl_getValue,                         // index: 504
   // Frequency meter features -----------------------------------------------------------
   (FARPROC)FreqMeterFeatures_getChannelCountMax,              // index: 505
   (FARPROC)FreqMeterFeatures_getResolution,                   // index: 506
   (FARPROC)FreqMeterFeatures_getDataSize,                     // index: 507
   (FARPROC)FreqMeterFeatures_getCapabilities,                 // index: 508
   (FARPROC)FreqMeterFeatures_getNoiseFilterSupported,         // index: 509
   (FARPROC)FreqMeterFeatures_getNoiseFilterOfChannels,        // index: 510
   (FARPROC)FreqMeterFeatures_getNoiseFilterBlockTimeRange,    // index: 511
   (FARPROC)FreqMeterFeatures_getFmMethods,                    // index: 512
   // FreqMeterCtrl ----------------------------------------------------------------------
   (FARPROC)FreqMeterCtrl_Dispose,                             // index: 513
   (FARPROC)FreqMeterCtrl_Cleanup,                             // index: 514
   (FARPROC)FreqMeterCtrl_UpdateProperties,                    // index: 515
   (FARPROC)FreqMeterCtrl_addRemovedListener,                  // index: 516
   (FARPROC)FreqMeterCtrl_removeRemovedListener,               // index: 517
   (FARPROC)FreqMeterCtrl_addReconnectedListener,              // index: 518
   (FARPROC)FreqMeterCtrl_removeReconnectedListener,           // index: 519
   (FARPROC)FreqMeterCtrl_addPropertyChangedListener,          // index: 520
   (FARPROC)FreqMeterCtrl_removePropertyChangedListener,       // index: 521
   (FARPROC)FreqMeterCtrl_getSelectedDevice,                   // index: 522
   (FARPROC)FreqMeterCtrl_setSelectedDevice,                   // index: 523
   (FARPROC)FreqMeterCtrl_getInitialized,                      // index: 524
   (FARPROC)FreqMeterCtrl_getCanEditProperty,                  // index: 525
   (FARPROC)FreqMeterCtrl_getDevice,                           // index: 526
   (FARPROC)FreqMeterCtrl_getModule,                           // index: 527
   (FARPROC)FreqMeterCtrl_getSupportedDevices,                 // index: 528
   (FARPROC)FreqMeterCtrl_getSupportedModes,                   // index: 529
   (FARPROC)FreqMeterCtrl_getChannel,                          // index: 530
   (FARPROC)FreqMeterCtrl_setChannel,                          // index: 531
   (FARPROC)FreqMeterCtrl_getEnabled,                          // index: 532
   (FARPROC)FreqMeterCtrl_setEnabled,                          // index: 533
   (FARPROC)FreqMeterCtrl_getRunning,                          // index: 534
   (FARPROC)FreqMeterCtrl_getNoiseFilter,                      // index: 535
   (FARPROC)FreqMeterCtrl_getFeatures,                         // index: 536
   (FARPROC)FreqMeterCtrl_getValue,                            // index: 537
   (FARPROC)FreqMeterCtrl_getMethod,                           // index: 538
   (FARPROC)FreqMeterCtrl_setMethod,                           // index: 539
   (FARPROC)FreqMeterCtrl_getCollectionPeriod,                 // index: 540
   (FARPROC)FreqMeterCtrl_setCollectionPeriod,                 // index: 541
   // One shot features ----------------------------------------------------------------------
   (FARPROC)OneShotFeatures_getChannelCountMax,                // index: 542
   (FARPROC)OneShotFeatures_getResolution,                     // index: 543
   (FARPROC)OneShotFeatures_getDataSize,                       // index: 544
   (FARPROC)OneShotFeatures_getCapabilities,                   // index: 545
   (FARPROC)OneShotFeatures_getNoiseFilterSupported,           // index: 546
   (FARPROC)OneShotFeatures_getNoiseFilterOfChannels,          // index: 547
   (FARPROC)OneShotFeatures_getNoiseFilterBlockTimeRange,      // index: 548
   (FARPROC)OneShotFeatures_getDelayCountRange,                // index: 549
   // OneShotCtrl ----------------------------------------------------------------------
   (FARPROC)OneShotCtrl_Dispose,                               // index: 550
   (FARPROC)OneShotCtrl_Cleanup,                               // index: 551
   (FARPROC)OneShotCtrl_UpdateProperties,                      // index: 552
   (FARPROC)OneShotCtrl_addRemovedListener,                    // index: 553
   (FARPROC)OneShotCtrl_removeRemovedListener,                 // index: 554
   (FARPROC)OneShotCtrl_addReconnectedListener,                // index: 555
   (FARPROC)OneShotCtrl_removeReconnectedListener,             // index: 556
   (FARPROC)OneShotCtrl_addPropertyChangedListener,            // index: 557
   (FARPROC)OneShotCtrl_removePropertyChangedListener,         // index: 558
   (FARPROC)OneShotCtrl_getSelectedDevice,                     // index: 559
   (FARPROC)OneShotCtrl_setSelectedDevice,                     // index: 560
   (FARPROC)OneShotCtrl_getInitialized,                        // index: 561
   (FARPROC)OneShotCtrl_getCanEditProperty,                    // index: 562
   (FARPROC)OneShotCtrl_getDevice,                             // index: 563
   (FARPROC)OneShotCtrl_getModule,                             // index: 564
   (FARPROC)OneShotCtrl_getSupportedDevices,                   // index: 565
   (FARPROC)OneShotCtrl_getSupportedModes,                     // index: 566
   (FARPROC)OneShotCtrl_getChannel,                            // index: 567
   (FARPROC)OneShotCtrl_setChannel,                            // index: 568
   (FARPROC)OneShotCtrl_getEnabled,                            // index: 569
   (FARPROC)OneShotCtrl_setEnabled,                            // index: 570
   (FARPROC)OneShotCtrl_getRunning,                            // index: 571
   (FARPROC)OneShotCtrl_getNoiseFilter,                        // index: 572
   (FARPROC)OneShotCtrl_addOneShotListener,                    // index: 573
   (FARPROC)OneShotCtrl_removeOneShotListener,                 // index: 574
   (FARPROC)OneShotCtrl_getFeatures,                           // index: 575
   (FARPROC)OneShotCtrl_getDelayCount,                         // index: 576
   (FARPROC)OneShotCtrl_setDelayCount,                         // index: 577
   // Timer/Pulse features ----------------------------------------------------------------------
   (FARPROC)TimerPulseFeatures_getChannelCountMax,             // index: 578
   (FARPROC)TimerPulseFeatures_getResolution,                  // index: 579
   (FARPROC)TimerPulseFeatures_getDataSize,                    // index: 580
   (FARPROC)TimerPulseFeatures_getCapabilities,                // index: 581
   (FARPROC)TimerPulseFeatures_getNoiseFilterSupported,        // index: 582
   (FARPROC)TimerPulseFeatures_getNoiseFilterOfChannels,       // index: 583
   (FARPROC)TimerPulseFeatures_getNoiseFilterBlockTimeRange,   // index: 584
   (FARPROC)TimerPulseFeatures_getTimerFrequencyRange,         // index: 585
   (FARPROC)TimerPulseFeatures_getTimerEventSupported,         // index: 586
   // TimerPulseCtrl ----------------------------------------------------------------------
   (FARPROC)TimerPulseCtrl_Dispose,                            // index: 587
   (FARPROC)TimerPulseCtrl_Cleanup,                            // index: 588
   (FARPROC)TimerPulseCtrl_UpdateProperties,                   // index: 589
   (FARPROC)TimerPulseCtrl_addRemovedListener,                 // index: 590
   (FARPROC)TimerPulseCtrl_removeRemovedListener,              // index: 591
   (FARPROC)TimerPulseCtrl_addReconnectedListener,             // index: 592
   (FARPROC)TimerPulseCtrl_removeReconnectedListener,          // index: 593
   (FARPROC)TimerPulseCtrl_addPropertyChangedListener,         // index: 594
   (FARPROC)TimerPulseCtrl_removePropertyChangedListener,      // index: 595
   (FARPROC)TimerPulseCtrl_getSelectedDevice,                  // index: 596
   (FARPROC)TimerPulseCtrl_setSelectedDevice,                  // index: 597
   (FARPROC)TimerPulseCtrl_getInitialized,                     // index: 598
   (FARPROC)TimerPulseCtrl_getCanEditProperty,                 // index: 599
   (FARPROC)TimerPulseCtrl_getDevice,                          // index: 600
   (FARPROC)TimerPulseCtrl_getModule,                          // index: 601
   (FARPROC)TimerPulseCtrl_getSupportedDevices,                // index: 602
   (FARPROC)TimerPulseCtrl_getSupportedModes,                  // index: 603
   (FARPROC)TimerPulseCtrl_getChannel,                         // index: 604
   (FARPROC)TimerPulseCtrl_setChannel,                         // index: 605
   (FARPROC)TimerPulseCtrl_getEnabled,                         // index: 606
   (FARPROC)TimerPulseCtrl_setEnabled,                         // index: 607
   (FARPROC)TimerPulseCtrl_getRunning,                         // index: 608
   (FARPROC)TimerPulseCtrl_getNoiseFilter,                     // index: 609
   (FARPROC)TimerPulseCtrl_addTimerTickListener,               // index: 610
   (FARPROC)TimerPulseCtrl_removeTimerTickListener,            // index: 611
   (FARPROC)TimerPulseCtrl_getFeatures,                        // index: 612
   (FARPROC)TimerPulseCtrl_getFrequency,                       // index: 613
   (FARPROC)TimerPulseCtrl_setFrequency,                       // index: 614
   // Pulse width meter features -------------------------------------------------------------
   (FARPROC)PwMeterFeatures_getChannelCountMax,                // index: 615
   (FARPROC)PwMeterFeatures_getResolution,                     // index: 616
   (FARPROC)PwMeterFeatures_getDataSize,                       // index: 617
   (FARPROC)PwMeterFeatures_getCapabilities,                   // index: 618
   (FARPROC)PwMeterFeatures_getNoiseFilterSupported,           // index: 619
   (FARPROC)PwMeterFeatures_getNoiseFilterOfChannels,          // index: 620
   (FARPROC)PwMeterFeatures_getNoiseFilterBlockTimeRange,      // index: 621
   (FARPROC)PwMeterFeatures_getPwmCascadeGroup,                // index: 622
   (FARPROC)PwMeterFeatures_getOverflowEventSupported,         // index: 623
   // PwMeterCtrl ----------------------------------------------------------------------------
   (FARPROC)PwMeterCtrl_Dispose,                               // index: 624
   (FARPROC)PwMeterCtrl_Cleanup,                               // index: 625
   (FARPROC)PwMeterCtrl_UpdateProperties,                      // index: 626
   (FARPROC)PwMeterCtrl_addRemovedListener,                    // index: 627
   (FARPROC)PwMeterCtrl_removeRemovedListener,                 // index: 628
   (FARPROC)PwMeterCtrl_addReconnectedListener,                // index: 629
   (FARPROC)PwMeterCtrl_removeReconnectedListener,             // index: 630
   (FARPROC)PwMeterCtrl_addPropertyChangedListener,            // index: 631
   (FARPROC)PwMeterCtrl_removePropertyChangedListener,         // index: 632
   (FARPROC)PwMeterCtrl_getSelectedDevice,                     // index: 633
   (FARPROC)PwMeterCtrl_setSelectedDevice,                     // index: 634
   (FARPROC)PwMeterCtrl_getInitialized,                        // index: 635
   (FARPROC)PwMeterCtrl_getCanEditProperty,                    // index: 636
   (FARPROC)PwMeterCtrl_getDevice,                             // index: 637
   (FARPROC)PwMeterCtrl_getModule,                             // index: 638
   (FARPROC)PwMeterCtrl_getSupportedDevices,                   // index: 639
   (FARPROC)PwMeterCtrl_getSupportedModes,                     // index: 640
   (FARPROC)PwMeterCtrl_getChannel,                            // index: 641
   (FARPROC)PwMeterCtrl_setChannel,                            // index: 642
   (FARPROC)PwMeterCtrl_getEnabled,                            // index: 643
   (FARPROC)PwMeterCtrl_setEnabled,                            // index: 644
   (FARPROC)PwMeterCtrl_getRunning,                            // index: 645
   (FARPROC)PwMeterCtrl_getNoiseFilter,                        // index: 646
   (FARPROC)PwMeterCtrl_addOverflowListener,                   // index: 647
   (FARPROC)PwMeterCtrl_removeOverflowListener,                // index: 648
   (FARPROC)PwMeterCtrl_getFeatures,                           // index: 649
   (FARPROC)PwMeterCtrl_getValue,                              // index: 650
   // Pulse width modulator features ----------------------------------------------------------
   (FARPROC)PwModulatorFeatures_getChannelCountMax,            // index: 651
   (FARPROC)PwModulatorFeatures_getResolution,                 // index: 652
   (FARPROC)PwModulatorFeatures_getDataSize,                   // index: 653
   (FARPROC)PwModulatorFeatures_getCapabilities,               // index: 654
   (FARPROC)PwModulatorFeatures_getNoiseFilterSupported,       // index: 655
   (FARPROC)PwModulatorFeatures_getNoiseFilterOfChannels,      // index: 656
   (FARPROC)PwModulatorFeatures_getNoiseFilterBlockTimeRange,  // index: 657
   (FARPROC)PwModulatorFeatures_getHiPeriodRange,              // index: 658
   (FARPROC)PwModulatorFeatures_getLoPeriodRange,              // index: 659
   // PwModulatorCtrl ----------------------------------------------------------------------
   (FARPROC)PwModulatorCtrl_Dispose,                           // index: 660
   (FARPROC)PwModulatorCtrl_Cleanup,                           // index: 661
   (FARPROC)PwModulatorCtrl_UpdateProperties,                  // index: 662
   (FARPROC)PwModulatorCtrl_addRemovedListener,                // index: 663
   (FARPROC)PwModulatorCtrl_removeRemovedListener,             // index: 664
   (FARPROC)PwModulatorCtrl_addReconnectedListener,            // index: 665
   (FARPROC)PwModulatorCtrl_removeReconnectedListener,         // index: 666
   (FARPROC)PwModulatorCtrl_addPropertyChangedListener,        // index: 667
   (FARPROC)PwModulatorCtrl_removePropertyChangedListener,     // index: 668
   (FARPROC)PwModulatorCtrl_getSelectedDevice,                 // index: 669
   (FARPROC)PwModulatorCtrl_setSelectedDevice,                 // index: 670
   (FARPROC)PwModulatorCtrl_getInitialized,                    // index: 671
   (FARPROC)PwModulatorCtrl_getCanEditProperty,                // index: 672
   (FARPROC)PwModulatorCtrl_getDevice,                         // index: 673
   (FARPROC)PwModulatorCtrl_getModule,                         // index: 674
   (FARPROC)PwModulatorCtrl_getSupportedDevices,               // index: 675
   (FARPROC)PwModulatorCtrl_getSupportedModes,                 // index: 676
   (FARPROC)PwModulatorCtrl_getChannel,                        // index: 677
   (FARPROC)PwModulatorCtrl_setChannel,                        // index: 678
   (FARPROC)PwModulatorCtrl_getEnabled,                        // index: 679
   (FARPROC)PwModulatorCtrl_setEnabled,                        // index: 680
   (FARPROC)PwModulatorCtrl_getRunning,                        // index: 681
   (FARPROC)PwModulatorCtrl_getNoiseFilter,                    // index: 682
   (FARPROC)PwModulatorCtrl_getFeatures,                       // index: 683
   (FARPROC)PwModulatorCtrl_getPulseWidth,                     // index: 684
   (FARPROC)PwModulatorCtrl_setPulseWidth,                     // index: 685
   // Up-Down counter features -------------------------------------------------------------
   (FARPROC)UdCounterFeatures_getChannelCountMax,              // index: 686
   (FARPROC)UdCounterFeatures_getResolution,                   // index: 687
   (FARPROC)UdCounterFeatures_getDataSize,                     // index: 688
   (FARPROC)UdCounterFeatures_getCapabilities,                 // index: 689
   (FARPROC)UdCounterFeatures_getNoiseFilterSupported,         // index: 690
   (FARPROC)UdCounterFeatures_getNoiseFilterOfChannels,        // index: 691
   (FARPROC)UdCounterFeatures_getNoiseFilterBlockTimeRange,    // index: 692
   (FARPROC)UdCounterFeatures_getCountingTypes,                // index: 693
   (FARPROC)UdCounterFeatures_getInitialValues,                // index: 694
   (FARPROC)UdCounterFeatures_getSnapEventSources,             // index: 695
   // UdCounterCtrl ------------------------------------------------------------------------
   (FARPROC)UdCounterCtrl_Dispose,                             // index: 696
   (FARPROC)UdCounterCtrl_Cleanup,                             // index: 697
   (FARPROC)UdCounterCtrl_UpdateProperties,                    // index: 698
   (FARPROC)UdCounterCtrl_addRemovedListener,                  // index: 699
   (FARPROC)UdCounterCtrl_removeRemovedListener,               // index: 700
   (FARPROC)UdCounterCtrl_addReconnectedListener,              // index: 701
   (FARPROC)UdCounterCtrl_removeReconnectedListener,           // index: 702
   (FARPROC)UdCounterCtrl_addPropertyChangedListener,          // index: 703
   (FARPROC)UdCounterCtrl_removePropertyChangedListener,       // index: 704
   (FARPROC)UdCounterCtrl_getSelectedDevice,                   // index: 705
   (FARPROC)UdCounterCtrl_setSelectedDevice,                   // index: 706
   (FARPROC)UdCounterCtrl_getInitialized,                      // index: 707
   (FARPROC)UdCounterCtrl_getCanEditProperty,                  // index: 708
   (FARPROC)UdCounterCtrl_getDevice,                           // index: 709
   (FARPROC)UdCounterCtrl_getModule,                           // index: 710
   (FARPROC)UdCounterCtrl_getSupportedDevices,                 // index: 711
   (FARPROC)UdCounterCtrl_getSupportedModes,                   // index: 712
   (FARPROC)UdCounterCtrl_getChannel,                          // index: 713
   (FARPROC)UdCounterCtrl_setChannel,                          // index: 714
   (FARPROC)UdCounterCtrl_getEnabled,                          // index: 715
   (FARPROC)UdCounterCtrl_setEnabled,                          // index: 716
   (FARPROC)UdCounterCtrl_getRunning,                          // index: 717
   (FARPROC)UdCounterCtrl_getNoiseFilter,                      // index: 718
   (FARPROC)UdCounterCtrl_addUdCntrEventListener,              // index: 719
   (FARPROC)UdCounterCtrl_removeUdCntrEventListener,           // index: 720
   (FARPROC)UdCounterCtrl_SnapStart,                           // index: 721
   (FARPROC)UdCounterCtrl_SnapStop,                            // index: 722
   (FARPROC)UdCounterCtrl_CompareSetTable,                     // index: 723
   (FARPROC)UdCounterCtrl_CompareSetInterval,                  // index: 724
   (FARPROC)UdCounterCtrl_CompareClear,                        // index: 725
   (FARPROC)UdCounterCtrl_ValueReset,                          // index: 726
   (FARPROC)UdCounterCtrl_getFeatures,                         // index: 727
   (FARPROC)UdCounterCtrl_getValue,                            // index: 728
   (FARPROC)UdCounterCtrl_getCountingType,                     // index: 729
   (FARPROC)UdCounterCtrl_setCountingType,                     // index: 730
   (FARPROC)UdCounterCtrl_getInitialValue,                     // index: 731
   (FARPROC)UdCounterCtrl_setInitialValue,                     // index: 732
   (FARPROC)UdCounterCtrl_getResetTimesByIndex,                // index: 733
   (FARPROC)UdCounterCtrl_setResetTimesByIndex,                // index: 734

   // Add new method ------------------------------------------------------------------------
   (FARPROC)InstantDiCtrl_ReadBit,                             // index: 735
   (FARPROC)InstantDoCtrl_WriteBit,                            // index: 736
   (FARPROC)InstantDoCtrl_ReadBit,                             // index: 737

};                                                           
#endif                                                         
}
